/*
FILE:     pdb_extract.C
*/
/*
VERSION:  3.25
*/
/*
DATE:     3/29/2019
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2019 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               RCSB PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/
/* program to extract information from crystallographic softwares */

#include <stdio.h> //defines three variable types, several macros, and various functions for performing input and output
#include <string.h> //defines one variable type, one macro, and various functions for manipulating arrays of characters.
#include <stdlib.h> //defines four variable types, several macros, and various functions for performing general functions
#include <ctype.h> //declares several functions that are useful for testing and mapping characters.
//All the functions accepts int as a parameter, whose value must be EOF or representable as an unsigned char.
//All the functions return non-zero (true) if the argument c satisfies the condition described, and zero(false) if not.

#include <sys/types.h> //defines a collection of typedef symbols and structures.
#include <unistd.h> //is the C/C++ header file that is your code's entry point to various constant, type and function declarations that comprise the POSIX operating system API
#include <iostream> //like the cstdio header inherited from C's stdio.h, iostream provides basic input and output services for C++ programs. iostream uses the objects cin , cout , cerr , and clog for sending data to and from the standard streams input, output, error 

#include "TableFile.h" //../../tables/include/, Header file for Block and TableFile classes.
#include "CifFile.h" //../../cif-file/include/, Header file for CifFile class.
#include "CifParserBase.h" //../../cif-parser/include/, Header file for CifParser class.
#include "structure.h" //collection of struct definitions
#include "structure-ini.h"  //initiation parameters of cif structure 
#include "harv.h" //header for data harvesting
#include "util.h" //header of util.C, the utilities workhorse module
#include "version.h" //version

using std::cout;
using std::endl;

extern PDBX_VALIDATE_OVERALL pdbx_validate_overall;
extern COMPUTING computing;
extern EXPTL exptl;
extern FILE *FW_LOG;
int SPLIT;
extern char INFO[1000];
const char *REFINE_ID[]= { "X-RAY DIFFRACTION", "NEUTRON DIFFRACTION"  };

void check_log(const char* file);
void final_check(char* outfile);


int main(int argc, char **argv)
{
    char outfile[512],method[80],expt[80], pdbfile[512] ; 
    int  i,nline=0, tem_type=0;    
    FILE *fout=NULL;

    
    printf("\n==============================================================\n");
    printf("|           pdb_extract (version-%s ; %s)        |\n",
          PDB_EXTRACT_VERSION,PDB_EXTRACT_DATE );
    printf("==============================================================\n");
    printf("|You can use the option 'pdb_extract -version' to check if    |\n");
    printf("|the version that you are using is our latest release.        |\n");
    printf("==============================================================\n");


    FW_LOG=fopen("pdb_extract.log", "w"); //open log file object
    
    SPLIT=0;
    
    strcpy(outfile,"");
    //Step 1: Separate NMR/EM
    //Parse experimental method from arguments
    for(i=0; i<argc; i++){
        if(!strcmp_case(argv[i], "-NMR")){
            strcpy(expt,"NMR");
        }else if(!strcmp_case(argv[i], "-EM")){
            strcpy(expt,"EM");
        }else if(i<argc-1 && !strcmp_case(argv[i], "-O")){
            strcpy(outfile,argv[i+1]);
        }else if(!strcmp_case(argv[i], "-version")){
            check_version();
            exit(0);
        }else if(!strcmp_case(argv[i], "-split")){
            SPLIT=1;
        }else if(!strcmp_case(argv[i], "-ipdb")){
            strcpy(pdbfile, argv[i+1]);
        }
    }
    
    if(strlen(outfile) <= 0) strcpy(outfile, "pdb_extract.mmcif");  
    extract_seq (argc, argv); // only for the CCP4i interface

    //Separate proces for NMR and EM (NMR/EM entry process not in the following steps)
    if(!strcmp_case(expt, "NMR")){
        strcpy(exptl.method, "NMR");
        proc_nmr_structure(outfile, argc, argv);
        exit(0); 
    }else if (!strcmp_case(expt, "EM")){
        proc_em_structure(outfile, argc, argv);
        exit(0); 
    }

    
    printf("\nExtracting information about X-RAY structure\n\n");
    strcpy(exptl.method, "X-RAY DIFFRACTION");

    //Step 2: Preprocess Xray
    initialize(); //from pdb2cif, setup certain parameters
    check_input_error(argc, argv); //check enviroments and arguments
    
    //START of outfile process
    fout = fopen(outfile,"w"); //open fout file object that will be passed around from writting

    write_block_method(fout, argc, argv,method); //write data_UNNAMED header first
    pdb_extract_info(fout); //write _data_extraction.extraction_date, _entry.id, and _database_2 cat
    get_software_info(fout, argc, argv); //write _software, parse refinement software version from PDB header or refinement log

    if(!strcmp_case(method,"SIR") || strstr_case(method,"MIR")){
        strcpy(method,"MIR");
    }else{
        strcpy(method,"MAD");
    }
    
    if(SPLIT){get_segid_entity(fout, pdbfile); } //for -split option only, -split is not a parameter for public version
   
    tem_type=read_entity_poly(fout, argc, argv, method); //parse template and write to fout

    //Step 3: Extract
    extraction_new(fout, argc, argv); //process -sp -s -i -v options, extract log files
    extraction(fout, argc, argv, method, "-P",  "PHS"); //process -p option, extract log file
    extraction(fout, argc, argv, method, "-D",  "DM"); //process -d option, extract log file
    extraction(fout, argc, argv, method, "-M",  "MR"); //process -m option, extract log file
    extraction(fout, argc, argv, method, "-R",  "REF"); //process -r option, extract coordinates
    
    write_mmcif(fout, argc, argv); //extract refinement statistics
    fclose(fout); // close fout file object

    //Step 4: Postprocess
    cifparse(outfile, outfile,250); //re-organize outfile 
    reorder_shell(argc, argv, outfile); //correct order of shell in outfile, if necessary
    
    (SPLIT)? cifparse(outfile, outfile,120) : cifparse(outfile, outfile,100); //re-organize outfile again
    //END of outfile process, followed by further checking but no more update from here
    
    dic_check(outfile, 1) ;
    cif_check(outfile) ;
    
    if(tem_type==0){ //not mmcif template
        enumerate_value_check(outfile) ;
//        final_check(outfile);
    }
    
    nline = num_of_line(outfile);
    printf("\nOutput file=  %s (%d lines)\n\n", outfile, nline);
    
    fclose(FW_LOG); //close log file object
    check_log("pdb_extract.log"); 
    
    return 0;   
}

char **get_category(char *infile, int *ncate)
{
    char **line, **cate, tmpstr[90], category[100],item[100];
    int i,id_dic, nline=0, nc=0;
    
    cate = new char *[200];
    
    line=get_lines_from_file(infile, &nline); //from template
    strcpy(tmpstr,"?");
    for(i=0; i<nline; i++){
        id_dic=check_category(line[i], category, item);
        if (id_dic<=0 || !(strlen(category)>2 && strlen(item)>1)) continue;
       
        if (!strcmp(category, tmpstr)){
            continue;
        }
        cate[nc]=new char[strlen(category)+1];        
        strcpy(cate[nc], category);
        strcpy(tmpstr, category);
        nc++;
    }
    *ncate=nc;
    return cate;
}
    
void final_check(char *outfile)
//check the missing category. If missing add them from the full template.
{
    char **cate_tmp, **cate_out, **lines_tmp;
    char dicfile[512], tmpfile[512], str[512], tmpstr[512];    
    int i,j,k, nc_tmp, nc_out,n ,nid=0, index[100], nline_tmp;    
    FILE *ftmp, *fout;
    
    printf("Doing final check.\n");
    
    strcpy(tmpfile, "data_template_full.cif");
    get_local_file(dicfile, tmpfile);

    cate_tmp=get_category(dicfile, &nc_tmp);
    cate_out=get_category(outfile, &nc_out);
        
    for (i=0; i<nc_tmp; i++){
        n=0;
        for (j=0; j<nc_out; j++){
            if (!strcmp(cate_tmp[i], cate_out[j])){
                n=1;
                break;
            }
        }
        if (n==0){// not matched
            index[nid]=i;
            nid=nid+1;
        }
    }

    
    ftmp=get_temp_file(outfile);
    fout=fopen(outfile,"w");
    while(fgets(str, sizeof str, ftmp)!= NULL){
        fprintf(fout, "%s", str);
    }

    lines_tmp=get_lines_from_file(dicfile, &nline_tmp); //from template
    
    for (i=0; i<nid; i++){ //Adding the missing tables
        sprintf(INFO,"\nWarning: The category (%s) is empty. Add values to the category if applicable.\n", cate_tmp[index[i]]);
        pinfo(INFO,1);
        
        for (j=0; j<nline_tmp; j++){
            sprintf(tmpstr,"%s.", cate_tmp[index[i]]);
            
            if (j>0 && strstr(lines_tmp[j], tmpstr)){
                fprintf(fout,"\n#\n%s", lines_tmp[j-1]);
                for (k=j; k<nline_tmp; k++){
//                    printf("tmpcate=%d, %d, %s %s | %s", i, j, tmpstr, lines_tmp[j], lines_tmp[k]);
                    if (k-j>1 && strchr(lines_tmp[k], '#')) {
                        j=k-1;
                        break;
                    }
                    fprintf(fout,"%s",lines_tmp[k]);
                }
            }
        }
    }

    fclose(fout);
    

}

    
void check_log(const char *logfile)
{
    FILE *fp;
    char str[500];
    int n=0;
    
    
    if((fp=fopen(logfile, "r"))==NULL){
        return;
    }
    
    while (fgets(str, sizeof str, fp) != NULL) {
        if (strstr_case(str,"Error:") || strstr_case(str,"Warning:")){
            n=1;
            break;
        }
    }
    if (n){
        printf("Error/Warning messages were found in pdb_extract.log. \nFollow instructions in the pdb_extract.log to correct the errors if applicable. \n\n");               
    }
            
}

void extraction_new(FILE *fout, int argc, char **argv)
// the way for extraction_new is different from extraction
{
    int i=0;
    
    
    for(i=1; i<argc-2; i++){ //scaling for phase. separate with scaling
        if (!strcmp_case(argv[i],"-sp")){
            extract_scale_phase(fout, argc, argv, i+2, argv[i+1]);
        }
    }
    
    for(i=1; i<argc-2; i++){
        if(!strcmp_case(argv[i],"-s") ){ //scaling/averaging
            extract_scale(fout, argc, argv, i+2, argv[i+1]);
        
        }else if (!strcmp_case(argv[i],"-i")){ //indexing
            extract_index(fout, argc, argv, i+2, argv[i+1]);
  
        }else if (!strcmp_case(argv[i],"-v")){ //validation
            strcpy(pdbx_validate_overall.program, argv[i+1]);
            extract_valid(fout, argc, argv, i+2, argv[i+1]);
        }
    }
}

void extract_valid(FILE *fout, int argc, char **argv, int start, char *prog)
{
    int i=0;
    char infile[512];
    const char *id = "-i -p -e -d -m -r -s -sp";


    for(i=start; i<argc-1; i++){
        if(strstr_case(id, argv[i])) break;
        if(!strcmp_case(argv[i],"-ILOG") ){
            strcpy(infile, argv[i+1]);
            if(!strcmp_case(prog, "phenix")||!strcmp_case(prog, "molprobity")){
                extract_valid_phenix(infile);
            }
        }
    }
}

void extract_valid_phenix(char *file)
{
   int i, nline=0;
   char **line;
 
   line=get_lines_from_file(file, &nline);
    for(i=0; i<nline; i++){
        if (pattern5(line[i], "    outliers ", ":", "(", "%", ")")){
            pdbx_validate_overall.ramachandran_outlier_percent=get_float(line[i], "(", "%");
        }else if (pattern5(line[i], "favored ", ":", "(", "%", ")")){
            pdbx_validate_overall.ramachandran_favored_percent=get_float(line[i], "(", "%");
        }else if (pattern5(line[i], "clashscore ", ":", "(", ")", "")){
            pdbx_validate_overall.clashscore=get_float(line[i], ":", "(");
        }
    }
}

int check_options(char *id)
{
    int i, n=0;
    const char *options[] = {"-o", "-e", "-i",  "-s", "-sp", "-r", "-p", "-d", "-m", "END"};

    for(i=0; i<1000; i++){
        if(!strcmp(options[i], "END")) break;
        if(!strcmp_case(id, options[i])){
            n=1;
            break;
        }
    }
    return n;
}


void extract_index(FILE *fout, int argc, char **argv, int start, char *prog)
{
    int i=0;
    char infile[512];
    
    for(i=start; i<argc-1; i++){
        if(check_options(argv[i])) break;
        if(!strcmp_case(argv[i],"-ILOG") ){ //parsse log files
            strcpy(infile, argv[i+1]);
            if(!strcmp_case(prog, "HKL")|| !strcmp_case(prog,"scalepack")||
               !strcmp_case(prog, "HKL2000") || !strcmp_case(prog, "DENZO")){
                
                printf("HKL/Scalepack/DENZO output Log file: (%s)\n",infile);
                
                extract_HKL_log_index(infile);
                
            }else if (!strcmp_case(prog,"DTREK")){

                printf("D*TREK output Log file (%s)\n",infile);
                extract_DTREK_log_index(infile); 
                        
                        
            }else if(!strcmp_case(prog,"mosflm")){

                printf("MOSFLM output Log file (%s)\n",infile);
                extract_MOSFLM_log_index(infile);
            }
        }else if (!strcmp_case(argv[i],"-ICIF") ){ //for cif format input model
            strcpy(infile, argv[i+1]);
            write_cif(fout, infile, "");
        }
    }

}

void extraction(FILE *fout, int argc, char **argv, char *method,
               const char *option_id, const char *step_id)
// option_id == -R, -P, -D, -M:   step_id == REF, PHS, DM, MR
    
{
    int i=0, j=0;
    char prog[512];

    
    for(i=1; i<argc-2; i++){
        
        if(!strcmp_case(argv[i],option_id)){
            strcpy(prog, argv[i+1]);
           
            for(j=i+2; j<argc; j++){
                if(check_options(argv[j])) break;
//                if(argv[j][0] == '-' && toupper(argv[j][1]) != 'I' ) break;
      
                if(!strcmp_case(argv[j],"-ILOG") ){
                    process_files(fout,&j,argc,argv,prog,"LOG",method,step_id);
                    
                }else if (!strcmp_case(argv[j],"-ICIF")){
                    process_files(fout,&j,argc,argv,prog, "CIF",method,step_id);
                    
                }else if (!strcmp_case(argv[j],"-IPDB")){
//                    if(!strcmp_case(option_id, "-R") && NPDB ==1) continue;
                    
                    process_files(fout,&j,argc,argv,prog, "PDB",method,step_id);
                    
                }
                
            }
            
        }
    }

}

void process_files(FILE *fout, int *j, int argc, char **argv,
                   char *prog, const char *format_in,  char *method, const char *id)
/* continue to take all the argument until meet - or end
 special treatment for SHARP.
*/    
{
    int k=0;
    char infile[512],infile_new[512], str[512], tmp[512], format[30];
    FILE *fp=NULL, *fpp=NULL;
    

    strcpy(format, format_in);
    if(!strcmp_case(prog, "SHARP") && !strcmp_case(format, "LOG")){
        if((fp=fopen("sharp_phs_files.tmp", "w"))==NULL){
            printf("Sorry disk is writen-protected! please give permision\n");
            exit(0);
        }
    } 
    
    sprintf(tmp,"%s%s%s", prog, format,id);
    upperstr(tmp);
    
    for(k= (*j)+1; k<argc; k++){
        if(argv[k][0] == '-' || k>=argc) {
            *j=k-1;
            break;
        }
        strcpy(infile, argv[k]);
        
        if(!strcmp_case(tmp, "SHARPLOGPHS")){
            if((fpp = fopen(infile, "r"))==NULL){
                sprintf(INFO, "Error! I can not open the file %s\n", infile);
                pinfo(INFO,0);
                continue;
            }
            printf("SHARP phasing;  file_format=%s:  file_name=%s\n", format, infile);
            
            while(fgets(str, sizeof str,fpp)!=NULL){
                fprintf(fp, "%s", str);
            }
            fclose(fpp);
        }
        
        if(strstr(id,"REF")){
            
            if(non_ascii(infile)){
                sprintf(infile_new, "%s__new", infile);
                clean_file(infile, infile_new);
                for_refine(fout, prog, infile_new, format, method);
                delete_file(infile_new);
            }else{
                for_refine(fout, prog, infile, format, method);
            }
        }else if(strstr(id,"PHS")){
            
            if(strcmp_case(tmp, "SHARPLOGPHS")){
//                printf("prog, infile,=%s, %s\n",  prog, infile);
                for_phasing(fout, prog, infile, format, method);
            }
            
        }else if(strstr(id,"DM")){
            for_dm(fout, prog, infile, format, method);
            
        }else if(strstr(id,"MR")){
            for_mr(fout, prog, infile, format, method);
            
        }

        
    }
    
    if(!strcmp_case(tmp, "SHARPLOGPHS")){
        fclose(fp);
        for_phasing(fout, prog,(char *)  "sharp_phs_files.tmp", format, method);
    }
}


int look_for_alt_pair(int i,int ns,int **residx,char **atom,char **comp,
                      char **seq, float *occ)
// check if residue i has alt with the rest residues and return the residue number and nskip
{
    int key=0, k,j, n1,n2, alt2 ;
    
    
    n1=residx[i][0];
    for(j=i+1; j<ns;j++){ //check rest residue if alt, and add them.
        n2=residx[j][0];
        alt2=0;
        for(k=residx[j][0]; k<residx[j][1];k++){ // check occ
            if(occ[k]<0.999){
                alt2=1;
                break;
            }
        }
        if(alt2 == 1){ // check if the alt residue 
            if(!strcmp(comp[n1], comp[n2]) && !strcmp(seq[n1], seq[n2]) ){
                key=atom_atom_check(i,j, residx,atom,occ);
                
                if(key>0){
//                    printf("residue %d and %d are alt partner!\n",i, j);
                    break;
                }
            }
        }
    }
    return key;
 
}


int atom_atom_check(int i,int j, int **residx, char **atom, float *occ)
//check if the sum of occupancy of two atoms <= 1.00
{
    int k, m, natm=0, na=0;
    
    for(k=residx[i][0]; k<residx[i][1]; k++){
        for(m=residx[j][0]; m<residx[j][1];m++){
            if(!strcmp(atom[k], atom[m])){
                if(occ[k]<0.99 || occ[m]<0.99){
                    na++;
                    if(occ[k] + occ[m]<1.001){ //two atoms are alt.
                        natm++;
                    }
                }
            }
        }
    }
    if (na==natm && na>0)
        return 1;
    else
        return 0;
    
}

int ** get_residx2(int nline, char **line, char ** seq, int *nres)
{
    char  tmp[256], tmp1[256];
    int **residx,i,j=0, ns;
    
    residx = new int *[nline];
    for(i=0; i<nline;i++) residx[i]=new int [2];
    
    ns=0;
    for(i=0; i<nline; i++){
        strncpy(tmp,line[i]+17, 9);
        tmp[9]='\0';
        residx[ns][0]=i;
        for(j=i+1; j<nline; j++){
            strncpy(tmp1,line[j]+17, 9);
            tmp1[9]='\0';            
            
            if(strcmp(tmp, tmp1)){ // different residue
                residx[ns][1]=j;
                ns++;
                i=j-1;
                break;
            }
        }
        
    }
    if(ns>0){
        residx[ns][0]=residx[ns-1][1];
        residx[ns][1]=nline;
    }
    
    *nres=ns+1;
    return residx;
}



char get_chainid(int n1, int ns, char *chain, int **residx)
{
    char c, c1,c2;
    int i,nline;

    c= ' ';
    c1= ' ';
    c2= ' ';
    
    c=chain[n1];
    nline=residx[ns-1][1];
    
    if(c != ' ') return c;
    
    for (i=n1; i<=0; i--){
        if(chain[i] != ' ') {
            c1=chain[i];
            break;
        }
    }
    for (i=n1; i<nline; i++){
        if(chain[i] != ' ') {
            c2=chain[i];
            break;
        }
    }
    if(c1==c2 && c1!=' '){
        return c1;
    }else if(c1==c2 && c1 ==' '){
        sprintf(INFO,"Warning! No chain ID in the pdbfile.\n");
        pinfo(INFO, 0);
        return c1;
    }else if( c1 ==' ' && c2!=' '){
        return c2;
    }else if( c2 ==' ' && c1!=' '){
        return c1;
    }if(c1!=c2 && c1!=' ' && c2!=' '){ //middle of two chains
        return c1;
        
    }else
        return c2;    
}

void dic_check(char *inpfile, int key)
//id=0; check cif items only.
//id==1, replace them if they are not the stardard in the dic. 
{
    char str[512], **all_items, category[100],item[100], dic[512];
    char category_dic[100],item_dic[100] ;
    int i, nitem=0, id=0,id_dic=0,  ns=0, seq=0;
    FILE *ftmp, *fout;

    
    printf("Checking file (%s) by the mmCIF dictionary (v5).\n", inpfile); 

    get_local_file(dic,"mmcif_pdbx_items.dic");
    ftmp=get_temp_file(inpfile);
    fout=fopen(inpfile,"w");

    all_items = get_lines_from_file(dic,&nitem);
    
    while(fgets(str, sizeof str, ftmp)!= NULL){
        id=check_category(str, category, item);
        if (strstr(str, "_entity_poly.pdbx_seq_one_letter_code")) seq=1;
        if(id>0 && strlen(category)>2 && strlen(item) >1 ){
            if (key==1){
                if(!strcmp_case(category,"_atom_site") && !strcmp_case(item,"atom_type_symbol")){
                    replace_first_str(str,"atom_type_symbol", "type_symbol", str);
                    sprintf(INFO, "Warning!  The item (_atom_site.atom_type_symbol) is not in the mmCIF dictionary(v5)! \
It is replaced by _atom_site.type_symbol. \n");
                    pinfo(INFO,0);

                }else if(!strcmp_case(category,"_refine_analyze") && !strcmp_case(item,"Luzzati_d_res_high_obs")){
                    replace_first_str(str,"Luzzati_d_res_high_obs", "pdbx_Luzzati_d_res_high_obs", str);
                    sprintf(INFO, "Warning! The item (_refine_analyze.Luzzati_d_res_high_obs) is not in the mmCIF dictionary(v5)! \
It is replaced by _refine_analyze.pdbx_Luzzati_d_res_high_obs.\n");
                    pinfo(INFO,0);
                }
            }

            ns=0;
            for(i=0; i<nitem; i++){
                id_dic=check_category(all_items[i], category_dic, item_dic);
                if (id_dic<=0) continue;
                if(!strcmp_case(category_dic, category) && !strcmp_case(item_dic,item)){
                    if(key==1 && (strcmp(category_dic, category) || strcmp(item_dic,item))){
                        strcat(category_dic, ".");
                        strcat(category_dic, item_dic);
                        strcat(category, ".");
                        strcat(category, item);
                        replace_first_str(str,category, category_dic, str);
                        sprintf(INFO, "Warning!  The item (%s) not match dictionary(v5)! It is replaced by (%s).\n", category, category_dic);
                        pinfo(INFO,0);
                    }
                    ns=1;
                    break;
                }
            }
            if (ns==0 && strcmp(item, "extraction_date")){
                sprintf(INFO, "Warning!  Data item (%s.%s) is not in the mmCIF dictionary(v5)!\n", category, item);
                pinfo(INFO,0);
            }
        }
        fprintf(fout, "%s",str);    
    }
    if(seq==1){
        rewind(ftmp);
        while(fgets(str, sizeof str, ftmp)!= NULL){
            if (strstr(str, "_entity_poly.pdbx_seq_one_letter_code")){
                while(fgets(str, sizeof str, ftmp)!= NULL){
                    id=check_category(str, category, item);
                    if (id>0 && !strstr(str, "entity_poly")) break;
                    if(strstr(str,"????")){
                        sprintf(INFO,"Error: question marks are in sequence (file=%s): Replace the ???? by the missing residues.\n", inpfile);
                        pinfo(INFO,0);
                        break;
                    }
                }
                break;
            }
        } 
    }            
    if(fout) fclose(fout);
    if(ftmp) fclose(ftmp);
    if(nitem) free_memory_2d(all_items, nitem);
}

int check_mandatory_cif(char *item)
{
    int i, nkey=0;
    const char *mandatory[]={"_entity.type",
                             "_entity.src_method",
                             "_diffrn_radiation.wavelength_id",
                             "_entity_src_gen.pdbx_src_id",
                             "_entity_src_nat.pdbx_ncbi_taxonomy_id",
                             "_entity_src_nat.pdbx_src_id",
                             "_entity_src_nat.pdbx_ncbi_taxonomy_id",
                             "_pdbx_entity_src_syn.pdbx_src_id",
                             "_pdbx_entity_src_syn.ncbi_taxonomy_id",
                             "_entity_poly.pdbx_seq_db_id",
                             "_entity_poly.pdbx_seq_db_name",
                             "_reflns.d_resolution_low",
                             "END" };

    for (i=0; i<100; i++){
        if (!strcmp(mandatory[i], "END")) break;
        if (!strcmp_case(item, mandatory[i])){
            nkey=1;
            break;
        }
    }
    return nkey;
}
    

void enumerate_value_check(char *inpfile)
//inpfile: the ciffile from pdb_extract
//Compare the data items in inpfile and in the cif_enumerate.list
//If they are the same, then compare the corresponding values
{
    string diags, blockId;
    char str[2000], tmpstr[2000], enum_file[512], enum_val[300][150];
    char category1[200], category2[200], item1[200], item2[200];
    char **enum_items, **value1;
    FILE *ftmp;
    
    int id, i, ii, k, j,m1, m2,m=0, nenum,  nitem, nkey=0;
    long n1=0;
        
    CifFile * fobjR = NULL;
    CifParser* cifParserR = NULL;

    printf("Checking enumerate values in the V5 dictionary...\n");
    
    if (!check_file(1,inpfile)) return;
    
    fobjR = new CifFile(true);
    cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
    cifParserR->Parse(inpfile, diags);
    delete(cifParserR);

    if (!diags.empty()) {
        sprintf(INFO, "%s\n", diags.c_str());
        pinfo(INFO, 0);
        if(strstr_case(diags.c_str(), "Number of data values is not exact multiples")){
            sprintf(INFO, "This is considered as severe error. Program stopped.\n");
            pinfo(INFO, 0);
            exit(0);
       }
    }            

    get_local_file(enum_file,"cif_enumerate.list");
    enum_items = get_lines_from_file(enum_file,&nitem);
    
    ftmp=get_temp_file(inpfile);
    while(fgets(str, sizeof str, ftmp)!= NULL){ //items from cif file
        
        id=check_category(str, category1, item1);
        if (!(id>0 && strlen(category1)>2 && strlen(item1) >1)) continue;
       
        
        for(i=0; i<nitem; i++){ //check cif_enumerate.list, see if both co-exist
            id=check_category(enum_items[i], category2, item2);
            
            if (!(id>0 && strlen(category2)>2 && strlen(item2) >1)) continue;
                
            if (!strcmp_case(category1, category2) && !strcmp_case(item1, item2)){
                
                nenum=0;
                for (j=i+1; j<nitem; j++){ //get values from cif_enumerate.list
                    if (enum_items[j][0]=='_') break;
                    rid_of_front_end_space(enum_items[j]);
                    strcpy(enum_val[nenum],enum_items[j]);
                    nenum++;
                }

                blockId = fobjR->GetFirstBlockName();
                n1=0;
                value1=get_attribute_values(fobjR,blockId.c_str(),category1+1, item1, &n1);
                sprintf(tmpstr,"%s.%s", category1, item1);
                nkey=check_mandatory_cif(tmpstr);
                m2=0;
                m=0;    
//                printf ("string1=%ld|%d|%s|%s|%s",n1,nkey, category2, item2 ,enum_items[i]);
                for(ii=0; ii<n1; ii++){ //for each value in the cif file
                
                    if (strstr(value1[ii], ".") || strstr(value1[ii], "?")){
                        if (nkey){ //mandatory item
                            sprintf(INFO,"Error: mandatory cif item (%s) has no value.", tmpstr);
                            pinfo(INFO,1);
                        }else{
                            continue;
                        }
                    }
                    
                    m++;

                    m1=0;
                    for(k=0; k<nenum; k++){
                        if (!strcmp_case(value1[ii], enum_val[k])){
                            m1=1;
                            m2=1;
                            break;
                        }
                    }
                    if (m1==0){
                        sprintf(INFO,"\nError: The cif value (%s.%s=%s) is not defined in the dictionary below.\n", category1, item1,value1[ii]);
                        pinfo(INFO,1);       
                    }
                   
                }
                if (m2==0 && m){//no match!
                    for (k=0; k<nenum; k++){
                        sprintf(INFO, "%2d  %s\n", k+1, enum_val[k]);
                        pinfo(INFO, 1);
                    }
                }
                    
                    
                break;
               
            }
        }
        
    }

    if(ftmp) fclose(ftmp);

}


void cif_check(char *inpfile)
//check cif items and replace them if they are not the stardard in the dic. 
{
    long n1=0,n2=0,n4=0, i=0, j, m1,m2;
    int ntoken;
    
    string diags, blockId;
    char **seq, **entity_p, **poly_asym, /* **entity, */  **asym, **comp;
    char ch[10], **chains, *dat, tmp0[80],tmp[80],tmp1[80], tmp2[80];
    
    CifFile * fobjR = NULL;
    
    if (!check_file(1,inpfile)) return;
    
    
    CifParser* cifParserR = NULL;
    fobjR = new CifFile(true);
    cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
    cifParserR->Parse(inpfile, diags);
    delete(cifParserR);

    if (!diags.empty()) {
        sprintf(INFO, "%s\n", diags.c_str());
        pinfo(INFO, 0);
        if(strstr_case(diags.c_str(), "Number of data values is not exact multiples")){
	  //chenghua-20190313-start
	  //for cif error (found in refmac cif output), change from program exit to continue trying parse only the 1st datablock
	  sprintf(INFO, "This is considered as severe error. Program now attemps to process the 1st data block only\n"); 
            pinfo(INFO, 0);
            // exit(0);
	  //chenghua-20190313-end
       }
    }            
    
    blockId = fobjR->GetFirstBlockName();
    entity_p=get_attribute_values(fobjR,blockId.c_str(),"entity_poly", "entity_id", &n1);
    poly_asym=get_attribute_values(fobjR,blockId.c_str(),"entity_poly", "pdbx_strand_id", &n1);
    seq=get_attribute_values(fobjR,blockId.c_str(),"entity_poly", "pdbx_seq_one_letter_code", &n1);
    for(i=0; i<n1; i++){
        if (strstr(entity_p[i], "?")){
            sprintf(INFO, "Error: question mark can not be used as entity_id. \
Program stopped! (entity_poly=%ld; file=%s).\nPlease use other values.\n",i+1, inpfile);
            pinfo(INFO, 0);
            exit(0);
        }
        if (strstr(seq[i], "?")){
            sprintf(INFO, "Error: question marks are found in the sequence. \
Program stopped!(entity_poly=%ld; file=%s).\nPlease replace the ???? by the missing residues.\n",i+1,inpfile);
            pinfo(INFO, 0);
            exit(0);
        }
        if(i>0 && !strcmp(entity_p[i], entity_p[i-1])){
            sprintf(INFO, "Error: Duplicate entity_poly.entity_id between group %ld and %ld.\n",i,i+1);
            pinfo(INFO, 0);
        }
    }
    // entity=get_attribute_values(fobjR,blockId.c_str(),"entity", "id", &n2);
    if (n2>0 && n1>0 && n2<n1){
        sprintf(INFO, "Error: The # of entity (%ld) is less than the # of entity_poly (%ld).\n",n2,n1);
        pinfo(INFO, 0);
    }
/*    
    src_gen=get_attribute_values(fobjR,blockId.c_str(),"entity_src_gen", "entity_id", &n3);
    if (n3>0 && n1>0 && n1!=n3){
        sprintf(INFO, "Error: The # of entity_src_gen (%ld) not equal to the # of entity_poly (%ld).\n",n3,n1);
        pinfo(INFO, 0);
    }
*/    
    comp=get_attribute_values(fobjR,blockId.c_str(),"atom_site", "auth_comp_id", &n4);
    asym=get_attribute_values(fobjR,blockId.c_str(),"atom_site", "label_asym_id", &n4);

    if (comp && asym && n1 && seq && poly_asym){//check each entity-poly
        for(i=0; i<n1; i++){ //from entity-poly
            rid_of_space(seq[i]);
            delete_return(seq[i]);
            m1=strlen(seq[i]);
            chains=string_token(poly_asym[i], ",", &ntoken);
            strcpy(ch, chains[0]);
            
            m2=0;
            for(j=0; j<n4-1; j++){
                
                if ( !strcmp(ch,asym[j]) && strcmp(comp[j],comp[j+1])){
                    m2++;
                }
            }
            if (m2 >m1){
                sprintf(INFO, "Error: Coordinate (chain=%s) has more residues than entity-poly (group=%ld).\n",ch,i+1);
                pinfo(INFO, 0);
            }
            
        }
    }
    
    dat=get_attribute_value(fobjR,blockId.c_str(),"diffrn_detector", "pdbx_collection_date");
    if (dat && strlen(dat)>2){
        strcpy(tmp,dat);
        replace_char(dat,'-', ' ');
        if (sscanf(dat,"%s %s %s",tmp0, tmp1, tmp2)!=3 || strlen(tmp0)!=4
            || atoi(tmp0) <1970 || atoi(tmp0)>2050 || strlen(tmp1)!=2
            ||strlen(tmp2)!=2 ||atoi(tmp1)>12 || atoi(tmp2)>31){
            sprintf(INFO, "Error for date format (%s) in diffrn_detector.pdbx_collection_date(should be yyyy-mm-dd).\n", tmp);
            pinfo(INFO, 0);
        }
//        printf ("poly, m1, m2=%s \n",dat);
    }
    
    if (fobjR) delete fobjR;   
}


int read_entity_poly(FILE *fout, int argc, char **argv, char *method)
/*for reading ENTITY (write first)
 ntype=0 mmcif file; ntype=1,the text file (data_template.text)
*/
{
    char file1[512], **line;
    int  i, ntype=0,j, nline, m=0; 

    for(i=1; i<argc-1; i++){  
        if(!strcmp_case(argv[i],"-IENT")){
            strcpy(file1, argv[i+1]);
            
            printf("\nThe input file_name=%s\n",file1);
            if (!check_file(1,file1)) return 0;
            ntype = type_of_data_template(file1); //0, cif; 1, plain txt; 
            if(ntype ==0){
//                dic_check(file1,0); //only check, no correction
                cif_check(file1);
                write_cif(fout,  file1, "CIF");
            }else if(ntype ==1){
                parse_template_to_mmcif(fout, file1, 1);
            }else if(ntype ==-1){
                break;
            }
   
            line=get_lines_from_file(file1,&nline);
            for(j=0; j<nline; j++){
                if(strstr(line[j], "_exptl.crystals_number")){
                    
                    m=get_int_after_id(line[j], "_exptl.crystals_number");
                    if(m>0)exptl.crystals_number=m;
            
                    break;
                }
            }
            if(line)free_memory_2d(line,nline);
        }
    }
    return ntype;
}


int type_of_data_template(char *infile)
{
    int  mtext=0, mcif=0;
    char str[5120], cate[150],item[150];
    FILE *fp=NULL;
    
    if ((fp=fopen(infile, "r"))==NULL){
        open_file_error(infile, "type_of_data_template");
    }
    
    while (fgets(str, sizeof str, fp) != NULL) {
        if(is_space_line(str)<=0) continue;
        rid_of_front_end_space(str);
        
        if( str[0]=='<' &&
           (pattern5(str,"<", "entity_id", "=", "", "")||
            pattern5(str,"<", "contact_author_PI_last_name", "=", "", "")||
            pattern5(str,"<", "contact_author_PI_id", "=", "", "")||
            pattern5(str,"<", "contact_author_id", "=", "", "")||
            pattern5(str,"<", "molecule_name", "=", "", "")||
            pattern5(str,"<", "structure_title", "=", "", "")) ){
            
            if(mtext++ > 4) break;
            
        }else {
            if (check_category(str, cate, item)>0) {
               if(mcif++ >3 ) break;
            }
        }   
    }
    fclose(fp);
    
    if(mtext>3){
        return 1;
    }else if(mcif >2 ){
        return 0;
    }else{
        sprintf(INFO, "Error: Input file (%s) is not the template or mmCIF (or empty) file.\n",infile);
        pinfo(INFO, 0);
        
        return -1;
        
    }
}

void write_block_method(FILE *fout,int argc, char **argv, char *method)
/* write data block first, extract the first one from refinement
   and delete all others, if exists.
*/
{
    char ind_prog[80],sol_prog[80], ref_prog[80], scl_prog[80];  
    char dm_prog[80], mr_prog[80],  method_mr[40], tmp[80] ;
    int i=0;
    
    fprintf(fout, "data_%s\n", DATA_BLOCK);
    
    strcpy(ref_prog, "");
    strcpy(sol_prog, "");
    strcpy(scl_prog, "");
    strcpy(ind_prog, "");
    strcpy(dm_prog, "");
    strcpy(mr_prog, "");
    strcpy(method, "");
    strcpy(method_mr, "");
    
    for(i=1; i<argc-1; i++){
        if(!strcmp_case(argv[i], "-r")) {
            strcat(ref_prog, argv[i+1]);
            strcat(ref_prog, "  ");
            
        }else if(!strcmp_case(argv[i], "-e") ) {
            strcpy(tmp,  argv[i+1]);
            if (!strcmp_case(tmp, "mr")){
                strcpy(refine.pdbx_method_to_determine_struct, "MOLECULAR REPLACEMENT");
            }
            upperstr(tmp);
            strcat(method, tmp);
            strcat(method, "  ");
            
            
        }else if(!strcmp_case(argv[i], "-p") ) {
            strcat(sol_prog, argv[i+1]);
            strcat(sol_prog, "  ");
        }else if(!strcmp_case(argv[i], "-m")) {
            strcat(mr_prog, argv[i+1]);
            strcat(mr_prog, "  ");
            strcpy(method_mr, "MOLECULAR REPLACEMENT");
            
        }else if(!strcmp_case(argv[i], "-d")) {
            strcat(dm_prog, argv[i+1]);
            strcat(dm_prog, "  ");
        }else if(!strcmp_case(argv[i], "-s")) {
            strcat(scl_prog, argv[i+1]);
            strcat(scl_prog, "  ");
        }else if(!strcmp_case(argv[i], "-i")) {
            strcat(ind_prog, argv[i+1]);
            strcat(ind_prog, "  ");
        }
    }
    
    if(strlen(sol_prog)>1 && strlen(mr_prog)>1){
        strcat(sol_prog, " ;   ");
        strcat(sol_prog, mr_prog);
    }else if (strlen(sol_prog)<1 && strlen(mr_prog)>1){
        strcpy(sol_prog, mr_prog);
    }
    
            
    rid_of_front_end_space(ref_prog);
    rid_of_front_end_space(sol_prog);
    rid_of_front_end_space(dm_prog);
    rid_of_front_end_space(ind_prog);
    rid_of_front_end_space(scl_prog);
    rid_of_front_end_space(method);
    rid_of_front_end_space(method_mr);
    
    if(strlen(ref_prog)) strcpy(computing.structure_refinement , ref_prog);
    if(strlen(sol_prog)) strcpy(computing.structure_solution , sol_prog);
    if(strlen(dm_prog))  strcpy(computing.pdbx_density_modification , dm_prog);
    if(strlen(scl_prog)) strcpy(computing.data_reduction , scl_prog);
    if(strlen(ind_prog)) strcpy(computing.data_collection , ind_prog);
    if(strlen(ind_prog)) strcpy(computing.cell_refinement , ind_prog);
   

    if(strlen(method)<=0 && strlen(method_mr)<=0) return;
    
    if(strlen(method)>0 && strlen(method_mr)>0){
        fprintf(fout,"##\nloop_\n");
        fprintf(fout, "_phasing.method \n");
        fprintf(fout, "'%s' \n", method);
        
        if(!strstr_case(method, "MOLECULAR REPLACEMENT") && !strstr_case(method,"MR") ){
            fprintf(fout, " MR \n");
            sprintf(refine.pdbx_method_to_determine_struct, "%s, %s", method,  method_mr);
        }else
            sprintf(refine.pdbx_method_to_determine_struct, "%s",  method_mr);
            
        
    }else if(strlen(method)>0 && strlen(method_mr)<=0){
        fprintf(fout, "_phasing.method '%s' \n", method);
        sprintf(refine.pdbx_method_to_determine_struct, "%s", method);
        
    }else if(strlen(method_mr)>0 && strlen(method)<=0){
        sprintf(refine.pdbx_method_to_determine_struct, "MOLECULAR REPLACEMENT");
        fprintf(fout, "_phasing.method  MR \n");
    }
    
    fprintf(fout,"##\n");
}


void usage()
{
    printf("\n======================================================================\n");
    printf("Please type 'pdb_extract -h' for help \n");
    printf("======================================================================\n\n");
    exit(0);
}

int check_item(const char **str, char *item)
{
    int i, n=0;
    for(i=0; i<1000; i++){
        if(!strcmp_case(str[i],"END_STRING")) break;
        if(!strcmp_case(str[i],item)){
            n=1;
            break;
        }
    }
    return n;
}

void check_input_error(int argc, char **argv)
// check various input errors and return some parameters
{
    char refine_prog[1024];
    char *option = (char*)"-o -e -i -s -sp -r -p -d -m -v -iLOG -iCIF -iPDB -iENT -iDat -DATACUT";
    char *options = (char*)"-o -e -i -s -sp -r -p -d -m";
    const char *EXP[]={"MR","SAD","MAD","SIR","SIRAS","MIR","MIRAS",
                       "AB_INITIO","END_STRING"};
    const char *INDEX[]={"HKL","DENZO","XDS","DTREK","MOSFLM","END_STRING"};
    const char  *SCALE[]={"SCALA","AIMLESS","HKL","SCALEPACK","DTREK","SAINT","3DSCALE",
                          "XSCALE","XDS", "X-GEN","XGEN", "PROSCALE","XPREP","MARSCALE", "XIA2",
                          "ANY", "END_STRING"};
    const char *PHASE[]={"CNS","XPLOR","MLPHARE","SOLVE","SHELX","SNB","BnP",
                         "SHARP","PHASER","PHASES","WARP","BP3", "END_STRING"};
    const char *MR[]={"AMORE","CNS","XPLOR","EPMR","MOLREP","BEAST","PHASER",
                      "COMO", "END_STRING"};
    const char *DM[]={"CNS","XPLOR","DM","RESOLVE","SOLOMON","SHELXE","SHARP",
                      "END_STRING"};
    const char *REFINEMENT[]={"CNS","XPLOR","REFMAC5","REFMAC","SHELX","SHELXL","TNT",
                          "PROLSQ","NUCLSQ", "RESTRAIN","PHENIX","END_STRING"};
    
            
    char *temp=NULL;
    int i=0, last=0, c;
    FILE *fp;
    
    for(i=1; i<argc; i++){
        if(!strcmp_case(argv[i], "-h") || !strcmp_case(argv[i], "-help")){
            
            if(!strcmp_case(CCP4,"yes")){
                if((temp = getenv("CCP4")) != NULL) {
                    strcpy(refine_prog, temp);
                } 
                strcat(refine_prog,CCP4_2_DATA_HELP);
                if((fp = fopen(refine_prog,"r"))==NULL){
                    open_file_error(refine_prog, "check_input_error");
                }  
                while((c=getc(fp))!=EOF){
                    printf("%c", c);
                }
                exit(0);
            }else{
                if((temp = getenv("PDB_EXTRACT")) != NULL) {
                    strcpy(refine_prog, temp);

                }else{
                    sprintf(INFO, "Error! The PDB_EXTRACT environment is not set!\nPlease look at the README file in the package.\n");
                    pinfo(INFO,0);
                    return;
                }

                strcat(refine_prog,EXTRACT_2_DATA_HELP);
                if((fp = fopen(refine_prog,"r"))==NULL){
                    open_file_error(refine_prog, "check_input_error");
                }  
                while((c=getc(fp))!=EOF){printf("%c", c);}
                printf("\n");
            
                exit(0);
            }
            
        }
    }

    
    if(argc<3){ 
        printf("There are too few arguments!\n\n");
        usage();
    }

    last=argc-1;   
    if(!strcmp_case(argv[last],"-O")){
        printf("\nInput argument is wrong!!\n");
        printf("The option (%s) should be followed by an output file name!\n", argv[last]);
        usage();
    }

    if(!strcmp_case(argv[last],"-E")){
        printf("\nInput argument is wrong!!\n");
        printf("The option (%s) should be followed by one of the methods!\n", argv[last]);
        printf("(MR|SAD|MAD|SIR|SIRAS|MIR|MIRAS|AB_INITIO)\n");
        usage();
    }
    
    if(!strcmp_case(argv[last],"-P") || !strcmp_case(argv[last],"-R") ||
       !strcmp_case(argv[last],"-D") || !strcmp_case(argv[last],"-M") ||
       !strcmp_case(argv[last],"-S") || !strcmp_case(argv[last],"-SP")||
       !strcmp_case(argv[last],"-I")){
        printf("\nInput argument is wrong!!\n");
        printf("The option (%s) should be followed by a software name!\n", argv[last]);
        usage();
    }

    if(!strcmp_case(argv[last],"-ILOG") || !strcmp_case(argv[last],"-IPDB") ||
       !strcmp_case(argv[last],"-ICIF") || !strcmp_case(argv[last],"-IENT") ) {
        printf("\nInput argument is wrong!!\n");
        printf("The option (%s) should be followed by a input data file name(s)!\n",
               argv[last]);
        usage();
    }
 
    for(i=1; i<argc-1; i++){
        if(argv[i][0] == '-' && !strstr_case(option, argv[i]) ){
            printf("\nInput was wrong!\n");
            printf("You provided wrong option [%s]\n", argv[i]);
            printf("Option should be one of [%s]\n", option);
            usage();
        }			
						 
        if(argv[i][0] == '-' && argv[i+1][0]== '-'){
            printf("\nEither argument (%s) or (%s) is wrong!!\n",
                   argv[i],argv[i+1]);
            printf("The data file names can not be started with - !\n\n");
            usage();
        }

        if(!strcmp_case(argv[i],"-E") ){   /* experiment method  */
            if(!check_item(EXP,argv[i+1])){
                printf("\nInput is Wrong! The option [-e] should be ");
                printf("followed by one of (MR|SAD|MAD|SIR|SIRAS|MIR|MIRAS|AB_INITIO)\n\n");
                usage();
            }


            if(i<argc-2 &&
               (!strcmp_case(argv[i+2],"-ilog")||!strcmp_case(argv[i+2],"-ipdb") ||
                !strcmp_case(argv[i+2],"-idat") ||!strcmp_case(argv[i+2],"-icif") ||
                argv[i+2][0]!='-')){
                
                printf("\nThe argument(%s) after (%s  %s) is wrong!\n",
                       argv[i+2],argv[i], argv[i+1] );
                
                printf("(%s  %s) should be followed by one of (%s) or nothing.\n\n",
                       argv[i], argv[i+1], options );
                usage();
            }
        
        }

        
        if (!strcmp_case(argv[i],"-I") ){
            if(!check_item(INDEX,argv[i+1]))  print_error(argv[i+1], "Data Indexing");
            
        }else if (!strcmp_case(argv[i],"-S") ||  !strcmp_case(argv[i],"-SP") ){
            if(!check_item(SCALE,argv[i+1]))  print_error(argv[i+1], "Data Scaling");
            
        }else if(!strcmp_case(argv[i],"-P") ){
            if(!check_item(PHASE,argv[i+1]))  print_error(argv[i+1], "Heavy Atom Phasing");

        }else if (!strcmp_case(argv[i],"-M") ){
            if(!check_item(MR,argv[i+1]))  print_error(argv[i+1], "Molecular Replacement");
            
        }else if (!strcmp_case(argv[i],"-D") ){
            if(!check_item(DM,argv[i+1]))  print_error(argv[i+1], "Density Modification");
                
        }else if (!strcmp_case(argv[i],"-R") ){
            if(!check_item(REFINEMENT,argv[i+1]))  print_error(argv[i+1], "Structure Refinement");
        
        }
        
    }
    
         
}

void print_error(char *prog, const char *note)
{
    sprintf(INFO,"Warning! The program (%s used for %s) is not supported for LOG file extraction.\n", prog, note);
    pinfo(INFO, 0);
}


void unknown_prog(FILE *fout, char *inpfile, char *id, char *method)
/* at least to extract information from PDB or CIF files*/
{
    FILE *fp;
    
    if((fp = fopen(inpfile, "r"))==NULL) {        
        sprintf(INFO, "Can not open the file %s (routine:unknown_prog)\n",inpfile);
        pinfo(INFO, 0);
        
        return;
    }
    
    if(!strcmp_case(id, "PDB")){
        pdb_2_cif(fout, inpfile);
    }else if (!strcmp_case(id, "CIF")){
        write_cif(fout,  inpfile, method);
    }
    
    fclose(fp);
}
    

int check_block(char *str)
 // delete the [, {, or (
{
    int m=0, len=0, i=0;
    char str1[512], tmp1[512];
    
    
    m=sscanf(str, "%s %s", str1, tmp1);
    len = strlen(str1);
            
    if((m==1 || (m==2 && tmp1[0]=='#')) &&
       !strncmp_case(str1, "data_", 5) && len>5 ){
        for(i=0; i<len; i++){
            if(i< len-2 && (str[i]=='[' || str[i]=='(' || str[i]=='{' 
                            ||str[i]=='}' ||str[i]==')' ||str[i]==']'))
                str[i]='_';
            if(i>= len-2 && (str[i]=='}' ||str[i]==')' ||str[i]==']')) str[i]=' ';
        }
        
        return 1;
    }else return 0;
}

void write_cif_or_pdb(FILE *fout,  char *inpfile, char *format, char *method)
/* Note: if CIF, write everythings; if PDB, write heavy atom cood.*/
{
    FILE *fp;
    
    if((fp = fopen(inpfile, "r"))==NULL) {        
        printf("Can not open the file %s(routine:write_cif_or_pdb)\n",
               inpfile);
        return;
    }
    
    if(!strcmp_case(format, "CIF")) { /* take everthing */
        write_cif(fout, inpfile, method);
    }
    if(!strcmp_case(format, "PDB")) { /*take as heavy atom XYZ */
        heavy_coord_pdb(fout, inpfile, method);
    }
    
    fclose(fp);
}


void write_cif(FILE *fout, char *inpfile,const  char *id)
/* Note: CIF model input, write everythings after check; If error, try writing 1st block only*/
{
    int i, n=0, nline;
    char  **line, inpfile_tmp[512];
    
    n=is_cif(inpfile);
    if(n<1){
        sprintf(INFO,"Error! File (%s) is not in CIF format\n", inpfile);
        pinfo(INFO, 0);
        return;
    }

    sprintf(inpfile_tmp, "%s_tmp", inpfile);
    if(!strcmp_case(id, "scala")){
        convert_cif2rcsbcif(inpfile, inpfile_tmp, "scala");
    }else{
        convert_cif2rcsbcif(inpfile, inpfile_tmp, "");
    }
    
    cifparse(inpfile_tmp, inpfile_tmp, 500);
    line=get_lines_from_file(inpfile_tmp, &nline);
    
    n=0;
    for(i=0; i<nline; i++){
        rid_of_front_space(line[i]);
        if(line[i][0] !='#') n++;
           
        if((n==1 && !strncmp_case(line[i],"data_",5))||
           strstr_case(line[i],"_entry.id") ||
           strstr_case(line[i],"_audit.")||
           strstr_case(line[i],"_software.") 
           ){
            continue;
        }
        if (strstr_case(line[i], "_refine_ls_shell.R_factor_R_all")){
            replace_all_str(line[i],"_refine_ls_shell.R_factor_R_all", "_refine_ls_shell.R_factor_all");
        }
           
        fprintf(fout,"%s", line[i]);
    }
    delete_file(inpfile_tmp);
    
    if(nline) free_memory_2d(line, nline);
}


void extract_seq (int argc, char **argv)
// This is only for the CCP4i interface    
{
    int i, n=0;
    char outfile[512],command[512], *pth=NULL;

    strcpy(outfile, "extract_tmp");
    
    
    for(i=0; i<argc-1; i++){
        if( (!strcmp_case(argv[i],"-PDB") || !strcmp_case(argv[i],"-CIF") ||
             !strcmp_case(argv[i],"-EXT")) && argc<6 ){
            n=1;
           
        }else if(!strcmp_case(argv[i],"-O")&& argc<6){
            strcpy(outfile,argv[i+1]);
        }
    }
    if(n==0) return;

    pth = getenv("PDB_EXTRACT");
    for(i=0; i<argc-1; i++){
        if( !strcmp_case(argv[i],"-PDB")){
            sprintf(command,"%s/bin/extract -pdb %s \n", pth, argv[i+1]);
            n=system(command);
            check_system_call(n, "exract");
            change_file_name("data_template.text", outfile);            
//            delete_file("cgi_value");
            delete_file("entity-poly.mmcif");             
            exit (0);
            
        }else if(!strcmp_case(argv[i],"-CIF")){
            sprintf(command,"%s/bin/extract -cif %s \n", pth, argv[i+1]);
            n=system(command);
            check_system_call(n, "exract");
            change_file_name("data_template.text", outfile);            
//            delete_file("cgi_value");
            delete_file("entity-poly.mmcif");
            exit (0);
        }else if (!strcmp_case(argv[i],"-EXT")){
            sprintf(command,"%s/bin/extract -ext %s \n", pth, argv[i+1]);
            n=system(command);
            check_system_call(n, "exract");
            exit (0);
        }
      
     }
}

void reorder_shell(int argc, char **argv, char *inpfile)
{
    string blockId;
    char  **line=NULL, str[512], *shell[NBIN], **id1=NULL,**id2=NULL, **pdb=NULL;
    long nn=0,nid1=0,nid2=0, i, j,k,m ,npdb=0 ,npoly=0, nent=0;
    int nline;
    string diags; 
    
    FILE  *fout;
     
    line = get_lines_from_file(inpfile, &nline);
    
    
    CifParser* cifParserR = NULL;
    CifFile * fobjR = NULL;
    fobjR = new CifFile(true);
    cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
    cifParserR->Parse(inpfile, diags);

    delete(cifParserR);


    if (!diags.empty()) fprintf(stderr, "Diags for file %s: %s\n",inpfile,diags.c_str());

    blockId = fobjR->GetFirstBlockName();
    id1 = get_attribute_values(fobjR, blockId.c_str(), "refine_ls_shell","d_res_high", &nid1);
    id2 = get_attribute_values(fobjR, blockId.c_str(), "reflns_shell","d_res_high", &nid2);
    pdb =get_attribute_values(fobjR, blockId.c_str(), "atom_site", "Cartn_x", &npdb);
//    poly=get_attribute_values(fobjR, blockId.c_str(), "entity_poly", "entity_id", &npoly);
//    ent=get_attribute_values(fobjR, blockId.c_str(), "entity", "id", &npoly);

    nn=nline;
    fout=fopen(inpfile, "w");
    
    fprintf(fout,"%s", line[0]);
    fprintf(fout,"%s", line[1]);
    fprintf(fout,"%s", line[2]);
    
    for(i=4; i<nn; i++){
        fprintf(fout,"%s", line[i-1]);
                   
        if((!strncmp(line[i-4],"_refine_ls_shell.", 17) &&
            !strncmp(line[i-3],"_refine_ls_shell.", 17) &&
            !strncmp(line[i-2],"_refine_ls_shell.", 17) &&
            !strncmp(line[i-1],"_refine_ls_shell.", 17) &&
            
            sscanf(line[i], "%s%s%s%s", str, str, str, str)==4)) {
                
            if(nid1>0 && atof(id1[0]) < atof(id1[nid1-1])) continue;
            
            k=0;
            for(j=i; j<nn-1; j++){
                
                shell[k]=new char[strlen(line[j])+1];
                strcpy(shell[k], line[j]);
                k++;
                 
                if((sscanf(line[j+1], "%s%s%s%s", str, str, str, str )!=4 && k>0)||
                   (j+1==nn-1 && k>0) ){
                    i=j+1;
                    for(m=k-1; m>=0; m--){
                        fprintf(fout,"%s", shell[m]);
                    }
                    
                    break;
                }
            }
            
        }else if((!strncmp(line[i-4],"_reflns_shell.", 14) &&
                  !strncmp(line[i-3],"_reflns_shell.", 14) &&
                  !strncmp(line[i-2],"_reflns_shell.", 14) &&
                  !strncmp(line[i-1],"_reflns_shell.", 14) &&
            sscanf(line[i], "%s%s%s%s", str, str, str, str)==4)) {
            if(nid2>0 && atof(id2[0]) < atof(id2[nid2-1])) continue;
            k=0;
            for(j=i; j<nn-1; j++){
                
                shell[k]=new char[strlen(line[j])+1];
                strcpy(shell[k], line[j]);
                k++;
                 
                if((sscanf(line[j+1], "%s%s%s%s", str, str, str, str )!=4 && k>0)||
                   (j+1==nn-1 && k>0) ){
                    i=j+1;
                    for(m=k-1; m>=0; m--){
                        fprintf(fout,"%s", shell[m]);
                    }
                    
                    break;
                }
            }
        }
        
            
    }
    if (nent==0 && npoly>0){
        fprintf(fout,"\n#\nloop_\n");
        fprintf(fout,"_entity.id\n");
        fprintf(fout,"_entity.type\n");
        for (i=1; i<=npoly; i++){
            fprintf(fout, "%ld  polymer \n", i);
        }
    }
    if(nline)free_memory_2d(line, nline);
    fclose(fout);

    if(id1) free_memory_2d(id1, nid1);
    if(id2) free_memory_2d(id2, nid2);
    if(pdb) free_memory_2d(pdb, npdb);
    
    for(i=0; i<argc; i++){
        if(!strcmp_case(argv[i], "-r")){
            if(npdb<10){
                sprintf(INFO,"\nWarning! There are no atom coordinates in your file!\n");
                pinfo(INFO,0);
                
            }
        }
    }
}
    
void  test_cif(char *inpfile)
// this file is only for testing mmCIF
{
    string blockId;
    string diags;
    int i, colNo, rowNo, irow;

    CifFile * fobjR = NULL;

    CifParser* cifParserR = NULL;
     
    fobjR = new CifFile(true);

    cifParserR = new CifParser(fobjR, fobjR->GetVerbose());

    cifParserR->Parse(inpfile, diags);

    delete(cifParserR);

    if (!diags.empty()) fprintf(stderr, "Diags for file %s: %s\n",inpfile,diags.c_str());
    
    blockId = fobjR->GetFirstBlockName();
    
    Block& block = fobjR->GetBlock(blockId);
    vector<string> tableNames;
    block.GetTableNames(tableNames);

    for (unsigned int ii=0; ii<tableNames.size(); ii++)
        cout<<tableNames[ii]<<endl;


///////////////////////
    ISTable *Table = block.GetTablePtr("reflns");
    

//    if (is_empty_table(Table)) return;

    colNo = Table->GetNumColumns();
    rowNo = Table->GetNumRows();

    const vector<string>& ColumnNames = Table->GetColumnNames();

    for (irow = 0; irow < rowNo; irow++) {
        for (i = 0; i < colNo; i++) {
            const string& aString = (*Table)(irow, ColumnNames[i]);
            if (aString == "" || aString == "?" || aString == ".") continue;
            printf("items=%s\n", ColumnNames[i].c_str());
        }
    }
/*    
TblFileObj * fob= new TblFileObj("./TESTFILE1", WRITE_MODE);   
    s = new SSTable(cat_name[0]);
FillTestTable(s, 1, 1, "ABCDEFGHIJKLMNOPQRSTUVWXYZ012345689");
fob->WriteTable(s,blockId,cat_name[0]);

    cat_name = fobjR->GetTableNames(blockId, num);
    for (int ii=0; ii<num; ii++)
        cout<<cat_name[ii]<<endl;
*/    
}

void check_version()
{
    const char *URL="http://sw-tools.rcsb.org/apps/PDB_EXTRACT/source.html";
    char command[512] , ofile[512], **line;
    int nline, i, n=0;
    float version, max=0,local_version;
    
    
    
    strcpy(ofile, "extract_version.html");
//    sprintf(command ,"/bin/csh -c \"wget  -o wget_TMP.log -O  %s  %s\" ", ofile, URL);
    sprintf(command ,"wget  -o wget_TMP.log -O  %s  %s ", ofile, URL);
    n=system(command);
    check_system_call(n, "wget");

    line=get_lines_from_file(ofile, &nline);
    for(i=0; i<nline; i++){
        if(pattern5(line[i],"href", "pdb-extract-v", "-prod-src.tar.gz", "", "")){
            version=get_float(line[i], "extract-v", "-prod-src");
            if(version>max) max=version;
        }
    }
    
    local_version=atof( PDB_EXTRACT_VERSION);
    printf("The latest version on our WEB 'http://sw-tools.rcsb.org/apps/PDB_EXTRACT/' is  %.3f\n",max);
    if(max>local_version){
        printf("Note: your version (v%.3f) is lower than the latest version (v%.3f).\n",
               local_version,max);
        printf("      Please check http://sw-tools.rcsb.org/apps/PDB_EXTRACT/\n");
    }
    if(nline>0) free_memory_2d(line, nline);
    delete_file(ofile);
    delete_file("wget_TMP.log");
    
}

void pdb_extract_info(FILE *fout)
//write information about pdb_extract
{
    char current_time[80];
    struct tm   *newTime;
    time_t      szClock;


    fprintf(fout,"#\n_entry.id  UNNAMED \n\n");
    
    fprintf(fout,"#\nloop_  \n");
    fprintf(fout,"_database_2.database_id \n");
    fprintf(fout,"_database_2.database_code \n");
    fprintf(fout,"PDB  UNNAMED\n");
    fprintf(fout,"WWPDB  UNNAMED\n\n");
    

    strcpy(current_time,"N/A");
    
    time( &szClock ); // Get time in seconds
    newTime = localtime( &szClock );// Convert time to struct tm form 
    // Print local time as a string.
    strcpy(current_time, asctime( newTime ));
    rid_of_front_end_space(current_time);
    
    printf( "Current date and time: %s\n\n", current_time ); // C4996
    
    fprintf(fout,"#\n");
    fprintf(fout,"_data_extraction.extraction_date  '%s'\n", current_time);
    fprintf(fout,"#\n");
}


char ** get_mmcif_category(char *inpfile,  int *ntab)
//input inpfile and return table and ntable
{
    char  old[256],  category[100],item[100], **line, **table, tmp[256], str[256];
    int i, d=0, nline=0,  ncate=0, n1=0;
    FILE  *ftmp=NULL;
    
    ftmp=tmpfile();
    
    *ntab=0;
    
    line=get_lines_from_file(inpfile, &nline);
    if(nline<=0){ return NULL;}

    strcpy(old,"");
    for(i=0; i<nline; i++){
        d=check_category(line[i], category, item);
        if(d>0 && strlen(category)>3 && strlen(item) >1 ){
            if(strlen(old)>0 && !strcmp_case(old, category)) continue;
            ncate++;
            fprintf(ftmp,"%s\n",category);
            strcpy(old, category);
        }
    }
    rewind(ftmp);
    table=new char*[ncate+1];

    n1=0;
    while(fgets(str, sizeof str, ftmp) != NULL){ //# of tables
        sscanf(str,"%s", tmp);
        delete_return(tmp);
        table[n1]=new char[strlen(tmp)+1];
        strcpy(table[n1],tmp);
        n1++;
//        printf("table=%2d, %s\n", n1,tmp);
    }
        
    if(ftmp) fclose(ftmp);
    if(nline && line ) free_memory_2d(line, nline);

    *ntab=ncate;
    return table;
}

char ** get_mmcif_items(char *inpfile, int *nitem)
//input inpfile and return items, nitem
{
    char   category[100],item[100],**items, **line, tmp[256], str[256];
    int i, d=0, nline=0,  nit=0, n2=0;
    FILE  *ftmp=NULL;
    
    ftmp=tmpfile();
    *nitem=0;
    
    line=get_lines_from_file(inpfile, &nline);
    if(nline<=0){  return NULL;}
    
    for(i=0; i<nline; i++){
        d=check_category(line[i], category, item);
        if(d>0 && strlen(category)>3 && strlen(item) >1 ){
            fprintf(ftmp,"%s.%s\n",category,item);
            nit++;
        }
    }
    rewind(ftmp);
    items=new char*[nit];

    n2=0;
    while(fgets(str, sizeof str, ftmp) != NULL){ //# of tables
        sscanf(str,"%s", tmp);
        delete_return(tmp);
        items[n2]=new char[strlen(tmp)+1];
        strcpy(items[n2],tmp);
        n2++;
//        printf("items=%2d, %s\n", n2,tmp);
    }
    
//    printf("numbers= n1=%d; n2=%d :: nit=%d; ncate=%d :: %d\n", n1, n2,nit ,ncate, nline);

    if(ftmp) fclose(ftmp);
    if(nline && line ) free_memory_2d(line, nline);

    *nitem=nit;
    return items;
}

