/*
FILE:     text_parser.C
*/
/*
VERSION:  3.25
*/
/*
DATE:     3/29/2019
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2019 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               RCSB PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/
/* the input coordinate file either is PDF format or CIF */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "rcsb_math.h"
#include "structure.h"
#include "util.h"


void values_in_bracket(FILE *inp, char *first_item, char *value, int *key);
void write_values(FILE *fout, FILE *finp, int nitem, char *id1, char *id2,
                  char *id3, char *id4, char *id5, char *id6, char *id7,
                  char *id8, char *id9);
void initialize_id(char *id1,char *id2,char *id3,char *id4,char *id5,
                   char *id6, char *id7,char *id8, char *id9);

void test_sequence_correction(char *value);
void parse_infor_common(FILE *fout, int nitem, char **items, char **values);
void parse_infor_xray(FILE *fout, int nitem, char **items, char **values);
void parse_infor_nmr(FILE *fout, char *inpfile);
void write_text_parser_cif_head(FILE *fout, char *id);
int check_template(int k,int nitem,char **items, char **values,
                  const char *id1,const char *id2);
void token_value(FILE *fout,int k,int nitem,char **items,
                 char **values,char token[80][500], int ntoken,int key);
int test_first_line(char *item);

extern EXPTL exptl;
extern CELL cell ;

char REF_DETAIL[90000], REF_MODEL[900], INFO[1000];


//----------------------------------------------------------------//

void parse_template_to_mmcif(FILE *fout, char *inpfile_inp, int exp)
// inpfile_inp : input PDB file; exp: experiment type
{
    char  **values, **items,  item[80], *value,  inpfile[512];
    int i,nline=0,nitem=0,n=0,key=0,len;
    FILE *finp=NULL;

    strcpy(REF_DETAIL, "");
    strcpy(REF_MODEL, "");
    sprintf(inpfile, "%s_new", inpfile_inp);
    clean_file(inpfile_inp, inpfile);
    nline = num_of_line(inpfile);
    
    if(nline<1){
        sprintf(INFO, "Error in (parse_template_to_mmcif)! Too few lines in (%s)\n", inpfile);
        pinfo(INFO, 0);
        return;
    }
    
    items = new char *[nline];
    values = new char *[nline];
    value = new char [10000];
    
    finp=fopen(inpfile, "r") ;
    while(1) {
             

        values_in_bracket(finp,  item, value,  &key); //get item/value
        if(!strcmp_case(item, "refinement_detail") && strlen(value)>5){
            strcpy(REF_DETAIL, value);
        }else if (!strcmp_case(item, "refinement_start_model") && strlen(value)>0){
            strcpy(REF_MODEL, value);
        }
        
        if(key>0)break;
        len=strlen(item);
        if(len>0){
            if (test_first_line(item)){ //Add TER to the first line of the category
                items[n]=new char[5];
                values[n]=new char[2];
                strcpy(items[n], "TER");        
                strcpy(values[n],"");
                n++;
            }
            items[n]=new char[len+1];
            strcpy(items[n], item);
            
            len=strlen(value);
            values[n]=new char[len+1];
            strcpy(values[n],value);
            
            n++;
        }
        strcpy(item, "");
    }
    rid_of_front_end_space(REF_DETAIL);
    rid_of_front_end_space(REF_MODEL);
    nitem=n;
    for(i=0;i<nitem; i++){//modifed id in this version
        if (!strcmp(items[i], "contact_author_PI_phone_numer")){
            strcpy(items[i], "contact_author_PI_phone_number");
        }else if(!strcmp(items[i], "contact_author_phone_numer")){
            strcpy(items[i], "contact_author_phone_number");
        }else if (!strcmp(items[i], "data_collection_date")){ //chenghua-20190319 correct typo from "data_collection_data" to "data_collection_date"
            strcpy(items[i], "data_collection_date");
        }            
  //      printf("i=%3d nitem=%2d, items=%s, values=%s\n",i, nitem, items[i], values[i]);
    }
    
    parse_infor_common(fout, nitem, items, values);
    if(exp==0){
        parse_infor_nmr(fout,inpfile );
    }else if (exp==1){
        parse_infor_xray(fout, nitem, items, values);
    }
    
    delete [] value;
    for(i=0; i<nitem; i++){
        delete [] items[i];
        delete [] values[i];
    }
    fclose(finp);
    delete_file(inpfile);
    
}

int test_first_line(char *item){
    int n=0;
    if (strstr(item, "_id") || strstr(item, "radiation__experiment") ||
        strstr(item, "crystal_number") )  n=1;
    return n;
}

int check_bracket(int nitem, char **items, char **values, int nid, char id[][500])
//
{
    int i,j, check=0, key1=0,key2=0;
    if (nid<1) return 0;

    for (j=0; j<nid; j++){
        if (j>0 && !strcmp(id[j], id[j-1])) continue;
        
        for(i=0; i<nitem; i++){             
            if(!strcmp_case(items[i], id[j]) && strlen(values[i])>0 &&
               !strchr(values[i], '?')){
                if(j==0){
                    key1=1;
                }else{
                    key2++;
                }
                
            }
        }
    }
    if ((nid==1 && key1>0) || (nid>1 && key1>0 && key2>0)) check=1;
    return check;
}

void token_value(FILE *fout,int k,int nitem,char **items,
                 char **values,char token[80][500], int ntoken,int key)
//write the values parsed in the template file. if key=1, only for one item.
//data can be manipulated here.    
{
    char **val, tmp[90000];
    int  n=0,j, i=0, len=0, m=0,nloop=0,nonv=0, ndup=0;

    val=new char *[ntoken+1];
    for(j=0; j<ntoken+1; j++){val[j]=(char *)"?";}
    
    n=0;
    for(i=k; i<nitem; i++){
        ndup=0;
        for(j=0; j<ntoken; j++){
            if(!strcmp_case(items[i], token[j])){
                
                if (strstr(items[i], "TER")) {
            //        printf("warning %d %d %d %s ::: %s\n", i, j, n, items[i], tmp);
                    strcpy(tmp,"");
                    ndup=1;
                    break;
                }
                strcat (tmp, items[i]);
                strcat (tmp, " ");
                
                if(!strcmp_case(items[i],"molecule_one_letter_sequence") ){
                    test_sequence_correction(values[i]);
                }
                if(!strcmp_case(items[i],"Manipulated_entity_id") ||
                   !strcmp_case(items[i],"natural_source_entity_id")||
                   !strcmp_case(items[i],"synthetic_source_entity_id")
                   ){
		  // if (j>0) strcpy(values[i], "1"); //3.24 code to force assign _entity_src_gen.pdbx_src_id, _entity_src_nat.pdbx_src_id, _pdbx_entity_src_syn.pdbx_src_id 
		  // chenghua-20190319-start
		  // assign _entity_src_gen.pdbx_src_id only if _entity_src_gen.entity_id is not empty
		  // assign _entity_src_nat.pdbx_src_id only if _entity_src_nat.entity_id is not empty
		  // assign _pdbx_entity_src_syn.pdbx_src_id only if _pdbx_entity_src_syn.entity_id is not empty
		  if (j>0){  
		    if (strcmp_case(values[i],"")){ 
		      strcpy(values[i], "1");
		    }else{
		      strcpy(values[i], "");
		    }
		  }
		  //chenghua-20190319-end
                }
                if(!strcmp_case(items[i],"database_code") &&
                   strlen(values[i])==0 && i>0 && strlen(values[i-1])>0
                   ){
                    strcpy(values[i], ".");
                }
                
                len=strlen(values[i])+1;
                val[j]=new char [len];
                strcpy(val[j], values[i]);
                
                if(strstr_case(items[i],"_temperature") && strstr_case(val[j], "K")){
                    char2space(val[j], 'K');
                }
                rid_of_front_end_space(val[j]);
                
                n++;
            }
        }
            
        if(n==ntoken || ndup==1){ //
            
            strcpy(tmp,"");
            if(n==1 && strlen(val[0])<=0) continue;
            nonv=0;
            for(m=0; m<ntoken; m++){ if(strlen(val[m])==0)nonv++; }
            if(nonv == ntoken) continue; //all values are empty
            
            nloop++;
            for(m=0; m<ntoken; m++){
                len=strlen(val[m]);
                if(key==0){
                    (len>0) ? fprintf(fout,"\n;%s\n; ",val[m]):fprintf(fout,"\n;?\n;");
                }else if (key==1){   
                    (len>0) ? fprintf(fout,"%d \n;%s\n; ",nloop,val[m]):fprintf(fout,"\n;?\n;");
                }else if (key==2){       
                    if (m==0) fprintf(fout,"%d ", nloop);
                    (len>0) ? fprintf(fout,"\n;%s\n; ",val[m]):fprintf(fout,"\n;?\n;");
                }
            }
            n=0;
            fprintf(fout,"\n");
            
        }
    }
    
}

void get_enumerate_value(char **line, int nline, char *cif_item, char enum_val[300][150], int *nval)
//
{
    int i, j, n=0, nn=0;
    nn=strlen(cif_item);
    
    for (i=0; i<nline; i++){
        rid_of_front_end_space(line[i]);
        if(!strncmp_case(line[i],cif_item, nn)){
            n=0;
            for (j=i+1; j<nline; j++){
                rid_of_front_end_space(line[j]);
                
                if (j-i>250 || strlen(line[j])<1 || (j-i>0 && line[j][0]=='_' )){
                    break;
                }else{
                    strcpy(enum_val[n],line[j]);
                    n++;
                }
             }
             break;
         }
    }
    *nval=n;
    
//    for (i=0; i<n; i++){ printf("enumerate=%2d %s\n", i, enum_val[i]);}        
     
}


void check_enumerate(char **line, int nline, char *cif_item,  char *value, char *id)
//compare the values[i] with enumerate list. If not in the list, write in log
// singe cif item
{
    int  m=0, nn=0, nval=0;
    char tmp[200], enum_val[300][150];

    get_enumerate_value(line, nline, cif_item , enum_val, &nval);
    if (nval==0) return; // no enumerate value
    nn=0;
    for (m=0; m<nval; m++){
        clean_string(value,  tmp);
        
        if ((!strcmp_case(value, enum_val[m]) && strcmp(value, enum_val[m])) ||
            (!strcmp_case(tmp, enum_val[m]) && strcmp_case(value, enum_val[m])) ){//case or space problem
            sprintf(INFO, "\nWarning: the input value (%s) is replaced by (%s).\n", value, enum_val[m]);
            pinfo(INFO, 0);
            strcpy(value, enum_val[m]);
            nn=1;
            break;
        }else if(!strcmp(value, enum_val[m])){
            nn=1;
            break;
        }
    }

    if (nn==0 && strlen(value)>0){
        sprintf(INFO,"\nError: Input value (%s) to item (%s) is not in the defined list.\n", value, id);
        pinfo(INFO,0);
        printf("Details can be seen from the pdb_extract.log file.\n");
        for (m=0; m<nval; m++){
            sprintf(INFO, "%2d  %s\n", m+1, enum_val[m]);
            pinfo(INFO, 1);
        }
    }

}

void check_items(char **line, int nline, const char **cif, int nitem,
                 char **items, char **values, int ntoken, char id[80][500])
//compare the values[i] with enumerate list. If not in the list, write in log
{
    int i=0, j=0;
    char cif_item[200];

    
    for (i=0; i<nitem; i++){
        for (j=0; j<ntoken; j++){
            
            if(!strcmp_case(items[i], id[j])){
                strcpy(cif_item, cif[j]);
                check_enumerate(line, nline, cif_item, values[i], items[i]);
            }
        }
    }
}

void parse_infor_common(FILE *fout, int nitem, char **items, char **values)
// extract information common for Xray and NMR
{
    char id[80][500],cif_item[100], author_id[80], enum_file[512], **line;
    int  n,nn, n1=0,n2, ntoken=0, i, len=0, nline, taxid=0;

    get_local_file(enum_file, "cif_enumerate.list");
    line=get_lines_from_file(enum_file,&nline);
    
    static const char *author_pi[]={
        "contact_author_PI_id",
        "contact_author_PI_salutation",
        "contact_author_PI_first_name",
        "contact_author_PI_last_name",
        "contact_author_PI_middle_name",    
        "contact_author_PI_role",
        "contact_author_PI_organization_type",
        "contact_author_PI_email",      
        "contact_author_PI_address",
        "contact_author_PI_city",
        "contact_author_PI_State_or_Province",
        "contact_author_PI_Zip_Code",
        "contact_author_PI_Country",
        "contact_author_PI_fax_number",
        "contact_author_PI_phone_number",
        "END"
    };
    
    static const char *author_cont[]={
        "contact_author_id",
        "contact_author_salutation",
        "contact_author_first_name",
        "contact_author_last_name",
        "contact_author_middle_name",    
        "contact_author_role",
        "contact_author_organization_type",
        "contact_author_email",      
        "contact_author_address",
        "contact_author_city",
        "contact_author_State_or_Province",
        "contact_author_Zip_Code",
        "contact_author_Country",
        "contact_author_fax_number",
        "contact_author_phone_number",
        "END"
    };
    
    static const char *author_cif[]={
        "_pdbx_contact_author.id",
        "_pdbx_contact_author.name_salutation",
        "_pdbx_contact_author.name_first",
        "_pdbx_contact_author.name_last",
        "_pdbx_contact_author.name_mi",
        "_pdbx_contact_author.role",
        "_pdbx_contact_author.organization_type",
        "_pdbx_contact_author.email",
        "_pdbx_contact_author.address_1",
        "_pdbx_contact_author.city",
        "_pdbx_contact_author.state_province",
        "_pdbx_contact_author.postal_code",
        "_pdbx_contact_author.country",
        "_pdbx_contact_author.fax",
        "_pdbx_contact_author.phone",
        "END"
    };
         
    static const char *entity[]={
        "molecule_entity_id",
        "molecule_entity_type",
        "molecule_one_letter_sequence",
        "molecule_chain_id",
        "target_DB_id",
        "END"
    };

        
    static const char *entity_new[]={
        "molecule_entity_id",
        "molecule_entity_type",
        "molecule_one_letter_sequence",
        "molecule_chain_id",
        "target_DB_id",
        "sequence_database_id",
        "sequence_database_name",
        "END"
    };
        
    static const char *entity_cif[]={
        "_entity_poly.entity_id",
        "_entity_poly.type",
        "_entity_poly.pdbx_seq_one_letter_code",
        "_entity_poly.pdbx_strand_id",
        "_entity_poly.pdbx_target_identifier",
        "END"
    };
        
    static const char *entity_cif_new[]={
        "_entity_poly.entity_id",
        "_entity_poly.type",
        "_entity_poly.pdbx_seq_one_letter_code",
        "_entity_poly.pdbx_strand_id",
        "_entity_poly.pdbx_target_identifier",
        "_entity_poly.pdbx_seq_db_id",
        "_entity_poly.pdbx_seq_db_name",
        "END"
    };
    
    static const char *sg_cif[]={
        " _pdbx_SG_project.id",
        " _pdbx_SG_project.project_name",
        " _pdbx_SG_project.full_name_of_center",
        "END"
    };
    
//CATEGORY 2:   Molecular Entity Sequence
    nn=0;
    taxid=0;
    
    for (i=0; i<nitem; i++){
        if (strstr(items[i],"sequence_database_id")) {
            nn=1;
        }else if (strstr(items[i],"Source_organism_taxonomy_id")) {
            taxid=1;
        }
    }
    
    if (nn==0){
        
        ntoken = 5;
        for (i=0; i<ntoken; i++) strcpy(id[i],entity[i]);
        n=check_bracket(nitem, items, values, ntoken, id);
        if(n>0){
            fprintf(fout,"#\nloop_\n");
            for (i=0; i<ntoken; i++) fprintf(fout,"%s\n", entity_cif[i]);
            token_value(fout, 0, nitem, items, values, id, ntoken, 0);
        }
    }else{
        ntoken = 7;
        for (i=0; i<ntoken; i++) strcpy(id[i],entity_new[i]);
        n=check_bracket(nitem, items, values, ntoken, id);
        if(n>0){
            fprintf(fout,"#\nloop_\n");
            for (i=0; i<ntoken; i++) fprintf(fout,"%s\n", entity_cif_new[i]);
            token_value(fout, 0, nitem, items, values, id, ntoken, 0);
        }
    }
    
    
//CATEGORY 3, (Contact Authors for PI & others)
    ntoken = 15;
    for (i=0; i<ntoken; i++) strcpy(id[i],author_pi[i]);
    n1=check_bracket(nitem, items, values,ntoken, id);
    if(n1){
        fprintf(fout,"#\nloop_\n");
        for (i=0; i<ntoken; i++) { fprintf(fout,"%s\n", author_cif[i]);}
        check_items(line, nline, author_cif, nitem, items, values, ntoken, id);

        token_value(fout, 0, nitem, items, values, id, ntoken, 0);
    }
    
    ntoken = 15;
    for (i=0; i<ntoken; i++) strcpy(id[i],author_cont[i]);
    n2=check_bracket(nitem, items, values, ntoken, id);
    if(n2){
        if(n1<=0){
            sprintf(INFO, "Warning. You did not enter information for the PI. \n");
            pinfo(INFO, 0);
        }else{
            check_items(line, nline, author_cif, nitem, items, values, ntoken, id);
            token_value(fout, 0, nitem, items, values, id, ntoken, 0);
        }
    }
    
    
//CATEGORY  (Structure Genomics)

    ntoken= 3;
    strcpy(id[0],"SG_project_id");
    strcpy(id[1],"SG_project_name");
    strcpy(id[2],"full_name_of_SG_center");

 //   n=check_template( 0, nitem, items, values,id[0], id[2]);
    n=check_bracket(nitem, items, values, ntoken, id);
    if(n){
        fprintf(fout,"#\nloop_\n");
        for (i=0; i<ntoken; i++) { fprintf(fout,"%s\n", sg_cif[i]);}
        check_items(line, nline, sg_cif, nitem, items, values, ntoken, id);
    
        token_value(fout, 0, nitem, items, values, id, ntoken, 0);
    }
    

//CATEGORY 4,  (Release Status)
    n1=0;
    n2=0;
    for(i=0; i<nitem; i++){
        len = strlen(values[i]);
        if(len<0) continue;
        
        if(!strcmp_case(items[i], "Release_status_for_coordinates")){
            fprintf(fout,"_pdbx_database_status.entry_id  UNNAMED \n");
            fprintf(fout,"_pdbx_database_status.status_code  . \n");
            fprintf(fout,"_pdbx_database_status.dep_release_code_coordinates ");
            fprintf(fout,"\n;%s\n; \n",values[i]);
            
            sprintf(cif_item,"_pdbx_database_status.dep_release_code_coordinates");
            check_enumerate(line, nline, cif_item,values[i], items[i]);
        }else if(strstr_case(items[i], "Release_status_for_NMR_constraints")){
            fprintf(fout,"_pdbx_database_status.dep_release_code_nmr_constraints ");
            fprintf(fout," \n;%s\n;\n",values[i]);

            sprintf(cif_item,"_pdbx_database_status.dep_release_code_nmr_constraints");
            check_enumerate(line, nline, cif_item,values[i], items[i]);
        }else if(strstr_case(items[i], "Release_status_for_structure_factor")){
            fprintf(fout,"_pdbx_database_status.dep_release_code_struct_fact ");
            fprintf(fout," \n;%s\n;\n",values[i]);
            
            sprintf(cif_item,"_pdbx_database_status.dep_release_code_struct_fact");
            check_enumerate(line, nline, cif_item,values[i], items[i]);
        }else if(strstr_case(items[i], "Release_status_for_sequence")){
            fprintf(fout,"_pdbx_database_status.dep_release_code_sequence ");
            fprintf(fout," \n;%s\n;\n",values[i]);
            
            sprintf(cif_item,"_pdbx_database_status.dep_release_code_sequence");
            check_enumerate(line, nline, cif_item,values[i], items[i]);

        }else if(strstr_case(items[i], "binding_assay_ligand_descriptor_type")){
            sprintf(cif_item,"_pdbx_binding_assay.ligand_descriptor_type");
            check_enumerate(line, nline, cif_item,values[i], items[i]);
            
        }else if(strstr_case(items[i], "binding_assay_assay_value_type")){
            sprintf(cif_item,"_pdbx_binding_assay.assay_value_type");
            check_enumerate(line, nline, cif_item,values[i], items[i]);
            
//CATEGORY 5 (Structure Title)        
        }else if(!strcmp_case(items[i], "structure_title") && strlen(values[i])>1){  
            fprintf(fout,"\n_struct.entry_id UNNAMED \n");
            fprintf(fout,"_struct.title \n;%s\n;\n",values[i]);
            
        }else if(!strcmp_case(items[i], "structure_details") && strlen(values[i])>1){  
            fprintf(fout,"\n_struct.pdbx_model_details \n;%s\n;\n",values[i]);


//CATEGORY 12 (Keywords), 

        }else if(!strcmp_case(items[i], "structure_keywords")&& strlen(values[i])>1){
            fprintf(fout,"\n_struct_keywords.entry_id  UNNAMED \n");
            fprintf(fout,"_struct_keywords.text \n;%s\n;\n\n",values[i]);
        }
    }


//CATEGORY  (Author)
    ntoken = 1;
    strcpy(id[0],"structure_author_name");
    n=check_bracket(nitem, items, values, ntoken, id);
    if(n){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_audit_author.pdbx_ordinal \n");
        fprintf(fout,"_audit_author.name\n");
        token_value(fout, 0, nitem, items, values, id, ntoken, 1);
    }
    
            
//CATEGORY 8:   Molecule Information (the new template)
    ntoken = 4;
    strcpy(id[0],"molecule_id");
    strcpy(id[1],"molecule_name");
    strcpy(id[2],"molecule_type");
    strcpy(id[3],"molecule_source_method");
    n=check_bracket(nitem, items, values, ntoken, id);
    if(n){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_entity.id\n");
        fprintf(fout,"_entity.pdbx_description\n");
        fprintf(fout,"_entity.type\n");
        fprintf(fout,"_entity.src_method\n");
        token_value(fout, 0, nitem, items, values, id, ntoken, 0);
    }
    
//CATEGORY 8:   Molecule Information (the old template)
    ntoken = 1;
    strcpy(id[0],"molecule_name");
    n=check_bracket(nitem, items, values, ntoken, id);
    if(n){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_entity.id\n");
        fprintf(fout,"_entity.pdbx_description\n");
        token_value(fout, 0, nitem, items, values, id, ntoken, 1);
    }
    
//CATEGORY (Citation Authors)

    ntoken = 1;
    strcpy(id[0],"primary_citation_author_name");
    n=check_bracket(nitem, items, values, ntoken, id);
    if(n){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_citation_author.citation_id\n");
        fprintf(fout,"_citation_author.name\n");
        fprintf(fout,"_citation_author.ordinal\n");
    
        n1=0;
        n2=0;
        for(i=0; i<nitem; i++){
            len = strlen(values[i]);
            if(len<=0) continue;
            if(!strcmp_case(items[i], "primary_citation_author_name")){
                n1++;
                fprintf(fout,"primary   '%s' %d\n",  values[i], n1);
            }
    
            if(!strcmp_case(items[i], "citation_author_id")){
                strcpy(author_id,values[i]);
            }
        
            if(!strcmp_case(items[i], "citation_author_name") && strlen(author_id)>0){
                n2++;
                fprintf(fout,"'%s'  '%s' %d\n",author_id, values[i], n2);
            }
        }
    }
    
    

//CATEGORY 7, (Citation Article)
    ntoken = 7;
    strcpy(id[0],"primary_citation_id");
    strcpy(id[1],"primary_citation_journal_abbrev");
    strcpy(id[2],"primary_citation_title");
    strcpy(id[3],"primary_citation_year");
    strcpy(id[4],"primary_citation_journal_volume");
    strcpy(id[5],"primary_citation_page_first");
    strcpy(id[6],"primary_citation_page_last");

  //  n=check_template( 0, nitem, items, values,id[0],id[1]);
    n=check_bracket(nitem, items, values, ntoken, id);
    if(n){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_citation.id \n");
        fprintf(fout,"_citation.journal_abbrev \n");
        fprintf(fout,"_citation.title \n");
        fprintf(fout,"_citation.year \n");
        fprintf(fout,"_citation.journal_volume \n");
        fprintf(fout,"_citation.page_first \n");
        fprintf(fout,"_citation.page_last\n");
    
        token_value(fout, 0, nitem, items, values, id, ntoken, 0);

        
        ntoken = 7;
        strcpy(id[0],"citation_id");
        strcpy(id[1],"citation_journal_abbrev");
        strcpy(id[2],"citation_title");
        strcpy(id[3],"citation_year");
        strcpy(id[4],"citation_journal_volume");
        strcpy(id[5],"citation_page_first");
        strcpy(id[6],"citation_page_last");
      //  n1=check_template( 0, nitem, items, values,id[0],id[1]);
        n1=check_bracket(nitem, items, values, ntoken, id);
        if(n1){
            token_value(fout, 0, nitem, items, values, id, ntoken, 0);
        }
    }
    

    
//CATEGORY 9:   Molecule Details
    
    ntoken = 4;
    strcpy(id[0],"molecular_entity_id");
    strcpy(id[1],"Fragment_name");
    strcpy(id[2],"Specific_mutation");
    strcpy(id[3],"Enzyme_Comission_number");

   // n=check_template( 0, nitem, items, values,id[0],id[1]);
    n=check_bracket(nitem, items, values, ntoken, id);
    if(n){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_entity_keywords.entity_id\n");
        fprintf(fout,"_entity_keywords.pdbx_fragment\n");
        fprintf(fout,"_entity_keywords.pdbx_mutation\n");
        fprintf(fout,"_entity_keywords.pdbx_ec\n");
        token_value(fout, 0, nitem, items, values, id, ntoken, 0);
    }
    
//CATEGORY 10 (Genetically Manipulated Source)    
    
    ntoken = 11;
    if (taxid==0) ntoken = 10;
    strcpy(id[0],"Manipulated_entity_id");
    strcpy(id[1],"Manipulated_entity_id"); //both the same
    strcpy(id[2],"Source_organism_scientific_name");
    strcpy(id[3],"Source_organism_gene");
    strcpy(id[4],"Source_organism_strain");
    strcpy(id[5],"Expression_system_scientific_name");
    strcpy(id[6],"Expression_system_strain");
    strcpy(id[7],"Expression_system_vector_type");
    strcpy(id[8],"Expression_system_plasmid_name");
    strcpy(id[9],"Manipulated_source_details");
    strcpy(id[10],"Source_organism_taxonomy_id");
    
    //n=check_template( 0, nitem, items, values,id[0],"");
    n=check_bracket(nitem, items, values, ntoken, id);
    if(n){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_entity_src_gen.entity_id\n");
        fprintf(fout,"_entity_src_gen.pdbx_src_id\n");
        fprintf(fout,"_entity_src_gen.pdbx_gene_src_scientific_name\n");
        fprintf(fout,"_entity_src_gen.pdbx_gene_src_gene\n");
        fprintf(fout,"_entity_src_gen.gene_src_strain\n");
        fprintf(fout,"_entity_src_gen.pdbx_host_org_scientific_name\n");
        fprintf(fout,"_entity_src_gen.pdbx_host_org_strain\n");
        fprintf(fout,"_entity_src_gen.pdbx_host_org_vector_type\n");
        fprintf(fout,"_entity_src_gen.plasmid_name\n");
        fprintf(fout,"_entity_src_gen.gene_src_details\n");
        if (taxid>0)  fprintf(fout,"_entity_src_gen.pdbx_gene_src_ncbi_taxonomy_id\n");
        
        token_value(fout, 0, nitem, items, values, id, ntoken, 0);
    }

 //CATEGORY 11 (Category Natural Source)

    ntoken = 5;
    strcpy(id[0],"natural_source_entity_id");
    strcpy(id[1],"natural_source_entity_id");
    strcpy(id[2],"natural_source_scientific_name");
    strcpy(id[3],"natural_source_organism_strain");
    strcpy(id[4],"natural_source_details");

  //  n=check_template( 0, nitem, items, values,id[0],"");
    n=check_bracket(nitem, items, values, ntoken, id);
    if(n){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_entity_src_nat.entity_id \n");
        fprintf(fout,"_entity_src_nat.pdbx_src_id \n");
        fprintf(fout,"_entity_src_nat.pdbx_organism_scientific\n");
        fprintf(fout,"_entity_src_nat.strain\n");
        fprintf(fout,"_entity_src_nat.details\n");
        token_value(fout, 0, nitem, items, values, id, ntoken, 0);
    }
    
 //CATEGORY  (Category Synthetic Source)

    
    ntoken = 3;
    strcpy(id[0],"synthetic_source_entity_id");
    strcpy(id[1],"synthetic_source_entity_id");
    strcpy(id[2],"synthetic_source_description");
    
//    n=check_template( 0, nitem, items, values,id[0],id[1]);
    n=check_bracket(nitem, items, values, ntoken, id);
    if(n){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_pdbx_entity_src_syn.entity_id \n");
        fprintf(fout,"_pdbx_entity_src_syn.pdbx_src_id \n");
        fprintf(fout,"_pdbx_entity_src_syn.details\n");
        token_value(fout, 0, nitem, items, values, id, ntoken, 0);
    }

 //CATEGORY 20 (database)

    
    ntoken = 4;
    strcpy(id[0],"database_entity_id");
    strcpy(id[1],"database_name");
    strcpy(id[2],"database_code");
    strcpy(id[3],"database_accession");


  //  n=check_template( 0, nitem, items, values,id[0],id[1]);
    n=check_bracket(nitem, items, values, ntoken, id);
    if(n){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_struct_ref.id \n");
        fprintf(fout,"_struct_ref.entity_id \n");
        fprintf(fout,"_struct_ref.db_name \n");
        fprintf(fout,"_struct_ref.db_code \n");
        fprintf(fout,"_struct_ref.pdbx_db_accession \n");
        token_value(fout, 0, nitem, items, values, id, ntoken, 2);
    }

// CATEGORY 21 (ligand binding assay)
    
    ntoken = 10;
    strcpy(id[0],"binding_assay_id");
    strcpy(id[1],"binding_assay_target_sequence_one_letter_code");
    strcpy(id[2],"binding_assay_ligand_descriptor_type");
    strcpy(id[3],"binding_assay_ligand_descriptor");
    strcpy(id[4],"binding_assay_assay_type");
    strcpy(id[5],"binding_assay_assay_value_type");
    strcpy(id[6],"binding_assay_assay_value");
    strcpy(id[7],"binding_assay_assay_pH");
    strcpy(id[8],"binding_assay_assay_temperature");
    strcpy(id[9],"binding_assay_details");

    
    n=check_bracket(nitem, items, values, ntoken, id);
    if(n){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_pdbx_binding_assay.id\n");
        fprintf(fout,"_pdbx_binding_assay.target_sequence_one_letter_code\n");    
        fprintf(fout,"_pdbx_binding_assay.ligand_descriptor_type\n");
        fprintf(fout,"_pdbx_binding_assay.ligand_descriptor\n");
        fprintf(fout,"_pdbx_binding_assay.assay_type\n");
        fprintf(fout,"_pdbx_binding_assay.assay_value_type\n");    
        fprintf(fout,"_pdbx_binding_assay.assay_value\n");
        fprintf(fout,"_pdbx_binding_assay.assay_pH\n");
        fprintf(fout,"_pdbx_binding_assay.assay_temperature\n");
        fprintf(fout,"_pdbx_binding_assay.details\n");

        token_value(fout, 0, nitem, items, values, id, ntoken, 0);
    }
    
    
}

void parse_infor_xray(FILE *fout, int nitem, char **items, char **values)
// extracting the information related only to Xray  
{
    char **line, enum_file[512], sg[20], id[80][500];;
    int  n, ntoken=0, i, nline=0,  ncryst=0, len=0;

    static const char *source_cif[]={
        "_diffrn_source.diffrn_id",
        "_diffrn_source.source",
        "_diffrn_source.type",
        "_diffrn_source.pdbx_wavelength_list",
        "END"
    };

    static const char *detect_cif[]={
        "_diffrn_detector.diffrn_id",
        "_diffrn_detector.detector",
        "_diffrn_detector.type",
        "_diffrn_detector.pdbx_collection_date",
        "_diffrn_detector.details",
        "END"
    };

    static const char *proc_cif[]={
        "_diffrn_radiation.diffrn_id",
        "_diffrn_radiation.pdbx_diffrn_protocol",
        "_diffrn_radiation.monochromator",
        "_diffrn_radiation.pdbx_monochromatic_or_laue_m_l",
        "END"
    };

    static const char *grow_cif[]={
        "_exptl_crystal_grow.crystal_id",
        "_exptl_crystal_grow.method",
        "_exptl_crystal_grow.pH",
        "_exptl_crystal_grow.temp",
        "_exptl_crystal_grow.pdbx_details",
        "END"
    };

    get_local_file(enum_file, "cif_enumerate.list");
    line=get_lines_from_file(enum_file,&nline);


//CATEGORY 1  (Cell Parameters)
    for(i=0; i<nitem; i++){
        len=strlen(values[i]);
        if(len<=0)continue;
        
        if(!strcmp_case(items[i], "space_group") ){
            space_group_convertor(values[i], sg);
            fprintf(fout,"_symmetry.entry_id      UNNAMED \n");
            fprintf(fout,"_symmetry.space_group_name_H-M  '%s'\n",sg);
                    
        }else if(!strcmp_case(items[i],"space_group_number") ){
            fprintf(fout,"_symmetry.Int_Tables_number  '%s'\n",values[i]);
            
        }else if(!strcmp_case(items[i], "unit_cell_a") ){
            cell.length_a = atof(values[i]);
            
        }else if(!strcmp_case(items[i], "unit_cell_b")){
            cell.length_b = atof(values[i]);
            
        }else if(!strcmp_case(items[i], "unit_cell_c")){
            cell.length_c = atof(values[i]);
            
        }else if(!strcmp_case(items[i], "unit_cell_alpha")){
            cell.angle_alpha = atof(values[i]);
            
        }else if(!strcmp_case(items[i], "unit_cell_beta")){
            cell.angle_beta = atof(values[i]);
            
        }else if(!strcmp_case(items[i], "unit_cell_gamma")){
            cell.angle_gamma = atof(values[i]);

        }else if(!strcmp_case(items[i], "crystal_number")&& len>0 ){
            ncryst=atoi(values[i]);
        }
        
    }
    if(ncryst>0){
        fprintf(fout,"#\n");
        fprintf(fout,"\n_exptl.crystals_number %d\n",ncryst);
        fprintf(fout,"_exptl.entry_id   UNNAMED\n");
        fprintf(fout,"_exptl.method   'X-RAY DIFFRACTION'\n");
    }    
    
//CATEGORY   (Biological Assembly)
            
    ntoken = 1;
    strcpy(id[0],"biological_assembly_chain_number");
    //n=check_template( 0, nitem, items, values,id[0],"");
    n=check_bracket(nitem, items, values, ntoken, id);
    
    if(n){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_pdbx_struct_assembly_depositor_info.id\n");
        fprintf(fout,"_pdbx_struct_assembly_depositor_info.oligomeric_count\n");
        token_value(fout, 0, nitem, items, values, id, ntoken, 1);
    }

    
//CATEGORY  (Crystal Property)
    

    ntoken = 3;
    strcpy(id[0],"crystals_number");
    strcpy(id[1],"crystals_solvent_content");
    strcpy(id[2],"crystals_matthews_coefficient");
//    strcpy(id[3],"crystals_mosaicity");

 //   n=check_template( 0, nitem, items, values,id[0],"");
    n=check_bracket(nitem, items, values, ntoken, id);
    if(n){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_exptl_crystal.id \n");
        fprintf(fout,"_exptl_crystal.density_percent_sol\n");
        fprintf(fout,"_exptl_crystal.density_Matthews\n");
//        fprintf(fout,"_exptl_crystal.pdbx_mosaicity\n");

        token_value(fout, 0, nitem, items, values, id, ntoken, 0);
    }
    

    
//CATEGORY 15 (Methods and Conditions)
    ntoken = 5;
    strcpy(id[0],"crystal_number");
    strcpy(id[1],"crystallization_method");
    strcpy(id[2],"crystallization_pH");
    strcpy(id[3],"crystallization_temperature");
    strcpy(id[4],"crystallization_details");
    
    //n=check_template( 0, nitem, items, values,id[0],"");
    n=check_bracket(nitem, items, values, ntoken, id);
    if(n){
        fprintf(fout,"#\nloop_\n");
        for (i=0; i<ntoken; i++) { fprintf(fout,"%s\n", grow_cif[i]);}
        check_items(line, nline, grow_cif, nitem, items, values, ntoken, id);

        token_value(fout, 0, nitem, items, values, id, ntoken, 0);
    }
    
//CATEGORY  (Radiation Source)
    
    ntoken = 4;
    strcpy(id[0],"radiation_experiment");
    strcpy(id[1],"radiation_source");
    strcpy(id[2],"radiation_source_type");
    strcpy(id[3],"radiation_wavelengths");

  //  n=check_template( 0, nitem, items, values,id[0],"");
    n=check_bracket(nitem, items, values, ntoken, id);
    if(n){
        fprintf(fout,"#\nloop_\n");

        for (i=0; i<ntoken; i++) { fprintf(fout,"%s\n", source_cif[i]);}
        check_items(line, nline, source_cif, nitem, items, values, ntoken, id);

        token_value(fout, 0, nitem, items, values, id, ntoken, 0);
    }
    
//---    

    
    ntoken = 5;
    strcpy(id[0],"radiation_experiment");
    strcpy(id[1],"radiation_detector");
    strcpy(id[2],"radiation_detector_type");
    strcpy(id[3],"data_collection_date");
    strcpy(id[4],"radiation_detector_details");
   // n=check_template( 0, nitem, items, values,id[0],"");
    n=check_bracket(nitem, items, values, ntoken, id);
    if(n){
        fprintf(fout,"#\nloop_\n");
        for (i=0; i<ntoken; i++) { fprintf(fout,"%s\n", detect_cif[i]);}
        check_items(line, nline, detect_cif, nitem, items, values, ntoken, id);
        token_value(fout, 0, nitem, items, values, id, ntoken, 0);
    }
    
//---    
    
    ntoken = 4;
    strcpy(id[0],"radiation_experiment");
    strcpy(id[1],"data_collection_protocol");
    strcpy(id[2],"data_collection_monochromator");
    strcpy(id[3],"data_collection_monochromatic_or_laue");

//    n=check_template( 0, nitem, items, values,id[0], id[2]);
    n=check_bracket(nitem, items, values, ntoken, id);
    if(n){
        fprintf(fout,"#\nloop_\n");
        for (i=0; i<ntoken; i++) { fprintf(fout,"%s\n", proc_cif[i]);}
        check_items(line, nline, proc_cif, nitem, items, values, ntoken, id);

        token_value(fout, 0, nitem, items, values, id, ntoken, 0);
    }
    
//---    

    
    ntoken= 2;
    strcpy(id[0],"radiation_experiment");
    strcpy(id[1],"radiation_wavelengths");
  //  n=check_template( 0, nitem, items, values,id[0],id[1]);
    n=check_bracket(nitem, items, values, ntoken, id);
    if(n){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_diffrn_radiation_wavelength.id \n");
        fprintf(fout,"_diffrn_radiation_wavelength.wavelength \n");
        token_value(fout, 0, nitem, items, values, id, ntoken, 0);
    }
    

    
//CATEGORY   (Collection Temperature)
    
    ntoken= 3;
    strcpy(id[0],"radiation_experiment");
    strcpy(id[1],"crystal_number");
    strcpy(id[2],"data_collection_temperature");
    n=check_template( 0, nitem, items, values,id[0],id[1]);
    if(n){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_diffrn.id\n");
        fprintf(fout,"_diffrn.crystal_id\n");
        fprintf(fout,"_diffrn.ambient_temp\n");
        token_value(fout, 0, nitem, items, values, id, ntoken, 0);
    }
    
}


int check_template(int k,int nitem,char **items, char **values,
                  const char *id1, const char *id2)
//return 1 if it is OK.
{
    int i, key1=0, key2=0, check=0;


    if(strlen(id1)>0 && strlen(id2)>0){
        for(i=k; i<nitem; i++){
            if(!strcmp_case(items[i], id1) && strlen(values[i])>0){
                key1=1;
            }else if(!strcmp_case(items[i], id2) && strlen(values[i])>0){
                key2=1;
            }
            if(key1==1 && key2==1) {
                check=1;
                break;
            }
        }
    }else if(strlen(id1)>0 && strlen(id2)<=0){
        for(i=k; i<nitem; i++){
            if(!strcmp_case(items[i], id1) && strlen(values[i])>0){
                check=1;
                break;
            }
        }
    }else if(strlen(id2)>0 && strlen(id1)<=0){
        for(i=k; i<nitem; i++){
            if(!strcmp_case(items[i], id2) && strlen(values[i])>0){
                check=1;
                break;
            }
        }
    }
    
    return check;
}



/////////////////////////////////////////////////////////////////////

void parse_infor_nmr(FILE *fout, char *inpfile)
// extracting the information related only to NMR    
{
    char *value, item[80];
    char id1[60],id2[60],id3[60],id4[60],id5[60],id6[60],id7[60],id8[60],id9[60];
    int key,  nitem=0;
    FILE *finp=NULL;
    
    if((finp=fopen(inpfile, "r")) ==NULL)
        open_file_error(inpfile, "extract_infor_for_nmr");

    value = new char[10000];
    
//CATEGORY 12:   Ensemble

    rewind(finp);
    while(1) { 
        values_in_bracket(finp,  item, value,  &key);
        if(key>0)break;
        if( strlen(value)<=0 ) continue;
        if(!strcmp_case(item, "conformers_calculated_total_number")){
            fprintf(fout,"\n _pdbx_nmr_ensemble.conformers_calculated_total_number\n;%s\n;\n\n",value);
            
        }else if(!strcmp_case(item, "conformers_submitted_total_number")){
            fprintf(fout,"\n _pdbx_nmr_ensemble.conformers_submitted_total_number\n;%s\n;\n\n",value);
            fprintf(fout,"\n _pdbx_nmr_ensemble.entry_id  UNNAMED\n");
            
        }else if(!strcmp_case(item, "conformers_selection_criteria")){
            fprintf(fout,"\n_pdbx_nmr_ensemble.conformer_selection_criteria \n;%s\n;\n\n",value);
        }
        
    }

//CATEGORY 13:   Representative Conformers
    rewind(finp);
    while(1) { 
        values_in_bracket(finp,  item, value,  &key);
        if(key>0)break;
        if( strlen(value)<=0 ) continue;
        if(!strcmp_case(item, "conformer_id")){
            fprintf(fout,"\n_pdbx_nmr_representative.conformer_id \n;%s\n;\n\n",value);
            
        }else if(!strcmp_case(item, "conformer_selection_criteria")){
            fprintf(fout,"\n_pdbx_nmr_representative.selection_criteria \n;%s\n;\n\n",value);
        }
    }

//CATEGORY 14:   Sample Details
    fprintf(fout,"#\nloop_\n");
    fprintf(fout,"_pdbx_nmr_sample_details.solution_id\n");
    fprintf(fout,"_pdbx_nmr_sample_details.contents\n");
    fprintf(fout,"_pdbx_nmr_sample_details.solvent_system\n");
    
    nitem = 3;
    initialize_id(id1,id2,id3,id4,id5,id6,id7,id8,id9);
    strcpy(id1,"solution_id");
    strcpy(id2,"solution_content");
    strcpy(id3,"solvent_system");

    write_values(fout, finp,  nitem, id1,id2,id3,id4,id5,id6,id7 ,id8,id9);

//CATEGORY 15:   Sample Conditions

     fprintf(fout,"#\nloop_\n");
     fprintf(fout,"_pdbx_nmr_exptl_sample_conditions.conditions_id\n");
     fprintf(fout,"_pdbx_nmr_exptl_sample_conditions.temperature\n");
     fprintf(fout,"_pdbx_nmr_exptl_sample_conditions.pressure\n");
     fprintf(fout,"_pdbx_nmr_exptl_sample_conditions.pH\n");
     fprintf(fout,"_pdbx_nmr_exptl_sample_conditions.ionic_strength\n");
   
    
    nitem = 5;
    initialize_id(id1,id2,id3,id4,id5,id6,id7,id8,id9);
    strcpy(id1,"Conditions_id");
    strcpy(id2,"Temperature");
    strcpy(id3,"Pressure");
    strcpy(id4,"pH_value");
    strcpy(id5,"Ionic_strength");

    write_values(fout, finp,  nitem, id1,id2,id3,id4,id5,id6,id7,id8,id9 );

// CATEGORY 16:   Spectrometer
    
    fprintf(fout,"#\nloop_\n");
    fprintf(fout,"_pdbx_nmr_spectrometer.spectrometer_id\n");
    fprintf(fout,"_pdbx_nmr_spectrometer.manufacturer\n");
    fprintf(fout,"_pdbx_nmr_spectrometer.model\n");
    fprintf(fout,"_pdbx_nmr_spectrometer.field_strength\n");

    nitem = 4;
    initialize_id(id1,id2,id3,id4,id5,id6,id7,id8,id9);
    strcpy(id1,"spectrometer_id");
    strcpy(id2,"spectrometer_manufacturer");
    strcpy(id3,"spectrometer_model");
    strcpy(id4,"spectrometer_field_strength");
    write_values(fout, finp,  nitem, id1,id2,id3,id4,id5,id6,id7,id8,id9 );

// CATEGORY 17:    Experiment Type
    
    fprintf(fout,"#\nloop_\n");
    fprintf(fout,"_pdbx_nmr_exptl.experiment_id\n");
    fprintf(fout,"_pdbx_nmr_exptl.solution_id\n");
    fprintf(fout,"_pdbx_nmr_exptl.conditions_id\n");
    fprintf(fout,"_pdbx_nmr_exptl.type\n");

    nitem = 4;
    initialize_id(id1,id2,id3,id4,id5,id6,id7,id8,id9);
    strcpy(id1,"experiment_type_id");
    strcpy(id2,"solution_type_id");
    strcpy(id3,"conditions_type_id");
    strcpy(id4,"Experiment_type");
    write_values(fout, finp,  nitem, id1,id2,id3,id4,id5,id6,id7,id8,id9 );

//CATEGORY 18:   Method and Details

    rewind(finp);
    while(1) { 
        values_in_bracket(finp,  item, value,  &key);
        if(key>0)break;
        if( strlen(value)<=0 ) continue;
        if(!strcmp_case(item, "NMR_method")){
            fprintf(fout,"\n_pdbx_nmr_refine.entry_id 	UNNAMED \n");
            fprintf(fout,"\n_pdbx_nmr_refine.method \n;%s\n;\n\n",value);
            
        }else if(!strcmp_case(item, "NMR_details")){
            fprintf(fout,"\n_pdbx_nmr_refine.details \n;%s\n;\n\n",value);
        }
    }
    fclose(finp);
    if(value) delete [] value;
}

void initialize_id(char *id1,char *id2,char *id3,char *id4,char *id5,
                   char *id6, char *id7,char *id8, char *id9)
{
    strcpy(id1," ?? ");
    strcpy(id2," ?? ");
    strcpy(id3," ?? ");
    strcpy(id4," ?? ");
    strcpy(id5," ?? ");
    strcpy(id6," ?? ");
    strcpy(id7," ?? ");
    strcpy(id8," ?? ");
    strcpy(id9," ?? ");
}

void test_sequence_correction(char *value)
{
    int i=0, n;
    
    n = strlen(value);
    for(i=0; i<n-1; i++){
        if(value[i] == '?'){
            sprintf(INFO, "\nError: in the chemical sequences: \n \
There are \'????\' in the chemical sequences in the template file.\n \
Please do the following corrections, before proceeding.\n \
If \'????\' are caused by missing residues, replace them by the missing residue names\n \
Otherwise, please delete the \'????\'.\n");
            pinfo(INFO,0);
            exit(0);
        }
    }
}

void write_values(FILE *fout, FILE *finp, int nitem, char *id1,char *id2,
                  char *id3,char *id4,char *id5, char *id6, char *id7,
                  char *id8, char *id9)
/*A general route for writting values */
{
    char item[80], *value=NULL;
    char *str1=NULL, *str2=NULL, *str3=NULL, *str4=NULL;
    char *str5=NULL, *str6=NULL, *str7=NULL, *str8=NULL, *str9=NULL;
    int  n1=0, n2=0, n3=0, n4=0, n5=0, n6=0, n7=0, n8=0, n9=0;
    int  nn1=0, key=0, len=0;
   
    rewind(finp);
    
    while(1) {
        value= new char[10000]; //max values
        values_in_bracket(finp,  item, value,  &key);

        rid_of_front_end_space(value);
        
        len=strlen(value)+2;
        
        if(key>0)break;

            
        if(strstr_case(item, id1)){
            
            str1=new char[len];
            strcpy(str1, value);
            n1++;
        }else if(strstr_case(item, id2)){
            str2=new char[len+2];
            strcpy(str2, value);
            n2++;
        }else if(strstr_case(item, id3)){
            str3=new char[len+2];
            strcpy(str3, value);
            n3++;
        }else if(strstr_case(item, id4)){
            
            str4=new char[len+2];
            strcpy(str4, value);
            n4++;
            
        }else if(strstr_case(item, id5)){
            str5=new char[len+2];
            strcpy(str5, value);
            n5++; 
        }else if(strstr_case(item, id6)){
            str6=new char[len+2];
            strcpy(str6, value);
            n6++; 
        }else if(strstr_case(item, id7)){
            str7=new char[len+2];
            strcpy(str7, value);
            n7++; 
        }else if(strstr_case(item, id8)){
            str8=new char[len+2];
            strcpy(str8, value);
            n8++; 
        }else if(strstr_case(item, id9)){
            str9=new char[len+2];
            strcpy(str9, value);
            n9++; 
        }
        delete [] value;
        
        if(nitem==1 &&  n1>0){
            if(strlen(str1)>0){
                nn1++;
                fprintf(fout,"\n;%s\n; ", str1);
                n1=0; 
                str1=NULL;
            }else{
                n1=0; 
                str1=NULL;
            }
            fprintf(fout,"\n");
            
        }else if(nitem==2 &&  n1>0 && n2>0){
            if(strlen(str1)>0){
                nn1++;
                fprintf(fout,"\n;%s\n; ", str1);
                (strlen(str2)>0) ? fprintf(fout,"\n;%s\n; ",str2):fprintf(fout,"\n;?\n;");
                n1=0; n2=0;
                str1=NULL; str2=NULL;
            }else{
                n1=0; n2=0;
                str1=NULL; str2=NULL;
            }
            fprintf(fout,"\n");
            
        }else if (nitem==3 &&  n1>0 && n2>0 && n3>0){
            if(strlen(str1)>0){
                nn1++;
                fprintf(fout,"\n;%s\n; ", str1);
                (strlen(str2)>0) ? fprintf(fout,"\n;%s\n; ",str2):fprintf(fout,"\n;?\n;");
                (strlen(str3)>0) ? fprintf(fout,"\n;%s\n; ",str3):fprintf(fout,"\n;?\n;");
                n1=0; n2=0; n3=0;
                str1=NULL; str2=NULL; str3=NULL;
            }else{
                n1=0; n2=0; n3=0;
                str1=NULL; str2=NULL; str3=NULL;
            }
            fprintf(fout,"\n");
            
        }else if (nitem==4 && n1>0 && n2>0 && n3>0 && n4>0){
            
            if(strstr_case(id1,"entity_id") && strstr_case(id2,"entity_type")&&
               strstr_case(id3,"one_letter") && strstr_case(id4, "chain_id"))
                test_sequence_correction(str3);
         
            
                                     
            if(strlen(str1)>0){
                
                nn1++;
                
                fprintf(fout,"\n;%s\n; ",str1);
                (strlen(str2)>0) ? fprintf(fout,"\n;%s\n; ",str2):fprintf(fout,"\n;?\n;");
                (strlen(str3)>0) ? fprintf(fout,"\n;%s\n; ",str3):fprintf(fout,"\n;?\n;");
                (strlen(str4)>0) ? fprintf(fout,"\n;%s\n; ",str4):fprintf(fout,"\n;?\n;");
                
                n1=0; n2=0; n3=0; n4=0;
                delete [] str1;delete [] str2;delete [] str3;delete [] str4;
                 
//                str1=NULL; str2=NULL; str3=NULL; str4=NULL;
            }else{
//                str1=NULL; str2=NULL; str3=NULL; str4=NULL;
                delete [] str1;delete [] str2;delete [] str3;delete [] str4;
                n1=0; n2=0; n3=0; n4=0;  
            }
            fprintf(fout,"\n");
            
            
            
        }else if (nitem==5 &&  n1>0 && n2>0 && n3>0 && n4>0 && n5>0 ){
            if(strlen(str1)>0){
                nn1++;
                fprintf(fout,"\n;%s\n; ", str1);
                (strlen(str2)>0) ? fprintf(fout,"\n;%s\n; ",str2):fprintf(fout,"\n;?\n;");
                (strlen(str3)>0) ? fprintf(fout,"\n;%s\n; ",str3):fprintf(fout,"\n;?\n;");
                (strlen(str4)>0) ? fprintf(fout,"\n;%s\n; ",str4):fprintf(fout,"\n;?\n;");
                (strlen(str5)>0) ? fprintf(fout,"\n;%s\n; ",str5):fprintf(fout,"\n;?\n;");
                n1=0; n2=0; n3=0; n4=0;n5=0;
                str1=NULL; str2=NULL; str3=NULL; str4=NULL; str5=NULL;
            }else{
                n1=0; n2=0; n3=0; n4=0;n5=0;
                str1=NULL; str2=NULL; str3=NULL; str4=NULL; str5=NULL;
            }
            fprintf(fout,"\n");
            
        }else if (nitem==6 && n1>0 && n2>0 && n3>0 && n4>0 && n5>0 && n6>0){
            if(strlen(str1)>0){
                nn1++;
                fprintf(fout,"\n;%s\n; ", str1);
                (strlen(str2)>0) ? fprintf(fout,"\n;%s\n; ",str2):fprintf(fout,"\n;?\n;");
                (strlen(str3)>0) ? fprintf(fout,"\n;%s\n; ",str3):fprintf(fout,"\n;?\n;");
                (strlen(str4)>0) ? fprintf(fout,"\n;%s\n; ",str4):fprintf(fout,"\n;?\n;");
                (strlen(str5)>0) ? fprintf(fout,"\n;%s\n; ",str5):fprintf(fout,"\n;?\n;");
                (strlen(str6)>0) ? fprintf(fout,"\n;%s\n; ",str6):fprintf(fout,"\n;?\n;");
                n1=0; n2=0; n3=0; n4=0;n5=0;n6=0;
                str1=NULL; str2=NULL; str3=NULL; str4=NULL; str5=NULL;str6=NULL;
            }else{
                n1=0; n2=0; n3=0; n4=0;n5=0;n6=0;
                str1=NULL; str2=NULL; str3=NULL; str4=NULL; str5=NULL;str6=NULL;
            }
            fprintf(fout,"\n");
            
        }else if (nitem==7 && n1>0 && n2>0 && n3>0 && n4>0 && n5>0 && n6>0 && n7>0){
            if(strlen(str1)>0){
                nn1++;
                fprintf(fout,"\n;%s\n; ", str1);
                (strlen(str2)>0) ? fprintf(fout,"\n;%s\n; ",str2):fprintf(fout,"\n;?\n;");
                (strlen(str3)>0) ? fprintf(fout,"\n;%s\n; ",str3):fprintf(fout,"\n;?\n;");
                (strlen(str4)>0) ? fprintf(fout,"\n;%s\n; ",str4):fprintf(fout,"\n;?\n;");
                (strlen(str5)>0) ? fprintf(fout,"\n;%s\n; ",str5):fprintf(fout,"\n;?\n;");
                (strlen(str6)>0) ? fprintf(fout,"\n;%s\n; ",str6):fprintf(fout,"\n;?\n;");
                (strlen(str7)>0) ? fprintf(fout,"\n;%s\n; ",str7):fprintf(fout,"\n;?\n;");
                n1=0; n2=0; n3=0; n4=0;n5=0;n6=0;n7=0;
                str1=NULL; str2=NULL; str3=NULL; str4=NULL; str5=NULL;str6=NULL;str7=NULL;
            }else{
                n1=0; n2=0; n3=0; n4=0;n5=0;n6=0;n7=0;
                str1=NULL; str2=NULL; str3=NULL; str4=NULL; str5=NULL;str6=NULL;str7=NULL;
            }
            fprintf(fout,"\n");
            
        }else if (nitem==8 && n1>0 && n2>0 && n3>0 && n4>0 && n5>0 && n6>0 && n7>0 && n8>0){
            if(strlen(str1)>0){
                nn1++;
                fprintf(fout,"\n;%s\n; ", str1);
                (strlen(str2)>0) ? fprintf(fout,"\n;%s\n; ",str2):fprintf(fout,"\n;?\n;");
                (strlen(str3)>0) ? fprintf(fout,"\n;%s\n; ",str3):fprintf(fout,"\n;?\n;");
                (strlen(str4)>0) ? fprintf(fout,"\n;%s\n; ",str4):fprintf(fout,"\n;?\n;");
                (strlen(str5)>0) ? fprintf(fout,"\n;%s\n; ",str5):fprintf(fout,"\n;?\n;");
                (strlen(str6)>0) ? fprintf(fout,"\n;%s\n; ",str6):fprintf(fout,"\n;?\n;");
                (strlen(str7)>0) ? fprintf(fout,"\n;%s\n; ",str7):fprintf(fout,"\n;?\n;");
                (strlen(str8)>0) ? fprintf(fout,"\n;%s\n; ",str8):fprintf(fout,"\n;?\n;");
                n1=0; n2=0; n3=0; n4=0;n5=0;n6=0;n7=0;n8=0;
                str1=NULL; str2=NULL; str3=NULL; str4=NULL; str5=NULL;str6=NULL;str7=NULL;str8=NULL;
            }else{
                n1=0; n2=0; n3=0; n4=0;n5=0;n6=0;n7=0;n8=0;
                str1=NULL; str2=NULL; str3=NULL; str4=NULL; str5=NULL;str6=NULL;str7=NULL;str8=NULL;
            }
            fprintf(fout,"\n");
            
        }else if (nitem==9 && n1>0 && n2>0 && n3>0 && n4>0 && n5>0 && n6>0 && n7>0 && n8>0 && n9>0){
            if(strlen(str1)>0){
                nn1++;
                fprintf(fout,"\n;%s\n; ", str1);
                (strlen(str2)>0) ? fprintf(fout,"\n;%s\n; ",str2):fprintf(fout,"\n;?\n;");
                (strlen(str3)>0) ? fprintf(fout,"\n;%s\n; ",str3):fprintf(fout,"\n;?\n;");
                (strlen(str4)>0) ? fprintf(fout,"\n;%s\n; ",str4):fprintf(fout,"\n;?\n;");
                (strlen(str5)>0) ? fprintf(fout,"\n;%s\n; ",str5):fprintf(fout,"\n;?\n;");
                (strlen(str6)>0) ? fprintf(fout,"\n;%s\n; ",str6):fprintf(fout,"\n;?\n;");
                (strlen(str7)>0) ? fprintf(fout,"\n;%s\n; ",str7):fprintf(fout,"\n;?\n;");
                (strlen(str8)>0) ? fprintf(fout,"\n;%s\n; ",str8):fprintf(fout,"\n;?\n;");
                (strlen(str9)>0) ? fprintf(fout,"\n;%s\n; ",str9):fprintf(fout,"\n;?\n;");
                n1=0; n2=0; n3=0; n4=0;n5=0;n6=0;n7=0;n8=0;n9=0;
                str1=NULL; str2=NULL; str3=NULL; str4=NULL;
                str5=NULL;str6=NULL;str7=NULL;str8=NULL;str9=NULL;
            }else{
                n1=0; n2=0; n3=0; n4=0;n5=0;n6=0;n7=0;n8=0;n9=0;
                str1=NULL; str2=NULL; str3=NULL; str4=NULL; str5=NULL;
                str6=NULL;str7=NULL;str8=NULL;str9=NULL;
            }
            fprintf(fout,"\n"); 
        }
        
    }
    
    if(nitem== 1 && nn1==0 ){
        fprintf(fout,"? \n"); 
    }else if(nitem==2  && nn1==0 ){
        fprintf(fout,"? ? \n"); 
    }else if(nitem==3  && nn1==0 ){
        fprintf(fout,"? ? ? \n"); 
    }else if(nitem==4  && nn1==0 ){
        fprintf(fout,"? ? ? ? \n"); 
    }else if(nitem==5  && nn1==0 ){
        fprintf(fout,"? ? ? ? ?\n"); 
    }else if(nitem==6  && nn1==0 ){
        fprintf(fout,"? ? ? ? ? ? \n"); 
    }else if(nitem==7  && nn1==0 ){
        fprintf(fout,"? ? ? ? ? ? ?\n"); 
    }else if(nitem==8  && nn1==0 ){
        fprintf(fout,"? ? ? ? ? ? ? ?\n"); 
    }else if(nitem==9  && nn1==0 ){
        fprintf(fout,"? ? ? ? ? ? ? ? ?\n"); 
    }    
}

void values_in_bracket(FILE *inp, char *first_item, char *value, int *key)
/* first_item: the identifier.  
   value: the total characters in " " <>.
   key: =0 not the end, =1 end of file
*/
{
    int i=0,j=0,n=1, c, nc=0;
    char lett[10000];

    strcpy(lett,"");
    strcpy(first_item,"");
    strcpy(value,"");
    
     
   *key=0;
/* skip every letter until < is met */
    for(j=0; (c=getc(inp)) != '<' ; j++){
        if(c==EOF)  {
            *key=1;
            break;
        }
    }
    
/*take off the space between < and the first letter */    
    for(j=0; (c=getc(inp)) == ' ' ; j++){
        if(c==EOF)  {
            *key=1;
            break;
        }
    }
    
    lett[0]=c; 
    for(j=0; (c=getc(inp)) != '>' ; j++){
        if (c=='<') nc++;
        
        if(c==EOF)  {
            *key=1;
            break;
        }
        if (n>9998){
            sprintf(INFO, "Error: problem with template file.\n");
            pinfo(INFO,0);
            break;
        }
        
        lett[n]=c;
        n=n+1;
    }
    lett[n] = '\0';
    rid_of_front_end_space(lett);
    if (strlen(lett)<10) return;
    
    if (nc>0){
        sprintf(INFO, "Warning: a sign (>) may be missed in %s \n", lett);
        pinfo(INFO,0);
    }
    
    if (!strchr(lett,'=')){
        sprintf(INFO, "Error: missing sign (=) in %s \n", lett);
        pinfo(INFO,0);
        return;        
    }
    
    i=strcspn(lett,"="); // get the first item 
    strncpy(first_item, lett, i);
    first_item[i]='\0';
    rid_of_front_end_space(first_item);
    
    strcpy(value, strchr(lett,'=')+1);


    if (strstr(first_item, "molecule_one_letter_sequence") && strstr(value, "????")){
        sprintf(INFO, "Error: question marks in sequence. replace the ???? by the missing residues!\n");
        pinfo(INFO, 0);
    }

    if (strstr(value, "???")) replace_char(value,'?', ' ');
    if (strchr(value, '"')) replace_char(value,'"', ' ');
    if (strchr(value, ';')) replace_char(value,';', ':');//cif related
    
        
    if (strchr(value, '?') && (strstr(first_item, "_id") ||
                               strstr(first_item, "assembly_chain_number")||
                               strstr(first_item, "crystal_number")||
                               strstr(first_item, "radiation_experiment")))
        {
        sprintf(INFO, "Error: The data field (%s) can not have question mark!\n", first_item);
        pinfo(INFO, 0);
    }
    
    rid_of_front_end_space(value);
    
//    printf("item=%s||val=%s||lett=%s|\n", first_item, value, lett);
    
}

void write_text_parser_cif_head(FILE *fout, char *id)
// Not used at the moment!
{
    
    
//CATEGORY 2:   Molecular Entity Sequence
    if(!strcmp_case(id, "entity_poly")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_entity_poly.entity_id\n");
        fprintf(fout,"_entity_poly.type\n");
        fprintf(fout,"_entity_poly.pdbx_seq_one_letter_code\n");
        fprintf(fout,"_entity_poly.pdbx_strand_id\n");
        fprintf(fout,"_entity_poly.pdbx_target_identifier\n");
        fprintf(fout,"_entity_poly.pdbx_seq_db_id\n");  //new
        fprintf(fout,"_entity_poly.pdbx_seq_db_name\n"); //new
    
//CATEGORY 3, (Contact Authors for PI & others)
    }else if(!strcmp_case(id, "pdbx_contact_author")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_pdbx_contact_author.id\n");
        fprintf(fout,"_pdbx_contact_author.name_salutation\n");
        fprintf(fout,"_pdbx_contact_author.name_first\n");
        fprintf(fout,"_pdbx_contact_author.name_last\n");
        fprintf(fout,"_pdbx_contact_author.name_mi\n");
        fprintf(fout,"_pdbx_contact_author.role\n");
        fprintf(fout,"_pdbx_contact_author.organization_type\n");
        fprintf(fout,"_pdbx_contact_author.email\n");
        fprintf(fout,"_pdbx_contact_author.address_1\n");
        fprintf(fout,"_pdbx_contact_author.city\n");
        fprintf(fout,"_pdbx_contact_author.state_province\n");
        fprintf(fout,"_pdbx_contact_author.postal_code\n");
        fprintf(fout,"_pdbx_contact_author.country\n");
        fprintf(fout,"_pdbx_contact_author.fax\n");
        fprintf(fout,"_pdbx_contact_author.phone\n");
        
//CATEGORY  (Structure Genomics)
    }else if(!strcmp_case(id, "pdbx_SG_project")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout," _pdbx_SG_project.id\n");
        fprintf(fout," _pdbx_SG_project.project_name\n");
        fprintf(fout," _pdbx_SG_project.full_name_of_center\n");
        
//CATEGORY  (structure Author)        
    }else if(!strcmp_case(id, "audit_author")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_audit_author.pdbx_ordinal\n");
        fprintf(fout,"_audit_author.name\n");
        
//CATEGORY 8:   Molecule Names
    }else if(!strcmp_case(id, "entity")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_entity.id\n");
        fprintf(fout,"_entity.pdbx_description\n");
        
//CATEGORY (Citation Authors)
    }else if(!strcmp_case(id, "citation_author")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_citation_author.citation_id\n");
        fprintf(fout,"_citation_author.name\n");
        
//CATEGORY 9, (Citation Article)
    }else if(!strcmp_case(id, "citation")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_citation.id \n");
        fprintf(fout,"_citation.journal_abbrev \n");
        fprintf(fout,"_citation.title \n");
        fprintf(fout,"_citation.year \n");
        fprintf(fout,"_citation.journal_volume \n");
        fprintf(fout,"_citation.page_first \n");
        fprintf(fout,"_citation.page_last\n");
        
//CATEGORY 9:   Molecule Details
    }else if(!strcmp_case(id, "entity_keywords")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_entity_keywords.entity_id\n");
        fprintf(fout,"_entity_keywords.pdbx_fragment\n");
        fprintf(fout,"_entity_keywords.pdbx_mutation\n");
        fprintf(fout,"_entity_keywords.pdbx_ec\n");

        
//CATEGORY 10 (Genetically Manipulated Source)    
    }else if(!strcmp_case(id, "entity_src_gen")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_entity_src_gen.entity_id\n");
        fprintf(fout,"_entity_src_gen.pdbx_gene_src_scientific_name\n");
        fprintf(fout,"_entity_src_gen.pdbx_gene_src_gene\n");
        fprintf(fout,"_entity_src_gen.gene_src_strain\n");
        fprintf(fout,"_entity_src_gen.pdbx_host_org_scientific_name\n");
        fprintf(fout,"_entity_src_gen.pdbx_host_org_strain\n");
        fprintf(fout,"_entity_src_gen.pdbx_host_org_vector_type\n");
        fprintf(fout,"_entity_src_gen.plasmid_name\n");
        fprintf(fout,"_entity_src_gen.gene_src_details\n");
        
 //CATEGORY 11 (Category Natural Source)
    }else if(!strcmp_case(id, "entity_src_nat")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_entity_src_nat.entity_id \n");
        fprintf(fout,"_entity_src_nat.pdbx_organism_scientific\n");
        fprintf(fout,"_entity_src_nat.strain\n");
        fprintf(fout,"_entity_src_nat.details\n");
        
 //CATEGORY  (Category Synthetic Source)
    }else if(!strcmp_case(id, "pdbx_entity_src_syn")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_pdbx_entity_src_syn.entity_id \n");
        fprintf(fout,"_pdbx_entity_src_syn.details\n");
        



// X-ray part ================================
 //CATEGORY   (Biological Assembly)
//    }else if(!strcmp_case(id, "struct_biol")){
//        fprintf(fout,"#\nloop_\n");
//        fprintf(fout,"_struct_biol.id\n");
//        fprintf(fout,"_struct_biol.details\n");

//        fprintf(fout,"_pdbx_struct_assembly_depositor_info.id\n");
//        fprintf(fout,"_pdbx_struct_assembly_depositor_info.oligomeric_count\n");

        
//CATEGORY  (Crystal Property)
    }else if(!strcmp_case(id, "exptl_crystal")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_exptl_crystal.id \n");
        fprintf(fout,"_exptl_crystal.density_percent_sol\n");
        fprintf(fout,"_exptl_crystal.density_Matthews\n");
        fprintf(fout,"_exptl_crystal.pdbx_mosaicity\n");
        
//CATEGORY 15 (Methods and Conditions)
    }else if(!strcmp_case(id, "exptl_crystal_grow")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_exptl_crystal_grow.crystal_id \n");
        fprintf(fout,"_exptl_crystal_grow.method\n");
        fprintf(fout,"_exptl_crystal_grow.pH \n");
        fprintf(fout,"_exptl_crystal_grow.temp\n");
        fprintf(fout,"_exptl_crystal_grow.pdbx_details\n");
        
//CATEGORY  (Radiation Source)
    }else if(!strcmp_case(id, "diffrn_source")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_diffrn_source.diffrn_id\n");
        fprintf(fout,"_diffrn_source.source\n");
        fprintf(fout,"_diffrn_source.type\n");
        fprintf(fout,"_diffrn_source.pdbx_wavelength_list\n");
        
//CATEGORY  detectors
    }else if(!strcmp_case(id, "diffrn_detector")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_diffrn_detector.diffrn_id \n");
        fprintf(fout,"_diffrn_detector.detector \n");
        fprintf(fout,"_diffrn_detector.type \n");
        fprintf(fout,"_diffrn_detector.pdbx_collection_date \n");
        fprintf(fout,"_diffrn_detector.details \n");
        
        
//CATEGORY   radiation protocol
    }else if(!strcmp_case(id, "diffrn_radiation")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_diffrn_radiation.diffrn_id \n");
        fprintf(fout,"_diffrn_radiation.pdbx_diffrn_protocol \n");
        fprintf(fout,"_diffrn_radiation.monochromator \n");
        
//CATEGORY   wavelength
    }else if(!strcmp_case(id, "diffrn_radiation_wavelength")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_diffrn_radiation_wavelength.id \n");
        fprintf(fout,"_diffrn_radiation_wavelength.wavelength \n");
        
//CATEGORY   (Collection Temperature)
    }else if(!strcmp_case(id, "diffrn")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_diffrn.id\n");
        fprintf(fout,"_diffrn.ambient_temp\n");
        



// NMR================================
//CATEGORY 14:   NMR Sample Details

    }else if(!strcmp_case(id, "pdbx_nmr_sample_details")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_pdbx_nmr_sample_details.solution_id\n");
        fprintf(fout,"_pdbx_nmr_sample_details.contents\n");
        fprintf(fout,"_pdbx_nmr_sample_details.solvent_system\n");
        
//CATEGORY 15:  NMR  Sample Conditions
    }else if(!strcmp_case(id, "pdbx_nmr_exptl_sample_conditions")){
            fprintf(fout,"#\nloop_\n");
         fprintf(fout,"_pdbx_nmr_exptl_sample_conditions.conditions_id\n");
         fprintf(fout,"_pdbx_nmr_exptl_sample_conditions.temperature\n");
         fprintf(fout,"_pdbx_nmr_exptl_sample_conditions.pressure\n");
         fprintf(fout,"_pdbx_nmr_exptl_sample_conditions.pH\n");
         fprintf(fout,"_pdbx_nmr_exptl_sample_conditions.ionic_strength\n");
        
// CATEGORY 16: NMR  Spectrometer
    }else if(!strcmp_case(id, "pdbx_nmr_spectrometer")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_pdbx_nmr_spectrometer.spectrometer_id\n");
        fprintf(fout,"_pdbx_nmr_spectrometer.manufacturer\n");
        fprintf(fout,"_pdbx_nmr_spectrometer.model\n");
        fprintf(fout,"_pdbx_nmr_spectrometer.field_strength\n");
        
// CATEGORY 17:  NMR   Experiment Type
    }else if(!strcmp_case(id, "pdbx_nmr_exptl")){
        fprintf(fout,"#\nloop_\n");
        fprintf(fout,"_pdbx_nmr_exptl.experiment_id\n");
        fprintf(fout,"_pdbx_nmr_exptl.solution_id\n");
        fprintf(fout,"_pdbx_nmr_exptl.conditions_id\n");
        fprintf(fout,"_pdbx_nmr_exptl.type\n");
        
    }else if(!strcmp_case(id, "")){
        
        
    }
}


