#include "TableFile.h"
#include "CifFile.h"
#include "CifParserBase.h"


# define VERSION "1.305 : 2018-09-12 "


struct SF
{
    // Constructor for struct - initialize
    SF();
    // Destructor (clears memory use)
    ~SF();
    // Empties (frees) current memory use
    void empty();
    char *a, *b, *c, *alpha, *beta, *gamma ,*sg , *nsg;
    char **wavelength, **symm, *resh, *resl, *nobs;
    char *hmax,*kmax,*lmax,*hmin,*kmin,*lmin, *nall;
    char **H, **K, **L, **Fo, **sFo, **Fo_au, **sFo_au, **Fc, **Fc_au;
    char **Io, **sIo, **F2o, **sF2o, **Ic, **F2c;
    char **fom, **phase_c, **phase_o, **status, **flag, **free1,**free2 ;
    char **wave_id, **cryst_id, **scale_id, **diffr_id;
    char **I_plus, **sI_plus, **I_minus, **sI_minus;
    char **F_plus, **sF_plus, **F_minus, **sF_minus;
    char **hla, **hlb,**hlc, **hld;
    char **hla1, **hlb1,**hlc1, **hld1, **anom, **anoms;
    char **aud_id, **aud_date, **aud_rec, **aud_meth;
    char **dif_id,  **dif_cid, **dif_detail, **temp, **treat;
    char **d_spacing,**a_calc,**b_calc,**weighted,**fs_uncorrected;
    char **f_uncorrected,**gsas_i100_meas,**i_unknown, **is_unknown;
    char **phase_part, **F_part_au, **phase_with, **fcalc_with;
    char **Fc_all, **Fc_all_ph, **fwt, **phwt, **delfwt, **delphwt;
    char **fib_co, **fib_fo, **fib_ly, **wt ;
    char **pow_sc, **pow_io, **pow_iop,**pow_wt, **pow_ic, **pow_ict;
    char **dH, **dK, **dL, **unmerge_i, **unmerge_si;
    
    long nref, dnref,  nwave, nsymm, naud, ndif, npow;

private:
  void sf_delete_attribute_value(char *&value);
  void sf_delete_attribute_values(char **&values);
};

// sf_convert.C

void check_argv(char *arg1, int i, int argc);
void get_sf_info(char *diagfile);

void check_input_error(int argc, char **argv);
void add_items_to_cif(char *inpfile, char *outfile, char *correct);
void validate_sf_format(char *inpfile);
void test_ciftable(char *file);
void update_audit(char *file);
void check_unwanted_cif_item(char *file);
void pinfo(char *info, int id);

void correct_list_file(char *file1, char *file2);
void cif_dic_check(char *ciffile);
void rename_cif_token(int nline, char **line, char *infile);


void correct_list(char *file1);
void reformat_sfhead(char *file1, char *outfile);
void check_inpfile_name(char *file, char *pdbid);
void precheck_sf(char *inpfile, char *outfile);
void cif2pdb(char *inpfile, char *outfile);

void test_phenix_free_flag(char *xyzfile);

// for cif2cif_sf.C

void write_sf_info(FILE *fout, char *info, int id);
void compare_refln(char *infile, char *item);


void write_hkl_values(FILE *fout,char *fname, CifFile *fobjR,
                      int i,int nd, char *pdb_id, int key);
void correct_status(long n, char **status, int id);
void check_sf_infor(char *file, char **H, char **K, char **L, char **Fo, char **sFo,
                   char **F2o,char **sF2o ,char **Io,char **sIo,char **I_plus,char **F_plus,
                   char **status, float *cell, int i, int *index, float *resol);

float get_resolution(float h, float k, float l, float *rcell);
void calc_recip_cell(float *cell, float *rcell);
void write_plus_minus(FILE *fout, long n, char **H, char **K, char **L, char **status,
                      char **I_plus,char **Is_plus, char **I_minus,char **Is_minus);

void get_num_file(char **cryst_id, char **wave_id,char **scale_id, int n, int *index, int *nf);
void num_file(char **cryst_id, char **wave_id, char **scale_id, int n, int *index, int *nf) ;
void write_sf_file(FILE *fout,  char *block_name, int key, int i, int *index, int id);
void write_sf_unmerge(FILE *fout,  char *block_name, char *pdbid, int i, int *index, int id);
void sf_related_head(FILE *fout, char *block_name, char *pdbid, int nb);
void get_refln_list(int start, int nline, char **line, char **refln_list, int *nlist);
int check_refln_list(char *token, int nlist, char **refln_list);

void clean_up_sf(char *iFile, char *ofile);



void make_sf_standard(char *sffile, char *pdbfile ,char *ofile ,
                      char *correct, char *date,char *date_new, char *pdbid);

void update_items(char *file, char *newfile, char *pdbid, int nblock);
void parse_and_update(FILE *fout, char *infile,  char *date,  char *correct,
                      char *comment, int nblock);
void guess_comment(char *line, char *comment);
void write_audit(FILE *fout, char *date, char *correct,long naud,char **aud_id, 
                 char **aud_rec,char **aud_date, char **aud_meth, int id);

void write_diffrn(FILE *fout, long ndif, char **dif_id,char **dif_cid, 
                  char **dif_detail,char **temp, char **treat, char *comment);



//





void get_items_pdb(char *pdbfile, char *pdb_id_inp); 
void get_cell_man(char **argv, int argc, int in, int *out);
void guess_sf_format(char *inpfile, char *format);
void write_format(FILE *fout, const char *format,int key);
void make_complete_SF(char *pdbfile, char *outfile);
int check_mtz_label(char *file);




void parse_sf_values(CifFile *fobjR, int i) ;

void usage(int n);
void cif_2_various(char *inpfile,char *pdbfile,char *outfile,char *format_out);
void various_2_cif(char *outfile, char *inpfile,char *format_inp, char *pdbid,
                   char *type);
void cif2cif_sf_all(char *inpfile, char *inpfile_news, char *pdbid, int key);

void parse_values(char *iFile, long *nref);
void get_sigma(long i, double *sigI, double *sigF);
void to_cns(FILE *fout,  long j, long *index);
void to_epmr(FILE *fout,  long j, long *index);
void to_tnt(FILE *fout,  long j, long *index);
void to_shelx(FILE *fout,  long j, long *index);
void to_hkl(FILE *fout,  long j, long *index);
void to_mtz(char *pdbfile, char *outfile, char *inpfile);
void to_dtrek(FILE *fout,  long j, long *index);
void to_various(FILE *fout, char *out_format, long j, long *index);
void to_xscale(FILE *fout,  long j, long *index);
void to_other(FILE *fout,  long j, long *index);
void conversion(char *iFile, char *out_file, char *out_format, long n);
void check_status(char **status, long n9, long *id);
void new_sf(FILE *fout, long n, char **H, char **K, char **L, char **Io, char **sIo, 
            char **Fo_au, char **sFo_au,char **status);
void cad_mtz(char *inpfile, char *outfile);
void get_F_I(long j, int *h, int *k, int *l,
            float *ff, float *sff, float *ii, float *sii) ;

void get_items(char **label_ptr, char **type_ptr,char **sets_ptr,
                int ntype, int nset, int id, char *items);
void add_labels(FILE *finp, char **label_ptr, char **type_ptr,
                char **sets_ptr,int ntype, int d_set, int id);
int check_file (int size, char *file);


void get_cns_html(char *inpfile, char *path, char *pdb_id_inp);
void get_cns_man(char *inpfile, char *outfile, char *cns_items,  char *pdb_id);
void write_cns_cif_items(FILE *fout,int cns, char *str, char **token, int ntoken, int key);
void mtz2mmcif_mult(char*, char*);

void mtz2mmcif_run(char *ifile, char *ofile, char *items, int id);

void write_data_type_head(FILE *fout, const char *I_F_type);
void sf_data_type(FILE *fout);

void cns2mmcif(FILE *fout,  char *inpfile);
void print_cns(FILE *fout, char *line);

void shelx2mmcif(FILE *fout,  char *inpfile, char *type);
void tnt2mmcif(FILE *fout,  char *inpfile);
void mtz2mmcif( char *inpfile, char *outfile);
void print_labin(FILE *finp,char *sffile);
void hkl2mmcif(FILE *fout,  char *inpfile);
void epmr2mmcif(FILE *fout,  char *inpfile, char *type);
void dtrek2mmcif(FILE *fout,  char *inpfile);
void other2mmcif(FILE *fout,  char *inpfile);
void cryst_infor(FILE *fout, float *a, char *sym,int num);
void cif2mmcif(FILE *fout, char *infile);
void write_new_mmcif(FILE *fout, int n, char **item, int *index, char **line,
                     int nitem_tot);

void get_cif_item_index(int n, char **item, int index[], int *nitem_tot);  

void check_flag_num(float *flag, long nref, const char *id, char *flag_cif);
int get_flag(long i);



void get_mtz_infor(char *inpfile,char *path,char *id);
void delete_tab(char *line);
void get_item_index(int n, char **item, int index[])  ;
void validate_sf_format(char *inpfile);
void cifparse(char *iFile, char  *oFile, int width);
void reformat_mtz_output(char *inpfile, char *pdb_id);
void mtz_output(char *inpfile);
void write_loop(FILE *ftmp, FILE *fout);


void get_mmcif_item_index(int n, char **item, int index[], int *nitem_tot);  

void sf_data_header(FILE *fout, int I_F_type, int status );
void audit_sf_header(FILE *fout, int);
void audit_sf_header(FILE *fout, int);
void add_audit(char *outfile);


////various2cif

void xgen2mmcif(FILE *fout,  char *inpfile, char *type);
void multan2mmcif(FILE *fout,  char *inpfile, char *type);

void run_mtz2cif(char *ifile, char *ofile, char *items,int id, int free);

int is_small_mol_cif(char *inpfile);
int check_mmcif_status(char *outfile);
void fi2mmcif(FILE *fout,  char *inpfile, char *type);
void mtz2mmcif_multi(char *inpfile, char *outfile);
void mtz2mmcif(char *filename, char *outfile, char **label_ptr,
               char **type_ptr,char **sets_ptr,int ntype, int set_num);

void xscale2mmcif(FILE *fout, char *filename);
void main2mmcif(FILE *fout,  char *inpfile);
void xtalview2mmcif(FILE *fout,  char *inpfile);
void saint2mmcif(FILE *fout,  char *inpfile);
void formated_4_8(char *str, int *h, int *k, int *l, float *a,float *sa);

int first_set_num(char **label_ptr,char **type_ptr,char **sets_ptr,int ntype);

void print_labin_1(FILE *finp, char **label_ptr,char **type_ptr,
                   char **sets_ptr,int ntype);

void print_labin_2(FILE *finp, char **label_ptr,char **type_ptr,
                   char **sets_ptr,int ntype);

void f_sig_pair(char **label_ptr, char **type_ptr,char **sets_ptr, int ntype,
                const char type1, const char type2, const char lab1, int nset,
                const  char *asign_lab1, const char *asign_lab2, FILE *finp);

void mtz2mmcif_mult( char *filename, char *outfile);
void mtz2mmcif_phs( char *filename, char *outfile, char **label_ptr,
               char **type_ptr,char **sets_ptr,int ntype, int set_num);

void get_type_label_sets(char *inpfile, char *type, char *label, char *sets);

int is_mtz(char *inpfile);
void cns_multi_sets(char *inpfile, char *outfile, char *cns_items, char *pdb_id);
void mtz_man_sets(char *inpfile, char *outfile, char *cns_items, char *pdb_id);
void single_cif_item(FILE *fout, char *str, int key, const char *cif_token, char *item, int fmt);
void get_mtz_man(char *inpfile, char *outfile, char *cns_items, char *pdb_id);


////cif2various

void to_multan(FILE *fout,  long j, long *index);
void to_xtalview(FILE *fout,  long j, long *index);
void to_main(FILE *fout,  long j, long *index);

////get_cns_html.C

void get_cns_data_items(char *inpfile, int *nitem, char **items);
void get_cns_labels(FILE *fout, char **items,  int nitem,  const char *fname);
void replace_char(char *str, char find);
void cns_label(const char *name, int i, char *label);
void write_cns_data_4_html(FILE *fout, char **items, int nitem, int i);




////get_mtz_infor.C
void get_labels(FILE *fout, int k, char **set_ptr, char **label_ptr,
                char **type_ptr, int ntype,  const char *fname);

void write_data_column(FILE *fout, char *align, const char *item_name,
                       const char *fname, int k, char **items,
                       int  nitem,char **type,  char **sets);
void write_mtz_data_4_html(FILE *fout,  char **sets, char **items, int nitem,char **type, int i);

void html_rcsb_header(FILE *fw, int id);
void check_data_block_name(char *file, char *block_id,char *pdbid, int num);
void multiple_file_conversion(char *inpfile_all, char *outfile);


void check_sf_status(char **status, int j, char *file);
void i_to_f(int i, char *I , char **sI, char* fp, char *sigfp, float sf_defaut);
void write_sf_4_validation();
void write_unmerged_refl_head(FILE *fout);
void check_sf(int j, float *resol);
void sf_stat(char *inpfile);
void extract_data(char *ext_data, char *infile);



/*
  starting from 1.203,
  The free set from PHENIX can be automatically recognized. (2013-04-30)
  Extract FC_ALL and PHIC_ALL instead of FC/PHIC with the same cif token to make COOT work.

  starting from 1.204
  The other_bin/mtz2cif exports three digits. But it was built separatly. no longer used.
  the one in CCP4 package will be used.

  Add -verb option to print the intemediate files.

  version 1.300:

  add function to convert unmerged reflections to the refln->diffrn_refln
  add two new options.
  ("-detail  (if add '-detail \" ?? \" ', give infor. will overwrite the old message in _diffrn.details ");
  ("-wave_cgi  (if add '-wave_cgi 0.9987 ', input wavelength. will overwrite the old. ");
  
  version 1.301:
  minor improvement. Add/modify error corrrection/detections. (2016-05-01)

  version 1.302:
  * Maps is auto added to final mmcif. (2016-09-21)
  * Added multiple file conversion (-sf file1 file2 ..) (2016-09-26)
  * re-writen the on line help 'sf_convert -h' (2016-09-26)
  * make error/warning messages related to _diffrn_refln (2016-11-16)
  * extend HLA(B,C,D) auto extraction, if pattern HLA exists. (2017-2-7)
  * only check block=1 for reported and calculated resolution match (2017-03-14)
  
*/
