/* a program to convert a structure factor file from mmCIF to various formats */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include "rcsb_math.h"
#include "sf_convert.h"
#include "util.h"

extern char *CCP4, SYMM[20], *PATHTMP;
extern int TEST,  VERB;
extern float CELL[6];
extern SF sf;

void to_cns_f(FILE *fout,  long j, long *index);



void cif_2_various(char *inpfile_in,char *pdbfile,char *outfile,char *format_out)
{
    char  file_tmp[512], pdbid[20];
    long nref;
    int pid=getpid();
    
    if(!strcmp_case(format_out, "mtz")) { //treat mtz separatly
        sprintf(file_tmp, "%s__%d", outfile,pid);
        strcpy(pdbid, "xxxx");

        cif2cif_sf_all(inpfile_in, file_tmp, pdbid, 0); //take off 'X'
        parse_values(file_tmp, &nref);
        to_mtz(pdbfile,file_tmp, outfile);
        
        if (VERB==0) delete_file(file_tmp);
        
    }else{
        parse_values(inpfile_in, &nref);
        conversion(inpfile_in, outfile, format_out, nref);
    }
}
    
void to_mtz(char *pdbfile, char *inpfile, char *outfile)
// conversion to mtz format
{
    int  pid=0;
    char script[256], str[512], cellp[80],symmetry[20];
    char  command[512], outfile_tmp[512];
    
    FILE *fp=NULL, *fpdb=NULL;

    printf("Converting to mtz ..\n");
    
    pid = getpid();
    sprintf(script, "%scif2mtz_%d.scr",PATHTMP, pid);
    if((fp=fopen(script, "w"))==NULL){
        printf("Error: no written permission in directory (%s)!\n", PATHTMP);
        exit(0);
    }
    rid_of_space(pdbfile);
    
    if(!outfile) sprintf(outfile,"%s.mtz",inpfile);
    
    
    if(sf.a && sf.b && sf.c  && sf.gamma && (sf.sg ||sf.nsg) && atof(sf.a)>1.1 ){
        rid_of_space(sf.sg);
        if(sf.sg) strcpy(symmetry, sf.sg);
        if(!sf.sg && sf.nsg) strcpy(symmetry, sf.nsg);
        
        sprintf(cellp, "%s %s %s %s %s %s\n",
                sf.a, sf.b,sf.c,sf.alpha,sf.beta,sf.gamma);
        
    }else if(CELL[0]>1 && CELL[2]>1){
        sprintf(cellp, "%.3f %.3f %.3f %.3f %.3f %.3f\n",
                CELL[0],CELL[1],CELL[2],CELL[3],CELL[4],CELL[5]);
        strcpy(symmetry, SYMM);
        
    }else if(strlen(pdbfile)>0){ 
        if((fpdb=fopen(pdbfile, "r"))==NULL){
            open_file_error(pdbfile, "to_mtz");
        }
    
        while(fgets(str, sizeof str, fpdb)!=NULL){
            if (!strncmp(str,"CRYST1 ", 7)){
                get_string(str,7,54, cellp);
                get_string(str,54,66, symmetry);
                rid_of_space(symmetry);
                break;
            }
        }
        fclose(fpdb);
            
        if(strlen(cellp)<48 || strlen(symmetry)<1){
            printf("Warning: either cell or symmetry is missing from PDBfile (%s)\n", pdbfile);
            exit(0);
        }
    }else{
        printf("\nWarning: There are no CELL parameters in the file.\n");
        printf("You must either provide a PDB/CIF file (containing symmetry and cell),\n");
        printf("or manually type in the CELL parameters (-man a,b,c,alpha,beta,gamma,sym).\n");
        printf("type 'sf_convert -h for details!\n\n");
        exit(0);
    }
    
    sprintf(outfile_tmp, "%s_tmp", outfile);
    
    
    fprintf(fp,"#!/bin/csh -f\n");
    fprintf(fp,"%s/bin/cif2mtz HKLIN %s hklout %s << end > /dev/null \n", CCP4, inpfile,outfile_tmp);
    fprintf(fp,"TITLE testing \n");
    (isdigit(symmetry[0]))?fprintf(fp,"symmetry  %s\n", symmetry):fprintf(fp,"symmetry  '%s'\n", symmetry);
    fprintf(fp,"cell  %s\n", cellp);
    
    fprintf(fp,"PNAME sf_convert\n");
    fprintf(fp,"DNAME data_1\n");
    fprintf(fp,"XNAME cryst_1\n");
    fprintf(fp,"end\n");

// # get correct sort order and asymmetric unit  > /dev/null

    fprintf(fp,"\n\n# use CAD to get correct sort order and asymmetric unit\n\n");
    fprintf(fp,"%s/bin/cad  HKLIN1 %s HKLOUT %s<<end > /dev/null \n", CCP4, outfile_tmp ,outfile);
    fprintf(fp,"TITLE Sorting HKL with cad \n");
    
    fprintf(fp,"monitor BRIEF\n");
    fprintf(fp,"labin file 1 ALL\n");
    fprintf(fp,"sort H K L\n");
    fprintf(fp,"end\n");
    fclose(fp);
    sprintf(command, "chmod +x %s ;  %s", script,script);
    int fake;
    fake = system(command);
    fake++;
    //    printf("comm=%s\n", command);
    if (VERB==0) delete_file(script);
    if (VERB==0) delete_file(outfile_tmp);
}


void conversion(char *iFile, char * out_file, char *out_format, long n)
// convert all the data sets
{
    long i=0,  m1, index[50];
    char outfile[512], **key=NULL, tmp[80];
    int nalloc = 0;
    FILE *fout;
    

    if(!(sf.Io || sf.F2o || sf.Fo || sf.Fo_au || sf.I_plus || sf.F_plus
        || sf.Fc || sf.Fc_au || sf.Ic || sf.F2c ) ){
        printf("Error: Conversion from mmcif to %s format can not be performed.\n", out_format);
        printf("There are no correct mmCIF items in the file(%s).\n", iFile);
        printf("Items should be the same as in the dictionary 'http://mmcif.pdb.org/'\n\n");
        return;
    }
    
    key = new char *[n+1];
    if(sf.wave_id && sf.cryst_id){
        
        strcpy(tmp, sf.wave_id[0]);
        strcat(tmp, sf.cryst_id[0]);
        key[0] = new char[strlen(tmp)+1];
        strcpy(key[0], tmp);
    
        m1=0;
        index[0]=0;
        for(i=1; i<n; i++){
            strcpy(tmp, sf.wave_id[i]);
            strcat(tmp, sf.cryst_id[i]);

            key[i] = new char[strlen(tmp)+1];
            strcpy(key[i], tmp);
        
            if(strcmp(tmp, key[i-1])){
                m1++;
                index[m1] = i;
            }
        }
        m1++;  // the last one
        index[m1] = n;
	nalloc = n;
        
    }else if(sf.wave_id){
        
        key[0] = new char[strlen(sf.wave_id[0])+1];
        strcpy(key[0], sf.wave_id[0]);
    
        m1=0;
        index[0]=0;
        for(i=1; i<n; i++){
            strcpy(tmp, sf.wave_id[i]);
            key[i] = new char[strlen(tmp)+1];
            strcpy(key[i], tmp);
            
            if(strcmp(tmp, key[i-1])){
                m1++;
                index[m1] = i;
            }
        }
        
        m1++;  // the last one
        index[m1] = n;
	nalloc = n;

    }else if(sf.cryst_id){
        
        key[0] = new char[strlen(sf.cryst_id[0])+1];
        strcpy(key[0], sf.cryst_id[0]);
    
        m1=0;
        index[0]=0;
        for(i=1; i<n; i++){
            strcpy(tmp, sf.cryst_id[i]);
            key[i] = new char[strlen(tmp)+1];
            strcpy(key[i], tmp);
            
            if(strcmp(tmp, key[i-1])){
                m1++;
                index[m1] = i;
            }
        }
        
        m1++;  // the last one
        index[m1] = n;
	nalloc = n;
        
    }else{
        m1=1;
        index[0]=0;
        index[m1]=n;
	nalloc = 0;
    }
    
    for(i=0; i<m1; i++){
        if(sf.cryst_id && sf.wave_id){
            printf("SF converting (Crystal ID = %s ;  Wavelength ID = %s):\n",
                   sf.cryst_id[ index[i] ], sf.wave_id[ index[i] ]);           
            sprintf(outfile, "%s_%s.%s",  iFile, key[index[i]], out_format);
            
        }else if(sf.cryst_id){    
            printf("SF converting (Crystal ID = %s):\n", sf.cryst_id[ index[i] ]);             
            sprintf(outfile, "%s_%s.%s",  iFile, key[index[i]], out_format);
        }else if(sf.wave_id){    
            printf("SF converting (Wavelength ID = %s):\n", sf.wave_id[ index[i] ]);
            sprintf(outfile, "%s_%s.%s",  iFile, key[index[i]], out_format);
        }else
            sprintf(outfile, "%s.%s",  iFile, out_format);
        
        if(strlen(out_file)>0){ // author assigned
            strcpy(outfile,  out_file);
        }
        
        if((fout = fopen(outfile, "w"))==NULL){        
            printf("I have no permision to write the file %s (in conversion).\n",outfile);
            exit(0);
        }
        to_various(fout, out_format, i, index);
        fclose(fout);
        
        
        if(strlen(out_file)>0) break;
    }
    if(key) {
      for (i = 0; i < nalloc; i++)
	delete [] key[i];
      
      delete [] key;
    }
}


void to_various(FILE *fout, char *out_format, long j, long *index)
// various cases
{
    if(!strcmp_case(out_format,"CNS")|| !strcmp_case(out_format,"CNX")||
      !strcmp_case(out_format,"XPLOR") ){
    
//printf("tmp=%ld, %ld, %ld,\n",j, index[j], index[j+1]);
        to_cns(fout, j, index);
        
    }else if(!strcmp_case(out_format,"CNS_F")){ //only produe F/sigF/Test
        to_cns_f(fout, j, index); 
        
    }else if(!strcmp_case(out_format,"TNT")){
        to_tnt(fout, j, index);
        
    }else if (!strcmp_case(out_format, "SHELX")|| !strcmp_case(out_format, "XPREP")||
             !strcmp_case(out_format, "SAINT") ){
        to_shelx(fout, j, index);

    }else if(!strcmp_case(out_format,"EPMR") ){
        to_epmr(fout, j, index);
        
    }else if(!strncmp_case(out_format, "HKL",3)|| !strcmp_case(out_format,"scalepack") ||
            !strcmp_case(out_format, "3DSCALE")  ) {
        to_hkl(fout, j, index);
        
        
    }else if(!strcmp_case(out_format,"DTREK")){
        to_dtrek(fout, j, index);
        
    }else if(!strcmp_case(out_format,"XSCALE")){
        to_xscale(fout, j, index);
        
        
    }else if(!strcmp_case(out_format,"MULTAN")){
        to_multan(fout, j, index);
        
    }else if(!strcmp_case(out_format,"XTALVIEW")){
        to_xtalview(fout, j, index);
        
    }else if(!strcmp_case(out_format,"MAIN")){
        to_main(fout, j, index);
        
    }else if(!strcmp_case(out_format,"OTHER")){
        to_other(fout, j, index);
        
    }else{
        printf("Error: The output file format (%s) is wrong!\n",out_format );
        printf("Type 'sf_convert -h' for details.\n" );
        exit(0);
    }
    if(index[j+1]-index[j]<10) {
        printf("Error! Conversion from mmcif to %s failed.\n", out_format);
        return;
    }
    
}

void to_hkl(FILE *fout,  long j, long *index)
 /*3I4,2F8.1 (or 4F8.1 for I+ or I-) */
{
    float tmp=0.0, ff, sff, ii, sii, f1,sf1,f2,sf2;
    int h, k, l;
    long i, max=999999, min=-999999;

    
    fprintf(fout,"1\n-987\n");

    if(sf.a && sf.b && sf.c && sf.alpha && sf.beta && sf.gamma){
        fprintf(fout,"%10.3f%10.3f%10.3f%10.3f%10.3f%10.3f",atof(sf.a), atof(sf.b),
                atof(sf.c), atof(sf.alpha), atof(sf.beta), atof(sf.gamma));
    }else{
        printf("\nWarning! no cell parameters. Please add them manually!\n");
        fprintf(fout,"%10.3f%10.3f%10.3f%10.3f%10.3f%10.3f",tmp,tmp,tmp,tmp,tmp,tmp);
    }
    (sf.sg)? fprintf(fout," %s \n",sf.sg) :  fprintf(fout," ? \n");
    
    for(i=index[j]; i<index[j+1]; i++){
        if(sf.status){ if (sf.status[i][0]=='x') continue;}
        
        get_F_I(i, &h, &k, &l, &ff, &sff, &ii, &sii);
        fprintf(fout, "%4d%4d%4d",h,k, l);
        
        if(sf.I_plus){//I(+) exist
            f1=atof(sf.I_plus[i]);  
            sf1=atof(sf.sI_plus[i]);
            f2=atof(sf.I_minus[i]);
            sf2=atof(sf.sI_minus[i]);
            (f1> max || f1<min)?fprintf(fout,"%8.2E",f1):fprintf(fout,"%8.1f",f1);
            (sf1>max || sf1<min)?fprintf(fout,"%8.2E",sf1):fprintf(fout,"%8.1f",sf1);
            (f2>max || f2<min)?fprintf(fout,"%8.2E",f2):fprintf(fout,"%8.1f",f2);
            (sf2>max || sf2<min)?fprintf(fout,"%8.2E\n",sf2):fprintf(fout,"%8.1f\n",sf2);
        }else{
            (ii> max || ii<min)?fprintf(fout,"%8.2E",ii):fprintf(fout,"%8.1f",ii);
            (sii>max || sii<min)?fprintf(fout,"%8.2E\n",sii):fprintf(fout,"%8.1f\n",sii);
        }  
    }
}


void to_dtrek(FILE *fout,  long j, long *index)
 /*
  h k l I sI batch(optional)
  h k l I sI I(+) sI(+) I(-) sI(-) exist
 */
{
    float  ff, sff, ii, sii, f1,sf1,f2,sf2;
    int h, k, l;
    long i;

    if(sf.I_plus){
        fprintf(fout, "3 6 0 5\n") ;
        fprintf(fout, "CRYSTAL_DESCRIPTION=unknown;\n");
    }else{
        fprintf(fout, "3 2 0 4\n");       
    }
    fprintf(fout, "CRYSTAL_MOSAICITY=?;\n");       
    fprintf(fout, "CRYSTAL_ORIENT_ANGLES=?, ?, ?;\n");       
    fprintf(fout, "CRYSTAL_SPACEGROUP=%s;\n", sf.nsg);       
    fprintf(fout, "CRYSTAL_UNIT_CELL=%s %s %s %s %s %s;\n",
            sf.a, sf.b, sf.c, sf.alpha, sf.beta, sf.gamma);

    fprintf(fout,"nH\nnK\nnL\nfIntensity\nfSigmaI\n");
    if(sf.I_plus)fprintf(fout,"fIntensity+\nfSigmaI+\nfIntensity-\nfSigmaI-\n");
    

    for(i=index[j]; i<index[j+1]; i++){
        if(sf.status){ if (sf.status[i][0]=='x') continue;}
        
        get_F_I(i, &h, &k, &l, &ff, &sff, &ii, &sii);
        if(sf.I_plus){//I(+) exist
            f1=atof(sf.I_plus[i]);  
            sf1=atof(sf.sI_plus[i]);
            f2=atof(sf.I_minus[i]);
            sf2=atof(sf.sI_minus[i]);
            fprintf(fout, "%5d%5d%5d %8.2f %8.2f %8.2f %8.2f %8.2f %8.2f\n",
                    h,k, l, ii, sii, f1,sf1,f2,sf2);
        }else{
            fprintf(fout, "%5d%5d%5d %8.2f %8.2f\n",
                    h, k, l, ii, sii);
        }  
    }
    
}

void to_xscale(FILE *fout,  long j, long *index)
 /*
  h k l I sI 
  h k l  I(+) sI(+) I(-) sI(-) exist
 */
{
    float  ff, sff, ii, sii, f1,sf1,f2,sf2;
    int h, k, l, nc=5;
    long i;

    if(sf.I_plus){
        nc=7;
        fprintf(fout, "!FORMAT=XDS_ASCII    MERGE=TRUE     FRIEDEL'S_LAW=FALSE\n");
    }else{
        fprintf(fout, "!FORMAT=XDS_ASCII    MERGE=FALSE    FRIEDEL'S_LAW=TRUE\n");
        nc=5;
    }
    fprintf(fout, "!CONVERTED BY SF_CONVERT\n");       
    fprintf(fout, "!SPACE_GROUP_NUMBER= %s\n", sf.nsg);       
    fprintf(fout, "!UNIT_CELL_CONSTANTS= %s %s %s %s %s %s;\n",
            sf.a, sf.b, sf.c, sf.alpha, sf.beta, sf.gamma);
    
    fprintf(fout, "!NUMBER_OF_ITEMS_IN_EACH_DATA_RECORD=  %d\n", nc);       

    fprintf(fout,"!ITEM_H=1\n!ITEM_H=2\n!ITEM_H=3\n!ITEM_IOBS=4\n!ITEM_SIGMA(IOBS)=5\n");
    if(sf.I_plus)fprintf(fout,"!ITEM_IOBS'=6\n!ITEM_SIGMA(IOBS')=7\n");
    

    for(i=index[j]; i<index[j+1]; i++){
        if(sf.status){ if (sf.status[i][0]=='x') continue;}
        get_F_I(i, &h, &k, &l, &ff, &sff, &ii, &sii);
        if(sf.I_plus){//I(+) exist
            f1=atof(sf.I_plus[i]);  
            sf1=atof(sf.sI_plus[i]);
            f2=atof(sf.I_minus[i]);
            sf2=atof(sf.sI_minus[i]);
            fprintf(fout, "%6d%6d%6d%11.3E%11.3E%11.3E%11.3E\n",
                    h,k, l, f1,sf1,f2,sf2);
        }else{
            fprintf(fout, "%6d%6d%6d%11.3E%11.3E\n",
                    h, k, l, ii, sii);
        }  
    }
    
}

void to_shelx(FILE *fout,  long j, long *index)
 /* FORMATTED!
   3I4, 2F8.2, I4  [for H,K,L, F,sF flag=1 (o), =-1 (f)] for HKLF 3
    or [for H,K,L, I,sI flag=1 (o), =-1 (f)] for HKLF 4 (prefered)
    !!the defaut output depends on the sf status.
 */
{
    float  ff, sff, ii, sii;
    int h, k, l, flag=1;
    long i, max=999999, min=-999999;

    for(i=index[j]; i<index[j+1]; i++){
        if(sf.status){ if (sf.status[i][0]=='x') continue;}
        get_F_I(i, &h, &k, &l, &ff, &sff, &ii, &sii);
        fprintf(fout, "%4d%4d%4d", h,k, l);
        
        if(TEST>0.001 || sf.status ){
            
            if(TEST>0.001)
                (rand()%TEST == 1) ? flag=-1 : flag=1;
            else {
                if (sf.status[i][0] == 'o') flag=1 ;
                else if(sf.status[i][0] == 'f') flag=-1;
            }
            if(sf.Io){
                (ii> max || ii<min)?fprintf(fout,"%8.2E",ii):fprintf(fout,"%8.1f",ii);
                (sii>max || sii<min)?fprintf(fout,"%8.2E",sii):fprintf(fout,"%8.1f",sii);
            }else{
                (ff> max || ff<min)?fprintf(fout,"%8.2E",ff):fprintf(fout,"%8.1f",ff);
                (sff>max || sff<min)?fprintf(fout,"%8.2E",sff):fprintf(fout,"%8.1f",sff);
            }
            fprintf(fout, "  %2d\n", flag); 
            
        }else{
            if(sf.Io){
                (ii> max || ii<min)?fprintf(fout,"%8.2E",ii):fprintf(fout,"%8.1f",ii);
                (sii>max || sii<min)?fprintf(fout,"%8.2E\n",sii):fprintf(fout,"%8.1f\n",sii);
            }else{
                (ff> max || ff<min)?fprintf(fout,"%8.2E",ff):fprintf(fout,"%8.1f",ff);
                (sff>max || sff<min)?fprintf(fout,"%8.2E\n",sff):fprintf(fout,"%8.1f\n",sff);
            }
        }
        
    }
    if(sf.Io)
        printf("The output for SHELX is the I and SigI.\n");
    else
        printf("The output for SHELX is the F and SigF.\n");
    
}



void to_tnt(FILE *fout,  long j, long *index)
/*
"HKL ", followed by  h, k, l, amplitude,sigma, phase, fom 
 in the format (A4, 3I4, 3F8.1, F8.4).
remark statement must begin with ``REM '' instead of ``HKL '' 
HKL    0   0   6  1076.0     0.0   180.0  1.0000
*/
{
    int h, k, l;
    float ff, sff, ii, sii, phase=1000, fom=0;
    long i, max=999999, min=-999999;
    
    fprintf(fout, "REM  This file is converted from cif format\n");
    if(sf.a && sf.b && sf.c && sf.alpha && sf.beta && sf.gamma){
        fprintf(fout,"REM  Cell parameters = %s %s %s %s %s %s\n",
                sf.a, sf.b, sf.c, sf.alpha, sf.beta, sf.gamma);
    }
    
    if(sf.sg ) {
        if(sf.sg[0]!='?')
            fprintf(fout,"REM  Space group = %s \n",sf.sg);
        else
            if(sf.nsg){
                fprintf(fout,"REM  Space group number= %s \n",sf.nsg);
            }
    }
    
    for(i=index[j]; i<index[j+1]; i++){
        if(sf.status){ if (sf.status[i][0]=='x') continue;}
        get_F_I(i, &h, &k, &l, &ff, &sff, &ii, &sii);
        fprintf(fout, "HKL %4d%4d%4d",h,k,l);

        if(sf.phase_o){
            phase = atof(sf.phase_o[i]);
        }else if(sf.phase_c){
            phase = atof(sf.phase_c[i]);
        }else{
            phase = 1000;
        }
        if(sf.fom){
            fom=atof(sf.fom[i]);
        } else
            fom=0.0;

//        fprintf(fout, "HKL %4d%4d%4d%8.2f%8.2f%8.2f%8.4f\n",h,k,l,ff,sff,phase,fom);

        (ff> max || ff<min)?fprintf(fout,"%8.2E",ff):fprintf(fout,"%8.2f",ff);
        (sff>max || sff<min)?fprintf(fout,"%8.2E",sff):fprintf(fout,"%8.2f",sff);
        
        fprintf(fout, "%8.2f%8.4f\n",phase,fom);
        
    }
    printf("The output for TNT is the F and SigF.\n");
    
}

void to_epmr(FILE *fout,  long j, long *index)
/* only H K L Fo as the first four items on each line,
   separated by spaces.
*/
{
    int h, k, l;
    float  ff, sff, ii, sii;
    long i;

    for(i=index[j]; i<index[j+1]; i++){
        if(sf.status){ if (sf.status[i][0]=='x') continue;}
        get_F_I(i,&h, &k, &l, &ff, &sff, &ii, &sii);
        fprintf(fout, "%5d%5d%5d%11.2f\n",h, k, l, ff);
    }
    printf("The output for EPMR is the F\n");
    
}

void to_other(FILE *fout,  long j, long *index)
 /* free FORMAT! H,K,L, F,sF */
{
    float  ff, sff, ii, sii;
    int h, k, l;
    long i;

    for(i=index[j]; i<index[j+1]; i++){
        if(sf.status){ if (sf.status[i][0]=='x') continue;}
        get_F_I(i, &h, &k, &l, &ff, &sff, &ii, &sii);
        
        if(sf.Fo_au){
            fprintf(fout, "%4d %4d %4d %8.2f %8.2f\n", h,k, l, ff, sff);
        }else
            fprintf(fout, "%4d %4d %4d %8.1f %8.1f\n", h,k, l, ii, sii);
    }
    
    if(sf.Fo_au)
        printf("The output for OTHER is the F and SigF.\n");
    else
        printf("The output for OTHER is the I and SigI.\n");   
}

void to_multan(FILE *fout,  long j, long *index)
 /*The output file has h, k, l, f, imt in FORMAT(3I4,7X,F7.0,I6),
   where imt=0 for a good reflection.
 */
{
    float  ff, sff, ii, sii;
    int h, k, l;
    long i;

    for(i=index[j]; i<index[j+1]; i++){
        if(sf.status){ if (sf.status[i][0]=='x') continue;}
        get_F_I(i, &h, &k, &l, &ff, &sff, &ii, &sii);
        
        fprintf(fout, "%4d%4d%4d       %7.0f     0\n", h,k, l, ff);
    }
}

void to_xtalview(FILE *fout,  long j, long *index)
/* this is a .fin format only !!!
File contains: h k l f1 sigma(f1) f2 sigma(f2). The f1 and f2 can be
Bijvoet pairs, fp and fph, or any other pairable data. If one of the
two observations is missing, then f should equal 0.0
and sigma(f) should equal 9999.0    
*/
{
    float  ff, sff, ii, sii, f1,sf1,f2,sf2,xv=0, sxv=9999.0;
    int h, k, l;
    long i;


    for(i=index[j]; i<index[j+1]; i++){
        if(sf.status){ if (sf.status[i][0]=='x') continue;}
        get_F_I(i, &h, &k, &l, &ff, &sff, &ii, &sii);
        f1=xv; sf1=sxv;
        f2=xv; sf2=sxv;
        
        if(sf.F_plus){//F(+) exist
            f1=atof(sf.F_plus[i]);  
            sf1=atof(sf.sF_plus[i]);
            f2=atof(sf.F_minus[i]);
            sf2=atof(sf.sF_minus[i]);
            
            if(sf.F_plus[i][0]== '?' || sf1<0 || f1<0){
                f1=0; sf1=9999.0;
            }
            if(sf.F_minus[i][0]== '?'|| sf2<0 || f2<0){
                f2=0; sf2=9999.0;
            }
            fprintf(fout, " %3d %3d %3d %8.2f %8.2f %8.2f %8.2f\n",
                    h,k, l, f1,sf1,f2,sf2);
            
        }else if (sf.I_plus){ //I(+) exist 

            f1=atof(sf.I_plus[i]);  
            sf1=atof(sf.sI_plus[i]);
            
            f2=atof(sf.I_minus[i]);
            sf2=atof(sf.sI_minus[i]);
            
            if(f1>0){
                f1=sqrt(f1);
                sf1=sf1/(2*f1);
            }
            if(f2>0){
                f2=sqrt(f2);
                sf2=sf2/(2*f2);
            }
            
            if(sf.I_plus[i][0]== '?' || sf1<0 || f1<0){
                f1=0; sf1=9999.0;
            }
            if(sf.I_minus[i][0]== '?' || sf2<0 || f2<0){
                f2=0; sf2=9999.0;
            }
            
            fprintf(fout, " %3d %3d %3d %8.2f %8.2f %8.2f %8.2f\n",
                    h,k, l, f1,sf1,f2,sf2);
        }else{
            fprintf(fout, " %3d %3d %3d %8.2f %8.2f %8.2f %8.2f\n",
                    h,k, l, ff,sff,xv,sxv);
        }  
    }
}
void to_main(FILE *fout,  long j, long *index)
/*
The default format "main", which assumes keyword based records
based on old X-plor version format.
INDEX    8     1   -20  FOBS  1262.150  SIGMA 344.3 PHA 132.23 WE 0.345 TEST 0
http://www-bmb.ijs.si/
  
*/
{
    int h, k, l, flag,  test;
    long nref,i, key;
    float  ff, sff, ii, sii,  pha,fom;


    nref=index[j+1]-index[j];
    if(sf.status) check_status(sf.status, nref, &key);
    
    (TEST>0.01)? test=TEST : test=int(100/8 + 0.5); 
    
    for(i=index[j]; i<index[j+1]; i++){
        if(sf.status){ if (sf.status[i][0]=='x') continue;}
        pha=0;
        fom=0;
        if(sf.fom) fom=atof(sf.fom[i]);
        if(sf.phase_o) pha=atof(sf.phase_o[i]);
        
        if(TEST>0.001){ //given by -flag
            (rand()%test == 1) ? flag=1 : flag=0;
        }else{ 
            if(sf.status && key==1){
                (sf.status[i][0]=='f' || sf.status[i][0]=='1')? flag=1 : flag=0;
            }else{
                (rand()%test == 1)?  flag=1: flag=0;
            }
        }
        
        get_F_I(i, &h, &k, &l, &ff, &sff, &ii, &sii);
            
        fprintf(fout, "INDEX %4d %4d %4d FOBS %8.2f SIGMA %6.2f PHA %6.1f WE %6.3f TEST %d\n",
                 h,k,l, ff, sff,pha, fom, flag);
    }  
   
}

int get_flag(long i)
{
    int flag=0, ran;
    
    if(sf.status){
        if(sf.status[i][0]=='f' ){
            flag=1;
        }else if( sf.status[i][0]=='o' ){
            flag=0;
        } 
    }else
        flag=0;
        
    if(TEST>0){// overwrite the old flag by author
        flag=0;
        ran= rand();
        
        if(ran%TEST == 1) flag=1;
    }

    return flag;
    
}

    
void to_cns_f(FILE *fout,  long j, long *index)
//only generate F, sigF, test for phenix validation    
{
    long i;
    int  flag, test=TEST, h,k,l;
    float ii=0, sii, ff, sff;
    
        
    fprintf(fout," NREFlection= %8ld\n", (index[j+1] - index[j]));
    fprintf(fout," ANOMalous=FALSe { equiv. to HERMitian=TRUE}\n");
    fprintf(fout," DECLare NAME=FOBS            DOMAin=RECIprocal   TYPE=REAL END\n");
    fprintf(fout," DECLare NAME=SIGMA           DOMAin=RECIprocal   TYPE=REAL END\n");
    fprintf(fout," DECLare NAME=TEST            DOMAin=RECIprocal   TYPE=INTE END\n");  

    
    for(i=index[j]; i<index[j+1]; i++){
        
        if(sf.status){ if (sf.status[i][0]=='x') continue;}
        
        if(TEST>0.001){ //given by -flag
            test=TEST;
            (rand()%test == 1) ? flag=1 : flag=0;
        }else{ 
            if(sf.status){
                (sf.status[i][0]=='f' || sf.status[i][0]=='1')? flag=1 : flag=0;
            }else{
                flag=0;
            }
        }
        
        get_F_I(i, &h, &k, &l, &ff, &sff, &ii, &sii);
            
        fprintf(fout, " INDE  %4d %4d %4d ", h,k,l);
        fprintf(fout, "FOBS=%10.2f SIGMA=%10.2f",ff, sff);
        fprintf(fout, " TEST=%10d \n", flag);
        
        
    }
}

    
void to_cns(FILE *fout,  long j, long *index)
{
    long i;
    int  flag, test=TEST, h,k,l;
    float ii=0, sii, ff, sff, f1,sf1, f2, sf2;
    
        
    fprintf(fout," NREFlection= %8ld\n", (index[j+1] - index[j]));
    fprintf(fout," ANOMalous=FALSe { equiv. to HERMitian=TRUE}\n");
    fprintf(fout," DECLare NAME=FOBS            DOMAin=RECIprocal   TYPE=REAL END\n");
    fprintf(fout," DECLare NAME=SIGMA           DOMAin=RECIprocal   TYPE=REAL END\n");
    fprintf(fout," DECLare NAME=TEST            DOMAin=RECIprocal   TYPE=INTE END\n");  

    if(sf.Io || sf.F2o  ){ //both I & F exist
        fprintf(fout," DECLare NAME=IOBS            DOMAin=RECIprocal   TYPE=REAL END\n");
        fprintf(fout," DECLare NAME=SIGI            DOMAin=RECIprocal   TYPE=REAL END\n");
    }

    if(sf.F_plus && sf.F_minus ) {
        fprintf(fout," DECLare NAME=F+           DOMAin=RECIprocal   TYPE=REAL END\n");
        fprintf(fout," DECLare NAME=SIGF+        DOMAin=RECIprocal   TYPE=REAL END\n");
        fprintf(fout," DECLare NAME=F-           DOMAin=RECIprocal   TYPE=REAL END\n");
        fprintf(fout," DECLare NAME=SIGF-        DOMAin=RECIprocal   TYPE=REAL END\n");
    }else if(sf.I_plus && sf.I_minus ) {
        fprintf(fout," DECLare NAME=I+           DOMAin=RECIprocal   TYPE=REAL END\n");
        fprintf(fout," DECLare NAME=SIGI+        DOMAin=RECIprocal   TYPE=REAL END\n");
        fprintf(fout," DECLare NAME=I-           DOMAin=RECIprocal   TYPE=REAL END\n");
        fprintf(fout," DECLare NAME=SIGI-        DOMAin=RECIprocal   TYPE=REAL END\n");
    }
    if(sf.fom) fprintf(fout,"DECLare NAME=FOM   DOMAin=RECIprocal   TYPE=REAL END\n");   
    if(sf.hla) {
        fprintf(fout," DECLare NAME=HLA   DOMAin=RECIprocal   TYPE=REAL END\n");   
        fprintf(fout," DECLare NAME=HLB   DOMAin=RECIprocal   TYPE=REAL END\n");   
        fprintf(fout," DECLare NAME=HLC   DOMAin=RECIprocal   TYPE=REAL END\n");   
        fprintf(fout," DECLare NAME=HLD   DOMAin=RECIprocal   TYPE=REAL END\n");   
    }
    
//    nref=index[j+1]-index[j];
    
    
    for(i=index[j]; i<index[j+1]; i++){
        if(sf.status){ if (sf.status[i][0]=='x') continue;}
        
        if(TEST>0.001){ //given by -flag
            test=TEST;
            (rand()%test == 1) ? flag=1 : flag=0;
        }else{ 
            if(sf.status){
                (sf.status[i][0]=='f' || sf.status[i][0]=='1')? flag=1 : flag=0;
            }else{
                flag=0;
//                test=int(100/8 + 0.5); 
//                (rand()%test == 1)?  flag=1: flag=0;
            }
        }
        
        get_F_I(i, &h, &k, &l, &ff, &sff, &ii, &sii);
            
        fprintf(fout, " INDE  %4d %4d %4d ", h,k,l);
        fprintf(fout, "FOBS=%10.2f SIGMA=%10.2f",ff, sff);
        fprintf(fout, " TEST=%10d \n", flag);
        
        if(sf.Io || sf.F2o) fprintf(fout,"           IOBS= %12.2f SIGI= %12.2f\n",ii, sii);

        if(sf.F_plus && sf.F_minus) {
            
            f1=atof(sf.F_plus[i]);
            sf1=atof(sf.sF_plus[i]);
            
            f2=atof(sf.F_minus[i]);
            sf2=atof(sf.sF_minus[i]);

            fprintf(fout, "            F+= %10.2f SIGF+= %10.2f\n",f1,sf1);
            fprintf(fout, "            F-= %10.2f SIGF-= %10.2f\n",f2,sf2);
            
        }else if (sf.I_plus && sf.I_minus ) {
            
            f1=atof(sf.I_plus[i]);  
            sf1=atof(sf.sI_plus[i]);
            
            f2=atof(sf.I_minus[i]);
            sf2=atof(sf.sI_minus[i]);
            
            fprintf(fout, "            I+= %10.2f SIGI+= %10.2f\n",f1,sf1);
            fprintf(fout, "            I-= %10.2f SIGI-= %10.2f\n",f2,sf2);
        }
        if(sf.fom) fprintf(fout, "            FOM= %10s \n",sf.fom[i]);
        if(sf.hla) {
            fprintf(fout, "            HLA=%8s  HLB=%8s  HLC=%8s  HLD=%8s\n",
                    sf.hla[i], sf.hlb[i], sf.hlc[i], sf.hld[i]);
        }
        
    }
}

   

void get_F_I(long j, int *h, int *k, int *l,
            float *ff, float *sff, float *ii, float *sii) 
// get H,K,L, F o& I from SF:  (or sf.F_plus or sf.I_plus)
{
    float  f=0, ssf,  i, si, f1,sf1, f2, sf2;

    *h = atoi(sf.H[j]);
    *k = atoi(sf.K[j]);
    *l = atoi(sf.L[j]);

//sigma
    if(sf.sIo)
        si = atof(sf.sIo[j]);
    else if(sf.sF2o)
        si = atof(sf.sF2o[j]);
    else
        si=0.0;
    
    if(sf.sFo_au)
        ssf = atof(sf.sFo_au[j]);
    else if(sf.sFo)
        ssf = atof(sf.sFo[j]);
    else
        ssf = 0.0;  


    
    if( sf.Fo_au ){ //for F
        f = atof(sf.Fo_au[j]);

    }else if( sf.Fo ){
        f = atof(sf.Fo[j]);

    }else if( sf.Fc_au ){
        f = atof(sf.Fc_au[j]);
            
    }else if( sf.Fc ){
        f = atof(sf.Fc[j]);
    }
    if(!sf.Io){
        i=f*f;
        si=2*f*ssf;
    }
    
    
    
    if( sf.Io ){ //for I
        i = atof(sf.Io[j]);
            
    }else if( sf.F2o ){
        i = atof(sf.F2o[j]);
             
    }else if( sf.Ic ){
        i = atof(sf.Ic[j]);
        
    }else if( sf.F2c ){
        i = atof(sf.F2c[j]);
    }
    if(!sf.Fo_au) {
        (i>=0)? f = sqrt(i) : f=0;
        (i>=0)? ssf = si/(2.*f) : ssf=0;
    }
    
    
    if(sf.F_plus && !(sf.Fo_au || sf.Io)){ //F_plus exist
        
        f1=atof(sf.F_plus[j]);
        sf1=atof(sf.sF_plus[j]);
            
        f2=atof(sf.F_minus[j]);
        sf2=atof(sf.sF_minus[j]);

        if(f1>0 && f2<0.0001){
            f=f1; ssf=sf1;
        }else if(f2>0 && f1<0.0001){
            f=f2; ssf=sf2;
        }else{
            f=0.5*(f1+f2); ssf=0.5*(sf1 + sf2);
        }
        if(!sf.I_plus){
            i=f*f;
            si=2.0*f*ssf;
        }
    }
    
    if(sf.I_plus && !(sf.Fo_au || sf.Io)){ //I_plus exist
        
        f1=atof(sf.I_plus[j]);  
        sf1=atof(sf.sI_plus[j]);
            
        f2=atof(sf.I_minus[j]);
        sf2=atof(sf.sI_minus[j]);

        if(f1>0 && f2<0.0001){
            i=f1;
            si=sf1;
            
        }else if(f2>0 && f1<0.0001){
            i=f2;
            si=sf2;

        }else{
            i=0.5*(f1+f2);
            si=0.5*(sf1 + sf2);
            
        }
        if(!sf.F_plus){
            (i>=0)? f=sqrt(i) : f=0;
            (f>0.0001)? ssf=si/(2*f) : ssf=0;
        }  
    }
    
    *ff=f; *sff=ssf;
    *ii=i; *sii=si;
    
}

void get_sigma(long i, double *sigI, double *sigF)
 // bring back the  sigI, sigF. sf.sIo and sf.sFo_au first 
{
    
    if(sf.sIo)
        *sigI = atof(sf.sIo[i]);
    else if(sf.sF2o)
        *sigI = atof(sf.sF2o[i]);
    else
        *sigI=0.0;
    
    if(sf.sFo_au)
        *sigF = atof(sf.sFo_au[i]);
    else if(sf.sFo)
        *sigF = atof(sf.sFo[i]);
    else
        *sigF = 0.0;  
}

void check_status(char **status, long n9, long *id)
// check if there exist the right free R flag
{
    long i, nf=0, no=0;

//    if(TEST>0.001){
        
    *id=0;
    if(!status || n9==0) return;
    
    for(i=0; i<n9; i++){
        if(status[i][0] == 'f')
            nf++;
        else if(status[i][0] == 'o')
            no++;
    }

    if(nf>10 && no >100){
        if(float(nf)/float(no) > 0.03){
            *id = 1;
        }else{
            *id = 0;
        }
    }else{
        *id = 0;
    }
    
}



void parse_values(char *iFile, long *nref)
//parse the SF values using the CIF parser
{
  //    const char *blockId=NULL;
    string diags;

    if(!is_cif(iFile)){
        printf("Error! the file %s is not a correct mmCIF file.\n",iFile);
        exit(0);
    }
    
    CifFile * fobjR = NULL;

    CifParser* cifParserR = NULL;
    fobjR = new CifFile(true);
    cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
    cifParserR->Parse(iFile, diags);
    delete(cifParserR);
    
    if (!diags.empty()) fprintf(stderr, "Diags for file %s: %s\n",iFile,diags.c_str());

    vector<string> blockNames;
    fobjR->GetBlockNames(blockNames);
    //    blockId = fobjR->GetFirstBlockName().c_str();

//    printf("data_block_name=%s\n", blockId);
    
    parse_sf_values(fobjR, 0);
    
    *nref = sf.nref;
    if (fobjR) delete fobjR;    
}

