/*
FILE:     pdb2cif.C
*/
/*
VERSION:  3.25
*/
/*
DATE:     3/29/2019
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2019 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               RCSB PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/
/* program to extract informat from crystallographic softwares */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <fnmatch.h>

#include "structure.h"  
#include "harv.h"
#include "util.h"
#include "rcsb_math.h"

#define NCSMAX 4000
#define TLSMAX 900
#define TWINMAX 40

extern const char *REFINE_ID[];
extern int SPLIT;
extern char INFO[1000];

extern CELL cell;
extern REFINE refine;
extern  REFINE refine_neutron;
//extern REFINE_NEUTRON refine_neutron;
extern PDBX_REFINE pdbx_refine;
extern SYMMETRY symmetry ;
extern REFINE_HIST refine_hist;
extern REFINE_ANALYZE refine_analyze;
extern STRUCT_CONN struct_conn;
extern DATABASE_PDB_MATRIX database_PDB_matrix;
extern ATOM_SITES atom_sites;
extern PDBX_REFINE_TLS pdbx_refine_tls_single;
extern PDBX_REFINE_TLS_GROUP pdbx_refine_tls_group_single;
extern REFLNS reflns;
extern DIFFRN_DETECTOR diffrn_detector;
extern DIFFRN_SOURCE diffrn_source;
extern COMPUTING computing;
extern EXPTL exptl;
extern EXPTL_CRYSTAL exptl_crystal;
extern EXPTL_CRYSTAL_GROW exptl_crystal_grow ;
extern DIFFRN diffrn;
extern DIFFRN_RADIATION diffrn_radiation;

REFLNS reflns_neut;
EXPTL exptl_neut;
DIFFRN_SOURCE diffrn_source_neut;
DIFFRN_DETECTOR diffrn_detector_neut;
DIFFRN diffrn_neut;
DIFFRN_RADIATION diffrn_radiation_neut;
REFLNS_SHELL reflns_shell_neut;

REFINE_ANALYZE refine_analyze_neut;
REFINE_LS_SHELL refine_ls_shell;
REFINE_LS_SHELL refine_shell_neut;
PDBX_REFINE_TLS  pdbx_refine_tls_single;
PDBX_REFINE_TLS_GROUP pdbx_refine_tls_group_single;
PDBX_REFINE_TLS_GROUP pdbx_refine_tls_group[TLSMAX];
PDBX_REFINE_TLS pdbx_refine_tls[TLSMAX];
REFLNS_SHELL reflns_shell;
STRUCT_NCS_DOM struct_ncs_dom[NCSMAX];
STRUCT_NCS_DOM_LIM struct_ncs_dom_lim[NCSMAX];
REFINE_LS_RESTR_NCS refine_ls_restr_ncs[NCSMAX];
REFINE_LS_RESTR refine_ls_restr[100];
PDBX_XPLOR_FILE pdbx_xplor_file[80];
PDBX_TWIN pdbx_twin[TWINMAX];
RESINFO resinfo;

//PHASING_MAD_SET  mad_set[100];

int NR=0, NTLS=0,NTLS_GROUP=0,  NCS=0, NLIM=0, NNCS=0, NTWIN=0;
char PROG[40];

void pdb_compnd_source(FILE *fout, int nc[], int ns[], int mol, char **lines);
void extract_refmac_ncs_new(int ii, int nline, char **lines, int *end);
void get_pdb_30_70(char *strin, int n, char *strout);  
void extract_ncs_matrix(FILE *fout, char **lines, int nline);
void chain_seq_info(int nline, char **lines );
void get_remarks(const char *id, int n[], char **lines);



void pdb_2_cif(FILE *fout, char *infile)
// extract information from PDB file    
{
    char **lines=NULL;
    int  nline=0 , ntmp=0;

    strcpy(PROG,"");

    
    lines = get_pdb_records(infile, &nline);//neutron removed for separate extraction
    
    chain_seq_info( nline, lines); //load resid infor.
    
    if(strstr_case(computing.structure_refinement, "phenix")){
        extract_phenix_pdb_head(lines, nline);
    }else{
        extract_tls(0, nline, lines,  &ntmp);//also get resi info.
        extract_refine_ls_restr(lines, nline, computing.structure_refinement);
    }
    extract_pdb_head(lines, nline); //remark 3
    
    extract_pdb_head_other(fout, lines, nline); //other remarks    
    extract_ncs_matrix(fout, lines, nline); //ncs matrix
     
    extract_pdb_ncs(lines, nline); //ncs statistics
   
    write_ncs_new(fout);
    write_pdbx_xplor_file(fout);
    write_refine_ls_restr(fout);
    
    if(NTLS>0) write_tls(fout);
    if(NTWIN>0) write_twin(fout);
    write_database_PDB_matrix(fout);
    write_atom_sites(fout);
    write_remark_txt(fout, lines, nline);
//    write_pdb_conn(fout, lines, nline);
    
    pdb2cif_coord(fout, lines, nline);
    
    if(strlen(PROG)>1)
        strcpy(computing.structure_refinement , PROG);
    if(nline && lines) free_memory_2d(lines, nline);    
}

int is_aa_na_residue(char *res)
// >0 is the standard residue
{
    int id=0;
    if(!strcmp(res,"LEU") || !strcmp(res,"PRO") || !strcmp(res,"ALA") ||
       !strcmp(res,"VAL") || !strcmp(res,"GLY") || !strcmp(res,"SER") ||
       !strcmp(res,"ASN") || !strcmp(res,"GLN") || !strcmp(res,"GLU") ||
       !strcmp(res,"MET") || !strcmp(res,"TRP") || !strcmp(res,"PHE") ||
       !strcmp(res,"ILE") || !strcmp(res,"CYS") || !strcmp(res,"TYR") ||
       !strcmp(res,"HIS") || !strcmp(res,"LYS") || !strcmp(res,"ARG") ||
       !strcmp(res,"MSE") || !strcmp(res,"THR") || !strcmp(res,"ASP") ||
       
       !strcmp(res,"A") || !strcmp(res,"T") || !strcmp(res,"G") ||
       !strcmp(res,"C") || !strcmp(res,"U") || !strcmp(res,"I") ||
       !strcmp(res,"DA") || !strcmp(res,"DT") || !strcmp(res,"DG") ||
       !strcmp(res,"DC") || !strcmp(res,"DI") ||
       !strcmp(res,"ADE") || !strcmp(res,"THY") || !strcmp(res,"GUA") ||
       !strcmp(res,"CYT") || !strcmp(res,"URA") ){
        id=1;
    }
    return id;
}

void get_pdb_xyz(char *str, float &x1, float &y1, float &z1)
{

    char tmp[100];
    
    strncpy(tmp, str + 30, 8);
    tmp[8] = '\0';
    x1=atof(tmp);
            
    strncpy(tmp, str + 38, 8);
    tmp[8] = '\0';
    y1=atof(tmp);
            
    strncpy(tmp, str + 46, 8);
    tmp[8] = '\0';
    z1=atof(tmp);
    
}

int check_distance(int in,int *residx1,int *residx2, char **line)
{
    int i, j;
    float x1,y1,z1,x2,y2,z2, dx,dy,dz, d=0;
    
    for (i=residx1[in]; i<residx2[in]; i++){
        if(!strncmp(line[i]+17, "HOH", 3) || !strncmp(line[i]+17, "DOD", 3)) continue;
        
        get_pdb_xyz(line[i], x1,y1,z1);
        for (j=residx1[in+1]; j<residx2[in+1]; j++){
            get_pdb_xyz(line[j], x2,y2,z2);
            dx=fabs(x2-x1);
            dy=fabs(y2-y1);
            dz=fabs(z2-z1);
            if(dx>1.5 || dy>1.5 || dz>1.5)  continue;
            d=sqrt(dx*dx + dy*dy + dz*dz);
            if(d<1.7) return 1;
            
        }
    }
    return 0;
}

    
void check_type(int start, int end, char **line, int nline, char **chain, char**atomname)
/*check residue type:  if it is ligand, give chain='.' */
{
    char tmp[100], tmp1[100], resn[10], resn0[10];
    int i, j, k, n=0,nlig=0, ns=0,id=0, residx1[200000], residx2[200000], atom=0;

    
    ns=0;
    for (i=start; i<end; i++){
        if(strlen(line[i])<30) continue;
        
        strncpy(tmp,line[i]+22, 4);
        tmp[4]='\0';
        residx1[ns]=i;
        
        for(j=i+1; j<end; j++){
            if(strlen(line[j])<30) continue;
            strncpy(tmp1,line[j]+22, 4);
            tmp1[4]='\0';
            
            residx2[ns]=j;
            if(strcmp(tmp, tmp1) || j==end-1){ // different residue
                residx2[ns]=j;
                if(j==end-1){ residx2[ns]=end;}
                ns++;
                i=j-1;
                
                break;
            }
        }  
    }


    id=0;
    for (i=0; i<ns; i++){
        strncpy(resn, line[ residx1[i] ] + 17 , 3);
        resn[3]='\0';
        rid_of_space(resn);
        id=is_aa_na_residue(resn);
        atom=find_atom(" CA ", atomname, residx1[i], residx2[i]);
        if(id==1 || atom>0) break;
    }
    
    if(id==0 || (id==1 && ns<3)){ //all water or ligands
        for (i=start; i<end; i++){
            strcpy(chain[i], ".");
        }
    }else{
        for (i=ns; i>=0; i--){
            strncpy(resn, line[ residx1[i] ] +17 , 3);
            resn[3]='\0';
            rid_of_space(resn);
            if (i>0){
                strncpy(resn0, line[ residx1[i-1] ] +17 , 3);
                resn0[3]='\0';
            }
            
            atom=find_atom(" CA ", atomname, residx1[i], residx2[i]);
            n = is_aa_na_residue(resn);
            
//            printf("testing=%d|%d|%d|%s|%d|%d|%s",ns,i, n,resn, residx1[i], residx2[i] ,line[ residx1[i] ]);

            if(n==1  ||  atom >0 || ((!strcmp(resn, "NH2")|| !strcmp(resn, "ACE"))
                         && i>0 &&  is_protein(resn0)) ){
                nlig=i;
                break;
            }
        }
    }
    
    for (i=nlig; i<ns-1; i++){
//        n=check_distance(i,residx1,residx2,line);
        if (n==0 || n !=0){ //not functional
            for (j=i+1; j<ns; j++){
                for (k=residx1[j]; k<residx2[j]; k++){strcpy( chain[k], ".");}
            }
            break;
        }
    }
    
    
}


char **get_label_asym_id(char **line, int nline)
// label asym to take care of the ligands
{
    char **chain,  ch1[4], ch2[4],  **atomname;
    int i, j, start=0, end=0;

    chain = set_memory_2d (nline, 4);
    atomname=set_memory_2d(nline, 4);
    
    for (i=0; i<nline; i++){
        strcpy(chain[i],".");
        if ((!strncmp(line[i], "ATOM", 4) || !strncmp(line[i], "HETATM", 6) ||
             !strncmp(line[i], "ANISOU", 6)) && strncmp(line[i]+17, "DOD", 3) &&
            strncmp(line[i]+17, "HOH", 3) && strncmp(line[i]+20, "  ", 2)){
            strncpy(chain[i], line[i]+20, 2);
            chain[i][2]='\0';
            if (!strcmp(exptl.method, "NMR")){chain[i][0]=' ';}
            
        }
    }
    
    for (i=0; i<nline; i++){
        if (strchr(chain[i],'.')) continue;
        
        strcpy(ch1, chain[i]);
        start=i;
        for (j=i; j<nline; j++){
            strcpy(ch2, chain[j]);
            if(!strncmp(line[j],"TER", 3) || !strncmp(line[j],"END", 3)||strcmp(ch1, ch2)|| j==nline-1) {
                end=j;
                i=j-1;
                if(j==nline-1){
                    end = nline;
                }else{
                   i=j-1;
                }
                check_type(start, end, line, nline, chain, atomname); //if water/ligand, chain[i]='.'
          //  printf ("ch1-ch2-chain= %4d %s %s %s %s", j, ch1, ch2,chain[j], line[j]);
                break;
            }
        }
    }
    if (end==0) check_type(0, nline, line, nline, chain , atomname);
            
    return chain;
}


void initialize(void)
{
    int i;
    diffrn_source.pdbx_wavelength=(char*)"?";
    diffrn_source_neut.pdbx_wavelength=(char*)"?";

 
    for(i=0; i<TWINMAX; i++){
        pdbx_twin[i].crystal_id=1;
        pdbx_twin[i].diffrn_id=1;
        pdbx_twin[i].domain_id=1;
        pdbx_twin[i].fraction=INI;
        strcpy(pdbx_twin[i].operators, "?");
        strcpy(pdbx_twin[i].type, "?");
    }
    
//    refine_ls_shell.nbin=0;
    refine_shell_neut.nbin=0;
    
    for(i=0; i<100; i++){ 
        refine_ls_restr[i].number = INI;
        refine_ls_restr[i].dev_ideal =INI ;
        refine_ls_restr[i].dev_ideal_target  = INI;
        refine_ls_restr[i].weight  = INI;
        strcpy(refine_ls_restr[i].pdbx_restraint_function, "?");
    }
    
    
    for(i=0; i<NCSMAX; i++){

        strcpy(struct_ncs_dom[i].pdbx_ens_id, "?");
        strcpy(struct_ncs_dom[i].id, "?");
        strcpy(struct_ncs_dom[i].details, "?");
        
        strcpy(struct_ncs_dom_lim[i].pdbx_ens_id, "?" );
        strcpy(struct_ncs_dom_lim[i].dom_id, "?" );
        strcpy(struct_ncs_dom_lim[i].beg_label_asym_id, "?");
        strcpy(struct_ncs_dom_lim[i].end_label_asym_id, "?");
        strcpy(struct_ncs_dom_lim[i].selection_details, "?");
        struct_ncs_dom_lim[i].pdbx_component=INI;
        struct_ncs_dom_lim[i].beg_label_seq_id = INI;
        struct_ncs_dom_lim[i].end_label_seq_id = INI;
    
        strcpy(refine_ls_restr_ncs[i].pdbx_ens_id, "?" );
        strcpy(refine_ls_restr_ncs[i].dom_id, "?" );
        strcpy(refine_ls_restr_ncs[i].pdbx_type, "?");
        strcpy(refine_ls_restr_ncs[i].pdbx_asym_id, "?" ); 
        refine_ls_restr_ncs[i].pdbx_number = INI; 
        refine_ls_restr_ncs[i].pdbx_rms = INI; 
        refine_ls_restr_ncs[i].pdbx_weight = INI; 
    }
}


void extract_pdb_head_other(FILE *fout, char **lines, int nline)
{
    char *head=NULL, *title=NULL,*keywds=NULL, *author=NULL, **token=NULL;
    int i, j, nend=0,ntok, pj1=0, pj2=0, sj1=0, sj2=0;
    int nc[2],ns[2], r200[2], r230[2], r280[2],  mol_1=0, mol_2=0;
    
    
    author=new char[2000];
    keywds=new char[10000];
    title=new char[1000];
    head=new char[1000];

    for(i=0; i<2; i++){
        nc[i]=0; ns[i]=0; r200[i]=0; r230[i]=0; r280[i]=0;
    }
    
    strcpy(head, "");
    strcpy(author, "");
    strcpy(keywds, "");
    strcpy(title, "?");
    for(i=0; i<nline; i++){
        if(!strncmp(lines[i], "ATOM  ", 6) || !strncmp(lines[i], "HETATM", 6) ) break;
        
        if(!strncmp(lines[i], "HEADER", 6)){
            head=get_string_after_id_length(lines[i],"HEADER", 40);
            rid_of_front_end_space(head);
           
        }else if(!strncmp(lines[i], "TITLE ", 6)){
            nend=get_record_line(i, nline, lines, "TITLE ", 6);
            get_item_string(i, nend, lines, title , 10);
            rid_of_front_end_space(title);
            
            if(nend-i>1) i=nend-1;
            
        }else if(!strncmp(lines[i], "COMPND", 6)){
            nend=get_record_line(i, nline, lines, "COMPND", 6);
            nc[0]=i;  nc[1]=nend; 
            if(nend-i>1) i=nend-1;
            
        }else if(!strncmp(lines[i], "SOURCE", 6)){
            nend=get_record_line(i, nline, lines, "SOURCE", 6);
            ns[0]=i;  ns[1]=nend; 
            if(nend-i>1) i=nend-1;
            
        }else if(!strncmp(lines[i], "KEYWDS", 6)){
            nend=get_record_line(i, nline, lines, "KEYWDS", 6);
            get_item_string(i, nend, lines, keywds , 10);
            rid_of_front_end_space(keywds);
            if(nend-i>1) i=nend-1;
            
        }else if(!strncmp(lines[i], "EXPDTA", 6)){
            if (strstr(lines[i], "X-RAY ")) strcpy(exptl.method, "X-RAY DIFFRACTION");
            if (strstr(lines[i], "NEUTRON")) strcpy(exptl_neut.method, "NEUTRON DIFFRACTION");
            
        }else if(!strncmp(lines[i], "AUTHOR", 6)){
            nend=get_record_line(i, nline, lines, "AUTHOR", 6);
            get_item_string(i, nend, lines, author, 10);
            if(nend-i>1) i=nend-1;
            
        }else if(!strncmp(lines[i], "JRNL  ", 6)){
            nend=get_record_line(i, nline, lines, "JRNL  ", 6);
            pj1=i; pj2=nend;
            if(nend-i>1) i=nend-1;
            
        }else if(!strncmp(lines[i], "REMARK   1", 10)){
            nend=get_record_line(i, nline, lines, "REMARK   1", 10);
            sj1=i; sj2=nend;
            if(nend-i>1) i=nend-1;
            
        }else if(!strncmp(lines[i], "REMARK   3", 10)){
            nend=get_record_line(i, nline, lines, "REMARK   3", 10);
            if(nend-i>1) i=nend-1;
/*            
        }else if(!strncmp(lines[i], "REMARK  40", 10)){ 
            nend=get_record_line(i, nline, lines, "REMARK  40", 10);
            write_remark_40(fout, i, nend,  lines);
            if(nend-i>1) i=nend-1;
*/            
        }else if(!strncmp(lines[i], "REMARK 200", 10)){
            nend=get_record_line(i, nline, lines, "REMARK 200", 10);
            r200[0]=i; r200[1]=nend;
            if(nend-i>1) i=nend-1;
            
        }else if(!strncmp(lines[i], "REMARK 230", 10)){
            nend=get_record_line(i, nline, lines, "REMARK 230", 10);
            r230[0]=i; r230[1]=nend;
            if(nend-i>1) i=nend-1;
            
        }else if(!strncmp(lines[i], "REMARK 280", 10)){
            nend=get_record_line(i, nline, lines, "REMARK 280", 10);
            r280[0]=i; r280[1]=nend;
            if(nend-i>1) i=nend-1;
           
        }else if(!strncmp(lines[i], "CRYST1", 6)){
            extract_pdb_items(i,i, lines, "CRYST1");
            
        }else if(!strncmp(lines[i], "ORIGX1", 6)){
            extract_pdb_items(i,i, lines, "ORIGX1");
            
        }else if(!strncmp(lines[i], "ORIGX2", 6)){
            extract_pdb_items(i,i, lines, "ORIGX2");
            
        }else if(!strncmp(lines[i], "ORIGX3", 6)){
            extract_pdb_items(i,i, lines, "ORIGX3");
            
        }else if(!strncmp(lines[i], "SCALE1", 6)){
            extract_pdb_items(i,i, lines, "SCALE1");
            
        }else if(!strncmp(lines[i], "SCALE1", 6)){
            extract_pdb_items(i,i, lines, "SCALE1");
            
        }else if(!strncmp(lines[i], "SCALE2", 6)){
            extract_pdb_items(i,i, lines, "SCALE2");
            
        }else if(!strncmp(lines[i], "SCALE3", 6)){
            extract_pdb_items(i,i, lines, "SCALE3");

            
        }
    }

    if((r200[1]-r200[0])>2){get_remarks("200", r200, lines);}
    if((r230[1]-r230[0])>2){get_remarks("230", r230, lines);}
    if((r280[1]-r280[0])>2){get_remarks("280", r280, lines);}
    
    
    if(strlen(keywds)>1 && strcmp_case(keywds, " NULL")){
        fprintf(fout,"\n#\n");
        fprintf(fout,"_struct_keywords.entry_id  %s\n", DATA_BLOCK);
        if(strlen(head)>1) fprintf(fout,"_struct_keywords.pdbx_keywords  '%s'\n", head);
        fprintf(fout,"_struct_keywords.text  '%s'\n", keywds);
    }

    if(strlen(title)>2 && strcmp(title, "----") && strcmp(title, " NULL")){
        fprintf(fout,"\n#\n");
        fprintf(fout,"_struct.entry_id  %s\n", DATA_BLOCK);
        fprintf(fout,"_struct.title  '%s'\n", title);
    }
    
    if(strlen(author)>1 && strcmp(author, "NULL")){
        fprintf(fout,"\n#\nloop_\n_audit_author.pdbx_ordinal \n_audit_author.name\n");
        token=string_token(author, ",", &ntok);
        for(i=0; i<ntok; i++){
            rid_of_front_end_space(token[i]);
            fprintf(fout,"%d '%s'\n",i+1, token[i]);
        }
    }

    pdb_citation(fout, pj1,pj2,sj1,sj2,lines);
    
    for(j=nc[0]; j<nc[1]; j++)   if(strstr(lines[j],"MOL_ID:")) mol_1++;
    for(j=ns[0]; j<ns[1]; j++)   if(strstr(lines[j],"MOL_ID:")) mol_2++;
    
    if(mol_1>0 && mol_2>0 && mol_1 == mol_2 ){
        pdb_compnd_source(fout, nc, ns, mol_2, lines);
    }else if (mol_1>0 || mol_2>0){
        sprintf(INFO,"Warning! The entity in COMPND & SOURCE are different\n");
        pinfo(INFO,0);
    }
        
}

void parse_pdbitem_on_id(int n, int nline, char **lines, const char *id, char *str)
//
{
    char ss[500];
    int i;
    
    
    strcpy(str,"");
    for(i=n; i<nline; i++){
        strcpy(ss,lines[i]);
        replace_char(ss, '\n', ' ');
        if(i>n && strstr(ss, ": ")) break;
        if(strstr(ss, ": ")) {
            strcat(str, strstr(ss, ": ")+1);
        }else{
            strcat(str,ss + 10);
        }
    }
    
    if (strlen(str)>0){
        clean_string(str,str);
        replace_char(str, ';', ' ');
        rid_of_front_end_space(str);
    }
        //printf("hello=%d %d (%s) %s\n", n, nline, id, str);
    
}

void pdb_compnd_source(FILE *fout, int nc[], int ns[], int mol, char **lines)
// extract the infor about the  compnd & source                    
{
    char **chain,  **ec, **molname, **synonym,**frag, **mute, **molid, **eng;
    char **smolid, **sci, **comm, **gene, **esys, **esysc, **esyst,  **esyss;
    char **detail, **syn, **strain, **otaxid, **etaxid, **esysp;
    char  tmp[2560], ss[2560];
    int i, j, n, end=0, nsyn=0, nnat=0, neng=0;

    
    fprintf(fout,"\n#\nloop_\n_pdbx_pdb_compnd.id\n_pdbx_pdb_compnd.text\n");
    n=0;
    for(j=nc[0]; j<nc[1]; j++){
        n++;
        strcpy(tmp,lines[j]);
        rid_of_end_space(tmp);
        fprintf(fout,"%2d  '%s'\n", n, tmp+10);
    }
    
    fprintf(fout,"\n#\nloop_\n_pdbx_pdb_source.id\n_pdbx_pdb_source.text\n");
    n=0;
    for(j=ns[0]; j<ns[1]; j++){
        n++;
        strcpy(tmp,lines[j]);
        rid_of_end_space(tmp);
        fprintf(fout,"%2d  '%s'\n", n, tmp+10);
    }
    
    molname=set_memory_2d(mol, 2560);
    synonym=set_memory_2d(mol, 2560);
    molid=set_memory_2d(mol, 256);
    ec=set_memory_2d(mol, 256);
    chain=set_memory_2d(mol, 256);
    frag=set_memory_2d(mol, 256);
    mute=set_memory_2d(mol, 2560);
    eng=set_memory_2d(mol, 256);
    
    for(j=0; j<mol; j++){
        strcpy(molid[j],"?");
        strcpy(molname[j],"?");
        strcpy(chain[j],"?");
        strcpy(synonym[j],"?");
        strcpy(ec[j],"?");
        strcpy(mute[j],"?");
        strcpy(frag[j],"?");
        strcpy(eng[j],"?");
    }

    
    const char *field[]={  //never change the order
        "MOL_ID: ",
        "MOLECULE: ",
        "CHAIN: ",
        "SYNONYM: ",
        "EC: ",
        "MUTATION: ",
        "FRAGMENT: ",
        "ENGINEERED: "
    };
        
    n=0; end = nc[1];
    for(j=nc[0]; j<nc[1]; j++) { //COMPND
        strcpy(ss, lines[j]);
        
        if (strstr(ss, field[0])){
            parse_pdbitem_on_id(j, end, lines, field[0], tmp);
            strcpy(molid[n], tmp);
            
            for(i=j+1; i<nc[1]; i++){
                strcpy(ss, lines[i]);
                if (strstr(ss, field[1])){
                    parse_pdbitem_on_id(i, end, lines, field[1], tmp);
                    strcpy(molname[n], tmp);
                    
                }else if (strstr(ss, field[2])){
                    parse_pdbitem_on_id(i, end, lines, field[2], tmp);
                    strcpy(chain[n], tmp);
                    
                }else if (strstr(ss, field[3])){
                    parse_pdbitem_on_id(i, end, lines, field[3], tmp);
                    strcpy(synonym[n], tmp);
                    
                }else if (strstr(ss, field[4])){
                    parse_pdbitem_on_id(i, end, lines, field[4], tmp);
                    strcpy(ec[n], tmp);
                    
                }else if (strstr(ss, field[5])){
                    parse_pdbitem_on_id(i, end, lines, field[5], tmp);
                    strcpy(mute[n], tmp);
                    
                }else if (strstr(ss, field[6])){
                    parse_pdbitem_on_id(i, end, lines, field[6], tmp);
                    strcpy(frag[n], tmp);
                    
                }else if (strstr(ss, field[7])){
                    parse_pdbitem_on_id(i, end, lines, field[7], tmp);
                    strcpy(eng[n], tmp);
                    
                }else if(strstr(ss, field[0])){
                    j=i-1;
                    break;
                }

            }
            n++;
        }
    }

    
    smolid=set_memory_2d(mol, 2560);
    sci=set_memory_2d(mol, 2560);
    comm=set_memory_2d(mol, 2560);
    strain=set_memory_2d(mol, 2560);
    gene=set_memory_2d(mol, 2560);
    esys=set_memory_2d(mol, 2560);
    esysc=set_memory_2d(mol, 2560);
    esyst=set_memory_2d(mol, 2560);
    esysp=set_memory_2d(mol, 2560);
    esyss=set_memory_2d(mol, 2560);
    syn=set_memory_2d(mol, 2560);
    detail=set_memory_2d(mol, 2560);
    otaxid=set_memory_2d(mol, 2560);
    etaxid=set_memory_2d(mol, 2560);
    
    for(j=0; j<mol; j++){
        strcpy(smolid[j],"?");
        strcpy(sci[j],"?");
        strcpy(comm[j],"?");
        strcpy(otaxid[j],"?");
        strcpy(strain[j],"?");
        strcpy(gene[j],"?");
        
        strcpy(esys[j],"?");
        strcpy(etaxid[j],"?");
        strcpy(esysc[j],"?");
        strcpy(esyst[j],"?");
        strcpy(esysp[j],"?");
        strcpy(esyss[j],"?");
        strcpy(syn[j],"?");
        strcpy(detail[j],"?");
    }

    const char *sitem[]={
        "MOL_ID: ",
        "ORGANISM_SCIENTIFIC: ",
        "ORGANISM_COMMON: ",
        "ORGANISM_TAXID: ",
        "STRAIN: ",
        "GENE: ",
        
        "EXPRESSION_SYSTEM: ",
        "EXPRESSION_SYSTEM_TAXID: ",
        "EXPRESSION_SYSTEM_COMMON: ",
        "EXPRESSION_SYSTEM_VECTOR_TYPE: ",
        "EXPRESSION_SYSTEM_PLASMID: ",
        "EXPRESSION_SYSTEM_STRAIN: ",
        "SYNTHETIC: ",
        "DETAILS: "
    };

    
    n=0; end = ns[1];
    for(j=ns[0]; j<ns[1]; j++) { //SOURCE
        strcpy(ss, lines[j]);
        
        if (strstr(ss, sitem[0])){
            parse_pdbitem_on_id(j, end, lines, sitem[0], tmp);
            strcpy(smolid[n], tmp);
            
            for(i=j+1; i<ns[1]; i++){
                strcpy(ss, lines[i]);
                
                if (strstr(ss, sitem[1])){
                    parse_pdbitem_on_id(i, end, lines, sitem[1], tmp);
                    strcpy(sci[n], tmp);
                    
                }else if (strstr(ss, sitem[2])){
                    parse_pdbitem_on_id(i, end, lines, sitem[2], tmp);
                    strcpy(comm[n], tmp);
                    
                }else if (strstr(ss, sitem[3])){
                    parse_pdbitem_on_id(i, end, lines, sitem[3], tmp);
                    strcpy(otaxid[n], tmp);
                    
                }else if (strstr(ss, sitem[4]) && !strstr(ss, sitem[11]) ){
                    parse_pdbitem_on_id(i, end, lines, sitem[4], tmp);
                    strcpy(strain[n], tmp);
                    
                }else if (strstr(ss, sitem[5])){
                    parse_pdbitem_on_id(i, end, lines, sitem[5], tmp);
                    strcpy(gene[n], tmp);
                    
                }else if (strstr(ss, sitem[6])){
                    parse_pdbitem_on_id(i, end, lines, sitem[6], tmp);
                    strcpy(esys[n], tmp);
                    
                }else if (strstr(ss, sitem[7])){
                    parse_pdbitem_on_id(i, end, lines, sitem[7], tmp);
                    strcpy(etaxid[n], tmp);

                }else if (strstr(ss, sitem[8])){
                    parse_pdbitem_on_id(i, end, lines, sitem[8], tmp);
                    strcpy(esysc[n], tmp);

                }else if (strstr(ss, sitem[9])){
                    parse_pdbitem_on_id(i, end, lines, sitem[9], tmp);
                    strcpy(esyst[n], tmp);

                }else if (strstr(ss, sitem[10])){
                    parse_pdbitem_on_id(i, end, lines, sitem[10], tmp);
                    strcpy(esysp[n], tmp);

                }else if (strstr(ss, sitem[11])){
                    parse_pdbitem_on_id(i, end, lines, sitem[11], tmp);
                    strcpy(esyss[n], tmp);

                }else if (strstr(ss, sitem[12])){
                    parse_pdbitem_on_id(i, end, lines, sitem[12], tmp);
                    strcpy(syn[n], tmp);

                 }else if (strstr(ss, sitem[13])){
                    parse_pdbitem_on_id(i, end, lines, sitem[13], tmp);
                    strcpy(detail[n], tmp);
                    
                }else if(strstr(ss, sitem[0])){
                    j=i-1;
                    break;
                }

            }
            n++;
        }
    }


        //test ENGINEERED or SYN, or NAT
    for(j=0; j<mol; j++){
        if (strstr_case(syn[j], "YES"))  nsyn=1;
        if (!strstr_case(syn[j], "YES") && !strstr_case(eng[j], "YES")) nnat=1;
        if (strstr_case(eng[j], "YES") && !strstr_case(syn[j], "YES")) neng=1;
            
    }

    
    for(j=0; j<mol; j++){
        if(j==0){
            fprintf(fout,"\n#\n");
            fprintf(fout," loop_ \n \
_entity_keywords.entity_id \n \
_entity_keywords.pdbx_mutation \n \
_entity_keywords.pdbx_fragment \n \
_entity_keywords.pdbx_ec \n \
_entity_keywords.text \n ");
        }
        
        fprintf(fout,"'%s' '%s' '%s' '%s' ?\n", molid[j], mute[j], frag[j], ec[j]);
    }

    
    for(j=0; j<mol; j++){
        if(j==0){
            fprintf(fout,"\n#\n");
            fprintf(fout," loop_ \n \
_entity.id \n \
_entity.type \n \
_entity.src_method \n \
_entity.pdbx_description \n ");
        }
           
        strcpy(tmp, "man");
        if (!strstr_case(syn[j], "YES") && !strstr_case(eng[j], "YES")) {
            strcpy(tmp, "nat");
        }else if (strstr_case(syn[j], "YES")) {
            strcpy(tmp, "syn");
        }
        fprintf(fout,"'%s'  polymer '%s' \"%s\" \n",molid[j] ,tmp, molname[j]);
            
//        fprintf(fout,"'%s'  polymer ? ? \n",molid[j] );
        
    }
 
    
    n=0;
    for(j=0; j<mol; j++){
        if(strlen(synonym[j])>1) {
            n=1;
            fprintf(fout,"\n#\n");
            fprintf(fout," loop_ \n \
_entity_name_com.entity_id \n \
_entity_name_com.name \n ");
            break;
        }  
    }
    if(n>0){  
        for(j=0; j<mol; j++){
            if(strlen(synonym[j])>1)fprintf(fout,"%s '%s' \n",molid[j] ,synonym[j]);
        }
    }


    
    if (neng>0){
        n=0;
        for(j=0; j<mol; j++){
            if (j==0){
                fprintf(fout,"\n#\n");
                fprintf(fout," loop_ \n \
_entity_src_gen.entity_id              \n \
_entity_src_gen.gene_src_common_name             \n \
_entity_src_gen.pdbx_gene_src_gene                \n \
_entity_src_gen.gene_src_strain                  \n \
_entity_src_gen.pdbx_gene_src_scientific_name     \n \
_entity_src_gen.pdbx_gene_src_ncbi_taxonomy_id     \n \
_entity_src_gen.host_org_common_name             \n \
_entity_src_gen.pdbx_host_org_scientific_name     \n \
_entity_src_gen.pdbx_host_org_ncbi_taxonomy_id     \n \
_entity_src_gen.pdbx_host_org_strain         \n \
_entity_src_gen.pdbx_host_org_vector_type         \n \
_entity_src_gen.plasmid_name                     \n  \
_entity_src_gen.pdbx_description                 \n \
");
            }
            if (strstr_case(eng[j], "YES") && !strstr_case(syn[j], "YES")){
                fprintf(fout,"'%s' '%s' '%s' '%s' '%s' '%s'  '%s' '%s' '%s' '%s' '%s' '%s' '%s'\n",
                        smolid[j],comm[j],gene[j],strain[j],sci[j],otaxid[j],esysc[j], 
                        esys[j], etaxid[j], esyss[j],esyst[j],esysp[j],detail[j]);
            }
            
        }
    }

    if (nsyn>0){
        for(j=0; j<mol; j++){ //for synthetic
            if(j==0){
                fprintf(fout,"\n#\n");
                fprintf(fout," loop_ \n \
_pdbx_entity_src_syn.entity_id \n \
_pdbx_entity_src_syn.details \n ");
            }
            if (strstr_case(syn[j], "YES")){
                fprintf(fout,"'%s'  '%s' \n",smolid[j] ,detail[j]);
            }
        }
    }

    
    if (nnat>0){
        for(j=0; j<mol; j++){ //for natural
            if(j==0){
                fprintf(fout,"\n#\n");
                fprintf(fout," loop_ \n \
_entity_src_nat.entity_id \n \
_entity_src_nat.common_name \n \
_entity_src_nat.pdbx_organism_scientific \n \
_entity_src_nat.pdbx_ncbi_taxonomy_id  \n \
_entity_src_nat.strain \n \
_entity_src_nat.details \n ");
            }
            if (!strstr_case(syn[j], "YES") && !strstr_case(eng[j], "YES")){
                fprintf(fout,"'%s' '%s' '%s' '%s' '%s' '%s' \n",
                        smolid[j],comm[j],sci[j], otaxid[j], strain[j], detail[j]);
            }
            
        }
    }
    
}

void write_remark_40(FILE *fout, int start, int end, char **lines)
{
    
    int j, nrot=-1,nbrot=-1, nout=-1, nfav=0, prog=0;
    float score=0,brot=-1,out=0,fav=0;
    
    for(j=start; j<end; j++){
        if(strstr_space(lines[j], "REMARK  40  PROGRAMS    : MOLPROBITY")){
            prog=1;
            
        }else if(strstr_space(lines[j], "REMARK  40  ALL-ATOM CLASHSCORE     :")){
            score=get_float_after_id(lines[j],":");
            
        }else if(pattern5(lines[j], "BAD ROTAMERS", ":", "%", "/", "(")){
            brot=get_float(lines[j],":", "%");
            nbrot=get_int(lines[j],"%", "/");
            nrot=get_int(lines[j],"/", "(");
            
        }else if(pattern5(lines[j], "OUTLIERS", ":", "%", "/", "(")){
            out=get_float(lines[j],":", "%");
            nout=get_int(lines[j],"%", "/");
            
        }else if(pattern5(lines[j], "FAVORED", ":", "%", "/", "(")){
            fav=get_float(lines[j],":", "%");
            nfav=get_int(lines[j],"%", "/");
        }
    }
    if(prog==1 && nfav>0){
        fprintf(fout,"\n#\n");
        fprintf(fout,"_pdbx_validate_overall.program   MOLPROBITY\n");
        fprintf(fout,"_pdbx_validate_overall.clashscore  %.2f\n", score);
        fprintf(fout,"_pdbx_validate_overall.bad_rotamer_percent  %.2f\n",brot );
        fprintf(fout,"_pdbx_validate_overall.bad_rotamer_number  %d\n",nbrot );
        if(nrot>=0)fprintf(fout,"_pdbx_validate_overall.total_rotamer_number  %d\n", nrot);
        fprintf(fout,"_pdbx_validate_overall.ramachandran_outlier_percent  %.2f\n",out );
        if(nout>=0)fprintf(fout,"_pdbx_validate_overall.ramachandran_outlier_number  %d\n", nout);
        fprintf(fout,"_pdbx_validate_overall.ramachandran_favored_percent  %.2f\n", fav);
        if(nfav>=0)fprintf(fout,"_pdbx_validate_overall.ramachandran_favored_number  %d\n", nfav);
    }
    
}


void pdb_citation(FILE *fout,int pj1,int pj2,int sj1,int sj2, char **lines)
{
    char  jname[200], vol[40], page1[40], year[40], astm[40], contry[40];
    char  issn[40], pmid[80],doi[80],author[2560],title[1000],str[512] , **token;
    int i, n=0,m, nend=0, ntok=0, ref;

    strcpy(jname, "?"); strcpy( vol, "?"); strcpy(page1, "?"); strcpy(year, "?"); 
    strcpy(astm, "?"); strcpy(contry, "?"); strcpy(issn, "?"); strcpy(pmid, "?");
    strcpy(doi, "?"); strcpy(title, "?"); 
    

    if(pj2-pj1>1 || sj2-sj1>1){//primary or secondary journal authors
        fprintf(fout,"\n#\n");
        fprintf(fout," loop_ \n \
_citation_author.citation_id \n \
_citation_author.name \n \
_citation_author.ordinal \n ");
    }
    n=0;
    if(pj2-pj1>1){
        for(i=pj1; i<pj2; i++){
            strcpy(str, lines[i]);
            if(!strncmp(str, "JRNL        AUTH", 16)){
                nend=get_record_line(i, pj2, lines, "JRNL        AUTH", 16);
                get_item_string(i, nend, lines, author, 18);
                if(nend-i>1) i=nend-1;
            }
        }
        rid_of_front_end_space(author);
        if(strlen(author)){
            token=string_token(author, ",", &ntok);
            for(i=0; i<ntok; i++){
                n++;
                rid_of_front_end_space(token[i]);
                fprintf(fout,"primary '%s' %d\n", token[i], n);
            }
        }        
    }

    m=0;
    if(sj2-sj1>1){
        ref=1;
        for(i=sj1; i<sj2; i++){

            strcpy(str, lines[i]);
            if(!strncmp(str, "REMARK   1 REFERENCE", 20)){
                ref=get_int_after_id(str,"ENCE");
        
            }else if(!strncmp(str, "REMARK   1  AUTH", 16)){
                nend=get_record_line(i, sj2, lines, "REMARK   1  AUTH", 16);
                get_item_string(i, nend, lines, author, 18);
                if(strlen(author)>1){
                    token=string_token(author, ",", &ntok);
                    for(i=0; i<ntok; i++){
                        m++;
                        rid_of_front_end_space(token[i]);
                        fprintf(fout,"%d '%s' %d\n",ref, token[i], m);
                    }
                }
                
                if(nend-i>1) i=nend-1;
            }
        }
    }
    if((pj2-pj1>1 || sj2-sj1>1) && n==0 && m==0 ) fprintf(fout,". .  . \n");
    

        
    if(pj2-pj1>1 || sj2-sj1>1){//primary journal exist

        fprintf(fout,"\n#\n");
        fprintf(fout," loop_ \n \
_citation.id  \n \
_citation.title  \n \
_citation.journal_abbrev  \n \
_citation.journal_volume  \n \
_citation.page_first  \n \
_citation.page_last  \n \
_citation.year  \n \
_citation.journal_id_ASTM  \n \
_citation.country  \n \
_citation.journal_id_ISSN  \n \
_citation.journal_id_CSD  \n \
_citation.book_publisher  \n \
_citation.pdbx_database_id_PubMed  \n \
_citation.pdbx_database_id_DOI  \n ");
    }
    
    for(i=pj1; i<pj2; i++){
        strcpy(str, lines[i]);
        if(!strncmp(str, "JRNL        TITL", 16)){
            nend=get_record_line(i, pj2, lines, "JRNL        TITL", 16);
            get_item_string(i, nend, lines, title, 18);
            rid_of_front_end_space(title);

            if(nend-i>1) i=nend-1;
            
        }else if(!strncmp(str, "JRNL        REF ", 16)){
            get_string(str,19,48, jname);
            get_string(str,51,54,vol);
            get_string(str,55,60,page1);
            get_string(str,61,67,year);
            
        }else if(!strncmp(str, "JRNL        REFN", 16)){
            get_string(str,23,29,astm);
            get_string(str,30,33, contry);
            get_string(str,40,48,issn);
            
        }else if(!strncmp(str, "JRNL        PMID", 16)){
            sscanf(str+16, "%s", pmid);
            
        }else if(!strncmp(str, "JRNL        DOI ", 16)){
            sscanf(str+16, "%s", doi);
            
        }
    }
    
    if(pj2-pj1>1){
        fprintf(fout,"primary \n;%s\n;\n '%s' '%s' '%s' ? '%s' '%s' '%s' '%s' ? ? '%s' '%s' \n",
                title,jname, vol, page1,year, astm, contry, issn, pmid, doi);
    }

    if(sj2-sj1>1){
        ref=1;
        for(i=sj1; i<sj2; i++){
            strcpy(str, lines[i]);
            rid_of_end_space(str);
            if(!strncmp(str, "REMARK   1 REFERENCE", 20)){
                ref=get_int_after_id(str,"ENCE");

                strcpy(jname, "?"); strcpy( vol, "?"); strcpy(page1, "?"); strcpy(year, "?"); 
                strcpy(astm, "?"); strcpy(contry, "?"); strcpy(issn, "?"); strcpy(pmid, "?");
                strcpy(doi, "?"); strcpy(title, "?"); 

                
            }else if(!strncmp(str, "REMARK   1  TITL", 16)){
                nend=get_record_line(i, sj2, lines, "REMARK   1  TITL", 16);
                get_item_string(i, nend, lines, title, 18);
                rid_of_front_end_space(title);
                if(nend-i>1) i=nend-1;
                
            }else if(!strncmp(str, "REMARK   1  REF ", 16)){
                get_string(str,19,48,jname);
                get_string(str,51,54,vol);
                get_string(str,55,60,page1);
                get_string(str,61,67,year);
            
            }else if(!strncmp(str, "REMARK   1  REFN", 16)){
                get_string(str,23,29,astm);
                get_string(str,30,33,contry);
                get_string(str,40,48,issn);

            }else if(!strncmp(str, "REMARK   1  PMID", 16)){
                sscanf(str+16, "%s", pmid);

            }else if(!strncmp(str, "REMARK   1  DOI ", 16)){
                sscanf(str+16, "%s", doi);
            }
            
            if((strlen(str+9)>2 && !strncmp(lines[i+1], "REMARK   1 REFERENCE", 20)) || i==sj2-1){
                fprintf(fout,"%d \n;%s\n;\n '%s' '%s' '%s' ? '%s' '%s' '%s' '%s' ? ? '%s' '%s' \n",
                       ref, title,jname, vol, page1,year, astm, contry, issn, pmid, doi);
            }
        }
    }
    
}


void get_item_string(int start, int nend, char **lines, char *item, int cut)
//put sub-strings from start-nend into one line string (and clean space)
{
    char str[512];
    int j, len=0;
    
    strcpy(item,"");
    for(j=start; j<nend; j++){
        strcpy(str, lines[j]);
        replace_char(str, '\n', ' ');
        replace_char(str, '\r', ' ');
        len=strlen(str);
        if(len>cut)  strcat(item, str+cut);
    }
    clean_string(item,item);
}


void extract_pdb_items(int i, int nend, char **lines, const char *key)
{
    
    if(!strncmp(lines[i], "CRYST1 ", 7)){
        cell.length_a = get_float_by_length(lines[i],6, 14);
        cell.length_b = get_float_by_length(lines[i],15, 23);
        cell.length_c = get_float_by_length(lines[i],24,32) ;
        cell.angle_alpha = get_float_by_length(lines[i],33,39);
        cell.angle_beta = get_float_by_length(lines[i],40,46);
        cell.angle_gamma = get_float_by_length(lines[i],47,53);
        get_string(lines[i],55, 65, symmetry.space_group_name_H_M);
        rid_of_front_end_space(symmetry.space_group_name_H_M);
        cell.Z_PDB = get_int_by_length(lines[i],66,69);
        

    }else if(!strncmp(lines[i], "ORIGX1 ", 7)){
        database_PDB_matrix.origx11 = get_float_by_length(lines[i], 10, 19);
        database_PDB_matrix.origx12 = get_float_by_length(lines[i], 20, 29);
        database_PDB_matrix.origx13 = get_float_by_length(lines[i], 30, 39);
        database_PDB_matrix.origx_vector1 = get_float_by_length(lines[i], 45, 54);
    }else if(!strncmp(lines[i], "ORIGX2 ", 7)){
        database_PDB_matrix.origx21 = get_float_by_length(lines[i], 10, 19);
        database_PDB_matrix.origx22 = get_float_by_length(lines[i], 20, 29);
        database_PDB_matrix.origx23 = get_float_by_length(lines[i], 30, 39);
        database_PDB_matrix.origx_vector2 = get_float_by_length(lines[i], 45, 54);
    }else if(!strncmp(lines[i], "ORIGX3 ", 7)){
        database_PDB_matrix.origx31 = get_float_by_length(lines[i], 10, 19);
        database_PDB_matrix.origx32 = get_float_by_length(lines[i], 20, 29);
        database_PDB_matrix.origx33 = get_float_by_length(lines[i], 30, 39);
        database_PDB_matrix.origx_vector3 = get_float_by_length(lines[i], 45, 54);
            
    }else if(!strncmp(lines[i], "SCALE1 ", 7)){
        atom_sites.fract_transf_matrix11 = get_float_by_length(lines[i], 10, 19);
        atom_sites.fract_transf_matrix12 = get_float_by_length(lines[i], 20, 29);
        atom_sites.fract_transf_matrix13 = get_float_by_length(lines[i], 30, 39);
        atom_sites.fract_transf_vector1  = get_float_by_length(lines[i], 45, 54);
    }else if(!strncmp(lines[i], "SCALE2 ", 7)){
        atom_sites.fract_transf_matrix21 = get_float_by_length(lines[i], 10, 19);
        atom_sites.fract_transf_matrix22 = get_float_by_length(lines[i], 20, 29);
        atom_sites.fract_transf_matrix23 = get_float_by_length(lines[i], 30, 39);
        atom_sites.fract_transf_vector2  = get_float_by_length(lines[i], 45, 54);
    }else if(!strncmp(lines[i], "SCALE3 ", 7)){
        atom_sites.fract_transf_matrix31 = get_float_by_length(lines[i], 10, 19);
        atom_sites.fract_transf_matrix32 = get_float_by_length(lines[i], 20, 29);
        atom_sites.fract_transf_matrix33 = get_float_by_length(lines[i], 30, 39);
        atom_sites.fract_transf_vector3  = get_float_by_length(lines[i], 45, 54);
    }
        
}


int get_record_line(int start, int end, char **lines, const char *key, int len)
{
    int i, n=start+1;
    
    for(i=start; i<end; i++){
        if(strncmp(lines[i], key, len )){
            n=i;
            break;
        }
    }
    return n;

}


char **get_pdb_records(char *inpfile, int *nline)
 //there is no space between lines
{
    char str[512], **all_line=NULL, **line_ph=NULL, **line_nt=NULL;
    int i,j, m, nall, nl, xray=0, neut=0, nt=0;
    FILE *fp=NULL;
    

    printf("Extracting all record from pdb file ..\n");

    if((fp=fopen(inpfile, "r"))==NULL)
       open_file_error(inpfile, "get_pdb_records");
    
    nall=0;
    while (fgets(str, sizeof str, fp) != NULL) {
        if(is_space_line(str)<=0)continue;
        nall++;
    }
    rewind(fp);
    all_line=new char *[nall+1];

    m=0;
    while (fgets(str, sizeof str, fp) != NULL) {
        if(is_space_line(str)<=0)continue;
        all_line[m] = new char [strlen(str) +1];
        strcpy(all_line[m],str);
        if(strstr_space(str, "REMARK   3  X-RAY DATA")){
            xray=1;
        }else if(strstr_space(str, "REMARK   3  NEUTRON DATA")){
            neut=1;
        }        
        m++;
    }
    *nline =m;
    fclose(fp);

    
    if(xray && neut){ //extra for nuetron
        
        line_ph=new char *[m+1];
        line_nt=new char *[m+1];
        
        printf("This is a combined  Xray & Neutron refinement.\n");
        nl=0;
        for(i=0;i<m;i++){ //delete stat for neutron
            if(strstr_space(all_line[i], "REMARK   3  NEUTRON DATA")){
                nt=0;
                for(j=i;j<m;j++){
                    if(strstr_space(all_line[j],"REMARK   3  STRUCTURE FACTORS CALCU") ||
                       strstr_space(all_line[j],"REMARK   3     (t) = TRANSPOSE") ||
                       strstr_space(all_line[j],"REMARK   4") || 
                       !strncmp(all_line[j],"ATOM", 4) || !strncmp(all_line[j],"HETATM", 6)){
                        i=j+1;
                        break;
                    }
                    line_nt[nt] = new char [strlen(all_line[j]) +1];
                    strcpy(line_nt[nt], all_line[j]);
                    nt++;
                    continue;
                }
                
            }
            line_ph[nl] = new char [strlen(all_line[i]) +1];
            
            strcpy(line_ph[nl], all_line[i]);
            nl++;            
        }
        *nline =nl;
        free_memory_2d(all_line, m);
        get_neutron_stat(line_nt, nt);
        
        if(nt && line_nt) free_memory_2d(line_nt, nt);
        return (line_ph);
    }
    
    return (all_line);
}

void get_neutron_stat(char **lines, int nline)
{
    char str[512], tmp[512];
    int  i,m, j, n=0,n1,n2;
    int nbin=0, nconv=0, nwork;
    float  u11, u22, u33, u12, u23;

    refine_neutron.pdbx_refine_id=(char *)"NEUTRON DIFFRACTION";
    
    for(i=0;i<nline;i++){ //delete stat for neutron
        strcpy(str,lines[i]);
        
        if (strstr_space(lines[i],"REMARK   3   BIN  RESOLUTION RANGE  COMPL.   NWORK NFREE  RWORK RFREE")){//v-1.3b-rc3 ..rc6
            nbin=0;
            nconv=0;
            nwork=0;
            for(j=i+1; j<nline; j++){
                if(pattern5(lines[j], "REMARK   3", ".", " - ", ".","")){
                    strcpy(tmp, lines[j]);
                    if(strstr_case(lines[j], "None"))strcpy(tmp,replace_all_str(lines[j], "None", "0.0"));

                    
                    n=sscanf(tmp+10, " %*s %f %*s %f %f %d %d %f %f",
                             &u11,&u22,&u33, &n1, &n2, &u12, &u23 );
                    if((nbin>0 && n!=7 ) || strstr(lines[j], "REMARK   3  BULK SOLVENT")) break;
                    if(nbin>=NBIN-1){
                        sprintf(INFO, "Warning! Too many resolution bins (%d). BIN_MAX=%d\n", nbin,NBIN);
                        pinfo(INFO, 0);
                        nbin=0;
                        break;
                    }
                    
                    if(n==7){
                        refine_shell_neut.d_res_low[nbin] = u11;
                        refine_shell_neut.d_res_high[nbin] =u22;
                        if(u33<=1.0)u33=100*u33;
                        refine_shell_neut.percent_reflns_obs[nbin] = u33;
                        
                        refine_shell_neut.number_reflns_R_work[nbin] =n1;
                        nwork=nwork+n1;
                        refine_shell_neut.number_reflns_R_free[nbin] =n2;
                        if(u12>2. && u23>2. ){
                            u23=0.01*u23;
                            u12=0.01*u12;
                            nconv=1;
                        }
                        
                        refine_shell_neut.pdbx_refine_id[nbin] =( char *)"NEUTRON DIFFRACTION";               
                        
                        
                        refine_shell_neut.R_factor_R_work[nbin] = u12;               
                        refine_shell_neut.R_factor_R_free[nbin] = u23;               
                        nbin++;
                    }
                }
            }

            if(nbin>0){
                if(nconv)printf("\nNote: R_work/Rfree for shell have been converted to absolute values.\n") ;
                
                refine_shell_neut.nbin=nbin;
                for(m=0; m<nbin; m++){
                    refine_shell_neut.pdbx_total_number_of_bins_used[m]=nbin;
                }
            }
            
        

// stat for highest bin

        }else if (strstr_space(str, "REMARK 3 TOTAL NUMBER OF BINS USED           :")){
            refine_shell_neut.pdbx_total_number_of_bins_used[0] = get_int(str, ":", "\n");
            if(refine_shell_neut.nbin <=0) refine_shell_neut.nbin = 1;
            refine_shell_neut.pdbx_refine_id[0] = (char*)"NEUTRON DIFFRACTION";
            
        }else if (strstr_space(str, "REMARK 3 BIN RESOLUTION RANGE HIGH ") && strchr(str, ':')){  
            refine_shell_neut.d_res_high[0] = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3 BIN RESOLUTION RANGE LOW ") && strchr(str, ':')){
            refine_shell_neut.d_res_low[0] = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  BIN COMPLETENESS (WORKING+TEST) (%) :")){
            refine_shell_neut.percent_reflns_obs[0] = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3 REFLECTIONS IN BIN  (WORKING SET) :")||
                  strstr_space(str, "REMARK 3 REFLECTION IN BIN  (WORKING SET) :")){  //refmac
            refine_shell_neut.number_reflns_R_work[0] = get_int(str, ":", "\n");
               
        }else if (strstr_space(str, "BIN R VALUE        (WORKING + TEST SET) :")){
            refine_shell_neut.R_factor_all[0] = get_float(str, ":", "\n");
            
        }else if (strstr_space(str, "REMARK 3  BIN R VALUE           (WORKING SET) :")){
            refine_shell_neut.R_factor_R_work[0] = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3   BIN FREE R VALUE :")){
            refine_shell_neut.R_factor_R_free[0] = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3   BIN FREE R VALUE TEST SET SIZE (%) :")){ 
            refine_shell_neut.percent_reflns_R_free[0] = get_float(str, ":", "\n");

        }else if (strstr_space(str, "REMARK 3  BIN FREE R VALUE SET COUNT       :")||   //refmac
                  strstr_space(str, "REMARK 3  BIN FREE R VALUE TEST SET COUNT      :")){
            refine_shell_neut.number_reflns_R_free[0] = get_int(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3   ESTIMATED ERROR OF BIN FREE R VALUE :")){
            refine_shell_neut.R_factor_R_free_error[0] = get_float(str, ":", "\n");


//   refine_neutron             
        }else if (strstr_space(str, "REMARK   3  NEUTRON DATA.")){ //CNS/REFMAC5
            refine_neutron.pdbx_refine_id= (char*)"NEUTRON DIFFRACTION";

        }else if (strstr_space(str, "REMARK   3  REFINEMENT TARGET :")){ //CNS/REFMAC5
            refine_neutron.pdbx_stereochemistry_target_values= get_substr(str, ":", "\n");

        }else if(strstr_space(str, "REMARK 3  RESOLUTION RANGE HIGH (ANGSTROMS) :")){
            refine_neutron.ls_d_res_high = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  RESOLUTION RANGE LOW  (ANGSTROMS) :")){
            refine_neutron.ls_d_res_low = get_float(str, ":", "\n");

        }else if (strstr_space(str, "REMARK 3  DATA CUTOFF            (SIGMA(F)) :")||
                  strstr_space(str, "REMARK   3   MIN(FOBS/SIGMA_FOBS)              :")){
            
            if(strstr_space(str, ": NONE")){
                refine_neutron.pdbx_ls_sigma_F = 0.0;
            }else
                refine_neutron.pdbx_ls_sigma_F = get_float(str, ":", "\n");
        


        }else if (strstr_space(str, "REMARK 3 CROSS-VALIDATION METHOD       :") ){
            refine_neutron.pdbx_ls_cross_valid_method = get_substr(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  FREE R VALUE TEST SET SELECTION  :")){
            refine_neutron.pdbx_R_Free_selection_details =get_substr(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  R VALUE     (WORKING + TEST SET) :")){
            if(refine_neutron.pdbx_ls_sigma_F >0.001)
                refine_neutron.ls_R_factor_obs = get_float(str, ":", "\n");
            else
                refine_neutron.ls_R_factor_obs = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3 R VALUE      (WORKING SET) :") ){
            refine_neutron.ls_R_factor_R_work = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  FREE R VALUE         :")){
            refine_neutron.ls_R_factor_R_free = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  FREE R VALUE TEST SET SIZE   (%) :")){
            refine_neutron.ls_percent_reflns_R_free = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  FREE R VALUE TEST SET COUNT      :")){
            refine_neutron.ls_number_reflns_R_free = get_int(str, ":", "\n");
            
        }else if (strstr_space(str, "REMARK 3   ESTIMATED ERROR OF FREE R VALUE :")){
            refine_neutron.ls_R_factor_R_free_error = get_float(str, ":", "\n");



 // BULK SOLVENT MODELLING.(refine_neutron.)
        }else if (strstr_space(str, "REMARK   3   METHOD USED :")){
            refine_neutron.solvent_model_details=get_substr(str, ":", "\n");

        }else if (strstr_space(str, "REMARK   3   K_SOL     :") || strstr_space(str, "REMARK   3   K_SOL:")){
            refine_neutron.solvent_model_param_ksol  = get_float(str, ":", "\n");

        }else if (strstr_space(str, "REMARK   3    B_SOL    :") || strstr_space(str, "REMARK   3    B_SOL:")){
            refine_neutron.solvent_model_param_bsol  = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  VDW PROBE RADIUS   :")
                  || strstr_space(str, "REMARK   3   SOLVENT RADIUS     :")){
            refine_neutron.pdbx_solvent_vdw_probe_radii  = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  ION PROBE RADIUS   :")){
            refine_neutron.pdbx_solvent_ion_probe_radii  = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3   SHRINKAGE RADIUS   :")){
            refine_neutron.pdbx_solvent_shrinkage_radii  = get_float(str, ":", "\n");

        }else if (strstr_space(str, "REMARK   3  ISOTROPIC THERMAL MODEL  :")){
            refine_neutron.pdbx_isotropic_thermal_model= get_substr(str, ":", "\n");


        }else if (strstr_space(str, "REMARK 3 COMPLETENESS FOR RANGE    (%) :") ||
                  strstr_space(str, "REMARK 3 COMPLETENESS (WORKING+TEST) (%) :")){
            refine_neutron.ls_percent_reflns_obs = get_float(str, ":", "\n");
                     
        }else if (strstr_space(str, "REMARK 3 NUMBER OF REFLECTIONS             :")){
            
            refine_neutron.ls_number_reflns_obs = get_int(str, ":", "\n");

        }else if (strstr_space(str, "REMARK 3  B11 (A**2) :") || strstr_space(str, "REMARK   3    B11 :")){
            refine_neutron.aniso_B11 = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  B22 (A**2) :") || strstr_space(str, "REMARK   3    B22 :")){
            refine_neutron.aniso_B22 = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  B33 (A**2) :") || strstr_space(str, "REMARK   3    B33 :")){
            refine_neutron.aniso_B33 = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  B12 (A**2) :") || strstr_space(str, "REMARK   3    B12 :")){
            refine_neutron.aniso_B12 = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  B13 (A**2) :") || strstr_space(str, "REMARK   3    B13 :")){
            refine_neutron.aniso_B13 = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  B23 (A**2) :") || strstr_space(str, "REMARK   3    B23 :")){
            refine_neutron.aniso_B23 = get_float(str, ":", "\n");

                    // ESTIMATED COORDINATE ERROR. (refine_analyze_neut)
        }else if (strstr_space(str, "REMARK 3  ESD FROM LUZZATI PLOT (A) :")){
            refine_analyze_neut.pdbx_refine_id=(char*)"NEUTRON DIFFRACTION";
            refine_analyze_neut.Luzzati_coordinate_error_obs = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  ESD FROM SIGMAA (A) :")){
            refine_analyze_neut.Luzzati_sigma_a_obs = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3   LOW RESOLUTION CUTOFF (A) :")){
            refine_analyze_neut.Luzzati_d_res_low_obs = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  ESD FROM C-V LUZZATI PLOT (A) :")){
            refine_analyze_neut.Luzzati_coordinate_error_free = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  ESD FROM C-V SIGMAA (A) :")){
            refine_analyze_neut.Luzzati_sigma_a_free = get_float(str, ":", "\n");
            
        }
          
    }
}

void write_pdbx_xplor_file(FILE *fout)
{
    int i;

    for(i=0;i<80;i++){
        if(pdbx_xplor_file[i].param_file  || pdbx_xplor_file[i].topol_file){
            if(i==0){   
                fprintf(fout,"%s %s %s %s %s" ,"\nloop_\n",
                        "_pdbx_xplor_file.pdbx_refine_id\n",
                        "_pdbx_xplor_file.serial_no\n",
                        "_pdbx_xplor_file.param_file\n",
                        "_pdbx_xplor_file.topol_file\n"
                        );
                
            }
            fprintf(fout,"'%s' %d \n",REFINE_ID[0], i+1);
            
            if(pdbx_xplor_file[i].param_file) rid_of_file_path_unix(pdbx_xplor_file[i].param_file);
            if(pdbx_xplor_file[i].topol_file) rid_of_file_path_unix(pdbx_xplor_file[i].topol_file);
                         
            (pdbx_xplor_file[i].param_file)?fprintf(fout,"'%s'\n", pdbx_xplor_file[i].param_file):fprintf(fout," ? \n");
            (pdbx_xplor_file[i].topol_file)?fprintf(fout,"'%s'\n", pdbx_xplor_file[i].topol_file):fprintf(fout," ? \n");
            
        }
    } 
}
    
void write_database_PDB_matrix(FILE *fout)
{
    if(database_PDB_matrix.origx11>-9998 && database_PDB_matrix.origx22>-9998 &&
       database_PDB_matrix.origx33>-9998){ 
        fprintf(fout,"#\n");
        fprintf(fout,"_database_PDB_matrix.entry_id  UNNAMED \n" );
        fprintf(fout,"_database_PDB_matrix.origx[1][1] %.6f\n", database_PDB_matrix.origx11);
        fprintf(fout,"_database_PDB_matrix.origx[1][2] %.6f\n", database_PDB_matrix.origx12);
        fprintf(fout,"_database_PDB_matrix.origx[1][3] %.6f\n", database_PDB_matrix.origx13);
        fprintf(fout,"_database_PDB_matrix.origx[2][1] %.6f\n", database_PDB_matrix.origx21);
        fprintf(fout,"_database_PDB_matrix.origx[2][2] %.6f\n", database_PDB_matrix.origx22);
        fprintf(fout,"_database_PDB_matrix.origx[2][3] %.6f\n", database_PDB_matrix.origx23);
        fprintf(fout,"_database_PDB_matrix.origx[3][1] %.6f\n", database_PDB_matrix.origx31);
        fprintf(fout,"_database_PDB_matrix.origx[3][2] %.6f\n", database_PDB_matrix.origx32);
        fprintf(fout,"_database_PDB_matrix.origx[3][3] %.6f\n", database_PDB_matrix.origx33);
    }
    if(database_PDB_matrix.origx_vector1>-9998 && database_PDB_matrix.origx_vector2>-9998){
        fprintf(fout,"_database_PDB_matrix.origx_vector[1] %.6f\n", database_PDB_matrix.origx_vector1);
        fprintf(fout,"_database_PDB_matrix.origx_vector[2] %.6f\n", database_PDB_matrix.origx_vector2);
        fprintf(fout,"_database_PDB_matrix.origx_vector[3] %.6f\n", database_PDB_matrix.origx_vector3); 
    }
    
}

    
void write_atom_sites(FILE *fout)
{
    if(atom_sites.fract_transf_matrix11>-9998 && atom_sites.fract_transf_matrix22>-9998
       && atom_sites.fract_transf_matrix33>-9998){ 
        fprintf(fout,"#\n");
        fprintf(fout,"_atom_sites.entry_id  UNNAMED \n");
        fprintf(fout,"_atom_sites.fract_transf_matrix[1][1] %.6f\n", atom_sites.fract_transf_matrix11);
        fprintf(fout,"_atom_sites.fract_transf_matrix[1][2] %.6f\n", atom_sites.fract_transf_matrix12);
        fprintf(fout,"_atom_sites.fract_transf_matrix[1][3] %.6f\n", atom_sites.fract_transf_matrix13);
        fprintf(fout,"_atom_sites.fract_transf_matrix[2][1] %.6f\n", atom_sites.fract_transf_matrix21);
        fprintf(fout,"_atom_sites.fract_transf_matrix[2][2] %.6f\n", atom_sites.fract_transf_matrix22);
        fprintf(fout,"_atom_sites.fract_transf_matrix[2][3] %.6f\n", atom_sites.fract_transf_matrix23);
        fprintf(fout,"_atom_sites.fract_transf_matrix[3][1] %.6f\n", atom_sites.fract_transf_matrix31);
        fprintf(fout,"_atom_sites.fract_transf_matrix[3][2] %.6f\n", atom_sites.fract_transf_matrix32);
        fprintf(fout,"_atom_sites.fract_transf_matrix[3][3] %.6f\n", atom_sites.fract_transf_matrix33);
    }
    if(atom_sites.fract_transf_vector1>-9998 && atom_sites.fract_transf_vector2>-9998){
        fprintf(fout,"_atom_sites.fract_transf_vector[1] %.6f\n", atom_sites.fract_transf_vector1);
        fprintf(fout,"_atom_sites.fract_transf_vector[2] %.6f\n", atom_sites.fract_transf_vector2);
        fprintf(fout,"_atom_sites.fract_transf_vector[3] %.6f\n", atom_sites.fract_transf_vector3); 
    }
}


void write_twin(FILE *fout)
{
    int i;

    if(NTWIN<=0) return;
    
    if(strlen(pdbx_twin[0].operators)>1 && pdbx_twin[0].fraction>0 ){
        fprintf(fout,"#\n");
        fprintf(fout,"loop_\n");
        fprintf(fout,"\
_pdbx_reflns_twin.domain_id \n\
_pdbx_reflns_twin.crystal_id \n\
_pdbx_reflns_twin.diffrn_id \n\
_pdbx_reflns_twin.fraction \n\
_pdbx_reflns_twin.operator \n\
_pdbx_reflns_twin.type \n\
_pdbx_reflns_twin.mean_F_square_over_mean_F2 \n\
_pdbx_reflns_twin.mean_I2_over_mean_I_square \n");

        for(i=0; i<NTWIN; i++){
            fprintf(fout,"%d  1  1  %.3f '%s'  '%s' ? ?\n",pdbx_twin[i].domain_id,
                    pdbx_twin[i].fraction, pdbx_twin[i].operators, pdbx_twin[i].type);
        }
    }
}


void write_tls(FILE *fout)
{
    int i, ref;
    
    if(NTLS_GROUP>0){
       
        fprintf(fout,"#\n");
        fprintf(fout,"loop_\n");
        fprintf(fout,"_pdbx_refine_tls_group.pdbx_refine_id\n");
        fprintf(fout,"_pdbx_refine_tls_group.id\n");
        fprintf(fout,"_pdbx_refine_tls_group.refine_tls_id\n");
        fprintf(fout,"_pdbx_refine_tls_group.beg_auth_asym_id\n");
        fprintf(fout,"_pdbx_refine_tls_group.beg_auth_seq_id\n");
        fprintf(fout,"_pdbx_refine_tls_group.end_auth_asym_id\n");
        fprintf(fout,"_pdbx_refine_tls_group.end_auth_seq_id\n");
        fprintf(fout,"_pdbx_refine_tls_group.selection_details\n");
        for(i=0; i<NTLS_GROUP; i++){
            fprintf(fout,"'%s' '%s' '%s'  '%s' '%s' '%s'  '%s'  \n;%s\n;\n   \n",
                    REFINE_ID[0],
                    pdbx_refine_tls_group[i].id,
                    pdbx_refine_tls_group[i].refine_tls_id,
                    pdbx_refine_tls_group[i].beg_label_asym_id,
                    pdbx_refine_tls_group[i].beg_label_seq_id,
                    pdbx_refine_tls_group[i].end_label_asym_id,
                    pdbx_refine_tls_group[i].end_label_seq_id,
                    pdbx_refine_tls_group[i].selection);
        }
    }

    if(pdbx_refine_tls[0].T11<-9998 && pdbx_refine_tls[0].L11<-9998 && pdbx_refine_tls[0].S11<-9998) return;
    
    fprintf(fout,"#\n");
    fprintf(fout,"loop_\n");
    fprintf(fout,"_pdbx_refine_tls.pdbx_refine_id\n");
    fprintf(fout,"_pdbx_refine_tls.id\n");
    fprintf(fout,"_pdbx_refine_tls.details\n");
    fprintf(fout,"_pdbx_refine_tls.method\n");
    fprintf(fout,"_pdbx_refine_tls.origin_x\n");
    fprintf(fout,"_pdbx_refine_tls.origin_y\n");
    fprintf(fout,"_pdbx_refine_tls.origin_z\n");
    fprintf(fout,"_pdbx_refine_tls.T[1][1]\n");
    fprintf(fout,"_pdbx_refine_tls.T[2][2]\n");
    fprintf(fout,"_pdbx_refine_tls.T[3][3]\n");
    fprintf(fout,"_pdbx_refine_tls.T[1][2]\n");
    fprintf(fout,"_pdbx_refine_tls.T[1][3]\n");
    fprintf(fout,"_pdbx_refine_tls.T[2][3]\n");
    fprintf(fout,"_pdbx_refine_tls.L[1][1]\n");
    fprintf(fout,"_pdbx_refine_tls.L[2][2]\n");
    fprintf(fout,"_pdbx_refine_tls.L[3][3]\n");
    fprintf(fout,"_pdbx_refine_tls.L[1][2]\n");
    fprintf(fout,"_pdbx_refine_tls.L[1][3]\n");
    fprintf(fout,"_pdbx_refine_tls.L[2][3]\n");
    fprintf(fout,"_pdbx_refine_tls.S[1][1]\n");
    fprintf(fout,"_pdbx_refine_tls.S[2][2]\n");
    fprintf(fout,"_pdbx_refine_tls.S[3][3]\n");
    fprintf(fout,"_pdbx_refine_tls.S[1][2]\n");
    fprintf(fout,"_pdbx_refine_tls.S[1][3]\n");
    fprintf(fout,"_pdbx_refine_tls.S[2][3]\n");
    fprintf(fout,"_pdbx_refine_tls.S[2][1]\n");
    fprintf(fout,"_pdbx_refine_tls.S[3][1]\n");
    fprintf(fout,"_pdbx_refine_tls.S[3][2]\n");
//    fprintf(fout,"_pdbx_refine_tls.S22-S11\n");
//    fprintf(fout,"_pdbx_refine_tls.S11-S33\n");    

    ref=-9998;
    
    for(i=0; i<NTLS; i++){
        fprintf(fout,"'%s' '%s' '%s'  '%s' ",REFINE_ID[0], pdbx_refine_tls[i].id, pdbx_refine_tls[i].details, pdbx_refine_tls[i].method);
        
        (pdbx_refine_tls[i].origin_x >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].origin_x ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].origin_y >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].origin_y ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].origin_z >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].origin_z ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].T11 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].T11 ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].T22 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].T22 ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].T33 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].T33 ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].T12 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].T12 ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].T13 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].T13 ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].T23 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].T23 ) : fprintf(fout,"? ");

        (pdbx_refine_tls[i].L11 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].L11 ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].L22 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].L22 ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].L33 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].L33 ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].L12 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].L12 ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].L13 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].L13 ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].L23 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].L23 ) : fprintf(fout,"? ");

        (pdbx_refine_tls[i].S11 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].S11 ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].S22 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].S22 ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].S33 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].S33 ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].S12 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].S12 ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].S13 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].S13 ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].S23 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].S23 ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].S21 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].S21 ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].S31 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].S31 ) : fprintf(fout,"? ");
        (pdbx_refine_tls[i].S32 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].S32 ) : fprintf(fout,"? ");
//        (pdbx_refine_tls[i].S22_S11 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].S22_S11 ) : fprintf(fout,"? ");
//        (pdbx_refine_tls[i].S11_S33 >ref)? fprintf(fout, " %.4f ", pdbx_refine_tls[i].S11_S33 ) : fprintf(fout,"? ");
                
        fprintf(fout,"\n");

    }
    
}

void write_ncs_new(FILE *fout)
{
    int i, ens=0;
    
    if(NCS>0 && NNCS>0){       
        fprintf(fout,"#\n");
        fprintf(fout,"loop_\n");
        fprintf(fout,"_struct_ncs_dom.pdbx_ens_id\n");
        fprintf(fout,"_struct_ncs_dom.id\n");
        fprintf(fout,"_struct_ncs_dom.details\n");
        
        for(i=0; i<NCS; i++){
            rid_of_front_end_space(struct_ncs_dom[i].pdbx_ens_id);
            rid_of_front_end_space(struct_ncs_dom[i].id);
            rid_of_front_end_space(struct_ncs_dom[i].details);
            fprintf(fout,"'%s' '%s' \n;%s\n;\n",struct_ncs_dom[i].pdbx_ens_id,
                    struct_ncs_dom[i].id, struct_ncs_dom[i].details);
            ens=atoi(struct_ncs_dom[i].pdbx_ens_id);
            
        }
        if(ens>0){
            
            fprintf(fout,"#\n");
            fprintf(fout,"loop_\n");
            fprintf(fout,"_struct_ncs_ens.id\n");
            for(i=0; i<ens; i++){
                fprintf(fout,"%d\n",i+1);
            }
        }
    }
//    printf("infor= %d %d %d %s\n",NCS, NLIM, NNCS, struct_ncs_dom_lim[0].beg_label_asym_id);
    
    if(NLIM > 0){       
        fprintf(fout,"#\n");
        fprintf(fout,"loop_\n");
        fprintf(fout,"_struct_ncs_dom_lim.pdbx_ens_id\n");
        fprintf(fout,"_struct_ncs_dom_lim.dom_id\n");
        fprintf(fout,"_struct_ncs_dom_lim.pdbx_component_id\n");
        fprintf(fout,"_struct_ncs_dom_lim.pdbx_refine_code\n");
        fprintf(fout,"_struct_ncs_dom_lim.beg_auth_asym_id\n");
        fprintf(fout,"_struct_ncs_dom_lim.beg_auth_seq_id\n");
        fprintf(fout,"_struct_ncs_dom_lim.end_auth_asym_id\n");
        fprintf(fout,"_struct_ncs_dom_lim.end_auth_seq_id\n");
        fprintf(fout,"_struct_ncs_dom_lim.selection_details\n");
        
        for(i=0; i<NLIM; i++){
            fprintf(fout,"'%s' '%s'  %d   %s  '%s' %d '%s' %d   \n;%s\n;\n   \n",
                    struct_ncs_dom_lim[i].pdbx_ens_id,
                    struct_ncs_dom_lim[i].dom_id,struct_ncs_dom_lim[i].pdbx_component,
                    struct_ncs_dom_lim[i].pdbx_refine_code,
                    struct_ncs_dom_lim[i].beg_label_asym_id, struct_ncs_dom_lim[i].beg_label_seq_id,
                    struct_ncs_dom_lim[i].end_label_asym_id, struct_ncs_dom_lim[i].end_label_seq_id,
                    struct_ncs_dom_lim[i].selection_details);
        }
    }
    
//    if(NNCS > 0 && refine_ls_restr_ncs[0].pdbx_rms>-9998){
    if(NNCS > 0 ){
        fprintf(fout,"#\n");
        fprintf(fout,"loop_\n");
        fprintf(fout,"_refine_ls_restr_ncs.pdbx_ordinal\n");
        fprintf(fout,"_refine_ls_restr_ncs.pdbx_refine_id\n");
        fprintf(fout,"_refine_ls_restr_ncs.pdbx_ens_id\n");
        fprintf(fout,"_refine_ls_restr_ncs.dom_id\n");
        fprintf(fout,"_refine_ls_restr_ncs.pdbx_type\n");
        fprintf(fout,"_refine_ls_restr_ncs.pdbx_auth_asym_id\n");
        fprintf(fout,"_refine_ls_restr_ncs.pdbx_number\n");
        if (strstr(refine_ls_restr_ncs[0].pdbx_type, "TORSIONAL")){
            fprintf(fout,"_refine_ls_restr_ncs.pdbx_rms\n");
        }else{
            fprintf(fout,"_refine_ls_restr_ncs.rms_dev_position\n");
        }
        
        
        fprintf(fout,"_refine_ls_restr_ncs.weight_position\n");

        
        for(i=0; i<NNCS; i++){
            rid_of_front_end_space(refine_ls_restr_ncs[i].pdbx_ens_id);
            rid_of_front_end_space(refine_ls_restr_ncs[i].dom_id);
            rid_of_front_end_space(refine_ls_restr_ncs[i].pdbx_type);
            rid_of_front_end_space(refine_ls_restr_ncs[i].pdbx_asym_id);
            
                
            fprintf(fout,"%2d '%s' '%s' '%s' \"%s\"  '%s' ",i+1, REFINE_ID[0],
                    refine_ls_restr_ncs[i].pdbx_ens_id, refine_ls_restr_ncs[i].dom_id,
                    refine_ls_restr_ncs[i].pdbx_type, refine_ls_restr_ncs[i].pdbx_asym_id);

            (refine_ls_restr_ncs[i].pdbx_number >-9998)?
                fprintf(fout, " %d ", refine_ls_restr_ncs[i].pdbx_number ) : fprintf(fout,"? ");

            (refine_ls_restr_ncs[i].pdbx_rms>-9998)? fprintf(fout," %.3f ",refine_ls_restr_ncs[i].pdbx_rms) :  fprintf(fout,"? ");
            
            (refine_ls_restr_ncs[i].pdbx_weight >-9998)?
                fprintf(fout, " %.3f ", refine_ls_restr_ncs[i].pdbx_weight) : fprintf(fout,"? ");
            fprintf(fout,"\n");
            
        }
    }
    
    if(NCS > 0 && NNCS<=0 && NLIM<=0){ //CNS/XPLOR/
        fprintf(fout,"#\n");
        fprintf(fout,"loop_\n");
        fprintf(fout,"_refine_ls_restr_ncs.pdbx_ordinal\n");
        fprintf(fout,"_refine_ls_restr_ncs.pdbx_refine_id\n");
        fprintf(fout,"_refine_ls_restr_ncs.dom_id\n");
        fprintf(fout,"_refine_ls_restr_ncs.ncs_model_details\n");
        fprintf(fout,"_refine_ls_restr_ncs.rms_dev_position\n");
        fprintf(fout,"_refine_ls_restr_ncs.weight_position \n");
        fprintf(fout,"_refine_ls_restr_ncs.rms_dev_B_iso\n");
        fprintf(fout,"_refine_ls_restr_ncs.weight_B_iso\n");
        
        for(i=0; i<NCS; i++){
                
            fprintf(fout,"%2d '%s' %d  RESTRAINTS  ",i+1, REFINE_ID[0], i+1);
            (refine_ls_restr_ncs[i].rms_dev_position > -9998)?
                fprintf(fout," %.3f ",refine_ls_restr_ncs[i].rms_dev_position) :  fprintf(fout,"? ");
            
            (refine_ls_restr_ncs[i].weight_position >-9998)?
                fprintf(fout, " %.3f ", refine_ls_restr_ncs[i].weight_position) : fprintf(fout,"? ");
            
            (refine_ls_restr_ncs[i].rms_dev_B_iso >-9998)?
                fprintf(fout, " %.3f ", refine_ls_restr_ncs[i].rms_dev_B_iso) : fprintf(fout,"? ");
            
            (refine_ls_restr_ncs[i].weight_B_iso >-9998)?
                fprintf(fout, " %.3f ", refine_ls_restr_ncs[i].weight_B_iso) : fprintf(fout,"? ");
            
            fprintf(fout,"\n");
            
        }
        fprintf(fout,"#\n");
        fprintf(fout,"loop_\n");
        fprintf(fout,"_struct_ncs_dom.id\n");
        fprintf(fout,"_struct_ncs_dom.details\n");
        for(i=0; i<NCS; i++){
            fprintf(fout," %d  ? \n ", i+1);
        }
        
    }
    
}

void extract_phenix_pdb_head(char **lines, int nline)
//extract all the information from PHENIX 
{
    
    char str[256], tmp[256], tmp1[256];
    int i, j, m, n=0, n1, n2, nr=0, nbin=0, nconv=0, nwork;
    float  k_sol, b_sol, u11, u22, u33, u12, u13, u23, wgt, fom, a, a1,a2;
    
    
    for(i=0; i<nline; i++){
 //version 1.1
        strcpy(str, lines[i]);
        if(!strncmp(str,"ATOM  ", 6)|| !strncmp(str,"HETATM", 6)) break;
        
        if(strstr_space(str, "REMARK    resolution: d_max = ")){
            refine.ls_d_res_low = get_float(str, "=", "A");
            refine_hist.d_res_low = refine.ls_d_res_low;
                
        }else if (strstr_space(str, "REMARK                d_min =")){
            refine.ls_d_res_high = get_float(str, "=", "A");
            refine_hist.d_res_high = refine.ls_d_res_high;

                
        }else if (strstr_space(str, "REMARK    number:     total =  ")){
            refine.ls_number_reflns_obs= get_int(str, "=", "\n");
            
        }else if (pattern5(str, "REMARK                work  =", "(percent from total =", ")", "","")){
            refine.ls_number_reflns_R_work= get_int(str, "=", "(");
            refine.ls_percent_reflns_obs = get_float(str, "=", ")");

        }else if (pattern5(str, "REMARK                test  =", "(percent from total =", ")", "","")){
            refine.ls_number_reflns_R_free= get_int(str, "=", "(");
            refine.ls_percent_reflns_R_free = get_float(str, "=", ")");


            
        }else if (pattern5(str, "REMARK Final: r_work =", "r_free =", "bonds =", "angles =", "")){
            refine.ls_R_factor_R_work = get_float(str, "r_work =", "r_free");
            refine.ls_R_factor_R_free = get_float(str, "r_free = ", "bonds");
                
 // BULK SOLVENT MODELLING.(refine.)
        }else if (strstr_space(str, "REMARK Stage        k_sol   b_sol    u11      u22     u33     u12     u13     u23   u_iso")){
            for(j=i; j<nline; j++){
                strcpy(str, lines[j]);
                if(pattern5(str,"REMARK", ":", "", "", "")){
                    
                    strcpy(tmp,strstr(str, ":")+1);
                    n=sscanf(tmp, "%f%f %f%f %f%f %f%f",
                             &k_sol, &b_sol, & u11, &u22,&u33, &u12,  &u13, &u23);
                    if(n==8){
                        refine.solvent_model_param_ksol = k_sol;
                        refine.solvent_model_param_bsol =  b_sol; 
                        refine.aniso_B11 = u11;
                        refine.aniso_B22 = u22; 
                        refine.aniso_B33 = u33;
                        refine.aniso_B12 = u12;   
                        refine.aniso_B13 = u13;  
                        refine.aniso_B23 = u23;
                    }else{
                        i=j;
                        break;
                    }
                }
            }
        }else if (strstr_space(str,"REMARK Stage         Phase    FOM  Alpha         Beta")){
            
             for(j=i; j<nline; j++){
                strcpy(str, lines[j]);
                if(pattern5(str,"REMARK", ":", "", "", "")){
                    strcpy(tmp,strstr(str, ":")+1);
                    
                    n=sscanf(tmp, "%*s%f", &fom);
                    if(n==1){
                        refine.overall_FOM_work_R_set = fom;
                    }else{
                        i=j;
                        break;
                    }
                }
            }
                
        }else if (strstr_space(str,"REMARK                 max     min    mean       Target  Weight")){
            
             for(j=i; j<nline; j++){
                strcpy(str, lines[j]);
                if(pattern5(str,"REMARK", ":", "", "", "")){
                    
                    n=sscanf(strstr(str, ":")+1, "%f %f %f",&u11, &u22, &u33);
                    if(n==3){
                        refine.B_iso_max = u11;
                        refine.B_iso_min = u22;
                        refine.B_iso_mean = u33;
                    }else{
                        i=j;
                        break;
                    }
                }
            }

//new version    (1.22a)        
        }else if(strstr_space(str, "REMARK    resolution d_max            : ")){
            refine.ls_d_res_low = get_float(str, ":", "A");
            refine_hist.d_res_low = refine.ls_d_res_low;
                
        }else if (strstr_space(str, "REMARK    resolution d_min            : ")){
            refine.ls_d_res_high = get_float(str, ":", "A");
            refine_hist.d_res_high = refine.ls_d_res_high;

        }else if (strstr_space(str, "REMARK    number of reflections total : ")){
            refine.ls_number_reflns_obs= get_int(str, ":", "\n");
            
        }else if (pattern5(str, "REMARK    number of reflections total :", "(percent from total =", ")", "","")){
            refine.ls_number_reflns_R_work= get_int(str, ":", "(");
            refine.ls_percent_reflns_obs = get_float(str, "=", ")");

        }else if (pattern5(str, "REMARK    number of reflections test  :", "(percent from total =", ")", "","")){
            refine.ls_number_reflns_R_free= get_int(str, ":", "(");
            refine.ls_percent_reflns_R_free = get_float(str, "=", ")");
            
        }else if (pattern5(str, "REMARK    completeness", ":", "(in range:", ")","")){
            a = get_float(str, ":", "(");
            if(a<1 && a>0){
                a=100*a;
                sprintf(INFO, "Warning! The completeness %.3f is multiplyed by 100 in _refine.ls_percent_reflns_obs.\n", a);
                pinfo(INFO,0);
            }
            refine.ls_percent_reflns_obs = a;
            
        }else if(strstr_space(str, "REMARK Refinement target            :")){
            refine.pdbx_stereochemistry_target_values=get_substr(str, ":", "\n");
            
            
        }else if (strstr_space(str, "REMARK Number of geometry restraints:")){
            refine.ls_number_constraints = get_int(str, ":", "\n");
            
        }else if (strstr_space(str, "REMARK Wilson B-factor              : ")||
                  strstr_space(str, "REMARK   3   WILSON B :")){
            reflns.B_iso_Wilson_estimate =  get_float(str, ":", "\n");       
        }else if (strstr_space(str, "Wilson B =")){
            reflns.B_iso_Wilson_estimate =  get_float_after_id(str, "=");       

//new version    (1.24a)

        }else if (pattern5(str, "REMARK    number of reflections work  :", "(percent from total =", ")", "","")){
            refine.ls_number_reflns_R_work= get_int(str, ":", "(");

        }else if (pattern5(str, "REMARK    completeness, overall ", "(", ")", ":","")){
            a = get_float_after_id(str, ":");
            if(a<1 && a>0){
                printf("\nNote! The completeness %.3f is multiplyed by 100 to make a percentage\n", a);
                printf("If you disagree with this, please change the value at _refine.ls_percent_reflns_obs\n");
                a=100*a;
            }
            refine.ls_percent_reflns_obs=a;
            
        }else if (strstr_space(str, "REMARK Number of geometry restraints:")){
            refine.ls_number_constraints = get_int(str, ":", "\n");

        }else if (strstr_space(str, "REMARK  stage     r-work r-free  xray_target_w  xray_target_t")){
            for(j=i+1; j<nline-3; j++){
                if(pattern5(lines[j], "REMARK", "_", ":", ".",".")){
                    n=sscanf(lines[j], "%s %s %f%f", tmp, tmp1, &a1, &a2);
                    if(n==4){
                        refine.ls_R_factor_R_work = a1;
                        refine.ls_R_factor_R_free = a2;
                    }
                }else{
                    if(j-i>2){
                        i=j;
                        break;
                    }
                }
            }
            
        }else if (strstr_space(str, "REMARK  stage     k_sol   b_sol     b11     b22     b33     b12     b13     b23")){
            for(j=i+1; j<nline-3; j++){
                if(pattern5(lines[j], "REMARK", "_", ":", ".",".")){
                    strcpy(tmp,strstr(lines[j], ":")+1);
                    n=sscanf(tmp, "%f%f %f%f %f%f %f%f",
                             &k_sol, &b_sol, & u11, &u22,&u33, &u12,  &u13, &u23);
                    if(n==8){
                        refine.solvent_model_param_ksol = k_sol;
                        refine.solvent_model_param_bsol =  b_sol; 
                        refine.aniso_B11 = u11;
                        refine.aniso_B22 = u22; 
                        refine.aniso_B33 = u33;
                        refine.aniso_B12 = u12;   
                        refine.aniso_B13 = u13;  
                        refine.aniso_B23 = u23;
                    }
                }else{
                    if(j-i>2){
                        i=j;
                        break;
                    }
                }
            }
        }else if (strstr_space(str, "REMARK  stage     <pher>    fom   alpha")){
            for(j=i+1; j<nline-3; j++){
                if(pattern5(lines[j], "REMARK", "_", ":", ".",".")){
                    strcpy(tmp,strstr(lines[j], ":")+1);
                    n=sscanf(tmp, "%*s%f", &fom);
                    if(n==1){
                        refine.overall_FOM_work_R_set = fom;
                    }
                }else{
                    if(j-i>2){
                        i=j;
                        break;
                    }
                }
            }
            
        }else if (strstr_space(str, "angl   bond   chir   dihe   plan   repu  geom_target ")  ){      
            nr=0;
            for(j=0; j<10; j++){
                refine_ls_restr[j].number = INI;
                refine_ls_restr[j].weight  =INI;
            }
            
            for(j=i+1; j<nline-3; j++){
                if(pattern5(lines[j], "REMARK", "_", ":", ".",".")){
                    strcpy(tmp,strstr(lines[j], ":")+1);
                    
                    n=sscanf(tmp, "%f%f %f%f %f %f %*s %f",  &u11, &u22, &u33, &u12,  &u13,  &u23, &wgt);
                    if(n==7){
                        strcpy(refine_ls_restr[nr].type, "f_angle_d");
                        refine_ls_restr[nr].dev_ideal = u11;
                        refine_ls_restr[nr].weight  =wgt;
                        nr++;
                        NR=nr;
                        strcpy(refine_ls_restr[nr].type, "f_bond_d");
                        refine_ls_restr[nr].dev_ideal = u22;
                        refine_ls_restr[nr].weight  =wgt;
                        nr++;
                        NR=nr;
                        strcpy(refine_ls_restr[nr].type, "f_chiral_restr");
                        refine_ls_restr[nr].dev_ideal = u33;
                        refine_ls_restr[nr].weight  =wgt;
                        nr++;
                        NR=nr;
                        strcpy(refine_ls_restr[nr].type, "f_dihedral_angle_d");
                        refine_ls_restr[nr].dev_ideal = u12;
                        refine_ls_restr[nr].weight  =wgt;
                        nr++;
                        NR=nr;

                        strcpy(refine_ls_restr[nr].type, "f_plane_restr");
                        refine_ls_restr[nr].weight  =wgt;
                        refine_ls_restr[nr].dev_ideal = u13;
                        nr++;
                        NR=nr;

                        strcpy(refine_ls_restr[nr].type, "f_nbd_refined");
                        refine_ls_restr[nr].dev_ideal = u23;
                        refine_ls_restr[nr].weight  =wgt;
                        nr++;
                        NR=nr;
                        break;
                    }
                }else{
                    if(j-i>2){
                        i=j;
                        break;
                    }
                }
            }
        }else if (strstr_space(str, "REMARK   3  DEVIATIONS FROM IDEAL VALUES.")){//rc6
            nr=0;
            for(j=0; j<10; j++){
                refine_ls_restr[j].number = INI;
                refine_ls_restr[j].weight  =INI;
            }
            
            for(j=i; j<nline; j++){
                if(strstr(lines[j], ":")) strcpy(tmp,strstr(lines[j], ":")+1);
                n=sscanf(tmp, "%f%f%d",  &u11, &u22, &n1);
                if(nr>0 && n !=3 ){
                    i=j;
                    NR=nr;
                    break;
                }
                
                if(n==3 && strstr(lines[j], "REMARK   3   BOND      : ")){
                    get_restr_type(nr, "f_bond_d", n1, u11, -1);
                    nr++;
                }else if(n==3 && strstr(lines[j], "REMARK   3   ANGLE     :")){
                    get_restr_type(nr, "f_angle_d", n1, u11, -1);
                    nr++;
                }else if(n==3 && strstr(lines[j], "REMARK   3   CHIRALITY :")){
                    get_restr_type(nr, "f_chiral_restr", n1, u11, -1);
                    nr++;
                }else if(n==3 && strstr(lines[j], "REMARK   3   PLANARITY :")){
                    get_restr_type(nr, "f_plane_restr", n1, u11, -1);
                    nr++;
                }else if(n==3 && strstr(lines[j], "REMARK   3   DIHEDRAL  :")){
                    get_restr_type(nr, "f_dihedral_angle_d", n1, u11, -1);
                    nr++;
                }
            }
        }else if (strstr_space(str, "REMARK   3                    ISO.  ANISO.      MIN     MAX    MEAN")){//rc6
            if(strstr(lines[i+1], "REMARK   3    ALL         :")){
                n=sscanf(lines[i+1]+20, "%*s%*s%*s%f%f%f",  &u11, &u22, &u33);
                if(n==3){
                    refine.B_iso_max = u11;
                    refine.B_iso_min = u22;
                    refine.B_iso_mean = u33;
                }
            }

        }else if (strstr_space(str, "REMARK  stage      b_max   b_min   b_ave")){      
            
            for(j=i; j<nline; j++){
                if(strstr(lines[j+1], "REMARK -----")){
                    strcpy(tmp,strstr(lines[j], ":")+1);
                    n=sscanf(tmp, "%f%f %f",  &u11, &u22, &u33);
                    if(n==3){
                        refine.B_iso_max = u11;
                        refine.B_iso_min = u22;
                        refine.B_iso_mean = u33;
                        i=j;
                        break;
                    }else{
                        i=j;
                        break;
                    }
                   
                }
            }
            
        }else if (strstr_space(str,"REMARK    Bin     Resolution    Compl.  No.   Scale_k1(work) R-factor(work)")){//bin
            nbin=0;
            for(j=i+1; j<nline; j++){
                if(pattern5(lines[j], "REMARK", ":", " - ", ".","")){
                    n=sscanf(strstr(lines[j], ":")+1, "%f %*s %f %f %d %*s %f",
                             &u11,&u22,&u33, &nr, &u12 );
                    if((nbin>0 && n!=5 ) || strstr(lines[j], " where:")) break;
                    if(nbin>=NBIN-1){
                        sprintf(INFO, "Warning! The too many resolution bins\n");
                        pinfo(INFO,0);
                        
                        nbin=0;
                        break;
                    }
                    
                    if(n==5){
                        refine_ls_shell.d_res_high[nbin] =u22;
                        refine_ls_shell.d_res_low[nbin] = u11;
                        if(u33<=1.0)u33=100*u33;
                        refine_ls_shell.percent_reflns_obs[nbin] = u33;
                        refine_ls_shell.R_factor_R_work[nbin] = u12;               
                        refine_ls_shell.number_reflns_R_work[nbin] =nr;
                        refine_ls_shell.R_factor_R_work[nbin] = u12;               
                        nbin++;
                    }
                }
            }
            if(nbin>0){
                refine_ls_shell.nbin=nbin;
                for(m=0; m<nbin; m++){
                    refine_ls_shell.pdbx_total_number_of_bins_used[m]=nbin;
                }
            }
        }else if (strstr_space(str,"REMARK   3   BIN  RESOLUTION RANGE  COMPL.   NWORK NFREE  RWORK RFREE")){//v-1.3b-rc3 ..rc6
            nbin=0;
            nconv=0;
            nwork=0;
            for(j=i+1; j<nline; j++){
                if(pattern5(lines[j], "REMARK   3", ".", " - ", ".","")){
                    strcpy(tmp, lines[j]);
                    if(strstr_case(lines[j], "None"))strcpy(tmp,replace_all_str(lines[j], "None", "0.0"));
                    

                        
                    n=sscanf(tmp+10, " %*s %f %*s %f %f %d %d %f %f",
                             &u11,&u22,&u33, &n1, &n2, &u12, &u23 );
                    if((nbin>0 && n!=7 ) || strstr(lines[j], "REMARK   3  BULK SOLVENT")) break;
                    if(nbin>=NBIN-1){
                        sprintf(INFO,"Warning! Too many resolution bins (%d). BIN_MAX=%d\n", nbin,NBIN);
                        pinfo(INFO,0);
                        nbin=0;
                        break;
                    }
                    
                    if(n==7){
                        refine_ls_shell.d_res_low[nbin] = u11;
                        refine_ls_shell.d_res_high[nbin] =u22;
                        if(u33<=1.0)u33=100*u33;
                        refine_ls_shell.percent_reflns_obs[nbin] = u33;
                        
                        refine_ls_shell.number_reflns_R_work[nbin] =n1;
                        nwork=nwork+n1;
                        refine_ls_shell.number_reflns_R_free[nbin] =n2;
                        if(u12>2.) u12=0.01*u12;
                        if(u23>2.) {
                            u23=0.01*u23;
                            nconv=1;
                        }
                        
                        
                        refine_ls_shell.pdbx_refine_id[nbin] = (char*)"X-RAY DIFFRACTION";
                        refine_ls_shell.R_factor_R_work[nbin] = u12;               
                        refine_ls_shell.R_factor_R_free[nbin] = u23;               
                        nbin++;
                    }
                }
            }
            if(nwork>0)refine.ls_number_reflns_R_work=nwork;
            
            if(nbin>0){
                if(nconv)printf("\nNote: R_work/Rfree for shell have been converted to absolute values.\n") ;
                
                refine_ls_shell.nbin=nbin;
                for(m=0; m<nbin; m++){
                    refine_ls_shell.pdbx_total_number_of_bins_used[m]=nbin;
                }
            }
            

        }else if (strstr_space(str,"REMARK   3   NUMBER OF TLS GROUPS:") ||
                  strstr_space(str,"REMARK   3   TLS GROUP :")){//start pattern
            extract_tls(i, nline, lines, &i);
/*           
        }else if (strstr_space(str,"REMARK   3    SELECTION: name")){
            strcpy(mad_set[nan].pdbx_atom_type, get_substr(str,"name", "\n"));
            if (nan++ >100) {
                printf("Warning: Too many anomlous atoms (>100)\n");
                nan=0;
            }
        }else if (strstr_space(str,"REMARK   3    fp  :")){
            mad_set[nan].f_prime=get_float(str, " :", "\n");
        }else if (strstr_space(str,"REMARK   3    fdp :")){
            mad_set[nan].pdbx_atom_type=get_float(str, " :", "\n");
*/        
        }
            
    }
}


void residue_range_refmac(int ntmp, int j, char **lines,
                          int *res_beg, int *res_end, int *ncomp, char **ch1)
{
    char tmp[10],tmp1[10];
    int i, k, n, n1=0, n2=0;

    res_beg[0]=0;
    res_end[0]=0;
    *ncomp=1;

    n=0;
    for(k=j; k<ntmp; k++){
        if(strstr_space(lines[k], "RESIDUE RANGE :  -1  -1")){

            for(i=0; i<resinfo.nch; i++){
                res_beg[i]=resinfo.resbeg[i];
                res_end[i]=resinfo.resend[i];
                strcpy(ch1[i],resinfo.resid[i]);
            }
            *ncomp=resinfo.nch;
            break;
        }else if(strstr_space(lines[k], "RESIDUE RANGE :") && 
                 !strstr_space(lines[k], " -1  -1")){
            
            if(sscanf(lines[k]+29,"%s %d %s %d", tmp,&n1,tmp1,&n2)==4 ||
               sscanf(lines[k]+29,"%s %d %d", tmp,&n1,&n2)==3  ) {
                if (strcmp(tmp, tmp1)){
                    sprintf(INFO,"Error: different chain IDs at line \n%s",lines[k]);
                    pinfo(INFO, 0);
                }
                
                res_beg[n]=n1;
                res_end[n]=n2;
                strcpy(ch1[n],tmp);
                n++;
                *ncomp=n;
                
            }else{
                sprintf(INFO,"Warning: can not parse residue range for %s",lines[k]);
                pinfo(INFO, 0);
                
            }
        }
    }
}

void extract_tls(int start, int nline, char **lines, int *end)
//extracting TLS and return the end index of the TLS.
{
    char *select=NULL,**ch1,str[6024],tmp[6024],ch[60] ;
    int i,j, m,ntls=0,nend=0, group=0, nref=0, key=0, ntmp=0;
    int ncomp=0,res_beg[200],res_end[200];

    ch1=set_memory_2d(200, 5); //number of chains
    for(i=0; i<200; i++) strcpy(ch1[i],"?");
                                    
    *end = start+1;

    for(i=start; i<nline; i++){
        if(!strncmp(lines[i],"ATOM",4)) break;
        
        if(strstr_space(lines[i], "REMARK   3   TLS GROUP :")){
            nref=get_int_after_id(lines[i],":"); //ntls for refine
            
            key=0;
            if(i<nline-1 && strstr_space(lines[i+1], "REMARK   3    SELECTION:")){
                key=1; //phenix
            }else if(i<nline-1 && strstr_space(lines[i+1], "COMPONENTS GROUP")){
                key=2; //refmac
            }else if(i<nline-1 && strstr_space(lines[i+1], "REMARK   3    SET : ")){
                key=3; //Buster-TNT
            }

            nend=get_string_line_patt(i, nline, lines," S31:", " S33:", "");//get one group
            if(nend<i+10) continue;
            *end= nend+1;
            
            for(j=i; j<=nend; j++){
                ncomp=0;
                if(key==1){ //phenix
                    if(strstr_space(lines[j],"REMARK   3    SELECTION:")){
                        select=get_string_patt(j, nline, lines,"SELECTION:","REMARK   3    ORIGIN", &j);
                        if(strstr(select,"REMARK 3 :")){
                            select=replace_all_str(select,"REMARK 3 :", " ");
                        }else if(strstr(select,"REMARK 3 ")){
                            select=replace_all_str(select,"REMARK 3 ", " ");
                        }
                        
                        strcpy(tmp, select);
                        strcpy(pdbx_refine_tls[ntls].details, "?");  
                        residue_range_phenix(tmp, res_beg, res_end, &ncomp, ch1);
//                        printf ("tls=%s\n", tmp);
                        
                    }
                }else if(key==2){ //refmac
                    if(strstr(lines[j]," NUMBER OF COMPONENTS GROUP")){
                        ntmp=get_string_line_patt(j, nline, lines,"ORIGIN FOR THE GROUP", "", "");
                        if(ntmp==0) {
                            sprintf(INFO, "Error! No components found (%d)\n", ntmp);
                            pinfo(INFO,0);
                        }
                        
                        residue_range_refmac(ntmp, j, lines,res_beg, res_end, &ncomp, ch1);
                        j=ntmp;
                    }
                }else if(key==3){ //buster
                    if(strstr_space(lines[j],"REMARK   3    SET :")){
                        select=get_string_patt(j, nline, lines,"SET :","REMARK   3    ORIGIN", &j);
                        if(strstr(select,"REMARK 3 :") || strstr(select,"REMARK 3")){
                            select=replace_all_str(select,"REMARK 3 :", " ");
                            select=replace_all_str(select,"REMARK 3", " ");
                        }
                        strcpy(tmp, select);
                        
                        strcpy(pdbx_refine_tls[ntls].details, "?");  
                        residue_range_tnt(tmp, res_beg, res_end, &ncomp, ch1);
                        
                    }
                    
                }
                
                if(strstr_space(lines[j], " ORIGIN FOR THE GROUP (A):")){
                   
                    for(m=0; m<ncomp; m++){
                        strcpy(ch, ch1[m]);
                        if(key==1 || key==3)
                            sprintf(pdbx_refine_tls_group[group].selection,"%s", select );
                        else
                            strcpy(pdbx_refine_tls_group[group].selection,"?");
                        
                        sprintf(pdbx_refine_tls_group[group].id, "%d",group+1 );
                        sprintf(pdbx_refine_tls_group[group].refine_tls_id , "%d", nref ) ;
                        strcpy(pdbx_refine_tls_group[group].beg_label_asym_id, ch);
                        sprintf(pdbx_refine_tls_group[group].beg_label_seq_id, "%d", res_beg[m]);
                        strcpy(pdbx_refine_tls_group[group].end_label_asym_id,ch);
                        sprintf(pdbx_refine_tls_group[group].end_label_seq_id,"%d", res_end[m] );
                        group++;
                    }
                    if(select) delete []select;

                    strcpy(str, strchr(lines[j], ':')+1);
                    if(sscanf(str, "%f%f%f",
                              &pdbx_refine_tls[ntls].origin_x, 
                              &pdbx_refine_tls[ntls].origin_y, 
                              &pdbx_refine_tls[ntls].origin_z) !=3 )
                        continue;
                    
                   
                }else if(pattern5(lines[j], " T11:", "T22", "", "","")){
                    pdbx_refine_tls[ntls].T11=get_float(lines[j], " T11:", "T22");
                    pdbx_refine_tls[ntls].T22=get_float(lines[j], " T22:", "\n");

                }else if(pattern5(lines[j], " T33:", "T12", "", "","")){
                    pdbx_refine_tls[ntls].T33=get_float(lines[j], " T33:", "T12");
                    pdbx_refine_tls[ntls].T12=get_float(lines[j], " T12:", "\n");
                        
                }else if(pattern5(lines[j], " T13:", "T23", "", "","")){
                    pdbx_refine_tls[ntls].T13=get_float(lines[j], " T13:", "T23");
                    pdbx_refine_tls[ntls].T23=get_float(lines[j], " T23:", "\n");
                            
                }else if(pattern5(lines[j], " L11:", "L22", "", "","")){
                    pdbx_refine_tls[ntls].L11=get_float(lines[j], " L11:", "L22");
                    pdbx_refine_tls[ntls].L22=get_float(lines[j], " L22:", "\n");
                        
                }else if(pattern5(lines[j], " L33:", "L12", "", "","")){
                    pdbx_refine_tls[ntls].L33=get_float(lines[j], " L33:", "L12");
                    pdbx_refine_tls[ntls].L12=get_float(lines[j], " L12:", "\n");
                        
                }else if(pattern5(lines[j], " L13:", "L23", "", "","")){
                    pdbx_refine_tls[ntls].L13=get_float(lines[j], " L13:", "L23");
                    pdbx_refine_tls[ntls].L23=get_float(lines[j], " L23:", "\n");
                            
                }else if(pattern5(lines[j], " S11:", "S12:", "S13:", "","")){
                    pdbx_refine_tls[ntls].S11=get_float(lines[j], " S11:", "S12");
                    pdbx_refine_tls[ntls].S12=get_float(lines[j], " S12:", "S13");
                    pdbx_refine_tls[ntls].S13=get_float(lines[j], " S13:", "\n");
                        
                }else if(pattern5(lines[j], " S21:", "S22:", "S23:", "","")){
                    pdbx_refine_tls[ntls].S21=get_float(lines[j], " S21:", "S22");
                    pdbx_refine_tls[ntls].S22=get_float(lines[j], " S22:", "S23");
                    pdbx_refine_tls[ntls].S23=get_float(lines[j], " S23:", "\n");
                    
                } else  if(pattern5(lines[j], " S31:", "S32:", "S33:", "","")){ 
//must be the last one                        
                    pdbx_refine_tls[ntls].S31=get_float(lines[j], " S31:", "S32");
                    pdbx_refine_tls[ntls].S32=get_float(lines[j], " S32:", "S33");
                    pdbx_refine_tls[ntls].S33=get_float(lines[j], " S33:", "\n");
                    
                    ntls++;
                    
                    sprintf(pdbx_refine_tls[ntls-1].id, "%d", ntls);                    
                    strcpy(pdbx_refine_tls[ntls-1].method, "refined");
                    pdbx_refine_tls[ntls-1].S22_S11 = pdbx_refine_tls[ntls-1].S22 - pdbx_refine_tls[ntls-1].S11;
                    pdbx_refine_tls[ntls-1].S11_S33 = pdbx_refine_tls[ntls-1].S11 - pdbx_refine_tls[ntls-1].S33;

                    i=nend;
                    break;
                }
            }
        }
    }
    
    NTLS=ntls;
    NTLS_GROUP=group;
//    printf("TLS extraction: NTLS_GROUP=%d: NTLS_COMP=%d \n",ntls, group);
}

void chain_seq_info(int nline, char **lines )
//write chain ID and residue number
{
    char ch[5], segid[5];
    int i,j, seq1=0, seq2=0, k=0;
    
    
    for(i=0; i<200; i++){ 
        resinfo.resbeg[i]=0;
        resinfo.resend[i]=0;
        strcpy(resinfo.resid[i], "?");
        strcpy(resinfo.segid[i], "");
    }
    
    for(i=0; i<nline; i++){ //get chain, seq1, seq2, seqid
        if (!strncmp(lines[i],"ENDMDL", 6)) break;
        
        if(!strncmp(lines[i],"ATOM", 4) || !strncmp(lines[i],"HETATM", 6)){
            strncpy(ch, lines[i]+20, 2);
            ch[2]='\0';
            
            sscanf(lines[i]+22, "%d", &seq1);
            
            if(strlen(lines[i])>74 && lines[i][72] !=' ') {
                strncpy(segid, lines[i]+72,4);
                segid[4]='\0';
                rid_of_front_end_space(segid);
            }
            
            
            for(j=i; j<nline-1; j++){
                if(strncmp(lines[j+1]+20,ch,2) || !strncmp(lines[j+1], "TER",3)){
                    
                    sscanf(lines[j]+22, "%d", &seq2);
                    resinfo.resbeg[k]=seq1;
                    resinfo.resend[k]=seq2;
                    rid_of_front_end_space(ch);
                    strcpy(resinfo.resid[k], ch);
                    strcpy(resinfo.segid[k], segid);
                    if(k>199) break;
                    
                    k++;
                    i=j;
                    break;
                }else if(j==nline-2 && !strncmp(lines[j+1]+20, ch,2)){
                    sscanf(lines[j+1]+22, "%d", &seq2);
                   
                    resinfo.resbeg[k]=seq1;
                    resinfo.resend[k]=seq2;
                    rid_of_front_end_space(ch);
                    strcpy(resinfo.resid[k], ch);
                    strcpy(resinfo.segid[k], segid);
                    k++;
                    i=j;
                    break;
                }     
            } 
        }
    }
    
    resinfo.nch=k;
    
//   for (i=0; i<k; i++){ printf("resinfo: %s %d %d:%d\n", resinfo.resid[i], i, resinfo.resbeg[i], resinfo.resend[i]);}
}

void get_restr_type(int nr, const char* type, int count, float dev,  float wt)
{
    if(count>0)refine_ls_restr[nr].number  = count;
    strcpy(refine_ls_restr[nr].type,type );
    if(dev>0)refine_ls_restr[nr].dev_ideal = dev;
    if(wt>0)refine_ls_restr[nr].weight  = wt;
}
   
void date_conversion(char *temp)
//convert date e.g.  from 15-SEP-09 (in PDB) to 2009-09-15 (in cif)  format!
{
    char tmp[80], **tok, year[20], mon[20], dat[20];
    const char *datec[]={"JAN", "FEB","MAR","APR","MAY","JUN","JUL","AUG","SEP","OCT","NOV","DEC"};
    const char *daten[]={"01", "02","03","04","05","06","07","08","09","10","11","12"};
    int i,ntok=0;
    
    strcpy(year,""); strcpy(mon,""); strcpy(dat,"");
    strcpy(tmp, temp);
    tok=string_token(tmp,"-", &ntok);
    if(ntok !=3 || strlen(tok[0]) !=2 || strlen(tok[1]) !=3 || strlen(tok[2]) !=2 ) return;
    for (i=0;i<12; i++){
        if(!strcmp(tok[1], datec[i])) {
            strcpy(mon,daten[i]);
            break;
        }
    }
    strcpy(dat,tok[0]);  
    if (tok[2][0]=='0') {
        sprintf(year, "200%c" , tok[2][1]);
    }else if (atoi(tok[2])>70){
        sprintf(year, "19%s" , tok[2]);
    }else{
        sprintf(year, "20%s" , tok[2]);
    }
    
    if (strlen(mon)==2 && strlen(dat)==2 && strlen(year)==4){
        sprintf(temp, "%s-%s-%s" , year, mon,dat);
    }
}


void get_remarks(const char *id, int nrange[], char **line)
{
    char temp[80], detail[516];
    int  i;
//    int exptl_crystals_number=INI;
    float a1;
    
    strcpy(detail,"");
    strcpy(temp,"");
    if (strstr(id,"280")){
        for(i=nrange[0]; i<nrange[1]; i++){
            if (strstr_space(line[i], ":NULL") || strstr_space(line[i], ": NULL") ) continue;
            
            if(strstr_space(line[i],"SOLVENT CONTENT, VS   (%):"))
                exptl_crystal.density_percent_sol=get_float_after_id(line[i],":");
            
            else if(strstr_space(line[i],"VM (ANGSTROMS**3/DA):"))
                exptl_crystal.density_Matthews=get_float_after_id(line[i],":");
            
            else if(strstr_space(line[i],"CRYSTALLIZATION CONDITIONS:")){
                get_data_set_id(line[i],"S:", "\n", detail);
                strcpy(temp,"");
                if(i<nrange[1]-1){
                    get_data_set_id(line[i+1],"REMARK 280", "\n", temp);
                }
                if(strlen(temp)>0) strcat(detail,temp);
                clean_string(detail,detail);
                
                strcpy(exptl_crystal_grow.pdbx_details, detail);
                exptl_crystal_grow.crystal_id= (char*)"1";
            }
        }
         
    }else if(strstr(id,"200")){ 
        diffrn_source.diffrn_id=(char*)"1";
        diffrn_detector.diffrn_id=(char*)"1";
        diffrn_radiation.diffrn_id=(char*)"1";
        diffrn_radiation.wavelength_id=(char*)"1";
        diffrn_radiation.pdbx_scattering_type=(char*)"x-ray";
        reflns.pdbx_diffrn_id=(char*)"1";
        reflns.pdbx_ordinal=(char*)"1";
        diffrn.id=(char*)"1";
        exptl_crystal_grow.method=(char*)"?";
        exptl_crystal_grow.temp=-999;

        for(i=nrange[0]; i<nrange[1]; i++){
            if (strstr_space(line[i], ":NULL") || strstr_space(line[i], ": NULL") ) continue;
        
            if(strstr_space(line[i], "EXPERIMENT TYPE                :")){ //exptl.
                strcpy(exptl.method, get_substr(line[i],":", "\n"));
                
            }else if(strstr_space(line[i], "DATE OF DATA COLLECTION        :")){
                strcpy(temp,  get_substr(line[i],":", "\n"));
                date_conversion(temp);
                strcpy(diffrn_detector.pdbx_collection_date , temp);

            }else if(strstr_space(line[i], "TEMPERATURE           (KELVIN) :")){
                a1 = get_float(line[i],":", "\n");
                if (a1>-99) diffrn.ambient_temp=a1;
                
            }else if(strstr_space(line[i], "PH                             :")){
                exptl_crystal_grow.pH = get_float(line[i],":", "\n");
                
//            }else if(strstr_space(line[i], "NUMBER OF CRYSTALS USED        :")){
//                exptl_crystals_number= get_int(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "SYNCHROTRON              (Y/N) :")){
                diffrn_source.pdbx_synchrotron_y_n=get_substr(line[i],":", "\n");
                if (strchr(diffrn_source.pdbx_synchrotron_y_n, 'Y')) {
                    diffrn_source.source= (char *) "SYNCHROTRON";
                }
                
            }else if(strstr_space(line[i], "RADIATION SOURCE               :")){
                diffrn_source.source= get_substr(line[i],":", "\n");
                
                
            }else if(strstr_space(line[i], "BEAMLINE                       :")){
                diffrn_source.pdbx_synchrotron_beamline= get_substr(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "X-RAY GENERATOR MODEL          :")){
                diffrn_source.type = get_substr(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "MONOCHROMATIC OR LAUE    (M/L) :")){
                diffrn_radiation.pdbx_monochromatic_or_laue_m_l=get_substr(line[i],":", "\n");
                    
            }else if(strstr_space(line[i], "WAVELENGTH OR RANGE        (A) :")){
                diffrn_source.pdbx_wavelength=get_substr(line[i],":", "\n");
                diffrn_source.pdbx_wavelength_list=get_substr(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "MONOCHROMATOR                  :")){
                diffrn_radiation.monochromator=get_substr(line[i],":", "\n");

                    
            }else if(strstr_space(line[i], "OPTICS                         :")){
                diffrn_detector.details = get_substr(line[i],":", "\n");

            }else if(strstr_space(line[i], "DETECTOR TYPE                  :")){
                diffrn_detector.detector = get_substr(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "DETECTOR MANUFACTURER          :")){
                diffrn_detector.type = get_substr(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "INTENSITY-INTEGRATION SOFTWARE :")){
                strcpy(computing.data_collection, get_substr(line[i],":", "\n"));
                
            }else if(strstr_space(line[i], "DATA SCALING SOFTWARE          :")){
                strcpy(computing.data_reduction, get_substr(line[i],":", "\n"));

            }else if(strstr_space(line[i], "NUMBER OF UNIQUE REFLECTIONS   :")){
                reflns.number_obs= get_int(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "RESOLUTION RANGE HIGH      (A) :")){
                reflns.d_resolution_high= get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "RESOLUTION RANGE LOW       (A) :")){
                reflns.d_resolution_low= get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "REJECTION CRITERIA  (SIGMA(I)) :")){
                reflns.observed_criterion_sigma_I = get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "COMPLETENESS FOR RANGE     (%) :")){
                reflns.percent_possible_obs = get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "DATA REDUNDANCY                :")){
                reflns.pdbx_redundancy = get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "R MERGE                    (I) :")){
                reflns.pdbx_Rmerge_I_obs = get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "R SYM                      (I) :")){
                reflns.pdbx_Rsym_value= get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "<I/SIGMA(I)> FOR THE DATA SET  :")){
                reflns.pdbx_netI_over_sigmaI = get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "HIGHEST RESOLUTION SHELL, RANGE HIGH (A) :")){
                reflns_shell.nbin = 1;
                reflns_shell.d_res_high[0]= get_float(line[i],":", "\n");
                reflns_shell.pdbx_diffrn_id[0]=(char*)"1";
                
            }else if(strstr_space(line[i], "HIGHEST RESOLUTION SHELL, RANGE LOW  (A) :")){
                reflns_shell.d_res_low[0]= get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "COMPLETENESS FOR SHELL     (%) :")){
                reflns_shell.percent_possible_all[0]= get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "DATA REDUNDANCY IN SHELL       :")){
                reflns_shell.pdbx_redundancy[0]= get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "R MERGE FOR SHELL          (I) :")){
                reflns_shell.Rmerge_I_obs[0]= get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "R SYM FOR SHELL            (I) :")){
                reflns_shell.pdbx_Rsym_value[0] = get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "<I/SIGMA(I)> FOR SHELL         :")){
                reflns_shell.meanI_over_sigI_obs[0] = get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "DIFFRACTION PROTOCOL:")){
                diffrn_radiation.pdbx_diffrn_protocol=get_substr(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "METHOD USED TO DETERMINE THE STRUCTURE:")){
                strcpy(refine.pdbx_method_to_determine_struct , get_substr(line[i], ":", "\n"));
                
            }else if(strstr_space(line[i], "SOFTWARE USED:")){
            
                strcpy(computing.structure_solution, get_substr(line[i],":", "\n"));
                
            }else if(strstr_space(line[i], "STARTING MODEL:")){
                strcpy(refine.pdbx_starting_model, get_substr(line[i],":", "\n")); 
                
            }
        }
    }else if(strstr(id,"230")){ 
        diffrn_source_neut.diffrn_id=(char*)"2";
        diffrn_detector_neut.diffrn_id=(char*)"2";
        diffrn_radiation_neut.diffrn_id=(char*)"2";
        diffrn_radiation_neut.wavelength_id=(char*)"2";
        diffrn_radiation_neut.pdbx_scattering_type=(char*)"neutron";
        reflns_neut.pdbx_diffrn_id=(char*)"2";
        reflns_neut.pdbx_ordinal=(char*)"2";
        diffrn_neut.id=(char*)"2";

        for(i=nrange[0]; i<nrange[1]; i++){
            if (strstr_space(line[i], ":NULL") || strstr_space(line[i], ": NULL") ) continue;
        
            if(strstr_space(line[i], "EXPERIMENT TYPE                :")){ //exptl.
                strcpy(exptl_neut.method, get_substr(line[i],":", "\n"));
                
            }else if(strstr_space(line[i], "DATE OF DATA COLLECTION        :")){
                strcpy(temp,  get_substr(line[i],":", "\n"));
                date_conversion(temp);
                strcpy(diffrn_detector_neut.pdbx_collection_date, temp);

            }else if(strstr_space(line[i], "TEMPERATURE           (KELVIN) :")){
                a1 = get_float(line[i],":", "\n");
                if (a1>-99) diffrn_neut.ambient_temp=a1;
                
//            }else if(strstr_space(line[i], "PH                             :")){
//                exptl_crystal_grow_neut.pH = get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "NUMBER OF CRYSTALS USED        :")){
                exptl_neut.crystals_number= get_int(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "SYNCHROTRON              (Y/N) :")){
                diffrn_source_neut.pdbx_synchrotron_y_n=get_substr(line[i],":", "\n");
                if (diffrn_source_neut.pdbx_synchrotron_y_n[0] == 'Y') {
                    diffrn_source_neut.source= (char *) "SYNCHROTRON";
                }
                
            }else if(strstr_space(line[i], "NEUTRON SOURCE                 :")){
                diffrn_source_neut.source= get_substr(line[i],":", "\n");
                
                
            }else if(strstr_space(line[i], "BEAMLINE                       :")){
                diffrn_source_neut.pdbx_synchrotron_beamline= get_substr(line[i],":", "\n");
                
                    
            }else if(strstr_space(line[i], "WAVELENGTH OR RANGE        (A) :")){
                diffrn_source_neut.pdbx_wavelength=get_substr(line[i],":", "\n");
                diffrn_source_neut.pdbx_wavelength_list=get_substr(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "MONOCHROMATOR                  :")){
                diffrn_radiation_neut.monochromator=get_substr(line[i],":", "\n");

                    
            }else if(strstr_space(line[i], "OPTICS                         :")){
                diffrn_detector_neut.details = get_substr(line[i],":", "\n");

            }else if(strstr_space(line[i], "DETECTOR TYPE                  :")){
                diffrn_detector_neut.detector = get_substr(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "DETECTOR MANUFACTURER          :")){
                diffrn_detector_neut.type = get_substr(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "INTENSITY-INTEGRATION SOFTWARE :")){
                strcpy(computing.data_collection, get_substr(line[i],":", "\n"));
                
            }else if(strstr_space(line[i], "DATA SCALING SOFTWARE          :")){
                strcpy(computing.data_reduction, get_substr(line[i],":", "\n"));

            }else if(strstr_space(line[i], "NUMBER OF UNIQUE REFLECTIONS   :")){
                reflns_neut.number_obs= get_int(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "RESOLUTION RANGE HIGH      (A) :")){
                reflns_neut.d_resolution_high= get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "RESOLUTION RANGE LOW       (A) :")){
                reflns_neut.d_resolution_low= get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "REJECTION CRITERIA  (SIGMA(I)) :")){
                reflns_neut.observed_criterion_sigma_I = get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "COMPLETENESS FOR RANGE     (%) :")){
                reflns_neut.percent_possible_obs = get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "DATA REDUNDANCY                :")){
                reflns_neut.pdbx_redundancy = get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "R MERGE                    (I) :")){
                reflns_neut.pdbx_Rmerge_I_obs = get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "R SYM                      (I) :")){
                reflns_neut.pdbx_Rsym_value= get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "<I/SIGMA(I)> FOR THE DATA SET  :")){
                reflns_neut.pdbx_netI_over_sigmaI = get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "HIGHEST RESOLUTION SHELL, RANGE HIGH (A) :")){
                reflns_shell_neut.nbin = 1;
                reflns_shell_neut.d_res_high[0]= get_float(line[i],":", "\n");
                reflns_shell_neut.pdbx_diffrn_id[0]=(char*)"2";
                
            }else if(strstr_space(line[i], "HIGHEST RESOLUTION SHELL, RANGE LOW  (A) :")){
                reflns_shell_neut.d_res_low[0]= get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "COMPLETENESS FOR SHELL     (%) :")){
                reflns_shell_neut.percent_possible_all[0]= get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "DATA REDUNDANCY IN SHELL       :")){
                reflns_shell_neut.pdbx_redundancy[0]= get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "R MERGE FOR SHELL          (I) :")){
                reflns_shell_neut.Rmerge_I_obs[0]= get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "R SYM FOR SHELL            (I) :")){
                reflns_shell_neut.pdbx_Rsym_value[0] = get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "<I/SIGMA(I)> FOR SHELL         :")){
                reflns_shell_neut.meanI_over_sigI_obs[0] = get_float(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "DIFFRACTION PROTOCOL:")){
                diffrn_radiation_neut.pdbx_diffrn_protocol=get_substr(line[i],":", "\n");
                
            }else if(strstr_space(line[i], "METHOD USED TO DETERMINE THE STRUCTURE:")){
                strcpy(refine_neutron.pdbx_method_to_determine_struct , get_substr(line[i], ":", "\n"));
                
            }else if(strstr_space(line[i], "SOFTWARE USED:")){
            
                strcpy(computing.structure_solution, get_substr(line[i],":", "\n"));
                
            }else if(strstr_space(line[i], "STARTING MODEL:")){
                strcpy(refine_neutron.pdbx_starting_model, get_substr(line[i],":", "\n")); 
                
            }
        }
        
        
    }

}

void write_remark_txt(FILE *fout, char **line, int nline)
{
    char rnum_old[10], rnum[10];
    int i, n1=0, n=0, rm=0;
    

    
    for(i=0; i<nline; i++){
        if(!strncmp(line[i], "ATOM", 4) || !strncmp(line[i], "HETATM", 6) ) break;
        if(!strncmp(line[i], "REMARK ", 7)) {
            rm++;
            n1=get_int_by_length(line[i], 6, 9);
            if (n1<0){ break;} //non-standard
        }
    }
    if (rm==0){ return;}
    if(n1<0 ){// non-standard remarks
        fprintf(fout,"# \n");
        fprintf(fout,"_database_PDB_remark.id   1 \n");
        fprintf(fout,"_database_PDB_remark.text \n;\n");
   
        for(i=0; i<nline; i++){
            if(!strncmp(line[i], "ATOM  ", 6) || !strncmp(line[i], "HETATM", 6) ) break;
            if(!strncmp_case(line[i], "REMARK ", 7)) {
                fprintf(fout,"%s", line[i]);
            }
        }
        fprintf(fout,";\n");
        
    }else {
        
        fprintf(fout,"# \n");
        fprintf(fout,"loop_\n");
        fprintf(fout,"_database_PDB_remark.id   \n");
        fprintf(fout,"_database_PDB_remark.text \n");
        n=0;
        strcpy(rnum_old, "-1");
        for(i=0; i<nline; i++){
            if(!strncmp(line[i], "ATOM  ", 6) || !strncmp(line[i], "HETATM", 6) ) break;
            if(!strncmp_case(line[i], "REMARK ", 7)) {
                
                get_string(line[i], 6,9,rnum);
                if(strcmp_case(rnum_old, rnum)) {
                    if(n>0)fprintf(fout,";\n");
                    fprintf(fout,"\n%s\n;\n", rnum);
                }
                n++;
                
                fprintf(fout,"%s", line[i]);
                strcpy(rnum_old, rnum);
            }
        }
        fprintf(fout,";\n");
        
    }
}


void write_pdb_conn(FILE *fout, char **line, int nline)
 // for the connectivity
{
    char str[512], id[50], item[20];
    char  comp[6], sn[6], asym, seq[6], ins;
    int     i;

    strcpy(id, "SSBOND SLTBRG HYDBND LINK   ");

   
    for(i=0;i<nline;i++){
        if(!strncmp(line[i], "ATOM  ", 6) || !strncmp(line[i], "HETATM", 6) ) break;
        strncpy(item,line[i], 6);
        item[6] = '\0';
        if(strstr_case(id, item)){
            fprintf(fout,"#\n");
            fprintf(fout,"loop_\n");
            fprintf(fout,"_struct_conn.conn_type_id\n");
            fprintf(fout,"_struct_conn.id\n");
            
            fprintf(fout,"_struct_conn.ptnr1_auth_atom_id \n");
            fprintf(fout,"_struct_conn.ptnr1_auth_comp_id \n");
            fprintf(fout,"_struct_conn.ptnr1_auth_asym_id \n");
            fprintf(fout,"_struct_conn.ptnr1_auth_seq_id\n");
            fprintf(fout,"_struct_conn.pdbx_ptnr1_auth_alt_id \n");
            fprintf(fout,"_struct_conn.pdbx_ptnr1_PDB_ins_code\n");
            fprintf(fout,"_struct_conn.ptnr1_symmetry \n");
            
            fprintf(fout,"_struct_conn.ptnr2_auth_atom_id \n");
            fprintf(fout,"_struct_conn.ptnr2_auth_comp_id \n");
            fprintf(fout,"_struct_conn.ptnr2_auth_asym_id \n");
            fprintf(fout,"_struct_conn.ptnr2_auth_seq_id \n");
            fprintf(fout,"_struct_conn.pdbx_ptnr2_auth_alt_id \n");
            fprintf(fout,"_struct_conn.pdbx_ptnr2_PDB_ins_code \n");
            fprintf(fout,"_struct_conn.ptnr2_symmetry \n");
            
            fprintf(fout,"_struct_conn.pdbx_ptnr3_auth_atom_id \n");
            fprintf(fout,"_struct_conn.pdbx_ptnr3_auth_alt_id \n");
            fprintf(fout,"_struct_conn.pdbx_ptnr3_auth_asym_id \n");
            fprintf(fout,"_struct_conn.pdbx_ptnr3_auth_seq_id \n");
            fprintf(fout,"_struct_conn.pdbx_ptnr3_PDB_ins_code\n");
            fprintf(fout,"_struct_conn.details \n");
            break;
            
        }
    }
   
   
    for(i=0;i<nline;i++){
        if(!strncmp(line[i], "ATOM  ", 6) || !strncmp(line[i], "HETATM", 6) ) break;
        strncpy(item,line[i], 6);
        item[6] = '\0';
        
        if(!strcmp_case(item, "SSBOND")){
           get_string(line[i], 7,9, sn);
           rid_of_front_end_space(sn) ;       
           fprintf(fout,"SSBOND  '%s'  ", sn);

            write_conn(fout, line[i],  6,7,  6,  11,13,  15,  17,20,  21,  59,64);


            write_conn(fout, line[i],  6,7,  6,  25,27,  29,  31,34,  35,  66,71);
            fprintf(fout," ?  ?  ?  ?  ?  ? \n");
        }
        
    }
/*
    for(i=0;i<n;i++){
        strncpy(item,line[i], 6);
        item[6] = '\0';
        if(!strcmp_case(item, "LINK  ")){
            fprintf(fout,"LINK   .  ");
            
            write_conn(fout, line[i],  12,15,  16,  17,19,  21,  22,25,  26,  59,64);
            write_conn(fout, line[i],  42,45,  46,  47,49,  51,  52,55,  56,  66,71);

            fprintf(fout," ?  ?  ?  ?  ?  ? \n");
        }
    }
   
    
    for(i=0;i<n;i++){
        strncpy(item,line[i], 6);
        item[6] = '\0';
        if(!strcmp_case(item, "HYDBND")){
            fprintf(fout,"HYDBND   ?  ");
            
            write_conn(fout, line[i],  12,15,  16,  17,19,  21,  22,26,  27,  59,64);
            write_conn(fout, line[i],  43,46,  47,  48,50,  52,  53,57,  58,  66,71);
            
            get_string(line[i], 29,32, atom);
            alt = line[i][33];
            asym = line[i][35];
            get_string(line[i], 36,40, seq);
            
            ins = line[i][41];
            
            rid_of_front_end_space(atom);
            rid_of_front_end_space(seq);
            
            fprintf(fout," '%s' %c %c  '%s' %c ", atom, alt, asym, seq, ins);
            fprintf(fout," ?\n ");
        }
    }
   
    for(i=0;i<n;i++){
        strncpy(item,line[i], 6);
        item[6] = '\0';
        if(!strcmp_case(item, "SLTBRG")){
            fprintf(fout,"SLTBRG  ?  ");
            
            write_conn(fout, line[i],  12,15,  16,  17,19,  21,  22,25,  26,  59,64);
            write_conn(fout, line[i],  42,45,  46,  47,49,  51,  52,55,  56,  66,71);

            fprintf(fout," ?  ?  ?  ?  ?  ? \n");
        }
     }

*/ 
//for _struct_mon_prot_cis.
    for(i=0;i<nline;i++){
        if(!strncmp(line[i], "ATOM  ", 6) || !strncmp(line[i], "HETATM", 6) ) break;
        strncpy(item,line[i], 6);
        item[6] = '\0';
        if(strstr_case(item, "CISPEP")){
            fprintf(fout,"#\n");
            fprintf(fout,"loop_\n");
            fprintf(fout,"_struct_mon_prot_cis.pdbx_id  \n");
            fprintf(fout,"_struct_mon_prot_cis.auth_comp_id  \n");
            fprintf(fout,"_struct_mon_prot_cis.auth_asym_id  \n");
            fprintf(fout,"_struct_mon_prot_cis.auth_seq_id  \n");
            fprintf(fout,"_struct_mon_prot_cis.pdbx_PDB_ins_code  \n");
            fprintf(fout,"_struct_mon_prot_cis.pdbx_auth_comp_id_2  \n");
            fprintf(fout,"_struct_mon_prot_cis.pdbx_auth_asym_id_2  \n");
            fprintf(fout,"_struct_mon_prot_cis.pdbx_auth_seq_id_2  \n");
            fprintf(fout,"_struct_mon_prot_cis.pdbx_PDB_ins_code_2  \n");
            fprintf(fout,"_struct_mon_prot_cis.pdbx_PDB_model_num  \n");
            fprintf(fout,"_struct_mon_prot_cis.pdbx_omega_angle  \n");
            break;
        }
    }
    for(i=0;i<nline;i++){
        if(!strncmp(line[i], "ATOM  ", 6) || !strncmp(line[i], "HETATM", 6) ) break;
        
        strncpy(item,line[i], 6);
        item[6] = '\0';
        if(!strcmp_case(item, "CISPEP")){
           
            get_string(line[i],7,9, sn);
            rid_of_front_end_space(sn);
            
            fprintf(fout," '%s' ", sn);
            
            get_string(line[i],11,13, comp);
            rid_of_front_end_space(comp);
            asym=line[i][15];
            
            if(isspace(asym)) asym='?';
            
            get_string(line[i],17,20, seq);
            rid_of_front_end_space(seq);
            ins=line[i][21];
            if(isspace(ins)) {
                ins='?';
            }else if (ins=='@'){
                ins='.';
            }
             
            fprintf(fout," '%s' %c '%s' %c  ",comp, asym,seq,ins);
            
            get_string(line[i],25,27,comp);
            rid_of_front_end_space(comp);
            
            asym=line[i][29];
            if(isspace(asym)) asym='?';
            
            get_string(line[i],31,34,seq);
            rid_of_front_end_space(seq);

            ins=line[i][35];
            if(isspace(ins)) ins='?';
            fprintf(fout," '%s' %c '%s' %c  ",comp, asym,seq,ins);
            
            get_string(line[i],43,45,seq);
            get_string(line[i],53,58,str);
            rid_of_front_end_space(seq);
            rid_of_front_end_space(str);
            
            fprintf(fout," '%s' '%s' \n", seq, str);
        }
    }
    
}

void write_conn(FILE *fout, char *line, int atom1, int atom2, int alt1, int comp1, 
                int comp2,int asym1, int seq1, int seq2, int ins1, int sym1, int sym2)
{
    char atom[20], seq[20], comp[20], alt[20], asym[20], ins[20], sym[20];
    
    get_string(line, atom1,atom2, atom);
    get_string(line,alt1,alt1,alt);
    get_string(line, comp1,comp2,comp);
    get_string(line,asym1,asym1,asym);
    get_string(line, seq1,seq2,seq);
    get_string(line,ins1,ins1,ins);
    get_string(line, sym1,sym2,sym);
    
    rid_of_front_end_space(sym);
    rid_of_front_end_space(atom);
    rid_of_front_end_space(seq);
    rid_of_front_end_space(comp);
    rid_of_front_end_space(asym);
    rid_of_front_end_space(alt);
    rid_of_front_end_space(ins);
    
    fprintf(fout, " '%s'  '%s'  '%s'  '%s'  '%s'  '%s' '%s' ",
            atom, comp, asym, seq, alt, ins, sym);
   
   
}

int get_domid(char **tok, int ntok, char *c1)
{
    int i, n=1;
        //   printf("ntok=%d, ch=%s\n", ntok, c1);
    
    for(i=0; i<ntok; i++){
        if(strlen(tok[i])>1)continue;
        if(!strcmp(tok[i],c1)) {
            n=i+1;
            break;
        }
    }
    return n;
}


int get_ncs_group(int ns, int nline, char **line, char **ncsg, int *j)
// return the group of lines for the NCS group. (general)
// different program has different ID    
{
    int i,  n=0;
    for(i=ns; i<nline; i++){
        if(strstr(line[i], " NCS GROUP ") || strstr(line[i], " Histogram ") ||
           !strncmp(line[i], "CRYST1",6) || !strncmp(line[i], "ATOM",4) ||
           !strncmp(line[i], "HETATM", 6) || strstr(line[i], "OTHER REFINEMENT ")) {
            
            *j=i-1;
            break;
        }
        if (n>=4000){
            sprintf(INFO, "Warning: the lines for the NCS group is too large!\n");
            pinfo(INFO,0);
            return n;
        }
                
        strcpy(ncsg[n] ,line[i]);
        n++;
    }
    return n;
}


char *get_phenix_ncs_selection(int n, int nline, char **line,int *j)
// the return char striped off the multiple spaces.    
{
    char *str=NULL;
    int i, k=0;
    
    str=new char [30000];
    strcpy(str,"");
    k=0;
    for(i=n; i<nline; i++){
        if(k>0 && !strstr(line[i],"REMARK   3                        ")){
            *j=i;
            break;
        }
        k++;
        strcat(str,line[i] +35 );
    }
    
    replace_char(str, '\n', ' ');
    replace_char(str, '\r', ' ');
    clean_string(str,str);
//    printf("string=%s\n", str);
    
    return str;
}

char ** phenix_ncs_selection(int nncs, char **ncsg, float *rmsd, int *numb,int *tors, int*nndom)
// the return char striped off the multiple spaces
//Two type of NCS treated differently!
{
    char *temp=NULL, **dom=NULL;
    int j, start=0,ndom=0, refs=0;

    *tors=0;
    
    dom=new char *[400]; //max 50 selections for the NCS
    
    for(j=0; j<nncs; j++){
        if(strstr_space(ncsg[j]," REFERENCE SELECTION") ){
            refs=1;
            temp=get_phenix_ncs_selection(j,nncs, ncsg, &start);
            dom[ndom]= new char [strlen(temp) +1];
            strcpy(dom[ndom], temp);
            ndom++;
            break;
        }
        
    }
    
    for(j=0; j<nncs; j++){
        if (strstr_space(ncsg[j],"REMARK   3     SELECTION          :")){
            temp=get_phenix_ncs_selection(j,nncs, ncsg, &start);
            dom[ndom]= new char [strlen(temp) +1];
            strcpy(dom[ndom], temp);
            if (ndom>=399){
                sprintf(INFO, "Error: Too many selections for the NCS group.\n");
                pinfo(INFO,0);
                
                *nndom=ndom;
                return dom;
            }
            
            ndom++;
            j=start-1;
        
        }else if(strstr_space(ncsg[j],"REMARK   3     ATOM PAIRS NUMBER  : ") ){
            if (refs==1){
                numb[ndom-1]= get_int_after_id(ncsg[j],":");
                numb[0]=numb[1];  //just fill up the missing
            }else{
                numb[0]= get_int_after_id(ncsg[j],":");
            }
                    
        }else if( strstr_space(ncsg[j],"REMARK   3     RESTRAINED TORSIONS:")){
            if (refs==1){
                numb[ndom-1]= get_int_after_id(ncsg[j],":");
                numb[0]=numb[1];  //just fill up the missing
            }else{
                numb[0]= get_int_after_id(ncsg[j],":");
            }
            
            *tors=1;
          
        }else if(strstr_space(ncsg[j],"REMARK   3     RMSD               :") ||
                 strstr_space(ncsg[j],"REMARK   3     ALL RESTRAINT RMSD :") ){
            if (refs==1){
                rmsd[ndom-1] = get_float_after_id(ncsg[j],":");
                rmsd[0]=rmsd[1];
            }else{
                rmsd[0]= get_float_after_id(ncsg[j],":");
            }
            //printf("get_string_patt  %4d %4d %.3f  %s  \n", *numb, j, *rmsd, temp);
        }
    }
    if(refs==0){
        for(j=0; j<ndom; j++){
            rmsd[j]=rmsd[0];
            numb[j]=numb[0];
        }
    }
    *nndom=ndom;
    return dom;
}

void extract_phenix_ncs(int ii, int nline, char **lines, int *end)
//ensemble= each NCS group; dom=each selection; component=each segments
{
    char  **ncsg=NULL, **dom=NULL, **ch1;
    int j,k,n,  n_group=0, res_beg[400], res_end[400];
    int numb[400], nndom=0, ndom=0, nlim=0, tors=0,ncomp1=0,start=0, nncs=0;
    float rmsd[400];

    printf("PHENIX NCS extracting...\n");
    
    ncsg=set_memory_2d(6000, 400); //lines for the NCS
    ch1=set_memory_2d(400, 5); //lines for the NCS

    for(n=0; n<400; n++){ //for struct_ncs_dom_lim
        res_beg[n]=0;
        res_end[n]=0;
        rmsd[n]=0;
        numb[n]=0;
    }


    *end=ii+1;
    for(j=ii; j<nline; j++){
        if(strstr(lines[j], "REMARK   3   NUMBER OF NCS GROUPS :")){
//            n_group_tot = get_int(lines[j], " :", "\n");
        }else if(strstr(lines[j], "REMARK   3   NCS GROUP :")){
            n_group++;
           // n_group  = get_int(lines[j]," :", "\n");
            nncs=get_ncs_group(j+1, nline, lines, ncsg, &start);
            j=start;
            
            dom=phenix_ncs_selection(nncs, ncsg, rmsd, numb,&tors, &nndom);
            
                //        for(k=0; k<nncs; k++){printf ("tmp(%d:%d)=%s", k,tors, ncsg[k]); }
            for(k=0; k<nndom; k++){
                residue_range_phenix(dom[k], res_beg, res_end, &ncomp1, ch1);//selection
                
                sprintf(struct_ncs_dom[ndom].pdbx_ens_id, "%d", n_group);
                sprintf(struct_ncs_dom[ndom].id, "%d", k+1);
                strcpy(struct_ncs_dom[ndom].details, dom[k]);
                

                sprintf(refine_ls_restr_ncs[ndom].pdbx_ens_id,"%d",n_group );
                sprintf(refine_ls_restr_ncs[ndom].dom_id,"%d", k+1);
                if (tors==0){
                    strcpy(refine_ls_restr_ncs[ndom].pdbx_type, "POSITIONAL");
                    refine_ls_restr_ncs[ndom].pdbx_rms = rmsd[k];
                    refine_ls_restr_ncs[ndom].pdbx_number = numb[k];
                    
                }else{
                    strcpy(refine_ls_restr_ncs[ndom].pdbx_type, "TORSIONAL");
                    refine_ls_restr_ncs[ndom].pdbx_number =  numb[0];
                    refine_ls_restr_ncs[ndom].pdbx_rms = rmsd[0];
                }
                sprintf(refine_ls_restr_ncs[ndom].pdbx_asym_id, "%s",ch1[0]); //1 char
                 
                ndom++;
                

                for(n=0; n<ncomp1; n++){ //component
                    if(nlim >= NCSMAX-1 )break;

                    sprintf(struct_ncs_dom_lim[nlim].pdbx_ens_id,"%d", n_group);
                    sprintf(struct_ncs_dom_lim[nlim].dom_id,"%d", k+1);
                    struct_ncs_dom_lim[nlim].pdbx_component=n+1;

                    strcpy(struct_ncs_dom_lim[nlim].beg_label_asym_id, ch1[n]);
                    struct_ncs_dom_lim[nlim].beg_label_seq_id = res_beg[n];
                    strcpy(struct_ncs_dom_lim[nlim].end_label_asym_id, ch1[n]);
                    struct_ncs_dom_lim[nlim].end_label_seq_id = res_end[n];

                    strcpy(struct_ncs_dom_lim[nlim].pdbx_refine_code, "?");
                    strcpy(struct_ncs_dom_lim[nlim].selection_details, dom[k]);

                    nlim++;
                }
            }

        }
    
    }
    NNCS=ndom;
    NLIM=nlim;
    NCS=ndom;

    if(NCSMAX-NNCS<2 || NCSMAX-NLIM<2 || NCSMAX-NCS<2){
        sprintf(INFO,"Warning! The max number of NCS may not be set big enough for extraction.\n");
        pinfo(INFO, 0);
    }
//    printf("PHENIX:  NNCS=%d :: NLIM=%d : NDOM=%d :: NCS_GROUP_TOT=%d\n",NNCS,NLIM, NCS, n_group_tot);
}


void max_min_avg(int ii, char *ch, int nline, char **line, int *min, int *max, float *avg)
//
{
    char tmp1[10];
    int min_o=999999, max_o=-999999, occ ,i, n=0;
    float bavg=0;

    *min=0; *max=0; *avg=0;
    for(i=ii+1; i<nline; i++){ 
        if ((!strncmp(line[i], "ATOM  ", 6) || !strncmp(line[i], "HETATM", 6))
               && strlen(line[i])>65){
            if (ch[0] != line[i][21]) continue;
            
            n++;
            
            strncpy(tmp1,line[i]+22, 4);
            tmp1[4]='\0';
            if (!isdigit(tmp1[3])) continue;
            
            occ=atoi(tmp1);
            if(occ<min_o)min_o=occ;
            if(occ>max_o)max_o=occ;
            
            bavg=bavg + float(occ);
            
        }
    }
    if(n>0){
        *avg=bavg/n;
        *min=min_o;
        *max=max_o;
    }
}

void extract_refmac_ncs_new(int ii, int nline, char **lines, int *end)
{
    char  **tok=NULL, tmp[256];
    int i,j, nncs=0, ntok=0;

    *end=ii+1;
    
    for(i=ii; i<nline; i++){ 
        if(!strncmp(lines[i],"ATOM", 4) || !strncmp(lines[i],"HATM", 4) ) break;
        
        if(strstr_space(lines[i],"GROUP  CHAIN1    RANGE     CHAIN2     RANGE    COUNT RMS ")){
            
            for(j=i+1; j<nline; j++){
                strcpy(tmp,lines[j]+10);
                
                tok=string_token(tmp," ", &ntok);
                if (ntok != 10) break;
                
                strcpy(struct_ncs_dom[nncs].id, "1");
                strcpy(struct_ncs_dom[nncs+1].id, "2");
                
                strcpy(struct_ncs_dom[nncs].pdbx_ens_id, tok[0]);
                strcpy(struct_ncs_dom[nncs+1].pdbx_ens_id, tok[0]);
                
                strcpy(struct_ncs_dom[nncs].details, tok[1]);
                strcpy(struct_ncs_dom[nncs+1].details, tok[4]);


                
                
                strcpy(struct_ncs_dom_lim[nncs].pdbx_ens_id, tok[0]);
                strcpy(struct_ncs_dom_lim[nncs+1].pdbx_ens_id, tok[0]);
                
                strcpy(struct_ncs_dom_lim[nncs].dom_id,"1");
                strcpy(struct_ncs_dom_lim[nncs+1].dom_id,"2");
                
                struct_ncs_dom_lim[nncs].pdbx_component=0;
                struct_ncs_dom_lim[nncs+1].pdbx_component=0;

                strcpy(struct_ncs_dom_lim[nncs].pdbx_refine_code, "0");
                strcpy(struct_ncs_dom_lim[nncs+1].pdbx_refine_code, "0");
                
                strcpy(struct_ncs_dom_lim[nncs].beg_label_asym_id, tok[1]);
                strcpy(struct_ncs_dom_lim[nncs].end_label_asym_id, tok[1]);
                
                strcpy(struct_ncs_dom_lim[nncs+1].beg_label_asym_id, tok[4]);
                strcpy(struct_ncs_dom_lim[nncs+1].end_label_asym_id, tok[4]);

                struct_ncs_dom_lim[nncs].beg_label_seq_id = atoi(tok[2]);
                struct_ncs_dom_lim[nncs].end_label_seq_id = atoi(tok[3]);

                struct_ncs_dom_lim[nncs+1].beg_label_seq_id = atoi(tok[5]);
                struct_ncs_dom_lim[nncs+1].end_label_seq_id = atoi(tok[6]);

                

                
                strcpy(refine_ls_restr_ncs[nncs].pdbx_ens_id , tok[0]);
                strcpy(refine_ls_restr_ncs[nncs+1].pdbx_ens_id , tok[0]);
                
                strcpy(refine_ls_restr_ncs[nncs].dom_id , "1");
                strcpy(refine_ls_restr_ncs[nncs+1].dom_id , "2");
                
//                strcpy(refine_ls_restr_ncs[nncs].pdbx_type , "LOCAL");
//                strcpy(refine_ls_restr_ncs[nncs+1].pdbx_type , "LOCAL");
                strcpy(refine_ls_restr_ncs[nncs].pdbx_type , "interatomic distance");
                strcpy(refine_ls_restr_ncs[nncs+1].pdbx_type , "interatomic distance");

                strcpy(refine_ls_restr_ncs[nncs].pdbx_asym_id , tok[1]);
                strcpy(refine_ls_restr_ncs[nncs+1].pdbx_asym_id , tok[4]);
                
                refine_ls_restr_ncs[nncs].pdbx_number = atoi(tok[7]);
                refine_ls_restr_ncs[nncs+1].pdbx_number = atoi(tok[7]);
                
                refine_ls_restr_ncs[nncs].pdbx_rms =  atof(tok[8]);
                refine_ls_restr_ncs[nncs+1].pdbx_rms =  atof(tok[8]);
                
                refine_ls_restr_ncs[nncs].pdbx_weight = atof(tok[9]);
                refine_ls_restr_ncs[nncs+1].pdbx_weight = atof(tok[9]);
                
                nncs = nncs + 2;
            }

        }
    }
    NNCS=nncs;
    NLIM=nncs;
    NCS=nncs;
    if(NCSMAX-NNCS<2 || NCSMAX-NLIM<2 || NCSMAX-NCS<2){
        sprintf(INFO,"Warning! The max number of NCS may not be set big enough for extraction.\n");
        pinfo(INFO, 0);
    }
    
    printf("REFMAC5: NNCS=%d :: NLIM=%d : NDOM=%d \n",  NNCS,NLIM, NCS);
    
}

void extract_refmac_ncs(int ii, int nline, char **lines, int *end)
{
    char  c1[40],c2[40],c3[40], asym[80];
    char *chain=NULL, **ncs_tmp, **tok=NULL, *temp;
    int  np1=0, np2=0, np3=0, nt1=0, nt2=0, nt3=0, np=0, m, ns;
    int  i,j=0,k, n0=0,n1=0, n2=0, n=0,  ntok=0, min=0,max=0,start=0;
    int  n_group=0, nlim=0, nncs=0, ncomp=0, ndom=0, domid=0;
    float avg=0;
    	
    *end=ii+1;  	
    ncs_tmp=set_memory_2d(900, 200);
    printf("Extracting NCS ...\n");
    
    for(i=ii+1; i<nline; i++){
        if(!strncmp(lines[i],"ATOM", 4) || !strncmp(lines[i],"HATM", 4) )break;
        if(strstr_space(lines[i],"REMARK   3  NCS GROUP NUMBER    :")){
            n_group = get_int(lines[i]," :", "\n");
            ns=0;
            for(j=i+1; j<nline; j++){ //get the contents of this NCS
                strcpy(ncs_tmp[ns], lines[j]);
                
                ns++;
                if (j<nline-1 && ns>2 &&
                    (strstr(lines[j+1], "REMARK   3  NCS GROUP") ||
                     strstr(lines[j+1], " TLS ") || strstr(lines[j+1], "REMARK 200") ||
                     !strncmp(lines[j+1],"ATOM", 4) )){
                    i=j;
                    break;
                }

            }
            if (!ns) continue;
            
            np=0; //process the ncs contents
            for(m=0; m<ns; m++){
                if(strstr_space(ncs_tmp[m],"REMARK   3 CHAIN NAMES :")){
                    temp=get_string_patt(0, ns, ncs_tmp, ":", "NUMBER", &start);
                    chain=replace_all_str(temp,"REMARK 3", " ");
                    
                    if(temp) delete[]temp;
                    if(strstr(chain,"CHAIN "))replace_first_str(chain, "CHAIN ", " ",chain);
                    if(strstr(chain,"CHAINS "))replace_first_str(chain, "CHAINS ", " ",chain);
                    if(strchr(chain,':'))replace_char(chain, ':', ' ');
                    rid_of_front_end_space(chain);
                    
                    if(strlen(chain)<=0){
                        sprintf(INFO, "Error: No chainID was given (NCS group=%d).\n", n_group);
                        pinfo(INFO, 0);
                    }
                }else if(strstr_space(ncs_tmp[m],"COMPONENTS NCS GROUP :")){
                    ncomp=get_int_after_id(ncs_tmp[m],":");
                    if(ncomp<=0) {
                        sprintf(INFO,"Warning: Problem with components number(NCS group=%d).\n", n_group);
                        pinfo(INFO, 0);
                    }
                    np=m;
                }
            }
            
            tok=string_token(chain," ", &ntok);
                        
            for(k=0; k<ntok; k++){
                if(strlen(tok[k])>2)continue;
                if(ndom>=NCSMAX-2) {
                    sprintf(INFO,"Error: Too many NCS doms (%d), increase NCSMAX.\n", ndom);
                    pinfo(INFO, 0);
                    
                    break;
                }
                
                sprintf(struct_ncs_dom[ndom].id, "%d", k+1);
                sprintf(struct_ncs_dom[ndom].pdbx_ens_id, "%d", n_group);
                strcpy(struct_ncs_dom[ndom].details,tok[k]);
//                printf("chain=%d, %s\n", k,struct_ncs_dom[ndom].details);
                ndom++;
            }
            if (ntok==0){
                sprintf(struct_ncs_dom[ndom].id, "1");
                sprintf(struct_ncs_dom[ndom].pdbx_ens_id, "%d", n_group);
                strcpy(struct_ncs_dom[ndom].details,"?");
                ndom++;
            }
            

            for(m=np; m<ns; m++){
                if (strstr_space(ncs_tmp[m],"GROUP CHAIN")) break;
                if (strstr_space(ncs_tmp[m],"COMPONENT C  SSSEQI")) continue;

                n=sscanf(ncs_tmp[m]+11,"%d %s %d %s %d %s", &n0, c1, &n1, c2, &n2, c3);
                
                if(n==6 && ntok>0 ){ //components
                    if(nlim>=NCSMAX-1) break;
                    domid=get_domid(tok, ntok, c1);
                    sprintf(struct_ncs_dom_lim[nlim].pdbx_ens_id,"%d", n_group);
                    sprintf(struct_ncs_dom_lim[nlim].dom_id,"%d", domid);
                    struct_ncs_dom_lim[nlim].pdbx_component=n0;
                    strcpy(struct_ncs_dom_lim[nlim].beg_label_asym_id, c1);
                    strcpy(struct_ncs_dom_lim[nlim].end_label_asym_id, c2);
                            
                    if(!strcmp(c1, c2) && n1<0 && n2>=9999){ //possible default
                        max_min_avg(ii, c1, nline, lines, &min, &max, &avg);
                                
                        if (n1<=min && n2>=max){
                            n1=min;
                            n2=max;
                        }
                    }
                            
                    struct_ncs_dom_lim[nlim].beg_label_seq_id = n1;
                    struct_ncs_dom_lim[nlim].end_label_seq_id = n2;
                                
                    strcpy(struct_ncs_dom_lim[nlim].pdbx_refine_code, c3);
                    nlim++;
               //     printf("nlim=%d, %s %d %s %d\n", nlim, c1,n1,c2,n2);
                }
            }
            for(m=np; m<ns; m++){ //rms,type ...
                if(pattern5(ncs_tmp[m], "(", ")", ":", ";", ";")){ 
                    strncpy(c1,ncs_tmp[m]+35, 3);
                    c1[3]='\0';
                    rid_of_space(c1);
                    if (strlen(c1)==0){
                        sprintf(INFO,"Error: No chain ID in the NCS! %s",ncs_tmp[m]);
                        pinfo(INFO,0);
                        strcpy(c1, "?");
//                        continue;
                    }
                    
                    strcpy(asym,"?");
                    domid=1;
                    if(ntok>0 && isalpha(tok[0][0]) && isdigit(c1[0])){
                        domid=atoi(c1);
                        if(domid>0 && domid<=ntok) strcpy(asym,tok[domid-1]);
                    }else if(isalpha(c1[0])){
                        strcpy(asym,c1);
                    }else if(ntok>0 && strlen(c1) == 0) {
                                
                        if(strstr(ncs_tmp[m],"TIGHT POSITIONAL")){
                            strcpy(asym,tok[np1]);
                            if(np1++ >=ntok) np1=ntok-1;
                        }else if(strstr(ncs_tmp[m],"MEDIUM POSITIONAL")){
                            strcpy(asym,tok[np2]);
                            if(np2++ >=ntok) np2=ntok-1;
                        }else if(strstr(ncs_tmp[m],"LOOSE POSITIONAL")){
                            strcpy(asym,tok[np3]);
                            if(np3++ >=ntok) np3=ntok-1;
                                    
                        }else if(strstr(ncs_tmp[m],"TIGHT THERMAL")){
                            strcpy(asym,tok[nt1]);
                            if(nt1++ >=ntok) nt1=ntok-1;
                                    
                        }else if(strstr(ncs_tmp[m],"MEDIUM THERMAL")){
                            strcpy(asym,tok[nt2]);
                            if(nt2++ >=ntok) nt2=ntok-1;
                                    
                        }else if(strstr(ncs_tmp[m],"LOOSE THERMAL")){
                            strcpy(asym,tok[nt3]);
                            if(nt3++ >=ntok) nt3=ntok-1;
                                   
                        }
                    }
                            
                            
                    sprintf(refine_ls_restr_ncs[nncs].pdbx_ens_id,"%d", n_group);
                    sprintf(refine_ls_restr_ncs[nncs].dom_id,"%d", domid);
                                            
                    get_string(ncs_tmp[m], 11, 30, refine_ls_restr_ncs[nncs].pdbx_type);
                    
                    if (strlen(asym)>0)
                        strcpy(refine_ls_restr_ncs[nncs].pdbx_asym_id, asym );
                    else
                        strcpy(refine_ls_restr_ncs[nncs].pdbx_asym_id, "?");
                                            
                    refine_ls_restr_ncs[nncs].pdbx_number =  get_int(ncs_tmp[m],":",";");
                    refine_ls_restr_ncs[nncs].pdbx_rms =  get_float(ncs_tmp[m],";"," ;");
                    refine_ls_restr_ncs[nncs].pdbx_weight =  get_float(strstr(ncs_tmp[m], ";")+1 ,";","\n");
                    nncs++;
                    
                }
               
            }
                
        }
    }
    *end = j;
    NNCS=nncs;
    NLIM=nlim;
    NCS=ndom;
    if(NCSMAX-NNCS<2 || NCSMAX-NLIM<2 || NCSMAX-NCS<2){
        sprintf(INFO,"Warning! The max number of NCS may not be set big enough for extraction.\n");
        pinfo(INFO, 0);
    }
    printf("REFMAC5: NNCS=%d :: NLIM=%d : NDOM=%d\n",  NNCS,NLIM, NCS);
}


void extract_cns_ncs(int ii, int nline, char **lines, int *end)
{
    int  nlim=0, nncs=0;
    int i,j, npos=0, nbf=0, ndom=0;
    
    for(i=ii; i<nline; i++){
        for(j=i+1; j<nline; j++){
                
            if(pattern5(lines[j],"GROUP", "POSITIONAL", "(A)",  ":", ";")){
                refine_ls_restr_ncs[npos].rms_dev_position =  get_float(lines[j],":"," ;");
                refine_ls_restr_ncs[npos].weight_position =  get_float(lines[j],";","\n");
                if(refine_ls_restr_ncs[npos].rms_dev_position >-9998 ||
                   refine_ls_restr_ncs[npos].weight_position) npos++;
            }else if(pattern5(lines[j],"GROUP", "B-FACTOR", "(A**2)",  ":", ";")){
                refine_ls_restr_ncs[nbf].rms_dev_B_iso =  get_float(lines[j],":"," ;");
                refine_ls_restr_ncs[nbf].weight_B_iso =  get_float(lines[j],";","\n");
                if(refine_ls_restr_ncs[nbf].rms_dev_B_iso >-9998 ||
                   refine_ls_restr_ncs[nbf].weight_B_iso >-9998) nbf++;    
            }else{
                i=j;
                break;
            }

        }
    }
    
    NNCS=nncs;
    NLIM=nlim;
    NCS=ndom;
    
//    printf("CNS/XPLOR:  NNCS=%d :: NLIM=%d : NDOM=%d :: NCS_GROUP_TOT=%d\n",NNCS,NLIM, NCS, n_group_tot);

}

void parse_ncs(float mat[][5], char *ss, int nn)
{
    
    int ntok, i;
    char **tok;

    for(i=0; i<5; i++){mat[nn][i]=0;}
    
    rid_of_end_space(ss);
    tok=string_token(ss+10," ", &ntok);
    
    if (ntok>4){
        mat[nn][4] = atof(tok[4]);
    }else if (ntok == 4) {
        mat[nn][4] = 0 ;
    }else{
        sprintf(INFO, "Error! Matrix (%d) not have enough columns.\n", nn+1);
        pinfo(INFO,0);
        return;
    }
    
    
    mat[nn][0]=atof(tok[0]);
    mat[nn][1]=atof(tok[1]);
    mat[nn][2]=atof(tok[2]);
    mat[nn][3]=atof(tok[3]);
}

void extract_ncs_matrix(FILE *fout, char **lines, int nline)
{
    char ss[30], str[512];
    int i , nm1=0, nm2=0, nm3=0;
    float mat1[500][5], mat2[500][5], mat3[500][5];
    
    for(i=0; i<nline; i++){
        strcpy(str,lines[i]);
        if(!strncmp(str,"MTRIX1", 6)){
            parse_ncs(mat1, str,nm1);
            nm1=nm1+1;
        }else if(!strncmp(str,"MTRIX2", 6)){
            parse_ncs(mat2, str,nm2);
            nm2=nm2+1;
        }else if(!strncmp(str,"MTRIX3", 6)){
            parse_ncs(mat3, str,nm3);
            nm3=nm3+1;
        }else if(!strncmp(str,"ATOM", 4) || !strncmp(str,"HATM", 4)){
            break;
        }
    }
    if (nm1>0 && nm1==nm2 && nm2==nm3){
       
        write_ncs_tls_head(fout, "_struct_ncs_oper");
        for (i=0; i<nm1; i++){
            if (mat1[i][4] ==0){
                strcpy(ss, "generate");
            }else{
                strcpy(ss, "given");
            }
            
            fprintf(fout, "%s %d %8.6f %8.6f %8.6f %8.6f \
%8.6f %8.6f %8.6f %8.6f  %8.6f %8.6f %8.6f %8.6f \n", ss,i+1, \
                    mat1[i][0], mat1[i][1], mat1[i][2], \
                    mat2[i][0], mat2[i][1], mat2[i][2],  \
                    mat3[i][0], mat3[i][1], mat3[i][2],  \
                    mat1[i][3], mat2[i][3], mat3[i][3] );
        }

    }
    
}

void extract_pdb_ncs(char **lines, int nline)
/* extract NCS parameter from PDB head file (The new def)
   variable ncs='ncs group, dom_id' ;
   nlim='include component'
   nncs='only for refine_ls_restr_ncs'
*/
{
    int i , ncs=0, end=0;

    for(i=0; i<nline; i++){
        
        if(!strncmp(lines[i],"ATOM", 4) || !strncmp(lines[i],"HATM", 4) )break;
        
        if(strstr_space(lines[i],"REMARK   3   NUMBER OF DIFFERENT NCS GROUPS :")){ //REFMAC
            ncs = get_int_after_id(lines[i], ":");
            
            if(ncs>0){
                extract_refmac_ncs(i, nline, lines, &end);
                break;
            }
            
        }else if(strstr_space(lines[i],"REMARK   3   NUMBER OF DIFFERENT NCS PAIRS  :")){ //REFMAC 5.6.0117
            ncs = get_int_after_id(lines[i], ":");
            if(ncs>0){
                extract_refmac_ncs_new(i, nline, lines, &end);
                break;
            }
                   
        }else if(strstr_space(lines[i],"REMARK   3   NUMBER OF NCS GROUPS :") ){ //Phenix
            ncs = get_int_after_id(lines[i], " :");
            if(ncs>0) {
                extract_phenix_ncs(i, nline, lines, &end);
                break;
            }
            
        }else if(strstr_space(lines[i],"REMARK   3  NCS RESTRAINTS.          RMS   SIGMA/WEIGHT") ){ //CNS
            extract_cns_ncs(i, nline, lines, &end);
            break;
        }
        
    }
    
}

int get_beg_end_idx(char *chid, char *segid)
//get index value 
{
    int i, idx=-1;

    if(strstr(chid, "?") || strstr(chid, ".") ) {return idx;}
    
    for(i=0; i<resinfo.nch; i++){
        if( !strcmp(resinfo.resid[i],chid )){
            idx=i;
            break;
        }
    }
    
    return idx;
}
    
void get_res_segment(char *tmp, int k, int *res_beg, int *res_end, int *ncomp, char **ch)
//take out residue ranges from one chain and RETURN remaining residue ranges     
{
    char ** tok, chtmp[8], *temp;
    int i,m,n,m1,m2, ntok, n1[80], n2[80], nn=0, ntmp[200];

    replace_char(tmp, '(', ' ');
    replace_char(tmp, ')', ' ');
    if(tmp[0]!='-' && strchr(tmp, '-')) replace_char(tmp, '-', ' ');
    replace_char(tmp, ':', ' ');
    
    temp=replace_all_str(tmp,"resid", " ");
     
    strcpy(chtmp, resinfo.resid[k]);
    
    if(!strstr(temp," or ")){
        sscanf(temp,"%d%d", &n1[0],&n2[0]);
        nn=1;
    }else{
        tok=token_pattern(temp, " or ", &ntok);
        n=0;
        for(m=0; m<ntok; m++){
            sscanf(tok[m],"%d%d", &m1,&m2);
            if(m1>resinfo.resend[k] || m2>resinfo.resend[k]){
                sprintf(INFO, "Error! (%s) out of residue range for chain %s\n", tok[m], chtmp);
                pinfo(INFO,0);
                continue;
            }
            
            n1[n]=m1;
            n2[n]=m2;
            n++;
        }
        nn=n;
        if(ntok)free_memory_2d(tok, ntok);
    }
    
    m=0;
    for(i=0; i<nn; i++){
        ntmp[m]=n1[i];
        m++;
        ntmp[m]=n2[i];
        m++;
    }
    ntmp[m]=resinfo.resbeg[k];
    m++;
    ntmp[m]=resinfo.resend[k];
    m++;
    
    larray_sort(ntmp, m, 0);
    
    n=0;
    for(i=0; i<m-1; i++){
        res_beg[n]=ntmp[i];
        res_end[n]=ntmp[i+1];
        if(res_beg[n] == res_end[n]){
            i=i+1;
            continue;
        }
        
        strcpy(ch[n], chtmp);
        n++;
        i=i+1;
    }
    
    *ncomp=n;
//    for(i=0; i<n; i++){ printf("paired=%d :: %d  ch=%c\n",res_beg[i], res_end[i], ch[i]); }
}

void residue_range(char *ch, char *str, int *res_beg, int *res_end)
//get residue range after resid or resseq
{
    char tmp[256], resid[20], tmp2[256],  empty[1];
    int k=0,n1,n2,m, len;
    
    res_beg[0]=0;
    res_end[0]=0;

    strcpy(empty,"");
    if(strstr_case(str,"resid"))
        strcpy(resid, "resid");
    else if (strstr_case(str,"resseq"))
        strcpy(resid, "resseq");
    else{
        k=get_beg_end_idx(ch, empty);
        if (k>=0){
            res_beg[0]=resinfo.resbeg[k];
            res_end[0]=resinfo.resend[k];
        }
        return;
    }
    strcpy(tmp, strstr_case(str,resid)+strlen(resid));
    replace_char(tmp,'(', ' ');
    replace_char(tmp,')', ' ');
    rid_of_front_end_space(tmp);
    
    if((len=strlen(tmp))<1 || strlen(ch)<=0 ) return;
    strcpy(tmp2, tmp);
    if(tmp2[0]==':'){ // :421
        k=get_beg_end_idx(ch, empty);
        if(k>=0) res_beg[0]=resinfo.resbeg[k];
        res_end[0]=get_int_after_id(tmp,":");
        
    }else if(tmp2[len-1]==':'){// 421:
        k=get_beg_end_idx(ch, empty);
        if(k>=0) res_end[0]=resinfo.resend[k];
        
        replace_char(tmp, ':', ' ');
        if(sscanf(tmp, "%d", &n1)==1) res_beg[0]=n1;
        
    }else{
        replace_char(tmp, ':', ' ');
        if(tmp[0]!='-' && strchr(tmp, '-')) replace_char(tmp, '-', ' ');
        m=sscanf(tmp, "%d %d", &n1, &n2);
        if(m==2){
            res_beg[0]=n1;
            res_end[0]=n2;
        }else if(m==1){
            res_beg[0]=n1;
            res_end[0]=n1;
        } 
    }

}

int get_int_tnt(char *str, int key)
{
    char tmp[80];
    int n=0 ;
    
    rid_of_front_end_space(str);
    if(key==0){ //starting residue number
        if(strchr(str,' ')) {
            sscanf(str,"%*s%s", tmp);
        }else{
            strcpy(tmp, str);
        }
        
    }else{ //ending residues
        if(strchr(str,' ')) {
            sscanf(str,"%s%*s", tmp);
        }else{
            strcpy(tmp, str);
        }
    }
    
    if (strchr(tmp,'|')){
        n=get_int_after_id(tmp,"|");
    }else if(isdigit(tmp[0])){
        n = atoi(tmp);
    }
//    printf("tmp=%d :: %s\n", n, tmp);
    return n;
    
}
        
void residue_range_tnt(char *str,  int *res_beg, int *res_end, int *ncomp, char **ch)
//get residue range and chain id for buster-tnt
{
    char tmp[1000],tmp1[1000],tmp2[1000], **tok=NULL,**tok1=NULL,**tok2=NULL,empty[1];
    int i,j, m, k=0, ntok=0, ntok1=0, ntok2=0;
    
    
    *ncomp=1;
    res_beg[0]=0;
    res_end[0]=0;
    strcpy(empty, "");
    
    clean_string(str, tmp);
    if (strchr(tmp, '{') ) replace_char(tmp,'{', ' ');
    if (strchr(tmp, '}') ) replace_char(tmp,'}', ' ');
    rid_of_front_end_space(tmp);
    
    if( !strchr(tmp,'|') || strlen(tmp)<2) {
        sprintf(INFO, "\nWarning! Non-standard TLS set (%s). skipped parsing.\n", tmp);
        pinfo(INFO,0);
        return;
    }
    
    tok=token_pattern_case(tmp, " ", &ntok);
    
    m=0;
    for(i=0; i<ntok; i++){
        if (!(strstr(tok[i],"|") && strlen(tok[i])>2 )){
            continue;
        }
//        printf("note: %d %d :  %s\n", i, ntok,tok[i]);
        
        strcpy(tmp2, tok[i]);
        tok2=token_pattern_case(tmp2, "|", &ntok2);
        if(tok[i][0] == '*' && strchr(tok[i]+1,'*')){ // all chains && residues
            for(j=0; j<resinfo.nch; j++){
                strcpy(ch[m],resinfo.resid[j]);
                res_beg[m]=resinfo.resbeg[k];
                res_end[m]=resinfo.resend[k];
                m++;
            }
        }else if (isalnum(tok[i][0]) && tok[i][2] == '*') {//all atoms for chain
            if (ntok2>1){
                k=get_beg_end_idx(tok2[0], empty);
                if(k>=0) {
                    res_beg[m]=resinfo.resbeg[k];
                    res_end[m]=resinfo.resend[k];
                    strcpy(ch[m], tok2[0]);
                    m++;
                }
            }
            
        }else if (isalnum(tok[i][0]) && (tok2[1][0]== '-' || isdigit(tok2[1][0]))) {//regular case
            strcpy(ch[m],tok2[0]);
            if (ntok-i>1 && !strcmp(tok[i+1], "-")){
                res_beg[m]=get_int_tnt(tok[i], 1);
                
                strcpy(tmp, tok[i+2]); //next
                if (strstr(tmp,"|")){
                    res_end[m]=get_int_tnt(tok[i+2], 1);
                    m++;
                }else if (isdigit(tmp[0])){
                    sscanf(tmp, "%d", &res_end[m]);
                    m++;
                }else{
                    sprintf(INFO,"Warning: Residue range can not be determined for %s.\n", str);
                    pinfo(INFO,0);
                }
                        
                i=i+2;
                        
            }else{ //single residue                       
                res_beg[m]=get_int_tnt(tok[i], 1);
                res_end[m]=res_beg[m];
                m++;                        
            }

            
        }else if (isalnum(tok[i][0])  && !strstr( str, " - ")) {//One residue
            strcpy(ch[m], tok2[0]);
            if (!strchr(tok[i], '-') ||(tok[i][2]=='-' && !strchr(tok[i]+2, '-'))){
                res_beg[m]=get_int_tnt(tok[i], 1);
                res_end[m]= res_beg[m];
                m++;
              
            }else{
                strcpy(tmp1, tok[i]);
                if (strstr(tmp1,"--")){
                    tok1=token_pattern_case(tmp1, "--", &ntok1);
                    res_beg[m]=get_int_tnt(tok1[0], 1);
                    res_end[m]=-get_int_tnt(tok1[1], 1);
                    m++;
                }else{
                    if (tmp1[2]=='-'){
                        tok1=token_pattern_case(tmp1+3, "-", &ntok1);
                        res_beg[m]=-get_int_tnt(tok1[0], 1);
                        res_end[m]=get_int_tnt(tok1[1], 1);
                        m++;
                    }else{
                        tok1=token_pattern_case(tmp1, "-", &ntok1);
                        
                        res_beg[m]=get_int_tnt(tok1[0], 1);
                        res_end[m]=get_int_tnt(tok1[1], 1);
                        m++;
                    }
                }
                
            }  
        }                 
    }
    *ncomp=m;
    
    if (m==0){
        sprintf(INFO, "\nWarning! TLS set (%s) can not be parsed (check chain_ID in TLS group & coordinate).\n", str);
        pinfo(INFO,0);
        *ncomp=1;
    }
    
        
}
            

void residue_range_phenix(char *str,  int *res_beg, int *res_end, int *ncomp, char **ch)
//get residue range and chain id for phenix
{
    char **tok, tmp[51200], chtmp[60], empty[1];
    int n=0, m, ntok=0, n1=0, n2=0,  len, k=0, i;

    strcpy(empty,"");
    res_beg[0]=0;
    res_end[0]=0;
    for (i=0; i<200; i++) strcpy(ch[i], ".");
    
    if (strchr(str,'\'')) rid_of_char(str, '\'');
    if (strchr(str,'"')) rid_of_char(str, '"');
    if (strstr(str, " through "))str=replace_all_str(str, " through ", ":");
    if (strstr(str, "resseq"))str=replace_all_str(str, "resseq", "resid");

    if(strstr_case(str,"chain")) sscanf(strstr_case(str,"chain")+5, "%s", ch[0]);
    *ncomp=1;
//    printf("str=%s\n", str);
    
    if(strlen(ch[0])>2 || strlen(str)>5000) {return;}

    
    rid_of_front_end_space(str);
    len=strlen(str);

    if(strstr_case(str,"chain ") && len<10 ) { //only chain
        sscanf(strstr_case(str,"chain")+5, "%s", chtmp);
        k=get_beg_end_idx(chtmp, empty);
        strcpy(ch[0],chtmp);
        if(k<0) return;
        res_beg[0]=resinfo.resbeg[k];
        res_end[0]=resinfo.resend[k];
        *ncomp=1;
    }else if(strstr_case(str,"all")  && len<6 ){ //all 
        for(i=0; i<resinfo.nch; i++){
            res_beg[i]=resinfo.resbeg[i];
            res_end[i]=resinfo.resend[i];
            ch[i]=resinfo.resid[i];
        }
        *ncomp=resinfo.nch;

    }else if((pattern5(str, "chain ",  " and res", "", "", "") ||
              pattern5(str, "res",  " and chain ", "", "", "") ||
              pattern5(str, "chain ",  " and (", "res", "", "") )
              && len<30){//one comp.
        if(strstr_case(str,"chain")) sscanf(strstr_case(str,"chain")+5, "%s", ch[0]);
        residue_range(ch[0], str, res_beg, res_end);
        
        
    }else if((pattern5(str, "chain ", " and (", " or res", ")" , "") ||
              pattern5(str, "(", " or res", ")", " and ", "chain " ))
             && len>40){ //one chain & multi-comp.
        if(strstr_case(str,"chain")) sscanf(strstr_case(str,"chain ")+6, "%s", chtmp);
        get_data_set_id(str, "(", ")", tmp);
        
        tok=token_pattern_case(tmp, " or ", &ntok);
        n=0;
        for(m=0; m<ntok; m++){
            residue_range(chtmp, tok[m], &n1, &n2);
            res_beg[n]=n1;
            res_end[n]=n2;
            strcpy(ch[n],chtmp);
            n++;
            *ncomp=n;
        }
        if(ntok) free_memory_2d(tok, ntok);
        
    }else if(pattern5(str, "chain ", " or chain ", "", "", "") &&
             (!strstr_case(str, "and ") && !strstr_case(str, " not"))){//multi-chain
        strcpy(tmp, str);
        
        tok=token_pattern_case(tmp, " or ", &ntok);
        n=0;
        for(m=0; m<ntok; m++){
            if(strstr_case(tok[m],"chain")) sscanf(strstr_case(tok[m],"chain")+5, "%s", chtmp);
            k=get_beg_end_idx(chtmp, empty);
            if(k>=0){
                res_beg[n]=resinfo.resbeg[k];
                res_end[n]=resinfo.resend[k];
                strcpy(ch[n],chtmp);
                n++;
                *ncomp=n;
            }
        }
        if(ntok) free_memory_2d(tok, ntok);

    }else if(pattern5(str, "(", "chain ", "and res", ") or (", "chain") &&
             !strstr_case(str, " not")){//multi-chain, multi-range
        strcpy(tmp, str);
        tok=token_pattern_case(tmp, " or ", &ntok);
        n=0;
        for(m=0; m<ntok; m++){
            if(strstr_case(tok[m],"chain")) sscanf(strstr_case(tok[m],"chain")+5, "%s", chtmp);
            residue_range(chtmp, tok[m], &n1, &n2);
            
            res_beg[n]=n1;
            res_end[n]=n2;
            strcpy(ch[n],chtmp);
            n++;
            *ncomp=n;
        }
        if(ntok) free_memory_2d(tok, ntok);
        
    }else if (pattern5(str,"chain ","and not resid", "", "", "") ||
              pattern5(str,"chain ","and not (resid", "", "", "")){ //chain & not resid
        if(strstr_case(str,"chain")) sscanf(strstr_case(str,"chain")+5, "%s", chtmp);
        k=get_beg_end_idx(chtmp, empty);
        if(k<0){
            sprintf(INFO, "Error! No chainID is found (%s)\n",str);
            pinfo(INFO,0);
            
            return;
        }
        strcpy(tmp,strstr_case(str, "and not ")+8);
        get_res_segment(tmp, k, res_beg, res_end, ncomp,ch);
    }
}


int seq_num(FILE *ftmp, char ch, char *seq1, char *seq2)
//return seq1 & seq2 using ch & ftmp
{
    
    char str1[80],ch1='?';
    int n1=0, n2=0;
    
    rewind(ftmp);
    while(fgets(str1, sizeof str1, ftmp)!=NULL){
        
        n1=sscanf(str1,"%c %s%s",&ch1, seq1,seq2);
        if(n1==3 && ch==ch1) {
            n2=1;
            break;
        }
    }
    if(n2==0) {
        sprintf(INFO, "Error! The chain ID (%c) in TLS does not match coordinate chain ID\n",ch);
        pinfo(INFO, 0);
    }
    
    return n2;
}


void extract_refine_ls_restr(char **lines, int nline, char *prog)
//RMS DEVIATIONS FROM IDEAL VALUES  (refine_ls_restr)    
{
    char str[512], tmp[512], f1[40], f2[40];
    int  nr=0, b1=-9999, b2=-9999, i, j;
    float w1=-9999,w2=-9999;

    
    strcpy(f1,"?");
    strcpy(f2,"?");
    
        
    for(i=0; i<nline; i++){
        strcpy(str, lines[i]);
        
        if(strstr_space(str, "REMARK   3")<=0)continue;
        if(!strncmp_case(str, "ATOM  ", 6) || !strncmp_case(str, "HETATM", 6) ) break;

        if(strstr_case(prog, "refmac") || strstr_case(prog, "restrain")){ //refmac5            
            if(strstr_space(str, "RMS DEVIATIONS FROM IDEAL VALUES   COUNT    RMS    WEIGHT")|| 
               strstr_space(str, "ISOTROPIC THERMAL FACTOR RESTRAINTS.  COUNT  RMS   WEIGHT")||
               strstr_space(str, "ANISOTROPIC THERMAL FACTOR RESTRAINTS.    COUNT   RMS   WEIGHT")){
            
                for(j=i+1; j<nline; j++){
                    strcpy(str, lines[j]);
                    if(pattern5(str,"REMARK   3",  "):", ";", ";","")){
                        strcpy(refine_ls_restr[nr].type, get_substr(str,"REMARK   3", ":"));
                    
                        strcpy(tmp, strstr(str,"):"));                    
                        refine_ls_restr[nr].number = get_int(tmp, "):", ";");
                        refine_ls_restr[nr].dev_ideal = get_float(tmp, ";", ";");
                        refine_ls_restr[nr].dev_ideal_target  = get_float(strstr(tmp,";")+1, ";", "\n");
                        refine_ls_restr[nr].weight  =  -9998;
                        nr++;
                        NR=nr;
                        
                    }else{
                        i=j+1;
                        break;
                    }
                }
            }else if(strstr_space(str, "DISTANCE RESTRAINTS.   RMS    SIGMA")|| //rare case
                     strstr_space(str, "ISOTROPIC THERMAL FACTOR RESTRAINTS.    RMS    SIGMA" )){
                
                for(j=i+1; j<nline; j++){
                    strcpy(str, lines[j]);
                    if(pattern5(str, "REMARK   3","(", ")", ":", ";")){
                        strcpy(refine_ls_restr[nr].type, get_substr(str,"REMARK   3", " :"));
                        refine_ls_restr[nr].number = INI;
                        refine_ls_restr[nr].dev_ideal = get_float(str, ":", ";");
                        refine_ls_restr[nr].dev_ideal_target  = -9998;
                        refine_ls_restr[nr].weight  = get_float(str, ";", "\n");
                        nr++;
                        NR=nr;
                    }else{
                        break;
                    }
                }
            }
            
        }else if(strstr_case(prog, "shelx")){ //shelx
            if(strstr_space(str, " RMS DEVIATIONS FROM RESTRAINT TARGET VALUES.")){ 
                
                for(j=i+1; j<nline; j++){
                    strcpy(str, lines[j]);
                    if(pattern5(str, "REMARK ",  " 3 ", " (", ") ",":")){

                        strcpy(refine_ls_restr[nr].type, get_substr(str,"REMARK   3", ":"));
                        refine_ls_restr[nr].number = INI;
                        refine_ls_restr[nr].dev_ideal = get_float(str, ":", "\n");
                        refine_ls_restr[nr].weight  = INI;
                        nr++;
                        NR=nr;
                    }else{
                        break;
                    }  
                }
            }
        }else if(strstr_case(prog, "buster")){ //
            if(strstr_space(str, "REMARK   3   TERM                          COUNT    WEIGHT   FUNCTION")||
               strstr_space(str, "REMARK   3   RMS DEVIATIONS FROM IDEAL VALUES.")){ 
                
                for(j=i+1; j<nline; j++){
                    strcpy(str, lines[j]);
                    if(pattern5(str, "REMARK ",  " 3 ", " : ", ";",";")){
                        
                        if(strstr_space(str,"BOND LENGTHS              :")){
                            b1 = get_int(str, ":", ";");
                            w1 = get_float(str, ";", ";");
                            strcpy(f1, str+59);
                            rid_of_space(f1);
                               

                            continue;
                        }else if(strstr_space(str,"BOND ANGLES               :")){
                            b2 = get_int(str, ":", ";");
                            w2 = get_float(str, ";", ";");
                            strcpy(f2, str+59);
                            rid_of_space(f2);
                            
                            continue;
                        }
                        
                        
                        strcpy(refine_ls_restr[nr].type, get_substr(str,"REMARK   3", ":"));
                        refine_ls_restr[nr].number = get_int(str, ":", ";");
                        refine_ls_restr[nr].weight =  get_float(str, ";", ";");
                        
                        if(!strstr(str+59, "NULL"))strcpy(refine_ls_restr[nr].pdbx_restraint_function, str+59);
                        rid_of_space(refine_ls_restr[nr].pdbx_restraint_function);
                        
                        nr++;
                        NR=nr;
                        
                    }else if(pattern5(str, "REMARK ",  " 3 ", " (", ") ",":")){

                        strcpy(refine_ls_restr[nr].type, get_substr(str,"REMARK   3", ":"));
                        refine_ls_restr[nr].dev_ideal = get_float(str, ":", "\n");
                        
                        if(strstr_space(str,"BOND LENGTHS")){
                            refine_ls_restr[nr].number = b1;
                            refine_ls_restr[nr].weight = w1;
                            strcpy(refine_ls_restr[nr].pdbx_restraint_function, f1);
                        }else if (strstr_space(str,"BOND ANGLES")){
                            refine_ls_restr[nr].number = b2;
                            refine_ls_restr[nr].weight = w2;
                            strcpy(refine_ls_restr[nr].pdbx_restraint_function, f2);
                        }
                        
//                        printf("srdmg |%s| : %d %.2f %d %.2f %s\n",  refine_ls_restr[nr].type,b1,w1,b2,w2, str );
                        
                        nr++;
                        NR=nr;
                    }else{
                        break;
                    }  
                }
            }
        }else if(strstr_case(prog, "cns") || strstr_case(prog, "plor")||
                 strstr_case(prog, "cnx")){ //CNS/XPLOR/CNX
            if(strstr_space(str, "ISOTROPIC THERMAL FACTOR RESTRAINTS.  RMS  SIGMA")){ 
                
                for(j=i+1; j<nline; j++){
                    strcpy(str, lines[j]);
                    if(pattern5(str, "(", ") ", ":", ";", "\n")){

                        strcpy(refine_ls_restr[nr].type, "?");
                        strcpy(refine_ls_restr[nr].type, get_substr(str,"REMARK   3", ":"));
                        refine_ls_restr[nr].number = INI;
                        refine_ls_restr[nr].dev_ideal = get_float(str, ":", ";");
                        refine_ls_restr[nr].dev_ideal_target  =get_float(str, ";", "\n") ;
//                        refine_ls_restr[nr].weight  =get_float(str, ";", "\n") ;
                        nr++;
                        NR=nr;
                    }else{
                        break;
                    }
                }
            
            }else if(strstr_space(str, "REMARK   3  RMS DEVIATIONS FROM IDEAL VALUES.")){ 
                
                for(j=i+1; j<nline; j++){
                    strcpy(str, lines[j]);
                    if(pattern5(str, "(", ") ", ":", "", "")){

                        strcpy(refine_ls_restr[nr].type, "?");
                        strcpy(refine_ls_restr[nr].type, get_substr(str,"REMARK   3", ":"));
                        refine_ls_restr[nr].number = INI;
                        refine_ls_restr[nr].dev_ideal = get_float(str, ":", "\n");
                        refine_ls_restr[nr].weight  =INI ;
                        nr++;
                        NR=nr;
                    }else{
                        break;
                    }
                }
            }
        }else if(strstr_case(prog, "tnt")){ //TNT
            if(strstr_space(str, "REMARK   3  RMS DEVIATIONS FROM IDEAL VALUES") &&
               strstr_space(str, " RMS    WEIGHT  COUNT")){ 
                
                for(j=i+1; j<nline; j++){
                    strcpy(str, lines[j]);
                    if(pattern5(str, "REMARK", ":", ";", ";", "\n")){
                        

                        strcpy(refine_ls_restr[nr].type, "?");
                        strcpy(refine_ls_restr[nr].type, get_substr(str," 3 ", ":"));
                        refine_ls_restr[nr].dev_ideal = get_float(str, ":", ";");
                        refine_ls_restr[nr].weight  = get_float(str, ";", ";") ;
                        refine_ls_restr[nr].number = get_int(strstr_case(str,";")+1, ";", "\n");
                        nr++;
                        NR=nr;
                    }else{
                        break;
                    }  
                }
            }
            
        } else if(strstr_case(prog, "NUCLSQ")|| strstr_case(prog, "PROLSQ")){ //NUCLSQ
            if(strstr_space(str, " DISTANCE RESTRAINTS.      RMS     SIGMA")||
               strstr_space(str, " NON-BONDED CONTACT RESTRAINTS.") ||
               strstr_space(str, "ISOTROPIC THERMAL FACTOR RESTRAINTS.  RMS  SIGMA")){

                for(j=i+1; j<nline; j++){
                    strcpy(str, lines[j]);
                    if(pattern5(str, "REMARK", ") ", ":", ";",  "\n")){

                        strcpy(refine_ls_restr[nr].type, "?");
                        strcpy(refine_ls_restr[nr].type, get_substr(str," 3 ", ":"));
                        refine_ls_restr[nr].dev_ideal = get_float(str, ":", ";");
                        refine_ls_restr[nr].weight  = get_float(str, ";", "\n") ;
                        refine_ls_restr[nr].number = INI;
                        nr++;
                        NR=nr;
                    }else{
                        break;
                    }
                }
                
            } else if(strstr_space(str, "REMARK   3   PLANE RESTRAINT  (A) :")||  
                      strstr_space(str, "REMARK   3   CHIRAL-CENTER RESTRAINT   (A**3) :")){
            
                strcpy(refine_ls_restr[nr].type, "?"); 
                strcpy(refine_ls_restr[nr].type, get_substr(str," 3 ", ":"));
                refine_ls_restr[nr].dev_ideal = get_float(str, ":", ";");
                refine_ls_restr[nr].weight  = get_float(str, ";", "\n") ;
                refine_ls_restr[nr].number = INI;
                nr++;
                NR=nr;
            } 
        }
        
    }
    
}

 

void extract_pdb_head(char **lines, int nline)
{
    char str[512], tmp[512];
    int i,j, nr=0, ntop=0, nparam=0;
    
    refine.pdbx_refine_id=(char*)"X-RAY DIFFRACTION";
    
    strcpy(PROG, "?");
    for(i=0; i<nline; i++){
        strcpy(str, lines[i]);
        if(strstr_case(str,": NULL")) continue;
        
        if( !strncmp_case(str, "ATOM", 4) || !strncmp_case(str, "HETATM", 6)) break;
        
            //DATA USED IN REFINEMENT.  (refine.)
        if(strstr_space(str, "REMARK 3  RESOLUTION RANGE HIGH (ANGSTROMS) :")){
            refine.ls_d_res_high = get_float(str, ":", "\n");
//            refine_hist.d_res_high = refine.ls_d_res_high;
                
        }else if (strstr_space(str, "REMARK 3  RESOLUTION RANGE LOW  (ANGSTROMS) :")){
            refine.ls_d_res_low = get_float(str, ":", "\n");
//            refine_hist.d_res_low = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  DATA CUTOFF            (SIGMA(F)) :")||
                  strstr_space(str, "REMARK   3   MIN(FOBS/SIGMA_FOBS)              :")){
            if(strstr_space(str, ": NONE")){
                refine.pdbx_ls_sigma_F = 0.0;
            }else
                refine.pdbx_ls_sigma_F = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3   DATA CUTOFF HIGH (ABS(F)) :")){
            refine.pdbx_data_cutoff_high_absF = get_int_after_id(str, ":");
            
                
        }else if (strstr_space(str, "REMARK 3  DATA CUTOFF LOW (ABS(F)) :")){
            refine.pdbx_data_cutoff_low_absF = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3 COMPLETENESS FOR RANGE    (%) :") ||
                  strstr_space(str, "REMARK 3 COMPLETENESS (WORKING+TEST) (%) :")){
            refine.ls_percent_reflns_obs = get_float(str, ":", "\n");
                     
        }else if (strstr_space(str, "REMARK 3 NUMBER OF REFLECTIONS             :")){
            refine.ls_number_reflns_obs = get_int(str, ":", "\n");
/*            
            if(strstr_case(PROG, "REFMAC")){
                refine.ls_number_reflns_R_work= get_int(str, ":", "\n");
            }else{
                refine.ls_number_reflns_obs = get_int(str, ":", "\n");
            }
*/
        }else if (strstr_space(str, "REMARK 3 TOTAL NUMBER OF REFLECTIONS   (NO CUTOFF) :")){
            refine.ls_number_reflns_all = get_int(str, ":", "\n");
            refine.ls_number_reflns_obs = refine.ls_number_reflns_all;
            
                
                
 //FIT TO DATA USED IN REFINEMENT. (refine.)
        }else if (strstr_space(str, "REMARK 3 CROSS-VALIDATION METHOD       :") ){
            refine.pdbx_ls_cross_valid_method = get_substr(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  FREE R VALUE TEST SET SELECTION  :")){
            refine.pdbx_R_Free_selection_details =get_substr(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  R VALUE     (WORKING + TEST SET) :")){
            if(refine.pdbx_ls_sigma_F >0.001)
                refine.ls_R_factor_obs = get_float(str, ":", "\n");
            else
//                refine.ls_R_factor_all = get_float(str, ":", "\n");
                refine.ls_R_factor_obs = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3 R VALUE      (WORKING SET) :") ){
            refine.ls_R_factor_R_work = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  FREE R VALUE         :")){
            refine.ls_R_factor_R_free = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  FREE R VALUE TEST SET SIZE   (%) :")){
            refine.ls_percent_reflns_R_free = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  FREE R VALUE TEST SET COUNT      :")){
            refine.ls_number_reflns_R_free = get_int(str, ":", "\n");
            
        }else if (strstr_space(str, "REMARK 3   ESTIMATED ERROR OF FREE R VALUE :")){
            refine.ls_R_factor_R_free_error = get_float(str, ":", "\n");

            
// FIT TO DATA USED IN REFINEMENT (NO CUTOFF)._pdbx_refine.          
        }else if (strstr_space(str, "REMARK 3  R VALUE (WORKING + TEST SET, NO CUTOFF) :")){
            pdbx_refine.R_factor_all_no_cutoff = get_float(str, ":", "\n");
            if(strstr_case(PROG, "SHELX")){
                
                refine.ls_R_factor_all = pdbx_refine.R_factor_all_no_cutoff;
                refine.ls_R_factor_obs = pdbx_refine.R_factor_all_no_cutoff;
            }
            
        }else if (strstr_space(str, "REMARK 3 R VALUE          (WORKING SET, NO CUTOFF) :")){
            pdbx_refine.R_factor_obs_no_cutoff = get_float(str, ":", "\n");
            if(strstr_case(PROG, "SHELX")){
                refine.ls_R_factor_obs = pdbx_refine.R_factor_obs_no_cutoff;
                refine.ls_R_factor_R_work = pdbx_refine.R_factor_obs_no_cutoff;
            }
            
        }else if (strstr_space(str, "REMARK 3  FREE R VALUE           (NO CUTOFF) :")){
            pdbx_refine.free_R_factor_no_cutoff = get_float(str, ":", "\n");
            if(strstr_case(PROG, "SHELX"))
                refine.ls_R_factor_R_free = pdbx_refine.free_R_factor_no_cutoff;
            
        }else if ( strstr_space(str, "REMARK 3  FREE R VALUE TEST SET SIZE (%, NO CUTOFF) :")){
            pdbx_refine.free_R_val_test_set_size_perc_no_cutoff = get_float(str, ":", "\n");
            if(strstr_case(PROG, "SHELX"))
                refine.ls_percent_reflns_R_free =pdbx_refine.free_R_val_test_set_size_perc_no_cutoff;
            
        }else if (strstr_space(str, "REMARK 3  FREE R VALUE TEST SET COUNT   (NO CUTOFF) :")){
            pdbx_refine.free_R_val_test_set_ct_no_cutoff = get_int(str, ":", "\n");
            if(strstr_case(PROG, "SHELX"))
                refine.ls_number_reflns_R_free=pdbx_refine.free_R_val_test_set_ct_no_cutoff;

        }else if (strstr_space(str, "REMARK   3   ESTIMATED ERROR OF FREE R VALUE (NO CUTOFF) :")){
            pdbx_refine.free_R_error_no_cutoff = get_float_after_id(str, ":");
            
        }else if (strstr_space(str, "REMARK   3   TOTAL NUMBER OF REFLECTIONS   (NO CUTOFF) :")){
            pdbx_refine.number_reflns_obs_no_cutoff = get_int(str, ":", "\n");
            if(strstr_case(PROG, "SHELX")){
                refine.ls_number_reflns_all = pdbx_refine.number_reflns_obs_no_cutoff;
                refine.ls_number_reflns_obs = pdbx_refine.number_reflns_obs_no_cutoff;
            }
            
// FIT/AGREEMENT OF MODEL FOR DATA WITH F>4SIG(F).

        }else if (strstr_space(str, "REMARK   3   R VALUE   (WORKING + TEST SET, F>4SIG(F)) :")){
            pdbx_refine.R_factor_all_4sig_cutoff = get_float(str, ":", "\n");
            
        }else if (strstr_space(str, "REMARK   3   R VALUE          (WORKING SET, F>4SIG(F)) :")){
            pdbx_refine.R_factor_obs_4sig_cutoff = get_float(str, ":", "\n");
            
        }else if (strstr_space(str, "REMARK   3   FREE R VALUE                  (F>4SIG(F)) :")){
            pdbx_refine.free_R_factor_4sig_cutoff = get_float(str, ":", "\n");
            
        }else if (strstr_space(str, "REMARK   3   FREE R VALUE TEST SET SIZE (%, F>4SIG(F)) :")){
            pdbx_refine.free_R_val_test_set_size_perc_4sig_cutoff = get_float(str, ":", "\n");
            
        }else if (strstr_space(str, "REMARK   3   FREE R VALUE TEST SET COUNT   (F>4SIG(F)) :")){
            pdbx_refine.free_R_val_test_set_ct_4sig_cutoff = get_int(str, ":", "\n");
            
        }else if (strstr_space(str, "REMARK   3   TOTAL NUMBER OF REFLECTIONS   (F>4SIG(F)) :")){
            pdbx_refine.number_reflns_obs_4sig_cutoff = get_int(str, ":", "\n");

// B VALUES.(refine.)
        }else if (strstr_space(str, "REMARK 3  FROM WILSON PLOT    (A**2) :") ||
                  strstr_space(str, "REMARK 3  WILSON B VALUE (FROM FCALC, A**2) :" )){
            reflns.B_iso_Wilson_estimate = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  MEAN B VALUE      (OVERALL, A**2) :")){
            refine.B_iso_mean = get_float(str, ":", "\n");
            
        }else if (strstr_space(str, "REMARK 3  B11 (A**2) :") || strstr_space(str, "REMARK   3    B11 :")){
            refine.aniso_B11 = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  B22 (A**2) :") || strstr_space(str, "REMARK   3    B22 :")){
            refine.aniso_B22 = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  B33 (A**2) :") || strstr_space(str, "REMARK   3    B33 :")){
            refine.aniso_B33 = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  B12 (A**2) :") || strstr_space(str, "REMARK   3    B12 :")){
            refine.aniso_B12 = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  B13 (A**2) :") || strstr_space(str, "REMARK   3    B13 :")){
            refine.aniso_B13 = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  B23 (A**2) :") || strstr_space(str, "REMARK   3    B23 :")){
            refine.aniso_B23 = get_float(str, ":", "\n");

 //CORRELATION COEFFICIENTS.(refine.)

        }else if (strstr_space(str, "REMARK 3  CORRELATION COEFFICIENT FO-FC      :")){
            refine.correlation_coeff_Fo_to_Fc = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  CORRELATION COEFFICIENT FO-FC FREE      :")){
            refine.correlation_coeff_Fo_to_Fc_free = get_float(str, ":", "\n");
                

 // ESTIMATED OVERALL COORDINATE ERROR. (refine.)
        }else if (strstr_space(str, "REMARK 3  ESU BASED ON R VALUE       (A):")){
            refine.pdbx_overall_ESU_R  = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  ESU BASED ON FREE R VALUE   (A):")){
            refine.pdbx_overall_ESU_R_Free  = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  ESU BASED ON MAXIMUM LIKELIHOOD    (A):  ") ||
                  strstr_space(str, "REMARK   3   COORDINATE ERROR (MAXIMUM-LIKELIHOOD BASED)     :")){
            refine.overall_SU_ML  = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK   3   PHASE ERROR (DEGREES, MAXIMUM-LIKELIHOOD BASED) : ") ){
            refine.pdbx_overall_phase_error  = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  ESU FOR B VALUES BASED ON MAXIMUM LIKELIHOOD (A**2):")){
            refine.overall_SU_B = get_float(str, ":", "\n");
            


        }else if (strstr_space(str, "REMARK   3   DPI (BLOW EQ-10) BASED ON R VALUE        (A) :")){
            refine.pdbx_overall_SU_R_Blow_DPI = get_float(str, ":", "\n");

        }else if (strstr_space(str, "REMARK   3   DPI (BLOW EQ-9) BASED ON FREE R VALUE    (A) :") ||
                  strstr_space(str,"REMARK   3   BLOW DPI ALL ATOMS (EQN 7)  : " )){
            refine.pdbx_overall_SU_R_free_Blow_DPI = get_float(str, ":", "\n");

        }else if (strstr_space(str, "REMARK   3   DPI (CRUICKSHANK) BASED ON R VALUE       (A) :")){
            refine.overall_SU_R_Cruickshank_DPI = get_float(str, ":", "\n");

        }else if (strstr_space(str, "REMARK   3   DPI (CRUICKSHANK) BASED ON FREE R VALUE  (A) :") ||
                  strstr_space(str, "REMARK   3   CRUICKSHANK DPI ALL ATOMS (EQN 27)  :")){
            refine.pdbx_overall_SU_R_free_Cruickshank_DPI = get_float(str, ":", "\n");

            

 // BULK SOLVENT MODELLING.(refine.)
        }else if (strstr_space(str, "REMARK   3   METHOD USED :")){
            refine.solvent_model_details=get_substr(str, ":", "\n");

        }else if (strstr_space(str, "REMARK   3   KSOL  :") || strstr_space(str, "REMARK   3   K_SOL   :") || strstr_space(str, "REMARK   3   K_SOL:")  ){
            refine.solvent_model_param_ksol  = get_float(str, ":", "\n");

        }else if (strstr_space(str, "REMARK   3    BSOL    :") || strstr_space(str, "REMARK   3    B_SOL  :") || strstr_space(str, "REMARK   3    B_SOL:")){
            refine.solvent_model_param_bsol  = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  VDW PROBE RADIUS   :") || strstr_space(str, "REMARK   3   SOLVENT RADIUS     :")){
            refine.pdbx_solvent_vdw_probe_radii  = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  ION PROBE RADIUS   :")){
            refine.pdbx_solvent_ion_probe_radii  = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3   SHRINKAGE RADIUS   :")){
            refine.pdbx_solvent_shrinkage_radii  = get_float(str, ":", "\n");

        }else if (strstr_space(str, "REMARK   3  ISOTROPIC THERMAL MODEL  :")){
            refine.pdbx_isotropic_thermal_model= get_substr(str, ":", "\n");

                
        }else if (strstr_space(str, "REMARK   3 STEREOCHEMISTRY TARGET VALUES : ")){ //SHELX
            refine.pdbx_stereochemistry_target_values= get_substr(str, ":", "\n");
        }else if (strstr_space(str, "REMARK   3  REFINEMENT TARGET :")){ //CNS/REFMAC5
            refine.pdbx_stereochemistry_target_values= get_substr(str, ":", "\n");
        }else if (strstr_space(str, "REMARK   3  STEREOCHEMISTRY :")){ //TNT
            refine.pdbx_stereochemistry_target_values= get_substr(str, ":", "\n");
            
                
        }else if (strstr_space(str, "REMARK   3  SPECIAL CASE:")){
            refine.pdbx_stereochem_target_val_spec_case = get_substr(str, ":", "\n"); 
        }else if (strstr_space(str, "REMARK 200 METHOD USED TO DETERMINE THE STRUCTURE: ")){
            strcpy(refine.pdbx_method_to_determine_struct , get_substr(str, ":", "\n"));
        }else if (strstr_space(str, "REMARK   3  NUMBER OF LEAST-SQUARES PARAMETERS :")){
            refine.ls_number_parameters= get_int(str, ":", "\n");
        }else if (strstr_space(str, "REMARK   3  NUMBER OF RESTRAINTS    :")){
            refine.ls_number_restraints= get_int(str, ":", "\n");
        }else if (strstr_space(str, "REMARK   3  OTHER REFINEMENT REMARKS:")){
            get_data_set_id(str, ":", "\n", tmp);
            rid_of_front_end_space(tmp);
            if(strcmp_case(tmp,"NULL")) {
                strcpy(refine.details ,tmp);
                
                for(j=i+1; j<nline; j++){
                    if(!strncmp(lines[j], "REMARK   3", 10)){
                        strcat(refine.details,  " ");
                        get_data_set_id(lines[j], "REMARK   3 ", "\n", tmp);
                        rid_of_front_end_space(tmp);
                        
                        strcat(refine.details, tmp );
//                        strcat(refine.details, "\n");
                        
                    }else{
                        i=j;
                        break;
                    } 
                }
            }else{
                strcpy(refine.details , "");
            }
            
      

        }else if (!strncmp_case(str, "REMARK ", 7) && (strstr_space(str, "refinement resolution:")||
                                                      strstr_space(str, "REMARK map resolution:") )){ //cns_non-standard
            refine.ls_d_res_low = get_float(str, ":", "-");
            refine.ls_d_res_high = get_float(str, "-", "\n");
            
        }else if (!strncmp_case(str, "REMARK ", 7) && strstr_space(str, " number of reflections rejected:")){//cns_non-standard
            refine.pdbx_ls_sigma_F = get_float(str, ":", "( ");   

        }else if (!strncmp_case(str, "REMARK ", 7) && strstr_space(str, "total number of reflections used:") ){//cns_non-standard
            refine.ls_percent_reflns_obs = get_float(str, " ( ", " % ");
            refine.ls_number_reflns_obs = get_int(str, ":", "(");

        }else if (!strncmp_case(str, "REMARK ", 7) && strstr_space(str, "number of reflections in working set:") ){//cns_non-standard
            refine.ls_number_reflns_R_work = get_int(str, ":", "(");

        }else if (!strncmp_case(str, "REMARK ", 7) && strstr_space(str, "number of reflections in test set:") ){//cns_non-standard
            refine.ls_number_reflns_R_free = get_int(str, ":", "(");
            refine.ls_percent_reflns_R_free = get_float(str, "( ", "%");
            
        }else if (!strncmp_case(str, "REMARK ", 7) && strstr_space(str, " final    r= ") &&
                  pattern5(str,"r=", "free_r=", "", "", "")){//cns_non-standard
            refine.ls_R_factor_R_work = get_float(str, "r=", "free_r=");
            refine.ls_R_factor_R_free = get_float(str, "free_r=", "\n");


        }else if (strstr_space(str,"REMARK  BIN RESOLUTION RANGE  COMPL.  NWORK   NFREE  RWORK  RFREE")){//bin
            float resl, resh, comp, rw, rf; int  nf,nfr, nn=0, m, nbin;
            nbin=0;
            
            for(j=i+1; j<nline; j++){
                if(pattern5(lines[j], "REMARK", ".", " - ", ".",".")){
                    
                    nn=sscanf(lines[j]+8, "%*s %f %*s %f %f %d %d %f %f",
                              &resl ,&resh, &comp, &nf, &nfr, &rw, &rf );
                    
                    if((nbin>0 && nn!=7 ) || !strstr(lines[j]," - ")){
                        i=j;
                        break;
                    }
                    
                    if(nbin>=NBIN-1){
                        sprintf(INFO, "Warning! The too many resolution bins\n");
                        pinfo(INFO, 0);
                        nbin=0;
                        break;
                    }
                    
                    if(nn==7){
                        refine_ls_shell.d_res_high[nbin] =resh;
                        refine_ls_shell.d_res_low[nbin] = resl;
                        if(comp<=1.0)comp=100*comp;
                        refine_ls_shell.percent_reflns_obs[nbin] = comp;
                        refine_ls_shell.R_factor_R_work[nbin] = rw;               
                        refine_ls_shell.R_factor_R_free[nbin] = rf;               
                        refine_ls_shell.number_reflns_R_work[nbin] =nf;
                        refine_ls_shell.number_reflns_R_free[nbin] =nfr;
                        refine_ls_shell.pdbx_refine_id[nbin] = (char*)"X-RAY DIFFRACTION";
                        nbin++;
                    }
                }
            }
            if(nbin>0){
                refine_ls_shell.nbin=nbin;
                for(m=0; m<nbin; m++){
                    refine_ls_shell.pdbx_total_number_of_bins_used[m]=nbin;
                }
            }
            
            
        }else if (pattern5(str, "REMARK ", "operator=", "fraction=", "", "")){//cns_non-standard
            NTWIN=1;
            get_data_set_id(str,"operator="," twinning", tmp);
            rid_of_front_end_space(tmp);
            strcpy(pdbx_twin[0].operators, tmp);
            pdbx_twin[0].fraction=get_float_after_id(str, "fraction=");
        }else if (strstr_space(str, "REMARK   3   FRACTION:" ) ){//phenix
            NTWIN=1;
            pdbx_twin[0].fraction=get_float_after_id(str, ":");
            if(i<nline-1 && strstr_case(lines[i+1], "OPERATOR:")){
                get_data_set_id(lines[i+1],":","\n", tmp);
                rid_of_front_end_space(tmp);
                strcpy(pdbx_twin[0].operators, tmp);
            }
        }else if (!strncmp_case(str, "REMARK ", 7) && strstr_space(str, "NUMBER OF TWIN DOMAINS  :")){//REFMAC5.5
            int ntwin=0;
            strcpy(tmp, "");
            for(j=i+1; j<nline; j++){
                if(!pattern5(lines[j], "REMARK ", "3 ", "TWIN", ":", "") || strlen(lines[j])<20){
                    i=j;
                    break;
                }
                
                if(strstr_space(lines[j], "TWIN OPERATOR :")){
                    get_data_set_id(lines[j], ":", "\n", tmp);
                    rid_of_front_end_space(tmp);
                }else if(strstr_space(lines[j], "TWIN FRACTION :")){
                    pdbx_twin[ntwin].fraction=get_float_after_id(lines[j], ":");
                    strcpy(pdbx_twin[ntwin].operators, tmp);
                    pdbx_twin[ntwin].domain_id=ntwin+1;
                    ntwin++;
                }
            }
            NTWIN=ntwin;
            
        }else if (!strncmp_case(str, "REMARK ", 7) && strstr_space(str, "REMARK final    twinned r=") &&
                  pattern5(str,"r=", "twinned free_r", "", "", "")){//cns_non-standard
            refine.ls_R_factor_R_work = get_float(str, "r=", "twinned");
            refine.ls_R_factor_R_free = get_float(str, "free_r=", "\n");


        }else if (!strncmp_case(str, "REMARK ", 7) && pattern5(str, "sg=","a=", "b=","c=","gamma=") ){
            
                
            get_data_set_id(str,"sg=", "a=", symmetry.space_group_name_H_M);
            if(strchr(symmetry.space_group_name_H_M,'('))
                replace_char(symmetry.space_group_name_H_M,'(', ' ');
            if(strchr(symmetry.space_group_name_H_M,')'))
                replace_char(symmetry.space_group_name_H_M,')', ' ');
            
            rid_of_front_end_space(symmetry.space_group_name_H_M);


            cell.length_a = get_float_after_id(str,"a=");
            cell.length_b = get_float_after_id(str,"b=");
            cell.length_c = get_float_after_id(str,"c=") ;
            cell.angle_alpha = get_float_after_id(str,"alpha=");
            cell.angle_beta = get_float_after_id(str,"beta=");
            cell.angle_gamma = get_float_after_id(str,"gamma=");

            
        }else if (!strncmp_case(str, "REMARK ", 7) &&
                  pattern5(str, " rmsd bonds=", "rmsd angles=", "","","")){
            
            
            strcpy(refine_ls_restr[nr].type, "?");
            strcpy(refine_ls_restr[nr].type, "c_bond_d");
            refine_ls_restr[nr].number = INI;
            refine_ls_restr[nr].dev_ideal = get_float(str, "bonds=", "rmsd angles=");
            
            refine_ls_restr[nr].weight  =INI;
            nr++;
            NR=nr;
            
            strcpy(refine_ls_restr[nr].type, "?");
            strcpy(refine_ls_restr[nr].type, "c_angle_d");
            refine_ls_restr[nr].number = INI;
            refine_ls_restr[nr].dev_ideal = get_float(str,  "rmsd angles=", "\n");
            refine_ls_restr[nr].weight  =INI;
            nr++;
            NR=nr;
            
        }else if (!strncmp_case(str, "REMARK ", 7) &&
                  pattern5(str, "B rmsd for bonded mainchain", "atoms=", "target=","","")){
            strcpy(refine_ls_restr[nr].type, "?");
            strcpy(refine_ls_restr[nr].type, "c_mcbond_it");
            refine_ls_restr[nr].number = INI;
            refine_ls_restr[nr].dev_ideal = get_float(str, "atoms=", "target");
            refine_ls_restr[nr].dev_ideal_target  = get_float(str,  "target=", "\n");
            if(refine_ls_restr[nr].dev_ideal <0) continue;
            
            nr++;
            NR=nr;
            
        }else if (!strncmp_case(str, "REMARK ", 7) &&
                  pattern5(str, "B rmsd for bonded sidechain", "atoms=", "target=","","")){
            strcpy(refine_ls_restr[nr].type, "?");
            strcpy(refine_ls_restr[nr].type, "c_scbond_it");
            refine_ls_restr[nr].number = INI;
            refine_ls_restr[nr].dev_ideal = get_float(str, "atoms=", "target");
            refine_ls_restr[nr].dev_ideal_target  = get_float(str,  "target=", "\n");
            if(refine_ls_restr[nr].dev_ideal <0) continue;
            
            nr++;
            NR=nr;
            
        }else if (!strncmp_case(str, "REMARK ", 7) &&
                  pattern5(str, "B rmsd for angle mainchain", "atoms=", "target=","","")){
            strcpy(refine_ls_restr[nr].type, "?");
            strcpy(refine_ls_restr[nr].type, "c_mcangle_it");
            refine_ls_restr[nr].number = INI;
            refine_ls_restr[nr].dev_ideal = get_float(str, "atoms=", "target");
            refine_ls_restr[nr].dev_ideal_target = get_float(str,  "target=", "\n");
            if(refine_ls_restr[nr].dev_ideal <0) continue;
            nr++;
            NR=nr;
        }else if (!strncmp_case(str, "REMARK ", 7) &&
                  pattern5(str, "B rmsd for angle sidechain", "atoms=", "target=","","")){
            strcpy(refine_ls_restr[nr].type, "?");
            strcpy(refine_ls_restr[nr].type, "c_scangle_it");
            refine_ls_restr[nr].number = INI;
            refine_ls_restr[nr].dev_ideal = get_float(str, "atoms=", "target");
            refine_ls_restr[nr].dev_ideal_target  = get_float(str,  "target=", "\n");
            if(refine_ls_restr[nr].dev_ideal <0) continue;
            nr++;
            NR=nr;
            
            
        }else if (!strncmp_case(str, "REMARK ", 7) &&
                   (pattern5(str, " parameter file", " : ", "","","") ||
                  pattern5(str,"REMARK   3  PARAMETER FILE",  " : ", "","","") )){
            
            pdbx_xplor_file[nparam].param_file = get_substr(str," : ", "\n");
            
            pdbx_xplor_file[nparam].nparam = nparam;
            nparam++;
            
        }else if (!strncmp_case(str, "REMARK ", 7) && 
                  (pattern5(str, " topology file", " : ", "","","")||
                   pattern5(str, "REMARK   3  TOPOLOGY FILE", " : ", "","","") )){
            
            pdbx_xplor_file[ntop].topol_file = get_substr(str," : ", "\n");
            
            pdbx_xplor_file[ntop].ntop = ntop;
            ntop++;

            
            
        }else if (strstr_space(str, "REMARK number of reflections in test set:")){//cns_non-standard
            refine.ls_number_reflns_R_free = get_int(str, ":", "(");
            refine.ls_percent_reflns_R_free = get_float(str, "( ", "%");
        }else if (pattern5(str,"REMARK ", "B11=", "B22=", "B33=", "")){//cns_non-standard
            refine.aniso_B11 = get_float(str, "B11=", "B22=");
            refine.aniso_B22 = get_float(str, "B22=", "B33=");
            refine.aniso_B33 = get_float(str, "B33=", "\n");
        }else if (pattern5(str,"REMARK ", "B12=", "B13=", "B23=", "")){//cns_non-standard
            refine.aniso_B12 = get_float(str, "B12=", "B13=");
            refine.aniso_B13 = get_float(str, "B13=", "B23=");
            refine.aniso_B23 = get_float(str, "B23=", "\n");
        }else if (strstr_space(str,"REMARK bulk solvent: density level= ")){//cns_non-standard
            refine.solvent_model_param_bsol = get_float(str, "B-factor=", "A");
               
                
  // FIT IN THE HIGHEST RESOLUTION BIN. (refine_ls_shell (highest)
        }else if (strstr_space(str, "REMARK 3 TOTAL NUMBER OF BINS USED :")){
            refine_ls_shell.pdbx_total_number_of_bins_used[0] = get_int(str, ":", "\n");
            if(refine_ls_shell.nbin <=0) refine_ls_shell.nbin = 1;
            refine_ls_shell.pdbx_refine_id[0] = (char*)"X-RAY DIFFRACTION";
    
        }else if (strstr_space(str, "REMARK 3 BIN RESOLUTION RANGE HIGH ") && strchr(str, ':')){  
            refine_ls_shell.d_res_high[0] = get_float(str, ":", "\n");
            
        }else if (strstr_space(str, "REMARK 3 BIN RESOLUTION RANGE LOW ") && strchr(str, ':')){
            refine_ls_shell.d_res_low[0] = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  BIN COMPLETENESS (WORKING+TEST) (%) :")){
            refine_ls_shell.percent_reflns_obs[0] = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3 REFLECTIONS IN BIN  (WORKING SET) :")||
                  strstr_space(str, "REMARK 3 REFLECTION IN BIN  (WORKING SET) :")){  //refmac
            refine_ls_shell.number_reflns_R_work[0] = get_int(str, ":", "\n");
               
        }else if (strstr_space(str, "REMARK   3  BIN R VALUE        (WORKING + TEST ") ||
                  strstr_space(str, "REMARK   3  BIN R VALUE        (WORKING+TEST)")){
            refine_ls_shell.R_factor_all[0] = get_float(str, ":", "\n");
            
        }else if (strstr_space(str, "REMARK 3  BIN R VALUE           (WORKING SET) :")){
            refine_ls_shell.R_factor_R_work[0] = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3   BIN FREE R VALUE :")){
            refine_ls_shell.R_factor_R_free[0] = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3   BIN FREE R VALUE TEST SET SIZE (%) :")){ 
            refine_ls_shell.percent_reflns_R_free[0] = get_float(str, ":", "\n");

        }else if (strstr_space(str, "REMARK 3  BIN FREE R VALUE SET COUNT       :")||   //refmac
                  strstr_space(str, "REMARK 3  BIN FREE R VALUE TEST SET COUNT      :")){
            refine_ls_shell.number_reflns_R_free[0] = get_int(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3   ESTIMATED ERROR OF BIN FREE R VALUE :")){
            refine_ls_shell.R_factor_R_free_error[0] = get_float(str, ":", "\n");
                

                    // NUMBER OF NON-HYDROGEN ATOMS USED IN REFINEMENT.(refine_hist)
        }else if (strstr_space(str, "REMARK 3   PROTEIN ATOMS :")){
            refine_hist.pdbx_number_atoms_protein = get_int(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  NUCLEIC ACID ATOMS :")){
            refine_hist.pdbx_number_atoms_nucleic_acid = get_int(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3   HETEROGEN ATOMS :")){
            refine_hist.pdbx_number_atoms_ligand = get_int(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3   SOLVENT ATOMS :")){
                refine_hist.number_atoms_solvent = get_int(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3   ALL ATOMS                :")){
            refine_hist.number_atoms_total = get_int(str, ":", "\n");

                    // ESTIMATED COORDINATE ERROR. (refine_analyze)
        }else if (strstr_space(str, "REMARK 3  ESD FROM LUZZATI PLOT (A) :")){
            refine_analyze.pdbx_refine_id = (char*)"X-RAY DIFFRACTION";
            refine_analyze.Luzzati_coordinate_error_obs = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  ESD FROM SIGMAA (A) :")){
            refine_analyze.Luzzati_sigma_a_obs = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3   LOW RESOLUTION CUTOFF (A) :")){
            refine_analyze.Luzzati_d_res_low_obs = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  ESD FROM C-V LUZZATI PLOT (A) :")){
            refine_analyze.Luzzati_coordinate_error_free = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK 3  ESD FROM C-V SIGMAA (A) :")){
            refine_analyze.Luzzati_sigma_a_free = get_float(str, ":", "\n");
                
        }else if (strstr_space(str, "REMARK   3   PROGRAM     :")){
            get_one_string_after_id(str,  " : ",  PROG);
            delete_return(PROG);
            if (strlen(computing.structure_refinement)>1 && strlen(PROG)>1 &&
                strcmp_case(PROG, computing.structure_refinement)) {
                sprintf(INFO, "Warning: The give program name (%s) is not the same as the extracted (%s)\n",computing.structure_refinement, PROG );
                pinfo(INFO, 0);
            }
            
        }else if (strstr_space(str, "METHOD USED TO DETERMINE THE STRUCTURE:")){
            get_data_set_id(str, ":", "\n", tmp);
            rid_of_front_end_space(tmp);
            if (!strstr(tmp,"NULL") && strlen(tmp)>0){
                strcpy(refine.pdbx_method_to_determine_struct, tmp);
            }
            
        }else if (strstr_space(str, "STARTING MODEL:")){
            get_data_set_id(str, ":", "\n", tmp);
            rid_of_front_end_space(tmp);
            if (!strstr(tmp,"NULL") && strlen(tmp)>0){
                strcpy(refine.pdbx_starting_model, tmp);
            }
            
                
        }        

    }
    
}
    
void convert_residue_name1(char *resname, char *resid)
 // if it is DNA with o2' added, it should be written as single letter.
{
    if(!strcmp_case(resname, "ADE") || !strcmp_case(resname, " Ar") ){
        strcpy(resid, "  A");

    }else if(!strcmp_case(resname, "GUA") || !strcmp_case(resname, " Gr") ){
        strcpy(resid, "  G");

    }else if(!strcmp_case(resname, "URA")|| !strcmp_case(resname, "URI") || !strcmp_case(resname, "Ur")){
        strcpy(resid, "  U");

    }else if(!strcmp_case(resname, "CYT") || !strcmp_case(resname, " Cr") ){
        strcpy(resid, "  C");

//    }else if(!strcmp_case(resname, "THY") || !strcmp_case(resname, " DT") ){
//        strcpy(resid, "  T");


    }else{
        strcpy(resid, resname);
    }
}

void convert_residue_name(char *resname, char *resid)
 // convert non-standard residue name to the wwPDB format 
{
    if(!strcmp_case(resname, "ADE") || !strcmp(resname, "  A") || !strcmp(resname, " Ad")){
        strcpy(resid, " DA");
        
    }else if(!strcmp_case(resname, "GUA")|| !strcmp(resname, "  G") || !strcmp(resname, " Gd")){ 
        strcpy(resid, " DG");
        
    }else if(!strcmp_case(resname, "URA")|| !strcmp_case(resname, "URI")|| !strcmp_case(resname, "Ur") ){
        strcpy(resid, "  U");
        
    }else if(!strcmp_case(resname, "CYT") || !strcmp(resname, "  C") || !strcmp(resname, " Cd")){
        strcpy(resid, " DC");
        
    }else if(!strcmp_case(resname, "THY") || !strcmp(resname, "  T") || !strcmp(resname, " Td")){ 
        strcpy(resid, " DT");
        
    }else if(!strcmp(resname, "  I") || !strcmp(resname, " Id")){ 
        strcpy(resid, " DI");
        
    }else{
        strcpy(resid, resname);
    }
   
}

    
void write_PDB_resid(FILE *fout, char *str, long len)
{
    
    char resname[20], resid[10];
    
    if(len>=20){
        fprintf(fout,"Please adjust the length of string!\n");
        return;
    }
    
    strncpy(resname, str, len);
    resname[len] = '\0';
    
    rid_of_front_space(resname);
    rid_of_end_space(resname);
    
    if(strlen(resname)<=0) {
        fprintf(fout,". ");
        return;
    }
    strcpy(resid, resname);
    
    rid_of_front_end_space(resid);
    fprintf(fout," '%s' ", resid);
    
}


void modify_asym_id(int i, int n, int**ch_idx, char**resname, int **res_idx,
                    char **atomname, char **asym_id, float **xyz)
//i: entity(chain) id ; entity_type (-1 ligand, 0,1,2,3)
//remove the 'ligand' in the end of the sequence.
//
{
    int  j, k, n1, n2, c3p;
    
    
    if (n<0){// all ligands
        for (j=ch_idx[i][0]; j<ch_idx[i][1]; j++){
            for (k=res_idx[j][0]; k< res_idx[j][1]; k++){
                strcpy(asym_id[k], ".");
//            printf("ligand nch %d %d  %d %d %d  %s  %d  \n", n, i, res_idx[j][0], res_idx[j][1], j, resname[j], ch_idx[i][1]);
            }
        }
        return;
    }
    
    for (j=ch_idx[i][1]-1; j>ch_idx[i][0]; j--){
        if (strchr(asym_id[ res_idx[j][0] ],'.')) continue;
        if(!protein_na_residue(resname[j]) ){
            if (n>1){ //protein
                if ((!strcmp(resname[j], "NH2")||!strcmp(resname[j], "ACE"))
                    && j>0 && is_protein(resname[j-1])) continue;
                n1=find_atom(" CA ", atomname, res_idx[j][0],  res_idx[j][1]);
                n2=find_atom(" CA ", atomname, res_idx[j-1][0],  res_idx[j-1][1]);
                 
                if ((n1>=0 && n2>=0 && !strcmp(resname[j], "UNK")) || (
                    n1>=0 && n2>=0 && dist(xyz[n1],xyz[n2])<4.5)) continue;
                
            }else{ //na
                c3p = find_atom(" C3'", atomname, res_idx[j][0], res_idx[j][1]);
                if (c3p<0) c3p = find_atom(" C3*", atomname,res_idx[j][0],  res_idx[j][1]);
                n1=find_atom(" P  ", atomname, res_idx[j][0], res_idx[j][1]);
                n2=find_atom(" P  ", atomname, res_idx[j-1][0], res_idx[j-1][1]);
                
//                printf("testing2  %d %d %s %s  %d  %d %d  %.3f\n", res_idx[j][0],  res_idx[j][1], resname[j], asym_id[res_idx[j][0]],c3p, n1, n2, dd);
                
                if (c3p>=0 && n1>=0 && n2>=0 && dist(xyz[n1],xyz[n2])<7.5) continue;
                
            }
            printf("Note: The ligand (%s) is removed from chain (%s).\n", resname[j], asym_id[res_idx[j][0]]);
            for (k=res_idx[j][0]; k< res_idx[j][1]; k++) strcpy(asym_id[k], ".");
            
        }else {
            break;
        } 
    }

}

void modify_asym_id1(int nch, int *ptype, int **ch_idx, int **res_idx, char **chain, char **asym_id)
//take care of weird entry(normally by CNS).  
{
    int i,j, k, m, n1,n2, dn1,dn2;  
    for (i=0; i<nch-1; i++){
        n1=ch_idx[i][0];
        if (ptype[i]<0) continue;
        
        for (j=i+1; j<nch; j++){
            if (ptype[j]<0) continue;
            n2=ch_idx[j][0];
            if (!strcmp(chain[n1], chain[n2])){
                dn1=ch_idx[i][1] - ch_idx[i][0];
                dn2=ch_idx[j][1] - ch_idx[j][0];
                if (dn1>dn2){
                    for (k=ch_idx[j][0]; k< ch_idx[j][1]; k++){
                        for (m=res_idx[k][0]; m< res_idx[k][1]; m++){
                            strcpy(asym_id[m], ".");
//                        printf("Warning: %d %d %d \n", i,j,k);
                        }
                    }
                    
                }else{
                    for (k=ch_idx[i][0]; k< ch_idx[i][1]; k++) {
                        for (m=res_idx[k][0]; m< res_idx[k][1]; m++){
                            strcpy(asym_id[m], ".");
//                        printf("Warning: %d %d %d \n", i,j,k);
                        }
                    }
                }
                
                sprintf(INFO, "Warning: duplicate chains removed.(nch=%d vs nch=%d) (%d %d) %s\n", i,j,dn1,dn2, chain[n1]);
                pinfo(INFO, 0);
            }
        }
    }
}
void chain_entity_type(char **line, int nline, int *dna_id, char **asym_id)
//identify if the chain is RNA or DNA or protein. 
//dna_id is labeled for each atom. dna_id[i]=1, dna; dna_id[i]=12, dna with adding O2';
{
    char **chain, **resname, **atomname, **poly_type;
    int  **res_idx, **ch_idx, *ptype, **pptype;
    int  i, n=0, nch_tmp=0, nres_tmp=0, nch,nres, model=0;
    float **xyz;

    for (i=0; i<nline; i++){
        dna_id[i]=-1;
        strcpy(asym_id[i],".");
        if (!strncmp(line[i], "MODEL", 5)) model++;
        
        if ((!strncmp(line[i], "ATOM", 4) || !strncmp(line[i], "HETATM", 6)
             || !strncmp(line[i], "ANISOU", 6))
            && strncmp(line[i]+17, "DOD", 3) && strncmp(line[i]+17, "HOH", 3)
            && strncmp(line[i]+20, "  ", 2)){
            if (!strcmp(exptl.method, "NMR")){line[i][20]=' ';}
            strncpy(asym_id[i], line[i]+20, 2);
            asym_id[i][2]='\0';
        }
    }
    
    get_nchain_nres(nline, line, &nres_tmp, &nch_tmp);
//    printf("NRES, nch = %d %d \n", nres_tmp, nch_tmp);
    
    xyz=set_fmemory_2d(nline, 4);
    atomname=set_memory_2d(nline, 4);
    ch_idx=set_imemory_2d(nch_tmp + 40, 3);
    res_idx=set_imemory_2d(nres_tmp+4, 2);
    resname=set_memory_2d(nres_tmp+4,3);
    chain=set_memory_2d(nres_tmp+4,4); // chain and 2 charactor chainID

    atominfo(nline,line,atomname,resname,chain, xyz, res_idx, &nres, ch_idx, &nch);
    
    ptype=new int [nch+2];//number of entity
    poly_type=set_memory_2d(nch, 80);

    pptype=set_imemory_2d(nch, nres);
    for (i=0; i<nch; i++){ //remove ligand (chain identity)
        n=protein_or_na(i, ch_idx, resname, res_idx, atomname);
        ptype[i]=n;  //n=0 rna; 1 dna, n=2,3 protein, n=4 rna/dna, n=-1 other
//        printf("linelin=%3d %3d %3d %s : %s\n",nch, i, n, chain[ch_idx[i][0]], resname[ch_idx[i][0]]);
        modify_asym_id(i, n, ch_idx, resname, res_idx, atomname, asym_id,xyz);        
        get_poly_type(n,i, ch_idx, res_idx, atomname, pptype, poly_type);
    }
    if (model==0) modify_asym_id1(nch, ptype, ch_idx, res_idx, chain, asym_id);
    
/*
    int j, k;
    for (i=0; i<nch; i++){
        for (j=ch_idx[i][0]; j<ch_idx[i][1]; j++){
            for (k=res_idx[j][0]; k<res_idx[j][1]; k++){
                if(ch_idx[i][1]-ch_idx[i][0]>2)  dna_id[k]=pptype[i][j];
                printf ("pptype= %d %d %d %d %3d %s %s\n", i,j, k, dna_id[k], ptype[i], asym_id[k], resname[j]);  
            }
        }
    }
*/
    
}


void get_derived_values(int nline, char **line, char **chain)
//
{
    char str1[20], str2[20];
    int i, natom=0, natom_lig=0, natom_sol=0, nres=0,nocc=0, nbiso=0; 
    float ball=0, biso=0, biso_lig=0, biso_sol=0,  occ, socc=0, socc_lig=0, socc_sol=0;
    float min_b=99999, max_b=INI, min_o=99999, max_o=INI ;

    for (i=0; i<nline; i++){
         if (strncmp(line[i], "ATOM" ,4) && strncmp(line[i], "HETATM" ,6)) continue;
         occ=get_float_by_length(line[i], 54, 58);
         if (occ==0) continue;
         
         biso=get_float_by_length(line[i], 60, 66);
         

         if(occ<min_o)min_o=occ;
         if(occ>max_o)max_o=occ;
         if (occ>1 || occ<0){ 
             nocc++;
             sprintf(INFO, "Error: OCC problem at %s\n", line[i]);
             pinfo(INFO, 0);
         }

         if(biso<min_b)min_b=biso;
         if(biso>max_b)max_b=biso;
         
         if (strcmp(exptl.method, "NMR") && (biso>900 || biso<=0.001)){ nbiso++;
         }
         
         
         socc = socc + occ;
         ball = ball + biso*occ;
         
         natom++;
         if (chain[i][0]=='.' &&
             (!strncmp(line[i]+17, "HOH", 3)|| !strncmp(line[i]+17, "DOD", 3))){
             natom_sol++;
             biso_sol = biso_sol + biso*occ;
             socc_sol=socc_sol + occ;
             
         }else if (chain[i][0]=='.' && strncmp(line[i]+17, "HOH", 3) &&
                   strncmp(line[i]+17, "DOD", 3)){
             natom_lig++;
             biso_lig = biso_lig + biso*occ;
             socc_lig=socc_lig + occ;
         }
         strcpy(str1, "");
         strcpy(str2, "");
         
         if (i>0){
             strncpy(str1, line[i-1]+20, 7);
             str1[7]='\0';
         }
         strncpy(str2, line[i]+20, 7);
         str2[7]='\0';
         
         if (i>0 && chain[i][0]!='.' && strcmp(str1, str2)) nres++;
    }
    
    if (socc>0 && refine.B_iso_mean<=0) refine.B_iso_mean = ball/socc;
//    printf("test-B=%.2f|%.2f|%.2f|%.2f|%d\n", ball/socc,ball/natom,ball,socc,natom);
    

    if (nocc>0){
        sprintf(INFO, "Warning! There are %d atoms with occupancy <0 or >1.0.\n", nocc);
        pinfo(INFO, 0);
    }
             
    if (nbiso>0){
        sprintf(INFO, "Warning! There are %d atoms with Bfactor <=0 (or >900).\n", nbiso);
        pinfo(INFO, 0);
    }

    refine.B_iso_max = max_b ;
    refine.B_iso_min = min_b ;
    
    refine_hist.number_atoms_total=natom;
    refine_hist.pdbx_number_atoms_ligand=natom_lig;
    refine_hist.number_atoms_solvent=natom_sol;
    refine_hist.number_residue_total=nres;
    refine_hist.d_res_high=refine.ls_d_res_high;
    refine_hist.d_res_low=refine.ls_d_res_low;
        
    if(socc_lig>0) refine_hist.pdbx_B_iso_mean_ligand = biso_lig/socc_lig;
    if(socc_sol>0) refine_hist.pdbx_B_iso_mean_solvent = biso_sol/socc_sol;
    
}

void pdb2cif_coord(FILE *fout, char **line, int nline)
/* convert the  coordinates and aniso. temp of a PDB file to mmcif*/
{
    char **asym_id, lab_asym[4], str[256], group[10],atomid1[80],atomid2[80];
    char symbol[5],charge[5];
    int i=0, natom,nanis=0,  sigatm=0,siguij=0, *dna_id, model=0,len ;
    FILE *tmp1=NULL, *tmp2=NULL;

    const char *atom_site[]=  {
        "group_PDB",
        "id",
                
        "auth_atom_id",
        "label_atom_id",
                
        "label_alt_id",
                
        "auth_comp_id",
        "label_comp_id",
        "pdbx_auth_comp_id",
        
        "auth_asym_id",
        "label_asym_id",
                
        "auth_seq_id" ,
        "label_seq_id" ,
                
        "pdbx_PDB_ins_code",
                
        "Cartn_x", 
        "Cartn_y", 
        "Cartn_z", 
        "occupancy", 
        "B_iso_or_equiv",
        "pdbx_PDB_model_num",
        
        "label_entity_id",
        "type_symbol",
        
        "pdbx_formal_charge",
        
                
        "Cartn_x_esd", 
        "Cartn_y_esd", 
        "Cartn_z_esd", 
        "occupancy_esd", 
        "B_iso_or_equiv_esd",
        
        "END_CATE"
    };
    
    const char *atom_site_anisotrop[]=  {
        
        "id",
        "pdbx_auth_atom_id",
        "pdbx_label_atom_id",
        "pdbx_label_alt_id",
        "pdbx_auth_comp_id",
        "pdbx_label_comp_id",
        "pdbx_auth_asym_id",
        "pdbx_label_asym_id",
        "pdbx_auth_seq_id" ,
        "pdbx_label_seq_id" ,
        "pdbx_PDB_ins_code",            
        "U[1][1]", 
        "U[2][2]", 
        "U[3][3]", 
        "U[1][2]", 
        "U[1][3]", 
        "U[2][3]",
        "pdbx_PDB_model_num", 
                   
        "type_symbol", 
                   
                      
        "U[1][1]_esd", 
        "U[1][2]_esd", 
        "U[1][3]_esd", 
        "U[2][2]_esd", 
        "U[2][3]_esd", 
        "U[3][3]_esd",
        "END_CATE"
    };
    
    tmp1=tmpfile();
    tmp2=tmpfile();
    
    printf("Extracting coordinates ...\n");
    for(i=0; i<nline; i++){
        if (!strncmp(line[i], "SIGATM", 6)){ //sigma
            sigatm=1;
        }else if(!strncmp(line[i], "SIGUIJ", 6)){ //sigma
            siguij=1;
        }
    }
    
    dna_id = new int[nline+1];
    asym_id = set_memory_2d (nline, 4);
    chain_entity_type(line, nline, dna_id, asym_id);

//    asym_id = get_label_asym_id(line, nline);

    
    natom=0;
    
    model = 1;
    for(i=0;i<nline; i++){
        if (!strncmp(line[i], "MODEL", 5)){ //coordinate
            if (model++ >1) break;
        }
    }
    
    if (model>1) model=0;
    strcpy(symbol,"");
    strcpy(charge,"");
    
    for(i=0;i<nline; i++){
        len=strlen(line[i]);
        if ((!strncmp_case(line[i], "ATOM", 4) && strncmp(line[i], "ATOM  ", 6)) ||
            (!strncmp_case(line[i], "HETATM", 6) && strncmp(line[i], "HETATM", 6))){
            sprintf(INFO, "Error! Not standard coordinate format: %s",line[i]);
            pinfo(INFO, 0);
        }

        if ((!strncmp_case(line[i], "ATOM", 4)||!strncmp_case(line[i], "HETATM", 6)) && len>55){
            natom++;
            strncpy(group,line[i], 6);
            group[6]='\0';
            
            strcpy(lab_asym, asym_id[i]);
            get_atom_record_id(line[i], atomid1, lab_asym, dna_id[i], 0);
            fprintf(tmp1,"%s %d %s ",group, natom, atomid1);
            write_PDB_30_70(tmp1, line[i], 1);//column 30_70

            if(len >=78){
                strncpy(symbol, line[i]+76, 2);
                symbol[2] = '\0';
                rid_of_space(symbol);
            }
            
            if (strlen(symbol)==0) strcpy(symbol, ".");
            
            if(len >78){
                strncpy(charge, line[i]+78, 2);
                charge[2] = '\0';
                rid_of_space(charge);
            }
            
            if (strlen(charge)==0) strcpy(charge, "?");
            
            fprintf(tmp1, " %d . %s %s ", model, symbol,charge);
            
            if(sigatm==1){
                if (i<nline-1 && !strncmp(line[i+1], "SIGATM", 6)){ //sigma
                    write_PDB_30_70(tmp1, line[i+1], 1);
                    i=i+1;
                }else {
                    fprintf(tmp1, " ? ?  ? ? ? ");
                }
            }
            
            if (SPLIT) {
                strncpy(str,line[i]+72, 4);
                str[4]='\0';
                fprintf(tmp1," '%s' ", str);
            }
                
            fprintf(tmp1,"\n");
        
            if (i<nline-1 && !strncmp(line[i+1], "ANISOU" ,6)){
                strcpy(lab_asym, asym_id[i+1]);
                
                get_atom_record_id(line[i+1], atomid2, lab_asym, dna_id[i+1], 1);
                if (strncmp(line[i]+7,line[i+1]+7, 20)){
                    sprintf(INFO, "Error! ATOM and ANISOU have different id for the two lines. \n%s%s ",line[i],line[i+1]);
                    pinfo(INFO, 0);
                }          
                nanis++;
                fprintf(tmp2,"%d %s ", natom, atomid2);
                write_PDB_30_70(tmp2, line[i+1], 2);
                fprintf(tmp2, " %d . ", model);
                i=i+1;

                if(siguij==1){
                    if (i<nline-1 && !strncmp(line[i+1], "SIGUIJ", 6)){ //sigma
                        write_PDB_30_70(tmp2, line[i+1], 2);
                        i=i+1;
                    }else {
                        fprintf(tmp2, " ? ?  ? ? ? ? ");
                    }

                }
                
                fprintf(tmp2,"\n");
            }
            
        }else if (!strncmp(line[i], "MODEL", 5)){ //coordinate
            model++;
        }

    }
    rewind(tmp1);
    rewind(tmp2);

    if(natom>0){
        if(sigatm==1)
            write_mmcif_head(fout, "_atom_site", atom_site);
        else
            print_mmcif_head(fout, "_atom_site", atom_site, 22);

        if(SPLIT){fprintf(fout, "_atom_site.auth_seg_id\n");}
            
        while (fgets(str, sizeof str, tmp1) != NULL) {
            fprintf(fout, "%s", str);
        }
    }else{
        sprintf(INFO, "Error! Atom coordinates were not extracted!\n");
        pinfo(INFO,0);
    }
        
    
    
    if(nanis>0){
        
        if(siguij==1)
            write_mmcif_head(fout,"_atom_site_anisotrop",atom_site_anisotrop );
        else
            print_mmcif_head(fout,"_atom_site_anisotrop",atom_site_anisotrop, 19);
        
        while (fgets(str, sizeof str, tmp2) != NULL) {
            fprintf(fout, "%s", str);
        }
    }
    get_derived_values(nline, line, asym_id);
    
    

    fclose(tmp1);
    fclose(tmp2);
//    exit(0);
    
}


void get_atom_record_id(char *str, char *atomid, char *lab_asym, int na, int idd)
// idd=0, for atom record;  idd==1 for anisou record
{
    
    char  auth_atom[5], lab_atom[5], auth_comp[5], lab_comp[5], pdbx_comp[5];
    char  auth_asym[4], auth_seq[5], lab_seq[5], alt,icode;       
    
    
    strncpy(auth_atom, str+12, 4); 
    auth_atom[4] = '\0';        // _atom_site.auth_atom_id
    rid_of_space(auth_atom);
    strcpy(lab_atom, auth_atom);//_atom_site.label_atom_id
    
    if(strchr(lab_atom, '*'))replace_char(lab_atom,'*', '\'');
//    if(strstr(lab_atom, "C5A")) strcpy(lab_atom,"C5M");
    
    alt = str[16];
    if(isspace(alt)) alt = '.';  //_atom_site.label_alt_id 
    
    strncpy(pdbx_comp, str+17, 3); //_atom_site.pdbx_auth_comp_id 
    pdbx_comp[3]='\0';
    
    if(na==1 || na==31){ //DNA
        convert_residue_name(pdbx_comp, lab_comp); //_atom_site.label_comp_id
    }else if(na==12 || na==2 ||na==32){
        convert_residue_name1(pdbx_comp, lab_comp); //_atom_site.label_comp_id
    }else{
        strcpy(lab_comp, pdbx_comp);
    }
//    printf("type=%d %s %s \n", na,lab_comp, pdbx_comp);
    
    rid_of_space(pdbx_comp);
    rid_of_space(lab_comp);
    strcpy(auth_comp, lab_comp);  // _atom_site.auth_comp_id

    if (strncmp(str+20, "  ", 2)){
        strncpy(auth_asym, str+20, 2);
        auth_asym[2]='\0';
        if (!strcmp(exptl.method, "NMR")){auth_asym[0]=' ';}

        rid_of_space(auth_asym);
    }else{
        strcpy(auth_asym, ".");
    }
    
    
    strncpy(auth_seq, str+22, 4);  // _atom_site.auth_seq_id  
    auth_seq[4]='\0';
    rid_of_space(auth_seq);
    strcpy(lab_seq, auth_seq);   // _atom_site.lab_seq_id  
    
    
    icode=str[26];  // _atom_site.pdbx_PDB_ins_code
    if(isspace(icode) || icode=='@'){ icode = '.';}
    if (idd==0){ //atom record
        sprintf(atomid," %s %s  %c  %s %s %s  %s  %s %s %s %c ",
                auth_atom,lab_atom, alt, auth_comp, lab_comp, pdbx_comp,
                auth_asym,lab_asym, auth_seq, lab_seq, icode);
    }else{
        sprintf(atomid," %s %s  %c  %s %s  %s  %s %s %s %c ",
                auth_atom,lab_atom, alt, auth_comp, lab_comp, 
                auth_asym,lab_asym, auth_seq, lab_seq, icode);
    }
}

void pdb_30_70_item(char *str, int len, int n, char *strout )
{
    char tmp[20];
    int m=0;
    float a;
    
    if (n==1){
        strncpy(tmp, str, len);
        tmp[len] = '\0';
        (sscanf(tmp, "%f", &a)==1) ? strcat(strout,tmp): strcat(strout," ? ");
        strcat(tmp, " ");
    }else{
        strncpy(tmp, str, len);
        tmp[len] = '\0';
        m=sscanf(tmp, "%f", &a);
        if (m){
            sprintf(tmp," %.4f ", a*0.0001);
        }else{
            sprintf(tmp, " ? ");
        }
        strcat(strout,tmp);
    }
}

void get_pdb_30_70(char *str, int n, char *strout)  
{
    
    if(n==1){ // x,y,z,occu,Biso
        pdb_30_70_item(str+30,8,1, strout);
        pdb_30_70_item(str+38,8,1, strout);
        pdb_30_70_item(str+46,8,1, strout);
        pdb_30_70_item(str+54,6,1, strout);
        pdb_30_70_item(str+60,6,1, strout);

    }else if(n==2){ //uij scale by 0.0001

        pdb_30_70_item(str+28, 7, 2, strout);
        pdb_30_70_item(str+35, 7, 2, strout);
        pdb_30_70_item(str+42, 7, 2, strout);
        pdb_30_70_item(str+49, 7, 2, strout);
        pdb_30_70_item(str+56, 7, 2, strout);
        pdb_30_70_item(str+63, 7, 2, strout);
    }
}

void write_PDB_30_70(FILE *fout, char *str, int n)  
{
    char tmp[20];
    float a;
    
    strcpy(tmp,"");
    
    if(n==1){ // x,y,z,occu,Biso
        strncpy(tmp, str+30, 8);
        tmp[8] = '\0';
        (sscanf(tmp, "%f", &a)==1) ? fprintf(fout,"%.3f ", a):fprintf(fout, " ? ");

        strncpy(tmp, str+38, 8);
        tmp[8] = '\0';
        (sscanf(tmp, "%f", &a)==1) ? fprintf(fout,"%.3f ", a):fprintf(fout, " ? ");
        
        strncpy(tmp, str+46, 8);
        tmp[8] = '\0';
        (sscanf(tmp, "%f", &a)==1) ? fprintf(fout,"%.3f ", a):fprintf(fout, " ? ");
        
        strncpy(tmp, str+54, 6);
        tmp[6] = '\0';
        (sscanf(tmp, "%f", &a)==1) ? fprintf(fout,"%.2f ", a):fprintf(fout, " ? ");
        
        strncpy(tmp, str+60, 6);
        tmp[6] = '\0';
        (sscanf(tmp, "%f", &a)==1) ? fprintf(fout,"%.2f ", a):fprintf(fout, " ? ");


    }else if(n==2){ //uij scale by 0.0001

        strncpy(tmp, str+28, 7);
        tmp[7] = '\0';
        (sscanf(tmp, "%f", &a)==1) ? fprintf(fout,"%.4f ", a*0.0001):fprintf(fout, " ? ");

        strncpy(tmp, str+35, 7);
        tmp[7] = '\0';
        (sscanf(tmp, "%f", &a)==1) ? fprintf(fout,"%.4f ", a*0.0001):fprintf(fout, " ? ");

        strncpy(tmp, str+42, 7);
        tmp[7] = '\0';
        (sscanf(tmp, "%f", &a)==1) ? fprintf(fout,"%.4f ", a*0.0001):fprintf(fout, " ? ");

        strncpy(tmp, str+49, 7);
        tmp[7] = '\0';
        (sscanf(tmp, "%f", &a)==1) ? fprintf(fout,"%.4f ", a*0.0001):fprintf(fout, " ? ");

        strncpy(tmp, str+56, 7);
        tmp[7] = '\0';
        (sscanf(tmp, "%f", &a)==1) ? fprintf(fout,"%.4f ", a*0.0001):fprintf(fout, " ? ");

        strncpy(tmp, str+63, 7);
        tmp[7] = '\0';
        (sscanf(tmp, "%f", &a)==1) ? fprintf(fout,"%.4f ", a*0.0001):fprintf(fout, " ? ");
        
    }
}




void write_refine_ls_restr(FILE *fout)
{
    int i;
    char type[80];
        
    static const char *refmac_33[] = 
        {
            "r_bond_refined_d           BOND LENGTHS REFINED ATOMS        (A) ",
            "r_bond_other_d             BOND LENGTHS OTHERS               (A) ",
            "r_angle_refined_deg        BOND ANGLES REFINED ATOMS   (DEGREES) ",
            "r_angle_other_deg          BOND ANGLES OTHERS          (DEGREES) ",
            "r_dihedral_angle_1_deg     TORSION ANGLES, PERIOD 1    (DEGREES) ",
            "r_dihedral_angle_2_deg     TORSION ANGLES, PERIOD 2    (DEGREES) ",
            "r_dihedral_angle_3_deg     TORSION ANGLES, PERIOD 3    (DEGREES) ",
            "r_dihedral_angle_4_deg     TORSION ANGLES, PERIOD 4    (DEGREES) ",
            "r_chiral_restr             CHIRAL-CENTER RESTRAINTS       (A**3) ",
            "r_gen_planes_refined       GENERAL PLANES REFINED ATOMS      (A) ",
            "r_gen_planes_other         GENERAL PLANES OTHERS             (A) ",
            "r_nbd_refined              NON-BONDED CONTACTS REFINED ATOMS (A) ",
            "r_nbd_other                NON-BONDED CONTACTS OTHERS        (A) ",
            "r_nbtor_refined            NON-BONDED TORSION REFINED ATOMS  (A) ",
            "r_nbtor_other              NON-BONDED TORSION OTHERS         (A) ",
            "r_xyhbond_nbd_refined      H-BOND (X...Y) REFINED ATOMS      (A) ",
            "r_xyhbond_nbd_other        H-BOND (X...Y) OTHERS             (A) ",
            "r_metal_ion_refined        POTENTIAL METAL-ION REFINED ATOMS (A) ",
            "r_metal_ion_other          POTENTIAL METAL-ION OTHERS        (A) ", 
            "r_symmetry_vdw_refined     SYMMETRY VDW REFINED ATOMS        (A) ",
            "r_symmetry_vdw_other       SYMMETRY VDW OTHERS               (A) ",
            "r_symmetry_hbond_refined   SYMMETRY H-BOND REFINED ATOMS     (A) ",
            "r_symmetry_hbond_other     SYMMETRY H-BOND OTHERS            (A) ",
            "r_symmetry_metal_ion_refined SYMMETRY METAL-ION REFINED ATOMS  (A) ",
            "r_symmetry_metal_ion_other SYMMETRY METAL-ION OTHERS         (A) ",
            "r_mcbond_it                MAIN-CHAIN BOND REFINED ATOMS  (A**2) ",
            "r_mcangle_it               MAIN-CHAIN ANGLE REFINED ATOMS (A**2) ",
            "r_mcbond_other             MAIN-CHAIN BOND OTHER ATOMS    (A**2) ",
            "r_scbond_it                SIDE-CHAIN BOND REFINED ATOMS  (A**2) ",
            "r_scangle_it               SIDE-CHAIN ANGLE REFINED ATOMS (A**2) ",
            "r_rigid_bond_restr         RIGID-BOND RESTRAINTS          (A**2) ",
            "r_sphericity_free          SPHERICITY; FREE ATOMS         (A**2) ",  
            "r_sphericity_bonded        SPHERICITY; BONDED ATOMS       (A**2) "
        };
    
    static const char *cns_9[] = 
        {
            " c_bond_d   BOND LENGTHS                 (A)    ",
            " c_angle_d  ANGLE DISTANCES                      (A)  ",
            " c_angle_deg  BOND ANGLES            (DEGREES)   ",
            " c_dihedral_angle_d   DIHEDRAL ANGLES        (DEGREES) ",
            " c_improper_angle_d   IMPROPER ANGLES        (DEGREES) ",
            " c_mcbond_it  MAIN-CHAIN BOND              (A**2) ",
            " c_mcangle_it  MAIN-CHAIN ANGLE             (A**2) ",
            " c_scbond_it  SIDE-CHAIN BOND              (A**2) ",
            " c_scangle_it  SIDE-CHAIN ANGLE             (A**2) "
        };

    static const char *xplor_9[] = 
        {
            " x_bond_d   BOND LENGTHS                 (A)    ",
            " x_angle_d  ANGLE DISTANCES                      (A)  ",
            " x_angle_deg  BOND ANGLES            (DEGREES)   ",
            " x_dihedral_angle_d   DIHEDRAL ANGLES        (DEGREES) ",
            " x_improper_angle_d   IMPROPER ANGLES        (DEGREES) ",
            " x_mcbond_it  MAIN-CHAIN BOND              (A**2) ",
            " x_mcangle_it  MAIN-CHAIN ANGLE             (A**2) ",
            " x_scbond_it  SIDE-CHAIN BOND              (A**2) ",
            " x_scangle_it  SIDE-CHAIN ANGLE             (A**2) "
        };

    
    static const char *shelx_10[] = 
        {
            "s_bond_d           BOND LENGTHS                         (A) ",
            "s_angle_d          ANGLE DISTANCES                      (A) ",
            "s_similar_dist     SIMILAR DISTANCES (NO TARGET VALUES) (A) ",
            "s_from_restr_planes  DISTANCES FROM RESTRAINT PLANES      (A) ",
            "s_zero_chiral_vol    ZERO CHIRAL VOLUMES               (A**3) ",
            "s_non_zero_chiral_vol   NON-ZERO CHIRAL VOLUMES           (A**3)", 
            "s_anti_bump_dis_restr  ANTI-BUMPING DISTANCE RESTRAINTS     (A) ",
            "s_rigid_bond_adp_cmpnt  RIGID-BOND ADP COMPONENTS         (A**2) ",
            "s_similar_adp_cmpnt   SIMILAR ADP COMPONENTS            (A**2) ",
            "s_approx_iso_adps     APPROXIMATELY ISOTROPIC ADPS      (A**2) "
        };
    
/*            
    static const char *buster_9[] = 
        {
            "o_bond_d                BOND LENGTHS                       (A) ",
            "o_angle_deg             BOND ANGLES                  (DEGREES) ",
            "o_dihedral_angle_d      PEPTIDE OMEGA TORSION ANGLES (DEGREES) ",
            "o_dihedral_angle_o      OTHER TORSION ANGLES         (DEGREES) ",
            "o_incorr_chiral_ct      CHIRAL-CENTER RESTRAINT       (A**3)  ",
            "o_pseud_angle           PSEUDOROTATION ANGLES  (DEGREES) ",
            "o_trig_c_planes         TRIGONAL CARBON PLANES       (A) ",
            "o_gen_planes            GENERAL PLANES               (A) ",
            "o_it                    ISOTROPIC THERMAL FACTORS (A**2) ",
            "o_nbd                   NON-BONDED CONTACTS          (A) "
        };
*/
    static const char *buster_18[] = 
        {
            "t_bond_d                BOND LENGTHS                       (A) ",
            "t_angle_deg             BOND ANGLES                  (DEGREES) ",
            "t_dihedral_angle_d      TORSION ANGLES             ",
            "t_incorr_chiral_ct      CHIRAL-CENTER RESTRAINT       (A**3)  ",
            "t_pseud_angle           PSEUDOROTATION ANGLES        ",
            "t_trig_c_planes         TRIGONAL CARBON PLANES     ",
            "t_gen_planes          GENERAL PLANES              ",
            "t_it                  ISOTROPIC THERMAL FACTORS  ",
            "t_nbd                 BAD NON-BONDED CONTACTS      ",
            "t_omega_torsion     PEPTIDE OMEGA TORSION ANGLES (DEGREES)   ",
            "t_other_torsion     OTHER TORSION ANGLES         (DEGREES)  ",
            "t_improper_torsion    IMPROPER TORSIONS  ",
            "t_chiral_improper_torsion     CHIRAL IMPROPER TORSION    ",
            "t_sum_occupancies   SUM OF OCCUPANCIES          ",
            "t_utility_distance  UTILITY DISTANCES          ",
            "t_utility_angle     UTILITY ANGLES             ",
            "t_utility_torsion   UTILITY TORSION             ",
            "t_ideal_dist_contact  IDEAL-DIST CONTACT TERM   "
        };

    
            
    static const char *tnt_9[] = 
        {
            "t_bond_d   BOND LENGTHS                 (A) ",
            "t_angle_deg  BOND ANGLES            (DEGREES) ",
            "t_dihedral_angle_d   TORSION ANGLES         (DEGREES) ",
            "t_incorr_chiral_ct CHIRAL-CENTER RESTRAINT       (A**3)  ",
            "t_pseud_angle   PSEUDOROTATION ANGLES  (DEGREES) ",
            "t_trig_c_planes  TRIGONAL CARBON PLANES       (A) ",
            "t_gen_planes  GENERAL PLANES               (A) ",
            "t_it   ISOTROPIC THERMAL FACTORS (A**2) ",
            "t_nbd   NON-BONDED CONTACTS          (A) "
        };
  
             
    static const char *nuclsq_17[] = 
        {
           
            " n_bond_d             BOND LENGTH                     (A) ",
            " n_angle_d            ANGLE DISTANCE                  (A)",
            " n_planar_d           INTRAPLANAR 1-4 DISTANCE        (A) ",
            " n_hb_or_metal_coord  H-BOND OR METAL COORDINATION    (A) ",
            " n_sugar_bond_it      SUGAR-BASE BONDS             (A**2)",
            " n_sugar_angle_it     SUGAR-BASE ANGLES            (A**2)",
            " n_phos_bond_it       PHOSPHATE BONDS              (A**2)",
            " n_phos_angle_it      PHOSPHATE BOND ANGLE, H-BOND (A**2)",
            " n_plane_restr        PLANE RESTRAINT                  (A)",
            " n_chiral_restr       SINGLE TORSION CONTACT          (A)",
            " n_sugar_bond_d       SUGAR-BASE BOND DISTANCE        (A)",
            " n_sugar_bond_angle_d SUGAR-BASE BOND ANGLE DISTANCE  (A)",
            " n_phos_bond_d        PHOSPHATE BONDS DISTANCE        (A)",
            " n_phos_bond_angle_d  PHOSPHATE BOND ANGLE, H-BOND    (A)",
            " n_chiral_restr       CHIRAL-CENTER RESTRAINT       (A**3)",
            " n_singtor_nbd        SINGLE TORSION         (A)",
            " n_multtor_nbd        MULTIPLE TORSION CONTACT        (A)"
        };
    
    static const char *prolsq_17[] = 
        {
           
            " p_bond_d             BOND LENGTH                     (A) ",
            " p_angle_d            ANGLE DISTANCE                  (A)",
            " p_planar_d           INTRAPLANAR 1-4 DISTANCE        (A) ",
            " p_hb_or_metal_coord  H-BOND OR METAL COORDINATION    (A) ",
            " p_sugar_bond_it      SUGAR-BASE BONDS             (A**2)",
            " p_sugar_angle_it     SUGAR-BASE ANGLES            (A**2)",
            " p_phos_bond_it       PHOSPHATE BONDS              (A**2)",
            " p_phos_angle_it      PHOSPHATE BOND ANGLE, H-BOND (A**2)",
            " p_plane_restr        PLANE RESTRAINT                  (A)",
            " p_chiral_restr       SINGLE TORSION CONTACT          (A)",
            " p_sugar_bond_d       SUGAR-BASE BOND DISTANCE        (A)",
            " p_sugar_bond_angle_d SUGAR-BASE BOND ANGLE DISTANCE  (A)",
            " p_phos_bond_d        PHOSPHATE BONDS DISTANCE        (A)",
            " p_phos_bond_angle_d  PHOSPHATE BOND ANGLE, H-BOND    (A)",
            " p_chiral_restr       CHIRAL-CENTER RESTRAINT       (A**3)",
            " p_singtor_nbd        SINGLE TORSION         (A)",
            " p_multtor_nbd        MULTIPLE TORSION CONTACT        (A)"
        };
    
    static const char *other_17[] = 
        {
           
            " o_bond_d             BOND LENGTH                     (A) ",
            " o_angle_d            ANGLE DISTANCE                  (A)",
            " o_planar_d           INTRAPLANAR 1-4 DISTANCE        (A) ",
            " o_hb_or_metal_coord  H-BOND OR METAL COORDINATION    (A) ",
            " o_sugar_bond_it      SUGAR-BASE BONDS             (A**2)",
            " o_sugar_angle_it     SUGAR-BASE ANGLES            (A**2)",
            " o_phos_bond_it       PHOSPHATE BONDS              (A**2)",
            " o_phos_angle_it      PHOSPHATE BOND ANGLE, H-BOND (A**2)",
            " o_plane_restr        PLANE RESTRAINT                  (A)",
            " o_chiral_restr       SINGLE TORSION CONTACT          (A)",
            " o_sugar_bond_d       SUGAR-BASE BOND DISTANCE        (A)",
            " o_sugar_bond_angle_d SUGAR-BASE BOND ANGLE DISTANCE  (A)",
            " o_phos_bond_d        PHOSPHATE BONDS DISTANCE        (A)",
            " o_phos_bond_angle_d  PHOSPHATE BOND ANGLE, H-BOND    (A)",
            " o_chiral_restr       CHIRAL-CENTER RESTRAINT       (A**3)",
            " o_singtor_nbd        SINGLE TORSION         (A)",
            " o_multtor_nbd        MULTIPLE TORSION CONTACT        (A)"
        };
    
    
//    if(refine_ls_restr[0].dev_ideal <-9998 || NR<=0  )return;
//    printf("nr=%d\n", NR);
    
    if(NR>0){
        
        fprintf(fout,"#\n");
        fprintf(fout,"loop_\n");
        fprintf(fout,"_refine_ls_restr.pdbx_refine_id\n");
        fprintf(fout,"_refine_ls_restr.type\n");
        fprintf(fout,"_refine_ls_restr.number\n");
        fprintf(fout,"_refine_ls_restr.dev_ideal\n");
        fprintf(fout,"_refine_ls_restr.dev_ideal_target \n");
        fprintf(fout,"_refine_ls_restr.weight \n");
        fprintf(fout,"_refine_ls_restr.pdbx_restraint_function \n");
    }
    if(!strcmp(refine_ls_restr[0].type, "c_bond_d") ){ // only for non-standard CNS
        for(i=0; i<NR; i++){
            fprintf(fout," '%s' '%s' ?  %.3f ", REFINE_ID[0],
                    refine_ls_restr[i].type,refine_ls_restr[i].dev_ideal );
            (refine_ls_restr[i].dev_ideal_target>-9999) ?
                fprintf(fout," %.3f ", refine_ls_restr[i].dev_ideal_target):fprintf(fout," ? ");
            fprintf(fout," ?  ?\n");
        }
        return;
    }

    for(i=0; i<NR; i++){
        strcpy(type, refine_ls_restr[i].type);

        fprintf(fout,"'%s' ", REFINE_ID[0]);    

        if(strstr_case(PROG, "REFMAC") || strstr_case(PROG, "RESTRAIN")){
           convert_restr_2_standard(fout,type , i, refmac_33, 33 );
        } else if(strstr_case(PROG, "CNS") || strstr_case(PROG, "CNX")){
            convert_restr_2_standard(fout,type , i, cns_9, 9); 
        } else if(strstr_case(PROG, "PLOR")){
            convert_restr_2_standard(fout,type , i, xplor_9, 9); 
        } else if(strstr_case(PROG, "SHELX")){
            convert_restr_2_standard(fout,type , i, shelx_10, 10); 
        } else if(strstr_case(PROG, "BUSTER")){
            convert_restr_2_standard(fout,type , i, buster_18, 18); 
        } else if(strstr_case(PROG, "TNT")){
            convert_restr_2_standard(fout,type , i, tnt_9, 9); 
        } else if(strstr_case(PROG, "NUCLSQ")){
            convert_restr_2_standard(fout,type , i, nuclsq_17, 17); 
        } else if(strstr_case(PROG, "PROLSQ")){
            convert_restr_2_standard(fout,type , i, prolsq_17, 17); 
        }else
            convert_restr_2_standard(fout,type , i, other_17, 17);
        
        (refine_ls_restr[i].number>-9998)
            ? fprintf(fout, "%d ", refine_ls_restr[i].number) : fprintf(fout,"?  ");
        
        (refine_ls_restr[i].dev_ideal>-9998)
            ? fprintf(fout,"%.3f ", refine_ls_restr[i].dev_ideal): fprintf(fout,"?  ") ;
        
        (refine_ls_restr[i].dev_ideal_target >-9998)
            ? fprintf(fout, "%.3f ", refine_ls_restr[i].dev_ideal_target ) : fprintf(fout,"? ");
        
        (refine_ls_restr[i].weight >-9998)
            ? fprintf(fout, "%.3f ", refine_ls_restr[i].weight ) : fprintf(fout,"? ");
        
        fprintf(fout," %s ", refine_ls_restr[i].pdbx_restraint_function);
        fprintf(fout,"\n");
    }
    
}

void convert_restr_2_standard(FILE *fout, char *type, int k,  const char **pair, int npair)
{    
    char param[512], str[512];
    int id=0, i;
 
    for(i=0;i<npair;i++){
        if(sscanf(pair[i], "%s", param) !=1 ) continue;
        strcpy(str,pair[i] + strlen(param)+1);
        rid_of_front_space(str);
        rid_of_end_space(str);
        
        if(!strcmp_case(str,type)){
            fprintf(fout,"\"%s\" ",param );
            id=1;
            break;
        }
    }
    
    if(id==0) fprintf(fout,"\"%s\" ", type);    
 
 }
